# Migrator.py  -  implements migration in Python

import sys
sys.path.append("/home/data/schiotz/DigitalMaterial/Python")
sys.path.append("/home/data/schiotz/DigitalMaterial/src/DMSystem/alpha/obj3D")
sys.path.append("/home/data/schiotz/DigitalMaterial/src/DMSystem/Parallel/alpha/obj3D")
sys.path.append("T:/tcretegn/CVS/DigitalMaterial/src/MD/WinSWIGBuild")
sys.path.append("T:/tcretegn/CVS/DigitalMaterial/src/DMSystem/")
sys.path[:0] = ["/home/camp/schiotz/python/Scientific"] # New MPI

from cPickle import Pickler, Unpickler
try:
    from DMSystem import ShadowMigrator
except:
    from MD3D import *

from NetBuffers import *
import Numeric

class Migrator(ShadowMigrator):
    """Implements migration."""

    def __init__(self, positions, domain, protocol, synchronizer):
        """Constructs a migrator.

        The arguments are:
        positions:  Position DMArray.  It and its siblings will be migrated.
        domain: A DomainSubdivision.  Decides which atoms belong where.
        protocol: A communications protocol.
        sycronizer: A synchronizer object. It will be informed about migrations.
        """
        self._positions = positions
        self._domain = domain
        self._protocol = protocol
        self._synchronizer = synchronizer
        # Connect with the C++ shadow object
        ShadowMigrator.__init__(self, self.Migrate)
        # Place sanity checks here!

    def Migrate(self):
        """Do the migration."""
        # Which processor are we running on?
        procID = self._protocol.GetProcessorNumber()
        nproc =  self._protocol.GetNumberOfProcessors()
        # Get list of all DMArrays, but only the real atoms
        nchild = self._positions.NumberOfBranches()
        if nchild != 2:
            raise ValueError, "positions should have exactly two subtrees (real and ghost). It has "+str(nchild)
        realpositions = self._positions.GetBranch(0)
        # arrays = self._positions.GetSiblings()
        arrays = []
        for i in xrange(self._positions.GetNumberOfSiblings()):
            arrays.append(self._positions.GetSibling(i))
        # Which atoms go where?
        sendlists = self._domain.BuildSendLists(realpositions, procID)
        assert nproc == len(sendlists)
    
        writebufs = []
        nleaves = realpositions.NumberOfLeaves()
        leafLengths = []
        for leaf in range(nleaves):
            leafLengths.append(len(realpositions.GetLeaf(leaf)))
        for target in xrange(nproc):
            # TargetID is the target process, sendlist is a list of
            # atoms to send
            if target == procID:
                writebufs.append(None)
            else:
                sendlist = sendlists[target]  # List over leaves
                assert len(sendlist) == nleaves
                # Get a write buffer
                writebuffer = NetWriteBuffer()
                writebufs.append(writebuffer)
                wbuf = Pickler(writebuffer, 1)
                for a in arrays:
                    nch = a.NumberOfBranches()
                    if nch != 2:
                        raise ValueError, "DMarray with "+str(nch)+" branches."
                    areal = a.GetBranch(0)
                    aghost = a.GetBranch(1)
                    # The ghosts will become invalid
                    for ghleaf in xrange(aghost.NumberOfLeaves()):
                        aghost.GetLeaf(ghleaf).Clear()
                    # Now write to the buffers
                    wbuf.dump("newarray") # Only used for consistency checks
                    wbuf.dump(len(sendlist))  # Number of leaves.  Superfluous.
                    for leaf in range(len(sendlist)):
                        leaflist = sendlist[leaf]
                        wbuf.dump("newleaf") # Magic, marks new leaf.
                        wbuf.dump(len(leaflist)) # Number of atoms.
                        arrayleaf = areal.GetLeaf(leaf)[:]
                        if len(arrayleaf) != leafLengths[leaf]:
                            raise ValueError, "Inconsistent leaf size among siblings."
                        for atom in leaflist:
                            wbuf.dump(arrayleaf[atom])

        #Do the actual communication
        readbufs = self._protocol.Butterfly(writebufs)

        # Remove atoms.  Make a list of atoms to remove at each leaf
        nleaves = realpositions.NumberOfLeaves()
        remlist = []                # Create empty lists
        for i in xrange(nleaves):
            remlist.append([])
        for sl in sendlists:     # Loop over processors
            assert nleaves == len(sl)
            for leaf in xrange(nleaves):   # Loop over leaves
                leaflist = sl[leaf]
                remlist[leaf].extend(leaflist)  # Add atoms in this leaf sent
                                                # to this processor to the
                                                # removelist for this leaf

        # Do the removes
        for a in arrays:
            areal = a.GetBranch(0)
            for leaf in xrange(nleaves):
                l = areal.GetLeaf(leaf)
                #l.RemoveElements(remlist[leaf])
                remlist[leaf].sort()
                remlist[leaf].reverse()
                for i in remlist[leaf]:
                    l.RemoveElement(i)
                    
        # Add atoms
        for source in readbufs:   # Loop over processors
            if source is None:
                continue          # Skip this processor
            rbuf = Unpickler(source)
            for a in arrays:           # Loop over DMArrays
                areal = a.GetBranch(0)
                assert rbuf.load() == "newarray"
                assert rbuf.load() == nleaves
                for leaf in xrange(nleaves):
                    l = areal.GetLeaf(leaf)
                    assert rbuf.load() == "newleaf"
                    natoms = rbuf.load()  # Number of atoms to add
                    if natoms > 0:
                        n = len(l)            # Number of "old" atoms
                        l.AddElementsAtEnd(natoms)
                        adddata = []
                        for i in xrange(natoms):
                            adddata.append(rbuf.load())
                        l[n:n+len(adddata)] = Numeric.array(adddata)

        # Consistency check
        # NOT IMPLEMENTED: realpositions.ValidateSiblings()
        if self._synchronizer:
            self._synchronizer.Rebuild()

        # Done!


        
###########################################################
#
# TEST SUITE
#
###########################################################

if __name__ == "__main__":
    import sys
#    sys.path.append("/home/data/schiotz/DigitalMaterial/Python")

    from Numeric import *
    from RandomArray import *
    from DMSystem import ArrayDoubleVector
    from DomainSubdivision import *
    from NetProtocol import NetProtocol

    def printarray(text, id, data):
        print "%s (processor %d)" % (text, id)
        for i in range(data.NumberOfLeaves()):
            print "leaf", i
            print data.GetLeaf(i)[:]

    def sign(data, leaf, proc):
        for i in range(len(data)):
            data[i,2] = 500000 + 10000 * proc + 1000 * leaf + i
            
    protocol = NetProtocol()
    seed(3273,protocol.GetProcessorNumber())
    sys.stdout = open("output."+str(protocol.GetProcessorNumber()), "w")
    
    # Make a domain decomposition
    cpu = (2,2,1)
    ncpu = cpu[0] * cpu[1] * cpu[2]
    basis = identity(3) * array((10.0, 10.0, 10.0))
    domain = DomainSubdivision(basis, cpu, 1.0)
    
    # Make a "positions" array.
    positions = ArrayDoubleVector()
    a = ArrayDoubleVector()
    a1 = ArrayDoubleVector(10)
    pos = matrixmultiply(random((10,3)), basis)
    sign(pos, 0, protocol.GetProcessorNumber())
    a1[:] = pos
    a.Attach(a1)
    a2 = ArrayDoubleVector(5)
    pos = matrixmultiply(random((5,3)), basis)
    sign(pos, 1, protocol.GetProcessorNumber())
    a2[:] = pos        
    a.Attach(a2)
    a1.thisown = a2.thisown = 0
    # Ghosts:
    g = ArrayDoubleVector()
    g1 = ArrayDoubleVector(1)
    g1.Zero()
    g.Attach(g1)
    g2 = ArrayDoubleVector(1)
    g2.Zero()
    g.Attach(g2)
    g1.thisown = g2.thisown = 0
    positions.Attach(a)
    positions.Attach(g)
    a.thisown = g.thisown = 0
    print "a, leaves:", a.NumberOfLeaves()
    print "g, leaves:", g.NumberOfLeaves()
    print "positions, leaves:", positions.NumberOfLeaves()

    assert positions.NumberOfBranches() == 2
    assert positions.GetBranch(0).NumberOfLeaves() == 2
    
    # Make the migrator
    migrator = Migrator(positions, domain, protocol, None)

    printarray("Positions before migration", protocol.GetProcessorNumber(),
               positions.GetBranch(0))
    migrator.Migrate()
    printarray("Positions after migration", protocol.GetProcessorNumber(),
               positions.GetBranch(0))
    print
    ar = positions.GetBranch(0)
    pID =  protocol.GetProcessorNumber()
    print '@@@@@@@@@@', ar.NumberOfLeaves()
    for i in range(ar.NumberOfLeaves()):
        belong = domain.PointsInDomain(ar.GetLeaf(i)[:], pID)
        ok = logical_and.reduce(belong)
        if ok:
            print "Processor %d, leaf %d: OK" % (pID, i)
        else:
            print "Processor %d, leaf %d: FAILED ***********" % (pID, i)

    sys.stdout.close()
    protocol._comm.barrier()
    
