import sys
sys.path.append('../../src/QuasiContinuum/SWIGBuild')
sys.path.append('../../src/QuasiContinuum/PythonScripts')

from Numeric import *
import QC3D
from Detri import *
from plot import *
from Delaunay import *
from Thompson import Thompson_110

potential = QC3D.QCEMTPotential("Cu")
d = potential.GetLengthScale()
a = d * sqrt(2)
rc = d * (sqrt(4) + sqrt(3)) / 2
rc1 = d * (sqrt(2) + sqrt(1)) / 2
(A, B, C, D, alpha, beta, gamma, delta) = Thompson_110(a)
lat = array([B - A, C - A, D - A])
lattice = QC3D.GeneralLattice()
lattice.SetLatticeVectors(lat)
N = 3
lattice2 = QC3D.GeneralLattice()
lattice2.SetLatticeVectors(N * lat)

r1 = 1.1 * a
r2 = 2.1 * a
r3 = 6.1 * a

# nonlocal atoms:
nonLocalAtoms = QC3D.PrimitiveListOfAtoms()
QC3D.SphericalClusterInitializer(r2, lattice).Create(nonLocalAtoms)

# local atoms: (vertices)
localAtoms = QC3D.PrimitiveListOfAtoms()
QC3D.SphericalClusterInitializer(r3, lattice2).Create(localAtoms)
QC3D.OverlapPruner(0.1, nonLocalAtoms.GetCartesianPositionsPtr()).Transform(
    localAtoms)

# create vacancy island:
island = QC3D.PrimitiveListOfAtoms()
i1 = QC3D.SphericalClusterInitializer(r1, lattice)
i2 = QC3D.RectangularClusterInitializer([2 * r1, 2 * r1, d / 2], lattice)
i3 = QC3D.CompositeClusterInitializer(i1, i2, lattice)
i3.Create(island)
del(i1, i2, i3)

QC3D.OverlapPruner(0.1, island.GetCartesianPositionsPtr()).Transform(
    nonLocalAtoms)


masters = QC3D.QCListOfAtoms(nonLocalAtoms, localAtoms)

nl = QC3D.CellNeighborList(rc, 0.05 * rc)
masters.SetNeighborLocator(nl)
tetrahedra = DetriMesh(masters)

meshData = QC3D.MeshData()
for tetrahedron in tetrahedra:
    if sort(tetrahedron)[3] >= nonLocalAtoms.GetNumber():
	meshData.CreateElement(tetrahedron, masters, lattice)

meshData.InitializeSlaves(masters)
print "ok"
potential.SetMeshData(meshData)
print "ok"

e0 = potential.CalculateEnergy(masters) 
print "E0: ", e0

def relax(atoms, potential, tolerance = 1e-2):
    e = potential.CalculateEnergy(atoms)
    mover = QC3D.QuickMinAtomsMover(potential, 0.04, tolerance)
    mover.Verbose()
    mover.SetMaxIterations(5)
    mover.Move(atoms)
    mover.SetDt(0.15)
    mover.SetMaxIterations(100)
    while mover.GetFinalTFS() > tolerance:
	mover.Move(atoms)
	de = potential.CalculateEnergy(atoms) - e
	e = e + de
	print mover.GetFinalTFS(), e, de
    return potential.CalculateEnergy(atoms)

relax(masters, potential)
