#!/usr/bin/python

from Numeric import *
import QC3D
from plot import *
from Thompson import Thompson_LC
import os
from Boundary import *
from IPAF import *

potential = QC3D.QCEMTPotential("Cu")
d = potential.GetLengthScale()
a = d * sqrt(2)
rc = d * (sqrt(4) + sqrt(3)) / 2
rc1 = d * (sqrt(1) + sqrt(2)) / 2
N = 3
Nz = 4 * N  # 10?
n1 = 5
L1 = 6
L2 = 16
LL1 = 3 + L1
LL2 = 2 + L2
N1 = 5 * N
(A, B, C, D, alpha, beta, gamma, delta) = Thompson_LC(a)
Lz = Nz * abs(B[2])
e1 = (D - gamma) * 3.0 / 2.0
e2 = (C - delta) * 3.0 / 2.0
lattice = QC3D.GeneralLattice()
lat = array([B - A, C - A, D - A])
lattice.SetLatticeVectors(lat)

corners = array([A,
		 (2 * n1 + 1) * e1,
		 (2 * n1 + 1) * e1 + (2 * n1 + 1 + LL2) * e2,
		 (2 * n1 + 1 + LL2) * e2])
corners = concatenate((corners, corners + Nz * B)) - beta
nonLocals = QC3D.PrimitiveListOfAtoms()
QC3D.PolyClusterInitializer(corners, lattice).Create(nonLocals)
nNonLocals1 = nonLocals.GetNumber()

corners = array([(2 * n1 + 1) * e1,
		 (2 * n1 + 1 + LL1) * e1, 
		 (2 * n1 + 1 + LL1) * e1 + (2 * n1 + 1) * e2,
		 (2 * n1 + 1) * e1 + (2 * n1 + 1) * e2])
corners = concatenate((corners, corners + Nz * B)) - beta
tmp = QC3D.PrimitiveListOfAtoms()
QC3D.PolyClusterInitializer(corners, lattice).Create(tmp)
#nonLocals.Merge(tmp)
nNonLocals2 = nonLocals.GetNumber()

## corners = array([(2 * n1 + 1) * (e1 + e2),
## 		 (2 * n1 + 1) * (e1 + e2) + e1,
## 		 (2 * n1 + 1) * (e1 + e2) + e1 + e2,
## 		 (2 * n1 + 1) * (e1 + e2) + e2])
## corners = concatenate((corners, corners + Nz * B)) - beta
## tmp = QC3D.QCListOfAtoms()
## QC3D.PolyClusterInitializer(corners, lattice).Create(tmp)
## atoms.Merge(tmp)

lattice2 = QC3D.GeneralLattice()
lattice2.SetLatticeVectors(N * lat)
## corners = array([-N1 * (e1 + e2), -N1 * e2 + (N1 + 2 * n1 + LL1) * e1,
## 		 (N1 + 2 * n1 + LL1) * e1 + (N1 + 2 * n1 + LL2) * e2,
## 		 -N1 * e1 + (N1 + 2 * n1 + LL2) * e2])
c0 = -N1 * (e1 + e2)
x0, y0 = c0[0], c0[1]
x1 = x0 + (C - delta)[0] / 2 * 3 * N * 24
y1 = y0 - (delta - D)[1] * 3 * N * 6
corners = array([c0, [x0, y1, 0.0], [x1, y1, 0.0], [x1, y0, 0.0]])
corners = concatenate((corners, corners + Nz * B)) - beta
locals = QC3D.PrimitiveListOfAtoms()
QC3D.PolyClusterInitializer(corners, lattice2).Create(locals)
QC3D.OverlapPruner(0.1, nonLocals.GetCartesianPositionsPtr()).Transform(locals)

masters = QC3D.QCListOfAtoms(nonLocals, locals)

b1a = gamma - A
b1b = D - gamma
b2a = A - delta
b2b = delta - C
b0 = D - C
line = B - A
center = (n1 - 0.75) * (e1 + e2) + e1
center1a = center + 0.1 * e1
center1b = center + 1 * e1
center2a = center + 0.1 * e2
center2b = center + 2 * e2
nu = 0.322387
## disl = QC3D.DislocationMaker(b0, line, center, nu)
## disl.Transform(masters)
## disl = QC3D.DislocationMaker(b1b, -line, center, nu)
## disl.Transform(masters)
## disl = QC3D.DislocationMaker(b1b, line, center + L1 * b1b, nu)
## disl.Transform(masters)
## disl = QC3D.DislocationMaker(b2b, -line, center, nu)
## disl.Transform(masters)
## disl = QC3D.DislocationMaker(b2b, line, center - L2 * b2b, nu)
## disl.Transform(masters)

bc = QC3D.SmartPeriodicBoundaryConditions([0, 0, 1])
bc.SetLength([0.0, 0.0, Lz])
masters.SetBoundaryConditions(bc)
#nl = QC3D.CellNeighborList(rc, 0.05 * rc)
nl = QC3D.CellNeighborLocator(rc, 0.05 * rc)
masters.SetNeighborLocator(nl)
tetrahedra = IPAFMesh(masters, array([[x1 - x0, 0.0, 0.0],
				      [0.0, y0 - y1, 0.0],
				      [0.0, 0.0, Lz]]), [0, 0, 1],
		      N * 2 * a)

meshData = QC3D.MeshData()

for tetrahedron in tetrahedra:
##     t = sort(tetrahedron)
##     if not (t[3] < nNonLocals1 or 
## 	    (t[0] >= nNonLocals1 and t[3] < nNonLocals2)):
	meshData.CreateElement(tetrahedron, masters, lattice)

meshData.InitializeSlaves(masters)

potential.SetMeshData(meshData)


e0 = potential.CalculateEnergy(masters) 
stop
def relax(masters, potential, tolerance = 1e-5):
    e0 = potential.CalculateEnergy(masters)
    mover = QC3D.QuickMinAtomsMover(potential, 
				    0.04, tolerance)
    mover.SetMaxIterations(5)
    mover.Move(masters)
    mover.SetDt(0.15)
    mover.SetMaxIterations(100)
    while mover.GetFinalTFS() > tolerance:
	mover.Move(masters)
	e = potential.CalculateEnergy(masters)
	print mover.GetFinalTFS(), e, e - e0
    return potential.CalculateEnergy(masters)

relax(masters, potential, 0.005)
masters.Write("i.coord")
stop
for x in [0.1, 0.01, 0.005, 0.002, 0.001, 0.0005]:
    relax(masters, potential, x)
    masters.Write("i.coord")
