from Numeric import *
import QC3D
from plot import *
from Thompson import Thompson_LC
import os
from Boundary import *

potential = QC3D.QCEMTPotential("Cu")
d = potential.GetLengthScale()
a = d * sqrt(2)
rc = d * (sqrt(5) + sqrt(4)) / 2
rc1 = d * (sqrt(1) + sqrt(2)) / 2
N = 3
Nz = 4 * N  # 10?
n1 = 5
L1 = 4
L2 = int(L1 * 3.82 + 0.5)
LL1 = 9 + L1
LL2 = 9 + L2
N1 = 10 * N
(A, B, C, D, alpha, beta, gamma, delta) = Thompson_LC(a)
Lz = Nz * abs(B[2])
e1 = (D - gamma) * 3.0 / 2.0
e2 = (C - delta) * 3.0 / 2.0
lattice = QC3D.GeneralLattice()
lat = array([B - A, C - A, D - A])
lattice.SetLatticeVectors(lat)

corners = array([A, 2 * n1 * e1, 2 * n1 * e1 + (2 * n1 + LL2) * e2,
	      (2 * n1 + LL2) * e2])
corners = concatenate((corners, corners + Nz * B)) - beta
atoms = QC3D.QCListOfAtoms()
QC3D.PolyClusterInitializer(corners, lattice).Create(atoms)

corners = array([2 * n1 * e1, (2 * n1 + LL1) * e1, 
		 (2 * n1 + LL1) * e1 + 2 * n1 * e2, 2 * n1 * e1 + 2 * n1 * e2])
corners = concatenate((corners, corners + Nz * B)) - beta
tmp = QC3D.QCListOfAtoms()
QC3D.PolyClusterInitializer(corners, lattice).Create(tmp)
atoms.Merge(tmp)

nAtomic = atoms.GetNumber()

lattice2 = QC3D.GeneralLattice()
lattice2.SetLatticeVectors(N * lat)
corners = array([-N1 * (e1 + e2), -N1 * e2 + (N1 + 2 * n1 + LL1) * e1,
		 (N1 + 2 * n1 + LL1) * e1 + (N1 + 2 * n1 + LL2) * e2,
		 -N1 * e1 + (N1 + 2 * n1 + LL2) * e2])
corners = concatenate((corners, corners + Nz * B)) - beta
tmp = QC3D.QCListOfAtoms()
QC3D.PolyClusterInitializer(corners, lattice2).Create(tmp)
QC3D.OverlapPruner(0.1, atoms.GetCartesianPositionsPtr()).Transform(tmp)
atoms.Merge(tmp)

file = open("rm.af3d", 'w')
R = atoms.GetCartesianPositions()[:]
file.write("%d %d\n" % (nAtomic, len(R)))
for i in range(3):
    for c in range(3):
	file.write("%f " % (N * lattice.GetLatticeVector(i, c)))
file.write("\n%f %f %d\n" % (a * N * 1.5, Lz, N))
for r in R:
    file.write("%f %f %f\n" % tuple(r))
b1a = gamma - A
b1b = D - gamma
b2a = A - delta
b2b = delta - C
line = B - A
center = (n1 - 0.5) * (e1 + e2)
center1a = center + 2 * e1
center1b = center + L1 * e1
center2a = center + 2 * e2
center2b = center + L2 * e2
nu = 0.322387
disl = QC3D.DislocationMaker(b1a, line, center1a, nu)
disl.Transform(atoms)
disl = QC3D.DislocationMaker(b1b, line, center1b, nu)
disl.Transform(atoms)
disl = QC3D.DislocationMaker(b2a, line, center2a, nu)
disl.Transform(atoms)
disl = QC3D.DislocationMaker(b2b, line, center2b, nu)
disl.Transform(atoms)
R = atoms.GetCartesianPositions()[:]
for r in R:
    file.write("%f %f %f\n" % tuple(r))
file.close()

os.system("qcipaf < rm.af3d > rm4")

bc = QC3D.SmartPeriodicBoundaryConditions([0, 0, 1])
bc.SetLength([0.0, 0.0, Lz])
atoms.SetBoundaryConditions(bc)
atoms.DecorateWithSlaves(nAtomic, rc, lattice, Lz, "rm4")
e0 = potential.CalculateEnergy(atoms) 

def relax(atoms, potential, tolerance = 1e-5):
    e0 = potential.CalculateEnergy(atoms)
    mover = QC3D.QuickMinAtomsMover(potential, 
				    0.04, tolerance)
    mover.SetMaxSteps(5)
    mover.Move(atoms)
    mover.SetDt(0.15)
    mover.SetMaxSteps(100)
    while mover.GetFinalTFS() > tolerance:
	mover.Move(atoms)
	e = potential.CalculateEnergy(atoms)
	print mover.GetFinalTFS(), e, e - e0
    return potential.CalculateEnergy(atoms)


m= atoms.GetMeshData()
potential.Grrr(-1, 0)
adata = zeros(nAtomic, Float)
for i in range(nAtomic):
    adata[i] = potential.Grrr(i, 1)

nElements = m.GetNumberOfElements()
edata = zeros(nElements, Float)
for i in range(nElements):
    edata[i] = potential.CalculateElementStress(atoms, i)[1]


m.Cut(array((0.0, 0.0, -10.0)), array((12.0, 0.0, 0.0)), array((0.0, -12.0, 0.0)),20,10, 30, adata, edata)
stop
m.Cut(array((-1.0, 1.0, 1.0)), array((3.0, 0.0, 0.0)), array((0.0, 0.0, 3.0)), 15,15, 30,ones(5177,Float),ones(3534,Float))
#mplot(atoms)
#cnaplot(atoms,rc1)
relax(atoms, potential)
