import sys
sys.path.append('../../src/QuasiContinuum/SWIGBuild')
sys.path.append('../../src/QuasiContinuum/PythonScripts')

from Numeric import *
import QC3D
from plot import *
from Delaunay import *
from Thompson import Thompson_110

potential = QC3D.QCEMTPotential("Cu")
d = potential.GetLengthScale()
a = d * sqrt(2)
rc = d * (sqrt(5) + sqrt(4)) / 2
rc1 = d * (sqrt(2) + sqrt(1)) / 2
(A, B, C, D, alpha, beta, gamma, delta) = Thompson_110(a)
lat = array([B - A, C - A, D - A])
lattice = QC3D.GeneralLattice()
lattice.SetLatticeVectors(lat)
N = 3
lattice2 = QC3D.GeneralLattice()
lattice2.SetLatticeVectors(N * lat)
dx = a / 2
dy = d / 2
dz = dy
Lx = (2 * 2 * N + 1) * dx
Ly = (2 * 2 * N + 1) * dy
Lz = (4 * 2 * N + 0) * dz
Lx2 = (5 * 2 * N + 1) * dx
Ly2 = (6 * 2 * N + 1) * dy
Lz2 = (8 * 2 * N + 1) * dz

atoms = QC3D.QCListOfAtoms()
QC3D.RectangularClusterInitializer([Lx, Ly, Lz], lattice).Create(atoms)
nAtomic = atoms.GetNumber()

tmp = QC3D.QCListOfAtoms()
QC3D.RectangularClusterInitializer([Lx2, Ly2, Lz2], lattice2).Create(tmp)
QC3D.OverlapPruner(0.1, atoms.GetCartesianPositionsPtr()).Transform(tmp)
atoms.Merge(tmp)

file = open("rm.af3d", 'w')
R = atoms.GetCartesianPositions()[:]
file.write("%d %d\n" % (nAtomic, len(R)))
for i in range(3):
    for c in range(3):
	file.write("%f " % (N * lattice.GetLatticeVector(i, c)))
file.write("\n%f %f %d\n" % (a * N * 1.5, -Lz, N))
for r in R:
    file.write("%f %f %f\n" % tuple(r))
R = atoms.GetCartesianPositions()[:]
for r in R:
    file.write("%f %f %f\n" % tuple(r))
file.close()
#stop
os.system("qcipaf < rm.af3d > rm4")
#stop

atoms.DecorateWithSlaves(nAtomic, 1.0 * rc, lattice, Lz)
e0 = potential.CalculateEnergy(atoms) 
print "E0: ", e0
stop
def relax(atoms, potential, tolerance = 1e-2):
    e = potential.CalculateEnergy(atoms)
    mover = QC3D.QuickMinAtomsMover(potential, 0.04, tolerance)
    mover.SetMaxSteps(5)
    mover.Move(atoms)
    mover.SetDt(0.15)
    mover.SetMaxSteps(100)
    while mover.GetFinalTFS() > tolerance:
	mover.Move(atoms)
	de = potential.CalculateEnergy(atoms) - e
	e = e + de
	print mover.GetFinalTFS(), e, de
    return potential.CalculateEnergy(atoms)
m= atoms.GetMeshData()
stop
m.Cut(array((0.0, 0.0, 0.0)), array((1.0, 0.0, 0.0)), array((0.0, 1.0, 0.0)), 10,10, 30,ones(5177,Float),ones(3534,Float))
m.Cut(array((-1.0, 1.0, 1.0)), array((3.0, 0.0, 0.0)), array((0.0, 0.0, 3.0)), 15,15, 30,ones(5177,Float),ones(3534,Float))
#mplot(atoms)
#cnaplot(atoms,rc1)
relax(atoms, potential)
