// ASCIIWriter.cpp: implementation of the ASCIIWriter class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"  // because of the STL warnings under Windows

#include "ASCIIWriter.h"

#include "DMLastHeader.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

using namespace std;

ASCIIWriter::ASCIIWriter(const std::string &filename) : DMWriter(filename)
{
	// Create a new empty file;
	out.open(filename.c_str(), std::ios::out);
	out.close();
}

ASCIIWriter::~ASCIIWriter()
{

}

void ASCIIWriter::Open()
{
	out.open(filename.c_str(), std::ios::app);
}

void ASCIIWriter::Close()
{
	out.close();
}

void ASCIIWriter::Begin(const std::string &type, const std::string &name)
{
	if(offset == "") Open();
	out << offset << type << '\t' << name <<" {"<< endl;
	offset += "    ";
}

void ASCIIWriter::End()
{
	offset.resize(offset.length()-4);
	out << offset << "}" << endl;
	if(offset.length() == 0) Close();
}

void ASCIIWriter::Put(const Serializable *serializable, const string &name)
{
	Begin(serializable->GetType(), name);
	serializable->Save(this);
	End();
}

void ASCIIWriter::PutBool( bool b, const string& name )
{
	int iBool = (b) ? 1 : 0;
	PutInt( iBool, name );
}

void ASCIIWriter::PutInt(int n, const string &name)
{
	Begin("int", name);
	out << offset << n << endl;
	End();
}

void ASCIIWriter::PutDouble(double x, const string &name)
{
	Begin("double", name);
	out << offset << x << endl;
	End();
}

void ASCIIWriter::PutString(const string &str, const string &name)
{
	Begin("string", name);
	out << offset << str << endl;
	End();
}

void ASCIIWriter::PutIntArray(const int *data, const vector<int> &dims, const string &name)
{
	int tot = 1;
	for(unsigned int ii=1; ii<dims.size(); ++ii) tot *= dims[ii];
	Begin("intArray", name);
	if(dims.size() > 0) out << offset << dims[0];
	for(unsigned int k=1; k<dims.size(); ++k) out << "\t" << dims[k];
	out << endl;
	if(dims.size() != 0)
	{
		for(int i=0; i<dims[0]; ++i)
		{
			out << offset ;
			for(int j=0; j<tot; ++j) out << *data++ << " " ;
			out << endl;
		}
	}
	End();
}

void ASCIIWriter::PutDoubleArray(const double *data, const vector<int> &dims, const string &name)
{
	int tot = 1;
	for(unsigned int ii=1; ii<dims.size(); ++ii) tot *= dims[ii];
	Begin("doubleArray", name);
	if(dims.size() > 0) out << offset << dims[0];
	for(unsigned int k=1; k<dims.size(); ++k) out << "\t" << dims[k];
	out << endl;
	if(dims.size() != 0)
	{
		for(int i=0; i<dims[0]; ++i)
		{
			out << offset ;
			for(int j=0; j<tot; ++j) out << *data++ << " " ;
			out << endl;
		}
	}
	End();
}
