// BinaryWriter.cpp: implementation of the BinaryWriter class.
//
//////////////////////////////////////////////////////////////////////

#include "BinaryWriter.h"
#include "Serializable.h"
#include "DMLastHeader.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BinaryWriter::BinaryWriter(): DMWriter("")
{
    this->buffer      = 0;
    this->bufferSize  = 0;
    this->currentSize = 0;
    // lets say 10000 is enough to start with. It will be extended if necessary
    Allocate(10000);
}

BinaryWriter::~BinaryWriter()
{
    if (buffer) delete [] buffer;
}


void BinaryWriter::Allocate(int newSize)
{
    char *newBuffer = new char[newSize];
    if(this->buffer)
    {
        memcpy(newBuffer, buffer, bufferSize*sizeof(char));
        delete [] this->buffer;
    }
    this->buffer     = newBuffer;
    this->bufferSize = newSize;
}



void BinaryWriter::Put(const Serializable *serializable, const string &name)
{
    PutString(serializable->GetType(),"");
    serializable->Save(this);
}

void BinaryWriter::PutBool( bool b, const string& name )
{
    if(b) PutInt(1,"");
    else PutInt(0,"");
}

void BinaryWriter::PutInt(int n, const string &name)
{
    const int need = sizeof(int);
    if(need + currentSize > bufferSize)
        Allocate(bufferSize + 1024*need);
    memcpy(buffer+currentSize, &n, need);
    currentSize += need;
}

void BinaryWriter::PutDouble(double x, const string &name)
{
    const int need = sizeof(double);
    if(need + currentSize > bufferSize)
        Allocate(bufferSize + 1024*need);
    memcpy(buffer+currentSize, &x, need);
    currentSize += need;
}

void BinaryWriter::PutString(const string &str, const string &name)
{
    const int need = (1+str.size()) * sizeof(char);
    if(need + currentSize > bufferSize)
	Allocate(bufferSize + need);
    ASSERT(str.c_str()[str.size()] == 0);
    memcpy(buffer+currentSize, str.c_str(), need);
    currentSize += need;
}

void BinaryWriter::PutIntArray(const int *data,
                               const vector<int> &dims, 
                               const string &name)
{
    const int nDims = dims.size();
    PutInt(nDims,"");
    int tot = 1;
    for(int ii=0; ii<nDims; ++ii)
    {
        PutInt(dims[ii],"");
        tot *= dims[ii];
    }
    const int need = tot*sizeof(int);

    if(need + currentSize > bufferSize) Allocate(bufferSize + need);

    memcpy(buffer+currentSize, data, need);
    currentSize += need;
}

void BinaryWriter::PutDoubleArray(const double *data,
				  const vector<int> &dims, 
                                  const string &name)
{
    const int nDims = dims.size();
    PutInt(nDims,"");
    int tot = 1;
    for(int ii=0; ii<nDims; ++ii)
    {
        PutInt(dims[ii],"");
        tot *= dims[ii];
    }
    const int need = tot*sizeof(double);

    if(need + currentSize > bufferSize) Allocate(bufferSize + need);

    memcpy(buffer+currentSize, data, need);
    currentSize += need;
}
