#include "DMFirstHeader.h"
#include <iostream>
#include "DMException.h"
#include "DMLastHeader.h"

/*!	Convenient way to make an informative exception.

	\param sClass The class throwing the exception.
	\param sFunction The member function which throws it.
	\param sMessage What the exception is.
	*/
DMException::DMException( const std::string& sClass, const std::string& sFunction,
						 const std::string& sMessage ) :
sClass(sClass), sFunction(sFunction), sMessage(sMessage) {}

/*!	Derived classes may prefer this interface, where they can set the
	message themselves.

	\param sClass The class throwing the exception.
	\param sFunction The member function which throws it.
	*/
DMException::DMException( const std::string& sClass, const std::string& sFunction ) :
sClass(sClass), sFunction(sFunction) {}

/*! We have this just so the destructor can be virtual. */
DMException::~DMException() {}

/*! This returns a copy of the string so we don't have to hold
  on to storage inside.
*/
std::string DMException::GetString( ) const
{
	std::string msg(sClass);
	msg.append("::");
	msg.append(sFunction);
	msg.append(":  ");
	msg.append(sMessage);
	return msg;
}

/*! There is no way to make an operator<< virtual, so you cannot override it
	in base classes.  Instead, we make a single ostream<< for all classes
	derived from DMException, but all it does is call our virtual put function.
	That way, subclasses can choose what to print on error.
	*/
std::ostream& DMException::Put(std::ostream& s) const
{
	return s << "Exception in " << this->GetString() << std::endl;
}

//! This operator permits us to write a DMException to a stream.
/*! In order to override the printing behavior of DMException in
	a subclass, don't try to redefine this operator because you can't.
	Instead, supply a new virtual method put(std::ostream& s) in the
	subclass.

	\param s Is any kind of ostream.
	\param r Is anything derived from DMException.
	\return Returns the stream.  Standard signature of operator<<.
	*/
std::ostream& operator<<(std::ostream& s, const DMException& r)
{
	return r.Put(s);
}
