#ifndef _DMEXCEPTION_H_
#define _DMEXCEPTION_H_

#include "DMFirstHeader.h"
#include <iostream>
#include <string>
#include "DigitalMaterial.h"

//! Base exception class providing basic ostream services.
/*! This exception stores the class and function that threw it
	and any message you supply.  It also prints itself.  It does
	not know how to collect a stack trace or anything of that sort.

	We need to teach SWIGged modules to catch this exception in python.

	\date 15 Sept 2000
	\author ajd27
	\ingroup dmsystem
	*/
class DMException
{
protected:
	std::string sClass; //!< Class that threw exception.
	std::string sFunction; //!< Function where thrown.
	std::string sMessage; //!< Whatever you want to say.
public:
	//! Constructor
	DMException( const std::string& sClass, const std::string& sFunction,
		const std::string& sMessage );
	//! virtual destructor.
	virtual ~DMException();
	//! Get a string describing the error.
	virtual std::string GetString( ) const;
	//! Permit ostream to write us.
	friend std::ostream& operator<<(std::ostream& s, const DMException& r);
protected:
	//! Constructor
	DMException( const std::string& sClass, const std::string& sFunction );

private:
	//! Write message to ostream.
	virtual std::ostream& Put(std::ostream& s) const;
};


#endif //_DMEXCEPTION_H_
