// Protocol.h: interface for the Protocol class.
//
//////////////////////////////////////////////////////////////////////

#ifndef DMPROTOCOL
#define DMPROTOCOL

#include "DMFirstHeader.h"
#include <vector>
#include <string>
#include "DigitalMaterial.h"
#include "DMReader.h"
#include "DMWriter.h"

class DMProtocol;

extern DMProtocol *dmProtocol;

class DMProtocol  
{
protected:
    DMProtocol() {}
public:
    virtual ~DMProtocol() {}
    
    virtual int GetProcessorNumber() const = 0;
    virtual int GetNumberOfProcessors() const = 0;
    
    virtual void Broadcast(double *values, int size, int master) = 0;
    virtual void Broadcast(int *values, int size, int master) = 0;
    virtual void Broadcast(std::string &string, int master) = 0;
    
    virtual void Barrier() = 0;
    
    virtual std::vector<int> GatherValue(int number, int root) = 0;
    
    
    enum Operation{SUM, MAX, MIN} ;
    virtual void AllReduce(const double *in, double *out, int size, Operation op) = 0;
    virtual void AllReduce(const int *in   , int    *out, int size, Operation op) = 0;

    class Buffers
    {
    public:
        virtual ~Buffers() {}
        //! an alltoall communication using buffers.
        /*!Useful for mixed-type data and/or variable length messages
        */
        virtual void AllToAll() = 0;
        virtual DMWriter *GetSendBuffer(int proc) = 0;
        virtual DMReader *GetRecvBuffer(int proc) = 0;
    protected:
        Buffers() {}
    };

    virtual DMProtocol::Buffers *NewBuffers() = 0;
};
    

#endif // !defined PROTOCOL_H
