// DMReader.h: interface for the DMReader class.
//
//////////////////////////////////////////////////////////////////////

#ifndef DMREADER_H
#define DMREADER_H

#include "DMFirstHeader.h"
#include <string>
#include <vector>
#include "DigitalMaterial.h"
#include "Serializable.h"

class DMReader
{
public:
    DMReader(const std::string &filename) : filename(filename) {}
    virtual ~DMReader() {}
    virtual void Open() = 0;
    virtual void Close() = 0;
    virtual std::string GetType(const std::string &name="root") = 0; 
    virtual bool Fill(Serializable *toFill, const std::string &name="root") = 0;
    virtual Serializable *Get(const std::string &name="root") = 0;
    virtual bool GetBool( const std::string& name ) = 0;
    virtual int GetInt(const std::string &name) = 0;
    virtual double GetDouble(const std::string &name) = 0;
    virtual std::string GetString(const std::string &name) = 0;
    virtual int *GetIntArray(std::vector<int> &dims, const std::string &name) = 0;
    virtual double *GetDoubleArray(std::vector<int> &dims, const std::string &name) = 0;

    //! Returns the number of elements to allocate.
    /*! If you call this with a shape which has zero size, it will throw
        an exception.
    */
    static size_t DimsToSize( const std::vector<int>& dims )
    {
        std::vector<int>::const_iterator begin;
        
        size_t len = 1;
        for ( begin = dims.begin(); begin < dims.end(); begin++ )
            len *= *begin;
        
        return len;
    }


    template <class Type> 
        void GetArray(std::vector<int> &dims, const std::string &name, Type *&ip)
    { throw DMException("DMReader", "GetArray<>", "This type not implemented yet");}
    void GetArray(std::vector<int> &dims, const std::string &name, int *&ip)
                { ip = GetIntArray(dims, name);}
    void GetArray(std::vector<int> &dims, const std::string &name, double *&xp)
                { xp = GetDoubleArray(dims, name);}

protected:
    std::string filename;
};


#endif // DMREADER_H
