%module DMSystem    ###_DMSystem

%{
#include "DMFirstHeader.h"
#include <fstream>
#include <vector>
#include <string>
#include "Numeric/arrayobject.h"
#include "AttributeError.h"  // Should be included in the subsidiary .i files.

#ifdef _WINDOWS
#include "WinProcessFactory.h"
#else
#include "UnixProcessFactory.h"
#endif

%}

%init
%{
#ifdef _WINDOWS
	WinProcessFactory::Create();
#else
	UnixProcessFactory::Create();
#endif
	import_array();
%}

%include DigitalMaterial.h

#%include typemaps.i #seems to be necessary for stringTypeMap to be wrapped properly
%include DimensionArrayTypeMap.i
%include stringTypeMap.i

%include PythonListArrayChecker.i

### EXCEPTIONS ###
%include DMException.i


### WRITERS / READERS ###
class DMWriter;

%include DMReader.i

%include NetCDFWriter.i
%include NetCDFReader.i

%include ASCIIWriter.i
%include ASCIIReader.i

### SERIALIZABLE ###
%nodefault Serializable;
class Serializable {
public:
    std::string GetType();
	void Save(DMWriter *writer);
	void Load(DMReader *reader);
};


### RANDOM NUMBER GENERATOR ###
%include Rand.i

### MINIMIZERS ###
%{
#include "ConjugateGradientMinimizer.h"
#include "SteepestDescentMinimizer.h"
%}

%nodefault Minimizer;
class Minimizer{};

%include "ConjugateGradientMinimizer.h"
%include "SteepestDescentMinimizer.h"


### INTER-PROCESSES COMMUNICATION PROTOCOL   ###
%{
#include "DMProtocol.h"
%}

%nodefault DMProtocol;
class DMProtocol 
{
public:
    int GetProcessorNumber() const ;
    int GetNumberOfProcessors() const ;
    void Barrier();
};

DMProtocol *dmProtocol;


### PARALLEL ###

#ifdef DMPARALLEL
%include "Parallel/Parallel.i"
#endif
