// LocalProtocol.h: interface for the LocalProtocol class.
//
//////////////////////////////////////////////////////////////////////

#ifndef LOCALPROTOCOL_H
#define LOCALPROTOCOL_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"

#include "DMProtocol.h"
#include "BinaryReader.h"
#include "BinaryWriter.h"

#include "DMException.h"

class LocalProtocol : public DMProtocol
{
public:
    LocalProtocol() {}
    virtual ~LocalProtocol() {}
    
    int GetProcessorNumber() const {return 0;}
    int GetNumberOfProcessors() const {return 1;}
    
    void Broadcast(double *values, int size, int master) {}
    void Broadcast(int *values, int size, int master) {}
    void Broadcast(std::string &string, int master) {}
    
    void Barrier() {}
    
    std::vector<int> GatherValue(int number, int root) {return std::vector<int>(1,number);}
        
    void AllReduce(const double *in, double *out, int size, Operation op);
    void AllReduce(const int *in   , int    *out, int size, Operation op);
    
    Buffers *NewBuffers() {return new BinBuffers();}

    class BinBuffers : public DMProtocol::Buffers
    {
    public:
        BinBuffers();
        ~BinBuffers();
        void AllToAll();
        DMWriter *GetSendBuffer(int proc){return sendBuffer;}
        DMReader *GetRecvBuffer(int proc){return recvBuffer;}
    private:
        //! Initialize the private members
        void Initialize();
        BinaryWriter *sendBuffer;
        BinaryReader *recvBuffer;
    };
};



#endif // !defined(LOCALPROTOCOL_H)
