// Minimizable.h: interface for the Minimizable class.
//
//////////////////////////////////////////////////////////////////////

#ifndef MINIMIZABLE_H
#define MINIMIZABLE_H
#include "DMFirstHeader.h"

#include "DigitalMaterial.h"
#include "AttributeError.h"

//! This class provides the functions needed by Minimizer classes. ObjectiveFunction is the function being minimized and must always be provided. Gradient and Hessian must be provided if needed by the Minimizer algorithm, otherwise an exception is thrown.

/**
 *
 * \author nbailey 
 * \date 6/11/2000
 * \ingroup moleculardynamics
 */


class Minimizable
{
 public:
  
  virtual double ObjectiveFunction(double *parameters)=0;
  virtual  double *Gradient(double *parameters) {
	  throw AttributeError("Minimizable","Gradient");
  }
  virtual double **Hessian(double *parameters) {
	  throw AttributeError("Minimizable","Hessian");
  }
  virtual int GetNParameters()=0;

};



#endif // MINIMIZABLE_H
