// NetCDFReader.h: interface for the netCDFReader class.
//
//////////////////////////////////////////////////////////////////////

#ifndef NETCDFREADER_H
#define NETCDFREADER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "NetCDFStructure.h"
#include "DMReader.h"

//! Reads serialized data from Unidata NetCDF files.
/*! NetCDF provides a format for storing scientific data such as integers,
	doubles, and arrays of them.  We define our own conventions in a
	NetCDF data file in order to mimic having hierarchical storage.

	\author Thierry Cretegny, Andrew Dolgert
	\data 1 Mar 2001
	*/
class NetCDFReader: public DMReader
{
	int ncID;
	bool bFileOpen;
	//! Represents the current path in the hierarchical storage.
	// This is a memento from NetCDFStructure.
	NetCDFStructure::PathType path;
public:
	NetCDFReader(const std::string &filename = "data.nc");
	virtual ~NetCDFReader();

	void Open();
	void Close();

	//! Returns the specifier for the class of the stored object.
	std::string GetType(const std::string &name="root");

	bool Fill(Serializable *toFill, const std::string &name="root");
	Serializable *Get(const std::string &name="root");

	bool		GetBool   ( const std::string& name );
	int			GetInt    ( const std::string& name );
	double		GetDouble ( const std::string& name );
	std::string GetString ( const std::string& name);

	//! Returns the shape of a stored array of any sort.
	void GetDimensions( std::vector<int>& dims, const std::string& name );
	void FillIntArray	 ( int* data,    const std::string& name );
	void FillDoubleArray ( double* data, const std::string& name );

	/*! Deprecated because they allocate memory in the library. */
	//@{
	int*	GetIntArray   ( std::vector<int> &dims, const std::string& name);
	double*	GetDoubleArray( std::vector<int> &dims, const std::string& name);
	//@}

private:
	void GetRegularDimensions ( std::vector<int>& dims,
									const std::string& sAbsoluteItem );
	void GetZeroDimensions    ( std::vector<int>& dims,
									const std::string& sAbsoluteItem );
	void FillAbsoluteIntArray   ( int* data,    const std::string& sAbsoluteItem );
	void FillAbsoluteDoubleArray( double* data, const std::string& sAbsoluteItem );

	static int GetVariableID( int ncID, const std::string& sAbsoluteItem );
	static size_t GetDimensionLength( int ncID, int dimID );

	std::string GetOldString( const std::string& name );
};

#endif // NETCDFREADER_H
