#ifndef _NETCDF_STRUCTURE_H_
#define _NETCDF_STRUCTURE_H_

#include "DMFirstHeader.h"
#include <stack>
#include <string>
#include "DigitalMaterial.h"

//! Static class which knows conventions for storing serialized NetCDF data.
class NetCDFStructure
{
public:
	typedef std::stack<std::string> PathType;
	enum Access { AINVALID, READONLY, WRITE };

	/*! NetCDF file access. */
	//@{
	static void CreateFile( const std::string& filename, int& fileID );
	static void Open( const std::string& filename, Access access, int& fileID );
	static void Close( int fileID );
	//@}

	/*! Conventions for mangling names. */
	//@{
	static void CheckItemName( const std::string& sItem );
	static std::string LengthMangle( const std::string& sItem );
	static std::string DimMangle( const std::string& sItem, int dimIdx );
	static std::string DataMangle( const std::string& sItem );
	static std::string FakeDimMangle( const std::string& sItem );
	//@}

	/*! These modify the directory structure.  The PathType object,
		owned by the client class, is a token this class manipulates
		in order to set the current directory in the NetCDF file.
		*/
	//@{
	static std::string CurDir( const PathType& path );
	static std::string AbsolutePath( const PathType& path, const std::string& fileName );
	static void UpDir( PathType& path );
	static void DownDir( PathType& path, const std::string& dirName );
	static bool IsAtRoot( const PathType& path );
	//@}
private:
	//! Makes sure a filename is valid.
	static void CheckFilename( const std::string& filename,
		const std::string& function );
};

#endif // _NETCDF_STRUCTURE_H_
