#ifndef _NOT_ENOUGH_MEMORY_H_
#define _NOT_ENOUGH_MEMORY_H_

#include <string>
#include "DigitalMaterial.h"
#include "DMException.h"

//! This exception signals that a method failed to allocate and therefore failed to initialize.
/*! When you fail to allocate, often new() throws std::bad_alloc, but by the time
	the thread of execution has left the method, you usually have caught bad_alloc
	and freed any previous allocations, so std::bad_alloc is no longer the right exception
	to pass up the call stack.  We use this class as a simple sign that initialization
	has failed due to a lack of memory.  It may even be a recoverable exception.

	Caught in Python as a MemoryError.
	*/
class NotEnoughMemory : public DMException
{
public:
	//! Constructor
	/*! This exception says there was a memory error, so you just
		tell it what kind of allocation failed.

		@param sClass Class where occurred.
		@param sFunction Function where occurred.
		@param sVariable What variables you couldn't allocate.
		*/
	NotEnoughMemory( const std::string& sClass, const std::string& sFunction,
		const std::string& sVariable );
	//! Destructor.
	virtual ~NotEnoughMemory();
};

#endif //_NOT_ENOUGH_MEMORY_H_
