// Buffer.h: interface for the Buffer class.
//
//////////////////////////////////////////////////////////////////////

#ifndef MPIRECVBUFFER_H
#define MPIRECVBUFFER_H

#include "DMReader.h"
#include "mpi.h"

//! A class that provides a buffer where MPI can write to and allows to unpack passed data.
/*! The member data consist of the MPI communicator, the buffer where data is written,
the size of the buffer (\a bufferSize) and \a currentPos: the shift between the beginning 
of the buffer and the next data to be read.

These is a weakness in the present design: the size of the buffer can be changed, but this 
is not done automatically if the buffer is too small to receive what it should.
This could be fixed if we agreed to send the size of the buffer prior to sending the data.
*/

class MPIRecvBuffer: public DMReader
{
public:
	MPIRecvBuffer(const MPI_Comm comm = 0);
	virtual ~MPIRecvBuffer();
	void SetComm(MPI_Comm comm) {this->comm = comm;}
	void Allocate(int bufferSize);
	//! Not used
	void Open() {}
	//! Not used
	void Close() {}
	//! Resets \a currentPos to 0
	void EmptyBuffer() {this->currentPos = 0;}
	std::string GetType(const std::string &name="root"); 
	bool Fill(Serializable *toFill, const std::string &name="root");
	Serializable *Get(const std::string &name="root");
	bool GetBool( const std::string& name );
	int GetInt(const std::string &name);
	double GetDouble(const std::string &name);
	std::string GetString(const std::string &name);
	int *GetIntArray(std::vector<int> &dims, const std::string &name);
	double *GetDoubleArray(std::vector<int> &dims, const std::string &name);
	void *GetBuffer() {return buffer;}
	int GetBufferSize() {return bufferSize;}
private:
	MPI_Comm comm;
	char *buffer;
	int bufferSize;
	int currentPos;
};

#endif // !defined MPIRECVBUFFER_H



	
