// Buffer.h: interface for the Buffer class.
//
//////////////////////////////////////////////////////////////////////

#ifndef MPISENDBUFFER_H
#define MPISENDBUFFER_H


#include "BinaryWriter.h"
#include "mpi.h"

//! A class that provides a buffer where a user can dump data and that MPI can send.
/*! The member data consist of the MPI communicator, the buffer where data is written,
the size of the buffer (\a bufferSize) and \a currentSize.
*/

class MPISendBuffer : public BinaryWriter
{
public:
    MPISendBuffer(const MPI_Comm comm=0);
    virtual ~MPISendBuffer();
    void SetComm(MPI_Comm comm) {this->comm = comm;}
    void PutBool( bool b, const std::string& name );
    void PutInt(int n, const std::string &name);
    void PutDouble(double x, const std::string &name);
    void PutString(const std::string &str, const std::string &name);
    void PutIntArray(const int *data, const std::vector<int> &dims, const std::string &name);
    void PutDoubleArray(const double *data, const std::vector<int> &dims, const std::string &name);
    void *GetBuffer() {return buffer;}
    int GetSize() {return currentSize;}
private:
    MPI_Comm comm;
};


#endif // !defined MPISENDBUFFER_H




