###############################################################################
### First, a function to check that a given PyObject is a 2D list 
### with specified dimensions nRow and nCols
##########################################################################3


%{
bool PythonListDouble2DCheck(PyObject *pyObj, int nRows, int nCols)
{
	class ListCheckerError
	{
		public:
		ListCheckerError(int type){this->type = type;}
		int type;
	};

	int a, b;
	try
	{
	// first check that PyObj is a Python list at all
	if(!PyList_Check(pyObj))
	throw ListCheckerError(0);
		
	// next check the size is nRows
	int size1 = PyList_Size(pyObj);
	if(size1 != nRows)
		throw ListCheckerError(1);		
	for(a=0;a<size1;++a)
	{
		// for each item in the list, check that it is a list with nCols items
		PyObject *row = PyList_GetItem(pyObj,a);
		if(!PyList_Check(row))
			throw ListCheckerError(2);
		int size2 = PyList_Size(row);
		if(size2 != nCols)
			throw ListCheckerError(3);
	}

	// finally check that every element is a double	or int
	for(a=0;a<size1;++a)
	{
		PyObject *row = PyList_GetItem(pyObj,a);	
		for(b=0;b<nCols;++b)
		{
			PyObject *element = PyList_GetItem(row,b);
			if(!PyFloat_Check(element) && !PyInt_Check(element))
			throw ListCheckerError(4);			
		}
	}
	}
	catch(ListCheckerError error)
	{
		if (error.type == 0)
	// In this case we don't want to throw an exception because the caller
	// might yet check that it's a list
			return false;

		else if (error.type == 1)
		PyErr_SetString(PyExc_TypeError,"wrong number of rows in list");
		else if(error.type == 2)
		PyErr_SetString(PyExc_TypeError,"non-list found in outer list");
		else if(error.type == 3)
		PyErr_SetString(PyExc_TypeError,"at least one row had wrong number of columns");
		else if(error.type == 4)
		PyErr_SetString(PyExc_TypeError,"at least one element neither a double nor an int");
	}

	return true;
}
%}


###############################################################################
### Next, a function to check that a given PyObject is a 2D NumPy Array
### with specified dimensions nRow and nCols
##########################################################################3


%{
bool PythonArrayDouble2DCheck(PyObject *pyObj, int nRows, int nCols)
{
	class ArrayCheckerError
	{
		public:
		ArrayCheckerError(int type){this->type = type;}
		int type;
	};

	try
	{
	// first check that PyObj is a NumPy array at all
	if (!PyArray_Check(pyObj))
		return false;	
		
	PyArrayObject *array = (PyArrayObject *)(pyObj);
	// next check that it's a two dimensional array
	int numDimensions = array->nd;
	if (numDimensions != 2)
		throw ArrayCheckerError(0);	

	if (!(array->dimensions[0] == nRows && array->dimensions[1] == nCols))
		throw ArrayCheckerError(1);

	if(array->descr->type_num != PyArray_DOUBLE)
		throw ArrayCheckerError(2);
	}
	catch(ArrayCheckerError error)
	{
		 if (error.type == 0)
		PyErr_SetString(PyExc_TypeError,"array dimension not equal to 2");
		else if(error.type == 1)
		PyErr_SetString(PyExc_TypeError,"either nRows or nCols not correct in 2D NumPy array");
		else if(error.type == 2)
		PyErr_SetString(PyExc_TypeError,"wrongtype in NumPy array");
	}

	return true;
}
%}