/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * Vectorized calculation of dEds
 */

#include "vectools.h"

/*  r[] = a[] * (b[] * c[] * d + e[] * f + g[] * h[] * x + y[] * z)  */
void vec_df(double *res, const double *a, const double *b, const double *c, double d, const double *e,
	    double f, const double *g, const double *h, double x, const double *y, double z,
	    int n)
{
    /* Note that sometimes b==g and e==y */
#ifdef AIX
#pragma disjoint (*res, *a, *b, *c, *e)
#pragma disjoint (*res, *a, *g, *h, *y)
#endif
    
    int i;

    double a0, b0, c0, e0, g0, h0, y0;
    /* double a1, b1, c1, e1, g1, h1, y1; */

    for (i = 0; i < n; i++)
    {
	a0 = a[i];
	b0 = b[i];
	c0 = c[i];
	e0 = e[i];
	g0 = g[i];
	h0 = h[i];
	y0 = y[i];
	res[i] = a0 * (b0 * c0 * d + e0 * f + g0 * h0 * x + y0 * z);
    }
}
