/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * r = a * b + c * d   (with various ways to put the parentheses)
 */

#include "vectools.h"

/* r[] = (a[] * b + c[]) * d[] */
void vec_mul_scal_add_mul(double *res, const double *a, double b, const double *c,
			  const double *d, int n)
{
    /* In this case the pragma slows down things.  A compiler bug? */
    /* #pragma disjoint (*res, *a, b, *c, *d) */
    
    int i, limit;
    double a0, a1, a2, a3, a4, a5, a6, a7;
    double c0, c1, c2, c3, c4, c5, c6, c7;
    double d0, d1, d2, d3, d4, d5, d6, d7;
    
    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	c0 = c[i];
	c1 = c[i+1];
	c2 = c[i+2];
	c3 = c[i+3];
	c4 = c[i+4];
	c5 = c[i+5];
	c6 = c[i+6];
	c7 = c[i+7];
	d0 = d[i];
	d1 = d[i+1];
	d2 = d[i+2];
	d3 = d[i+3];
	d4 = d[i+4];
	d5 = d[i+5];
	d6 = d[i+6];
	d7 = d[i+7];
	res[i] = (a0 * b + c0) * d0;
	res[i+1] = (a1 * b + c1) * d1;
	res[i+2] = (a2 * b + c2) * d2;
	res[i+3] = (a3 * b + c3) * d3;
	res[i+4] = (a4 * b + c4) * d4;
	res[i+5] = (a5 * b + c5) * d5;
	res[i+6] = (a6 * b + c6) * d6;
	res[i+7] = (a7 * b + c7) * d7;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	c0 = c[i];
	d0 = d[i];
	res[i] = (a0 * b + c0) * d0;
    }
}

/* r[] = (a[] * b + c[]) * d */
void vec_mul_scal_add_mul_scal(double *res, const double *a, double b, const double *c,
			       double d, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, b, *c, d)
#endif
    
    int i, limit;
    double a0, a1, a2, a3, a4, a5, a6, a7;
    double c0, c1, c2, c3, c4, c5, c6, c7;
    
    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	c0 = c[i];
	c1 = c[i+1];
	c2 = c[i+2];
	c3 = c[i+3];
	c4 = c[i+4];
	c5 = c[i+5];
	c6 = c[i+6];
	c7 = c[i+7];
	res[i] = (a0 * b + c0) * d;
	res[i+1] = (a1 * b + c1) * d;
	res[i+2] = (a2 * b + c2) * d;
	res[i+3] = (a3 * b + c3) * d;
	res[i+4] = (a4 * b + c4) * d;
	res[i+5] = (a5 * b + c5) * d;
	res[i+6] = (a6 * b + c6) * d;
	res[i+7] = (a7 * b + c7) * d;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	c0 = c[i];
	res[i] = (a0 * b + c0) * d;
    }
}

/* r[] = (a[] * b + c) * d[] */
void vec_mul_scal_add_scal_mul(double *res, const double *a, double b, double c,
			       const double *d, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, b, c, *d)
#endif
    
    int i, limit;
    double a0, a1, a2, a3, a4, a5, a6, a7;
    double d0, d1, d2, d3, d4, d5, d6, d7;
    
    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	d0 = d[i];
	d1 = d[i+1];
	d2 = d[i+2];
	d3 = d[i+3];
	d4 = d[i+4];
	d5 = d[i+5];
	d6 = d[i+6];
	d7 = d[i+7];
	res[i] = (a0 * b + c) * d0;
	res[i+1] = (a1 * b + c) * d1;
	res[i+2] = (a2 * b + c) * d2;
	res[i+3] = (a3 * b + c) * d3;
	res[i+4] = (a4 * b + c) * d4;
	res[i+5] = (a5 * b + c) * d5;
	res[i+6] = (a6 * b + c) * d6;
	res[i+7] = (a7 * b + c) * d7;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	d0 = d[i];
	res[i] = (a0 * b + c) * d0;
    }
}

/* r[] = a[] * b + c[] * d */
void vec_dbl_mul_scal_add(double *res, const double *a, double b, const double *c,
			  double d, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, b, *c, d)
#endif
    
    int i, limit;
    double a0, a1, a2, a3, a4, a5, a6, a7;
    double c0, c1, c2, c3, c4, c5, c6, c7;
    
    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	c0 = c[i];
	c1 = c[i+1];
	c2 = c[i+2];
	c3 = c[i+3];
	c4 = c[i+4];
	c5 = c[i+5];
	c6 = c[i+6];
	c7 = c[i+7];
	res[i] = a0 * b + c0 * d;
	res[i+1] = a1 * b + c1 * d;
	res[i+2] = a2 * b + c2 * d;
	res[i+3] = a3 * b + c3 * d;
	res[i+4] = a4 * b + c4 * d;
	res[i+5] = a5 * b + c5 * d;
	res[i+6] = a6 * b + c6 * d;
	res[i+7] = a7 * b + c7 * d;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	c0 = c[i];
	res[i] = a0 * b + c0 * d;
    }
}

/* r[] = a[] * b[offset[]] + c[] * d[offset[]] */
void vec_dbl_mul_indir_add(double *res, const double *a, const double *b, const double *c,
			  const double *d, const int *offset, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, *b, *c, *d)
#endif
    
    int i, limit;
    double a0, a1, a2, a3;
    double b0, b1, b2, b3;
    double c0, c1, c2, c3;
    double d0, d1, d2, d3;
    
    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	b0 = b[offset[i]];
	b1 = b[offset[i+1]];
	b2 = b[offset[i+2]];
	b3 = b[offset[i+3]];
	c0 = c[i];
	c1 = c[i+1];
	c2 = c[i+2];
	c3 = c[i+3];
	d0 = d[offset[i]];
	d1 = d[offset[i+1]];
	d2 = d[offset[i+2]];
	d3 = d[offset[i+3]];
	res[i] = a0 * b0 + c0 * d0;
	res[i+1] = a1 * b1 + c1 * d1;
	res[i+2] = a2 * b2 + c2 * d2;
	res[i+3] = a3 * b3 + c3 * d3;
    }
    for (i = n - n % 4; i < n; i++)
    {
	a0 = a[i];
	b0 = b[offset[i]];
	c0 = c[i];
	d0 = d[offset[i]];
	res[i] = a0 * b0 + c0 * d0;
    }
}

/* res3[][0..2] = a3[][0..2] * b[offset[]] + c3[][0..2] * d[offset[]] */
void vec3_dbl_mul_indir_add(double (*res3)[3], const double (*a3)[3], const double *b,
			    const double (*c3)[3], const double *d, const int *offset, int n)
{
    int i, limit;
    double *res, *a, *c;
    double a0, a1, a2;
    double bx;
    double c0, c1, c2;
    double dx;
#ifdef AIX
#pragma disjoint (*res3, *a3, *b, *c3, *d)
#pragma disjoint (*res, *a, *b, *c, *d)    
#endif

    res = (double *) res3;
    a = (double *) a3;
    c = (double *) c3;
    
    limit = 3*n;
    for (i = 0; i < limit; i+=3)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	bx = b[*offset];
	c0 = c[i];
	c1 = c[i+1];
	c2 = c[i+2];
	dx = d[*offset++];
	res[i] = a0 * bx + c0 * dx;
	res[i+1] = a1 * bx + c1 * dx;
	res[i+2] = a2 * bx + c2 * dx;
    }
}

/* r[] = (a[] * b[offset[]] + c[offset[]]) * d[] */
void vec_mul_indir_add_indir_mul(double *res, const double *a, const double *b, const double *c,
				 const double *d, const int *offset, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, *b, *c, *d)
#endif
    
    int i, limit;
    double a0, a1, a2, a3;
    double b0, b1, b2, b3;
    double c0, c1, c2, c3;
    double d0, d1, d2, d3;
    
    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	b0 = b[offset[i]];
	b1 = b[offset[i+1]];
	b2 = b[offset[i+2]];
	b3 = b[offset[i+3]];
	c0 = c[offset[i]];
	c1 = c[offset[i+1]];
	c2 = c[offset[i+2]];
	c3 = c[offset[i+3]];
	d0 = d[i];
	d1 = d[i+1];
	d2 = d[i+2];
	d3 = d[i+3];
	res[i] = (a0 * b0 + c0) * d0;
	res[i+1] = (a1 * b1 + c1) * d1;
	res[i+2] = (a2 * b2 + c2) * d2;
	res[i+3] = (a3 * b3 + c3) * d3;
    }
    for (i = n - n % 4; i < n; i++)
    {
	a0 = a[i];
	b0 = b[offset[i]];
	c0 = c[offset[i]];
	d0 = d[i];
	res[i] = (a0 * b0 + c0) * d0;
    }
}

