/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * r = (a * b + c) * d * e
 */

#include "vectools.h"

/* r[] = (a[] * b + c[]) * d * e[] */
void vec_mul_scal_add_mul_scal_mul(double *res, const double *a, double b, const double *c,
			       double d, const double *e, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, b, *c, d, *e)
#endif
    
    int i, limit;
    double a0, a1, a2, a3, a4, a5, a6, a7;
    double c0, c1, c2, c3, c4, c5, c6, c7;
    double e0, e1, e2, e3, e4, e5, e6, e7;
    
    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	c0 = c[i];
	c1 = c[i+1];
	c2 = c[i+2];
	c3 = c[i+3];
	c4 = c[i+4];
	c5 = c[i+5];
	c6 = c[i+6];
	c7 = c[i+7];
	e0 = e[i];
	e1 = e[i+1];
	e2 = e[i+2];
	e3 = e[i+3];
	e4 = e[i+4];
	e5 = e[i+5];
	e6 = e[i+6];
	e7 = e[i+7];
	res[i] = (a0 * b + c0) * d * e0;
	res[i+1] = (a1 * b + c1) * d * e1;
	res[i+2] = (a2 * b + c2) * d * e2;
	res[i+3] = (a3 * b + c3) * d * e3;
	res[i+4] = (a4 * b + c4) * d * e4;
	res[i+5] = (a5 * b + c5) * d * e5;
	res[i+6] = (a6 * b + c6) * d * e6;
	res[i+7] = (a7 * b + c7) * d * e7;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	c0 = c[i];
	e0 = e[i];
	res[i] = (a0 * b + c0) * d * e0;
    }
}

/* res[] = (a[] * b[offset[]] + c[]) * d[offset[]] * e[] */
void vec_mul_indir_add_mul_indir_mul(double *res, const double *a, const double *b,
				     const double *c, const double *d, const double *e,
				     const int *offset, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, *b, *c, *d, *e)
#endif
    
    int i, limit;
    double a0, a1, a2, a3;
    double b0, b1, b2, b3;
    double c0, c1, c2, c3;
    double d0, d1, d2, d3;
    double e0, e1, e2, e3;
    
    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	b0 = b[offset[i]];
	b1 = b[offset[i+1]];
	b2 = b[offset[i+2]];
	b3 = b[offset[i+3]];
	c0 = c[i];
	c1 = c[i+1];
	c2 = c[i+2];
	c3 = c[i+3];
	d0 = d[offset[i]];
	d1 = d[offset[i+1]];
	d2 = d[offset[i+2]];
	d3 = d[offset[i+3]];
	e0 = e[i];
	e1 = e[i+1];
	e2 = e[i+2];
	e3 = e[i+3];
	res[i] = (a0 * b0 + c0) * d0 * e0;
	res[i+1] = (a1 * b1 + c1) * d1 * e1;
	res[i+2] = (a2 * b2 + c2) * d2 * e2;
	res[i+3] = (a3 * b3 + c3) * d3 * e3;
    }
    for (i = n - n % 4; i < n; i++)
    {
	a0 = a[i];
	b0 = b[offset[i]];
	c0 = c[i];
	d0 = d[offset[i]];
	e0 = e[i];
	res[i] = (a0 * b0 + c0) * d0 * e0;
    }
}
