/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * Apply periodic boundary conditions to an array of 3-vectors
 *
 * NOTE: This involves a rounding operation.  The fastest portabel way of
 * doing this appears to be casting to int.  That is a rather slow operation.
 * On IEEE-compliant processors a "trick" can be used, where a very large
 * "magic" number is added and the rounding in the CPU does the convertion
 * to integer.
 *
 * THIS ONLY WORKS ON IEEE-COMPLIANT PROCESSORS AND MAY BE SPOILED BY
 * AN OPTIMIZING COMPILER !
 *
 * This works on the IBM RS/6000 POWER and POWER2 CPUs using the AIX xlc
 * compiler with -O2 optimization/
 */

/* Select normal or super-fast IEEE routine: */
#ifndef PBCNORMAL
#define PBCIEEE
#endif

/* Do not let the optimizer ruin the IEEE magic trick. */
/* moved to makefile */
/* #ifdef PBCIEEE */
/* #pragma options optimize=3 strict */
/* #endif */

#include <stdio.h>
#include <stdio.h>
#include <math.h>

#include "main.h"
#include "vectools.h"

#ifdef PBCNORMAL
#define ASSUREPOSI 5.0
void vec_pbc(double (*r)[3], int n)
{
    int i, limit;
    double r00, r10, r20, r30;
    double r01, r11, r21, r31;
    double r02, r12, r22, r32;
    double p0, p1, p2;
    double assureposi = ASSUREPOSI;
    double assureposihalf = ASSUREPOSI + 0.5;

    p0 = dPeri[0];
    p1 = dPeri[1];
    p2 = dPeri[2];
    if (p0 == 0.0 && p1 == 0.0 && p2 == 0.0)
	return;
    
    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	r00 = r[i][0];
	r01 = r[i][1];
	r02 = r[i][2];
	r10 = r[i+1][0];
	r11 = r[i+1][1];
	r12 = r[i+1][2];
	r20 = r[i+2][0];
	r21 = r[i+2][1];
	r22 = r[i+2][2];
	r30 = r[i+3][0];
	r31 = r[i+3][1];
	r32 = r[i+3][2];
	r[i][0] = r00 - p0 * (((int) (r00 + assureposihalf)) - assureposi);
	r[i][1] = r01 - p1 * (((int) (r01 + assureposihalf)) - assureposi);
	r[i][2] = r02 - p2 * (((int) (r02 + assureposihalf)) - assureposi);
	r[i+1][0] = r10 - p0 * (((int) (r10 + assureposihalf)) - assureposi);
	r[i+1][1] = r11 - p1 * (((int) (r11 + assureposihalf)) - assureposi);
	r[i+1][2] = r12 - p2 * (((int) (r12 + assureposihalf)) - assureposi);
	r[i+2][0] = r20 - p0 * (((int) (r20 + assureposihalf)) - assureposi);
	r[i+2][1] = r21 - p1 * (((int) (r21 + assureposihalf)) - assureposi);
	r[i+2][2] = r22 - p2 * (((int) (r22 + assureposihalf)) - assureposi);
	r[i+3][0] = r30 - p0 * (((int) (r30 + assureposihalf)) - assureposi);
	r[i+3][1] = r31 - p1 * (((int) (r31 + assureposihalf)) - assureposi);
	r[i+3][2] = r32 - p2 * (((int) (r32 + assureposihalf)) - assureposi);
    }
    for (i = n - n % 4; i < n; i++)
    {
	r00 = r[i][0];
	r01 = r[i][1];
	r02 = r[i][2];
	r[i][0] = r00 - p0 * (((int) (r00 + assureposihalf)) - assureposi);
	r[i][1] = r01 - p1 * (((int) (r01 + assureposihalf)) - assureposi);
	r[i][2] = r02 - p2 * (((int) (r02 + assureposihalf)) - assureposi);
    }
}
#endif


#ifdef PBCIEEE
#define MAGIC 6755399441055744.0
void vec_pbc(double (*r)[3], int n)
{
    int i, limit;
    double r00, r10, r20, r30;
    double r01, r11, r21, r31;
    double r02, r12, r22, r32;
    double p0, p1, p2;
    double magic = MAGIC;

    p0 = dPeri[0];
    p1 = dPeri[1];
    p2 = dPeri[2];
    if (p0 == 0.0 && p1 == 0.0 && p2 == 0.0)
	return;
    
    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	r00 = r[i][0];
	r01 = r[i][1];
	r02 = r[i][2];
	r10 = r[i+1][0];
	r11 = r[i+1][1];
	r12 = r[i+1][2];
	r20 = r[i+2][0];
	r21 = r[i+2][1];
	r22 = r[i+2][2];
	r30 = r[i+3][0];
	r31 = r[i+3][1];
	r32 = r[i+3][2];
	r[i][0] = r00 - p0 * ((r00 + magic) - magic);
	r[i][1] = r01 - p1 * ((r01 + magic) - magic);
	r[i][2] = r02 - p2 * ((r02 + magic) - magic);
	r[i+1][0] = r10 - p0 * ((r10 + magic) - magic);
	r[i+1][1] = r11 - p1 * ((r11 + magic) - magic);
	r[i+1][2] = r12 - p2 * ((r12 + magic) - magic);
	r[i+2][0] = r20 - p0 * ((r20 + magic) - magic);
	r[i+2][1] = r21 - p1 * ((r21 + magic) - magic);
	r[i+2][2] = r22 - p2 * ((r22 + magic) - magic);
	r[i+3][0] = r30 - p0 * ((r30 + magic) - magic);
	r[i+3][1] = r31 - p1 * ((r31 + magic) - magic);
	r[i+3][2] = r32 - p2 * ((r32 + magic) - magic);
    }
    for (i = n - n % 4; i < n; i++)
    {
	r00 = r[i][0];
	r01 = r[i][1];
	r02 = r[i][2];
	r[i][0] = r00 - p0 * ((r00 + magic) - magic);
	r[i][1] = r01 - p1 * ((r01 + magic) - magic);
	r[i][2] = r02 - p2 * ((r02 + magic) - magic);
    }
}
#endif
