/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * Header files for all the vectorized functions
 */

/* XXX Added extern"C" to everything? */

#ifndef VECTOOLS_H
#define VECTOOLS_H

#ifdef _MSC_VER
// XXX Need to disable under Microsoft console and _WIN32: what is the analog for _MICROSOFT
#pragma warning(disable:4068) // turns off spurious Windows pragma warnings
#endif // _MSC_VER

#ifdef __cplusplus
extern "C" {
#endif

void vecnorm(int n, double *res, const double (*v)[3]);
// void mat_mul_mat_3(double (*res)[3], const double a[3][3], double (*b)[3], int n);
// double max_abs_vec_diff_transformed(double (*a)[3], double (*b)[3],
//				    double m[3][3], int n);
void vec_gauss_rnd2(double *gr1, double *gr2, double *tmp1, double *tmp2,
		    int n);
void vec_pbc(double (*r)[3], int n);

void vec_mul_scal_add_scal(double *res, const double *a, const double b, const double c, int n);
/*   res[] = a[] * b + c */ 

void vec_mul_indir_add_indir(double *res, const double *a, const double *b, const double *c,
			     const int *offset, int n);
/* r[] = a[] * b[offset[]] + c[offset[]] */

void vec_mul(double *res, const double *a, const double *b, int n);
/*   res[] = a[] * b[] */

void vec_eq_self_mul_addscal_mul(double *res, const double *a, double b, const double *c,
				 int n);
/*   res[] = res[] * a[] + b * c[] */

void vec_mul_mul_scal(double *res, const double *a, const double *b, double c, int n);
/*   res[] = a[] * b[] * c */

void vec_mul_mul(double *res, const double *a, const double *b, const double *c, int n);
/*   res[] = a[] * b[] * c[] */

void vec_mul_mul_indir(double *res, const double *a, const double *b, const double *c,
		       const int *offset, int n);
/* r[] = a[] * b[] * c[offset[]] */

void vec_self_mul_mul_indir(double *res, const double *a, const double *b,
			    const int *offset, int n);
/* r[] = r[] * a[] * b[offset[]] */

void vec_self_mul_mul_scal(double *res, const double *a, double b, int n);
/*   res[] = res[] * a[] * b */

void vec_diffweight(double *res, const double *a, double b, int n);
/*   res[] = (a[] - a[] * a[]) * b */

void vec_mul_scal_add_mul(double *res, const double *a, double b, const double *c,
			  const double *d, int n);
/*   res[] = (a[] * b + c[]) * d[] */

void vec_mul_scal_add_mul_scal(double *res, const double *a, double b, const double *c,
			       double d, int n);
/*   res[] = (a[] * b + c[]) * d  */

void vec_mul_scal_add_scal_mul(double *res, const double *a, double b, double c,
			       const double *d, int n);
/*   res[] = (a[] * b + c) * d[] */

void vec_mul_indir_add_indir_mul(double *res, const double *a, const double *b, const double *c,
				 const double *d, const int *offset, int n);
/* r[] = (a[] * b[offset[]] + c[offset[]]) * d[] */

void vec_dbl_mul_scal_add(double *res, const double *a, double b, const double *c,
			  double d, int n);
/*   res[] = a[] * b + c[] * d */

void vec_dbl_mul_indir_add(double *res, const double *a, const double *b, const double *c,
			   const double *d, const int *offset, int n);
/* r[] = a[] * b[offset[]] + c[] * d[offset[]] */

void vec3_dbl_mul_indir_add(double (*res3)[3], const double (*a3)[3], const double *b,
			    const double (*c3)[3], const double *d, const int *offset, int n);
/* res3[][0..2] = a3[][0..2] * b[offset[]] + c3[][0..2] * d[offset[]] */

void vec_self_add_mul_scal(double *res, const double *a, double b, int n);
/*   res[] = res[] + a[] * b */

void vec_mul_scal(double *res, const double *a, double b, int n);
/*   res[] = a[] * b */

void vec_mul_indir(double *res, const double *a, const double *b, const int *offset, int n);
/* r[] = a[] * b[offset[]] */

void vec3_mul_indir(double (*res3)[3], const double (*a3)[3], const double *b, const int *offset,
		    int n);
/* r[][0..2] = a[][0..2] * b[offset[]] */

void vec_self_dbl_mul_scal_add_add(double *res, const double *a, double b,
				   const double *c, double d, const double *e, int n);
/*   res[] = res[] + a[] * b + c[] * d + e[] */

void vec3_self_dbl_mul_indir_add_add(double (*res3)[3], const double (*a3)[3],
				     const double *b, const double (*c3)[3], const double *d,
				     const double (*e3)[3], const int *offset, int n);
/* res3[][0..2] += a3[][0..2] * b[offset[]] + c3[][0..2] * d[offset[]] +
                   e3[][0..2] */

void vec_self_mul_scal_add_mul_scal_add(double *res, double a, const double *b,
					double c, const double *d, int n);
/*   res[] = res[] * a + b[] * c + d[] */

void vec3_self_mul_scal_add_mul_indir_add(double (*res3)[3], double a,
					  const double (*b3)[3], const double *c,
					  const double (*d3)[3], const int *offset, int n);
/*   res[][0..2] = res[][0..2] * a + b[][0..2] * c[offset[]] + d[][0..2]   */

void vec3_self_mul_indir_add_mul_indir_add(double (*res3)[3], const double *a,
					   const double (*b3)[3], const double *c,
					   const double (*d3)[3],
					   const int *offset, int n);
/*   res[][0..2] = res[][0..2] * a[offset[]] + b[][0..2] * c[offset[]]
     + d[][0..2]   */

void vec_P_av_times_b_plus_c_p_times_av_times_dv(double *res, const double *a,
				  double b, double c, const double *d, int n);
/*   res[] = (a[] * b + c) * a[] * d[] */

void vec_mul_scal_add_mul_scal_mul(double *res, const double *a, double b, const double *c,
				   double d, const double *e, int n);
/*   res[] = (a[] * b + c[]) * d * e[] */

void vec_mul_indir_add_mul_indir_mul(double *res, const double *a, const double *b,
				     const double *c, const double *d, const double *e,
				     const int *offset, int n);
/* res[] = (a[] * b[offset[]] + c[]) * d[offset[]] * e[] */

void vec_av_times_bv_plus_cv_times_bv_plus_d_times_ev(double *res, const double *a,
         const double *b, const double *c, double d, const double *e, int n);
/*   res[] = a[] * b[] + c[] * b[] + d * e[]   */

void vec_outherprod(double *res, const double a[][3], int ix, int iy, int n);
/*   res[] = a[][ix] * a[][iy] */

void vec_dEds(double *res, const double *a, const double *b, const double *c, double d,
	      double e, double f, int n);
/*  res[] = (d * a[] + e) * b[] + f * c[]  */

void vec_df(double *res, const double *a, const double *b, const double *c, double d, const double *e,
	    double f, const double *g, const double *h, double x, const double *y, double z,
	    int n);
/*  r[] = a[] * (b[] * c[] * d + e[] * f + g[] * h[] * x + y[] * z)  */

void vec3_self_add_mul_indir(double (*res3)[3], const double (*a3)[3], const double *b,
			     const int *offset, int n);
/* r[][1..3] = r[][1..3] + a[1..3] * b[offset[]] */

void vec_add_scal(double *res, const double *a, double b, int n);
/* r[] = a[] + b */

void vec_self_add_mul_mul_scal(double *res, const double *a, const double *b, double c,
			       int n);
/* r[] = r[] + a[] * b[] * c */


#ifdef __cplusplus

}

#endif

#endif // VECTOOLS_H

