#include "DMFirstHeader.h"

#include <iostream>

#include "DigitalMaterial.h"

#include "SerialFactory.h"
#include "SerialCreator.h"
#include "DMException.h"

#include "DMLastHeader.h"


SerialFactory* SerialFactory::_instance = 0;

SerialFactory::SerialFactory()
{
}

/**
 * Deletes all the registered creators.
 **/
SerialFactory::~SerialFactory()
{
	creatorMap::iterator buildPtr;
	for (buildPtr = registry.begin(); buildPtr == registry.end(); buildPtr++) {
		delete buildPtr->second;
	}
}

SerialFactory* SerialFactory::Instance()
{
	if (_instance == 0) {
		try {
			_instance = new SerialFactory();
		} catch (std::bad_alloc) {
			_instance = 0;
		}
		ASSERT( 0 != _instance );
		if (0 == _instance) throw DMException("SerialFactory","Instance",
			"Failed to create an instance of the SerialFactory.");
	}
	return _instance;
}

/** Store a pointer to the creator object, indexed by type.
  * If the creator object is a null pointer, this throws a DMException.
  * If the type is already registered, this returns false.  You should
  * only register objects allocated with new() so that they persist.
  * This factory will delete them when it is destroyed.
  */
bool SerialFactory::Register( const SerialCreator* const creator )
{
	ASSERT( 0 != creator );
	if ( creator == 0 ) throw DMException("SerialFactory","Register",
		"Tried to register a null creator.");

	std::string type = creator->GetType();
	if ( registry.find( type ) == registry.end() ) {
		registry[ type ] = creator;
	} return false;
	
	return true;
}

/** If the factory cannot make the object you request, it
  * returns a null pointer.
  */
Serializable* SerialFactory::New( const std::string& type )
{
	const SerialCreator* creator;

	if ( registry.find( type ) == registry.end() ) return 0;

	creator = registry[ type ];
	return creator->Build();
}
