#ifndef _SERIALFACTORY_H_
#define _SERIALFACTORY_H_

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif // SWIG

#include "DMFirstHeader.h"
#include <string>
#include <map>
#include "DigitalMaterial.h"

class SerialCreator;
class Serializable;

//! This singleton returns a factory object given a subtype of Serializable.
/**
 * If you know the type of the class you want to create, here the
 * string name, this creates a new instance of that class.
 * It can create any class which has registered itself using the
 * SerialCreator class.  This is a kind of Pluggable Factory, related
 * to the ProductTrader pattern.
 *
 * \code
 * Serializable* LOA = SerialFactory::Instance()->New("ListOfAtoms");
 * ListOfAtoms* ploa = dynamic_cast<ListOfAtoms*>(LOA);
 * \endcode
 *
 * \author ajd27
 * \date 27 July 2000
 * \ingroup dmsystem
 **/

class SerialFactory
{
	static SerialFactory* _instance;
	typedef std::map<std::string, const SerialCreator*> creatorMap;
	creatorMap registry;
public:
	//! Retrieve an instance of the Singleton SerialFactory from anywhere in the code.
	static SerialFactory* Instance();
	//! Register a creator object with the factory.
	bool Register( const SerialCreator* const creator );
	//! Request an instance of a given type.
	Serializable* New( const std::string& type );
	~SerialFactory();
private:
	SerialFactory();
};

#endif //_SERIALFACTORY_H_
