// SteepestDescentMinimizer.cpp: implementation of the SteepestDescentMinimizer class.
//
//////////////////////////////////////////////////////////////////////
#include "DMFirstHeader.h"
#include <assert.h>
#include <iostream>
#include <fstream>
#include "DigitalMaterial.h"
#include "SteepestDescentMinimizer.h"
#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


SteepestDescentMinimizer::SteepestDescentMinimizer(double tolerance, int maximumIterations,double stepParameter)
{
	this->tolerance = tolerance;
	this->maximumIterations = maximumIterations; 
	this->stepParameter = stepParameter;
	verbose = false;
	nIterations = 0;
	gradNorm = 0.;
}


SteepestDescentMinimizer::~SteepestDescentMinimizer()

{

}

double SteepestDescentMinimizer::Minimize(double *parameters, Minimizable *minimizable)
{
	int j;
	const double *externalGradientPtr;
	this->minimizable = minimizable;
	nParameters = minimizable->GetNParameters();
	this->parameters = parameters;
	double *deltaParameters = new double[nParameters];
	double oldGradNorm,deltaParamSq;
	externalGradientPtr = minimizable->Gradient(parameters); // Initial gradient in xi
	gradNorm = 0.;
	for(j=0;j<nParameters;++j)
	  gradNorm += externalGradientPtr[j] * externalGradientPtr[j];
	oldGradNorm = gradNorm;

	//ofstream outFile("steepestDesc.dat",ios::out);
	//outFile.precision(8);
	
	for (nIterations=1;nIterations<=maximumIterations;++nIterations) 
	{
	  if(verbose)
	    {
		  std::cout << "Iteration " << nIterations << " GradNorm " << gradNorm << std::endl;
//	      for(n=0;n<nParameters;++n)
//		std::cout << parameters[n] << " ";
//	      std::cout << std::endl;
	      
	      /*
	      outFile << nIterations;
	      for(n=0;n<nParameters;++n)
		outFile << " " << parameters[n];
	      outFile << endl;
	      */
	    }

	  if (gradNorm <= tolerance)
	    return 0.;
	  
	  deltaParamSq = 0.;
	  for(j=0;j<nParameters;++j)
	    {
	      deltaParameters[j] = -externalGradientPtr[j]*stepParameter;
	      parameters[j] += deltaParameters[j];
	      deltaParamSq += deltaParameters[j] * deltaParameters[j];
	    }

	  externalGradientPtr = minimizable->Gradient(parameters);
	 
	  gradNorm = 0.;
	  for(j=0;j<nParameters;++j)
	    gradNorm += externalGradientPtr[j] * externalGradientPtr[j];


	  if(gradNorm > 1.1*oldGradNorm)
	    {
	      stepParameter *= 0.25;
	      std::cout << "Reducing step parameter to " << stepParameter << std::endl;
	    }
	  oldGradNorm = gradNorm;
	}
	  
	  delete [] deltaParameters;
	  return 0;
}

