#ifndef _THREAD_H_
#define _THREAD_H_

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif

#include "ThreadImp.h"

//! The system-independent implementation of threads.
/**
 * This is the actual Thread class.  It varies in its system-dependent
 * implementation which it gets from an abstract factory.
 * See ThreadImp for details.
 * \author Drew Dolgert
 * \date 21 Feb 2000
 * \ingroup dmsystem
 * \sa ThreadImp, WinThread, UnixThread, Threadable, Runnable
 */
class Thread : public ThreadImp
{
public:
	Thread(Runnable *runnable);
	Thread(Threadable *runnable);
	virtual ~Thread() { delete _thread; }
	void SetTask(Runnable *runnable) { _thread->SetTask(runnable); }
	void SetTask(Threadable *threadable) { _thread->SetTask(threadable); }
	int Start() { return _thread->Start(); }
	int Stop() { return _thread->Stop(); }
	int Pause() { return _thread->Pause(); }
	int Resume() { return _thread->Resume(); }
	int Toggle() { return _thread->Toggle(); }
	// Interface to return thread.
	bool IsRunning() { return _thread->IsRunning(); }
private:
	ThreadImp* _thread;
};

#endif // _THREAD_H_
