#ifndef _THREADABLE_
#define _THREADABLE_

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif
#include "Runnable.h"

class Thread; // need know no more here.

//! A base class for objects which can be run in a thread.
/**
 * \ingroup dmsystem
 *
 * Inherit from this class in order to create a class which can be run in
 * its own thread.  Your class must call IsRunning() every so often in order
 * to ask the owning Thread class whether it has been requested to stop.  Check
 * out this example:
 *
 * \code
 * class MyCalc : public Threadable {
 *     int i;
 *     void Run() {
 *         while (this->IsRunning()) {
 *             i = i + 1; // Do calculation here
 *             // Then notify display you have done a step.
 *         }
 *     }
 * };
 *
 * int main(void) {
 *     MyCalc sim();
 *     Thread lightning(&sim);
 *     lightning.Start();
 *     DoUserInterface();
 * }
 * \endcode
 * 
 * Because we use explicitly cooperative threads, our techniques are a bit different
 * than Java.  This class isn't just an interface.  It provides a point for
 * a thread to break for others.
 * \author ajd27
 * \sa Runnable, Thread
 */

class Threadable : public Runnable
{
public:
	Threadable();
	bool IsRunning();
	//! Tell the threadable who owns it.
	/*! This method is called only by the Thread class.  You should
		not call it directly. */
	void SetThread(Thread *const thread);
	virtual ~Threadable() {}
private:
	Thread * pThread;
};

#endif // _THREADABLE_
