// Timer.cpp: implementation of the Timer class.
//
//////////////////////////////////////////////////////////////////////

// Adapted from "Developing Optimized Code with Microsoft Visual C++ 6.0"
// by Martin Heller, March 1999 (http://www.msdn.microsoft.com/library/techart/optcode.htm)

#include "DMFirstHeader.h"

#include "DigitalMaterial.h"
#include "Timer.h"

#include "DMLastHeader.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Timer::Timer()
{
	isTiming = FALSE;
	Reset();
}

Timer::~Timer()
{

}

void Timer::Reset()
{
	DetermineTimer();	// Sets overhead and frequency of ticks per second
	totalTime = 0.0;
	latestTime = 0.0;
	numCalls = 0;

}

void Timer::StartClock()
{
	// Let any other stuff happen before we start
	MSG msg;
	PeekMessage(&msg,NULL,NULL,NULL,PM_NOREMOVE);
	Sleep(0);

	ASSERT(!isTiming);
	isTiming = TRUE;

	if (ttuHiRes)
	{
		SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL);
		QueryPerformanceCounter(&ticksStart);
	}
	else
	{
		SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL);
		timeStart = clock();
	}
}

double Timer::StopClock()
{
	if (ttuHiRes)
	{
		QueryPerformanceCounter(&ticksStop);
		SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);

		ASSERT(isTiming);
		isTiming = FALSE;

		double x = AverageTime();

		LARGE_INTEGER Freq = perfFreq;
		int Oht = overheadTicks;
		int ReduceMag = 0;

		// Results are 64 bits but we only do 32
		unsigned int High32 = ticksStop.HighPart - ticksStart.HighPart;
		while ( High32 )
		{
			High32 >>= 1;
			ReduceMag++;
		}
		if ( perfFreqAdjust || ReduceMag )
		{
			if ( perfFreqAdjust > ReduceMag )
				ReduceMag = perfFreqAdjust;
			ticksStart.QuadPart = Int64ShrlMod32(ticksStart.QuadPart, ReduceMag);
			ticksStop.QuadPart = Int64ShrlMod32(ticksStop.QuadPart, ReduceMag);
			Freq.QuadPart = Int64ShrlMod32(Freq.QuadPart, ReduceMag);
			Oht >>= ReduceMag;
		}
		// Reduced numbers to 32 bits, now can do the math
		if ( Freq.LowPart == 0 )
			latestTime = 0.0;
		else
			latestTime = ((double)(ticksStop.LowPart - ticksStart.LowPart - Oht))/Freq.LowPart;
	}
	else
	{
		timeStop = clock();
		SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);
		latestTime = ((double)(timeStop-timeStart)) / CLOCKS_PER_SEC;
	}

	totalTime += latestTime;
	numCalls++;
	return latestTime;
}

void Timer::DetermineTimer()
{
	// Assume the worst
	ttuHiRes = FALSE;
	if ( QueryPerformanceFrequency(&perfFreq) )
	{
		// We can use hires timer, determine overhead
		ttuHiRes = TRUE;
		overheadTicks = 2000;	// I changed from 200 to 2000: typical in debugger is 900 JPS
		perfFreqAdjust = 0;

		// JPS: He appears to be using lowest overhead for 20 tries
		for ( int i=0; i < 20; i++ )
		{
			int Ticks;
			StartClock();
			StopClock();
			// We assume less than 32 bits have been used by the null operation:
			// otherwise StopClock() will notice ReduceMag and screw up
			Ticks = ticksStop.LowPart - ticksStart.LowPart;
			if ( Ticks >= 0 && Ticks < overheadTicks )
				overheadTicks = Ticks;
		}
		// See if Freq fits in 32 bits; if not lose some precision
		int High32 = perfFreq.HighPart;
		while ( High32 )
		{         
			High32 >>= 1;
			perfFreqAdjust++;
		}
	}
	return;
}
