#include "DMFirstHeader.h"
#include <windows.h>
#include "SprintString.h"
#include "WinEggTimer.h"
#include "DMException.h"
#include "NotEnoughMemory.h"
#include "DMLastHeader.h"


#if(WINVER < 0x0500)
#pragma message ("WinEggTimer only works in Win2k, so you need to set WINVER=0x0500.")
#endif

//! The EggThread is the thread function used by WinEggTimer.
VOID CALLBACK WinEggCallBack( PVOID pvParam, BOOLEAN TimerOrWaitFired )
{
	PLONG plDone = static_cast<long*>( pvParam );
	::InterlockedIncrement( plDone );
}

WinEggTimer::WinEggTimer()
{
	hTimerQueue = 0;
	hTimer = 0;
	lDone = 0;
}

WinEggTimer::~WinEggTimer()
{
#ifdef WIN2K
	if ( 0 != hTimerQueue ) {
		BOOL bSuccess = DeleteTimerQueue( hTimerQueue );
		ASSERT( 0 != bSuccess );
		if ( 0 == bSuccess ) {
			DWORD dwError = GetLastError();
			throw DMException("WinEggTimer","~WinEggTimer",
				SprintString("Error %i deleting timer queue.", dwError));
		}
	}
#endif
}

bool WinEggTimer::DoneYet() const
{
#ifdef WIN2K
	LONG initialValue = ::InterlockedCompareExchange( &lDone, 1, 1 );
	return ( 0 != initialValue );
#else
	return true;
#endif
}


void WinEggTimer::SetDueTime( int secondsToExpire )
{
#ifdef WIN2K
	if ( 0 == hTimerQueue ) CreateTimerQueue();

	lDone = 0;
	if ( 0 == hTimer ) {
		// We give the timer a period because a period of zero
		// has the secondary meaning that this is a one-shot
		// timer.  We wouldn't be able to use ChangeTimerQueueTimer
		// on a one-shot timer but would have to create a new timer
		// instead.
		BOOL bSuccess = ::CreateTimerQueueTimer
			(
			&hTimer, // handle to timer
			hTimerQueue, // handle to timer queue
			WinEggCallBack, // timer callback function
			&lDone, // callback parameter
			1000*secondsToExpire, // timer due time
			1000*secondsToExpire, // timer period
			0 // flags
			);
		ASSERT( 0 != bSuccess );
		if ( 0 == bSuccess ) {
			DWORD dwError = ::GetLastError();
			throw DMException("WinEggTimer","SetDueTime",
				SprintString("Error %i creating a timer.",dwError));
		}

	} else {
		BOOL bSuccess = ::ChangeTimerQueueTimer
			(
			hTimerQueue,
			hTimer,
			1000*secondsToExpire,
			1000*secondsToExpire
			);
		ASSERT( 0 != bSuccess );
		if ( 0 == bSuccess ) {
			DWORD dwError = ::GetLastError();
			throw DMException("WinEggTimer","SetDueTime",
				SprintString("Error %i creating a timer.",dwError));
		}
	}
#endif
}

void WinEggTimer::CreateTimerQueue()
{
#ifdef WIN2K
	hTimerQueue = ::CreateTimerQueue( );
	ASSERT( NULL != hTimerQueue );
	if ( NULL == hTimerQueue ) {
		DWORD dwError = ::GetLastError();
		throw DMException("","",
			SprintString("Error %i creating timer queue.", dwError));
	}
#endif
}


