#ifndef _WINTHREAD_
#define _WINTHREAD_ 1

#include "DMFirstHeader.h"
#include <windows.h>
#include <process.h>
#include "DigitalMaterial.h"
#include "Threadable.h" // includes Runnable
#include "ThreadImp.h"

//! The Windows implementation of a Thread.
/**
 * Windows needs to put the simulation in a thread.
 * If we derive the simulation from a thread, we would
 * have trouble using the simulation class on different
 * platforms because threads are a hairy subject.
 * Therefore this thread will own a Threadable object.
 *
 * The rules of this thread class are as follows:
 * It is synchronous, so the methods do what they claim before they leave.
 * Only one client thread gets access at a time, so the thread is thread-safe.
 * Note no functions tell you how the thread is doing.  This is a thread class
 * for the decisive.
 * \author ajd27
 * \date 21 November 1999
 * \ingroup dmsystem
 * \sa Thread, ThreadImp, Runnable, Threadable
 */
class WinThread : public ThreadImp
{
public:
	WinThread();
	virtual void SetTask(Runnable *thread);
	virtual void SetTask(Threadable *thread);
	virtual int Start();
	virtual int Stop();
	virtual int Pause();
	virtual int Resume();
//	virtual void Lock();
//	virtual void Unlock();
	virtual int Toggle();
	// interface to simulation
	virtual bool IsRunning();
	virtual ~WinThread();

private:
	// This structure is used to pass data to the raw thread loop.
	typedef struct
	{
		Runnable *thread;
		CRITICAL_SECTION *csRunningRights;
	} PARAMS, *PPARAMS;
	
	// This is our raw thread loop.
	static unsigned __stdcall Loop(void * pvoid);

	Threadable *threadable;
	Runnable *runnable;
	// We implement the class so many client threads can call it, so
	// each function has to be locked before you enter. 
	// We could use a mutex (I was using a mutex for this), but a mutex
	// does no more and is not nearly as fast because it executes kernel-
	// level code.
	HANDLE hNotifyMama;
	HANDLE hClassLock; // mutex to lock class
	HANDLE hThread; // This is the thread itself.
	HANDLE hPauseEvent; // The thread waits on this event while paused.
	CRITICAL_SECTION csThreadLock;
	unsigned int dwThreadID;
	PARAMS params;
	bool bPleaseListen;
	bool bPleaseStop; // switch to get thread to stop its inner loop.
	bool bPaused;
	bool ThreadExists(); // private helper function to say if thread exists.
};


#endif // _WINTHREAD_
