#ifndef _HYSTERESIS_CMD_PARSERS_H_
#define _HYSTERESIS_CMD_PARSERS_H_

#include "DiscreteAvalanche.h"
#include "CmdParser.h"

class CmdStreamReader; // for help command.

class HysteresisSimulation;

class EmptyCmdParser : public CmdParser
{
public:
	EmptyCmdParser( const std::string& cmdString );
	virtual bool AcceptCmdLine( std::istream& s );
	virtual bool Accept() PUREVIRTUAL;
};

class IntCmdParser : public CmdParser
{
public:
	IntCmdParser( const std::string& cmdString );
	virtual bool AcceptCmdLine( std::istream& s );
	virtual bool AcceptInt( int arg ) PUREVIRTUAL;
};

class ToggleCmdParser : public CmdParser
{
public:
	ToggleCmdParser( const std::string& cmdString );
	virtual bool AcceptCmdLine( std::istream& s );
	virtual bool TurnOn( ) PUREVIRTUAL;
	virtual bool TurnOff( ) PUREVIRTUAL;
};


class SetupCmdParser : public EmptyCmdParser
{
	HysteresisSimulation* pSim;
public:
	SetupCmdParser( HysteresisSimulation* pSim );
	virtual bool Accept();
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class HelpCmdParser : public CmdParser
{
	CmdStreamReader* cmdReader;
public:
	HelpCmdParser( CmdStreamReader* cmdReader );
	virtual bool AcceptCmdLine( std::istream& s );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class DimCmdParser : public IntCmdParser
{
	int dim;
	HysteresisSimulation* pSim;
public:
	DimCmdParser( HysteresisSimulation* pSim );
	virtual bool AcceptInt( int dimension );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};



class LengthCmdParser : public CmdParser
{
	HysteresisSimulation* pSim;
public:
	LengthCmdParser( HysteresisSimulation* pSim );
	virtual bool AcceptCmdLine( std::istream& s );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class RestartCmdParser : public CmdParser
{
	HysteresisSimulation* pSim;
public:
	RestartCmdParser( HysteresisSimulation* pSim );
	virtual bool AcceptCmdLine( std::istream& s );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class QuitCmdParser : public EmptyCmdParser
{
public:
	QuitCmdParser();
	virtual bool Accept( );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};

class SeedCmdParser : public IntCmdParser
{
	int randSeed;
	HysteresisSimulation* pSim;
public:
	SeedCmdParser( HysteresisSimulation* pSim );
	virtual bool AcceptInt( int Seed );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class EndTimeCmdParser : public IntCmdParser
{
	HysteresisSimulation* pSim;
public:
	EndTimeCmdParser( HysteresisSimulation* pSim );
	virtual bool AcceptInt( int EndTime );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class TraceCmdParser : public IntCmdParser
{
	HysteresisSimulation* pSim;
public:
	TraceCmdParser( HysteresisSimulation* pSim );
	virtual bool AcceptInt( int traceFrequency );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class CheckpointCmdParser : public IntCmdParser
{
	HysteresisSimulation* pSim;
public:
	CheckpointCmdParser( HysteresisSimulation* pSim );
	virtual bool AcceptInt( int checkpointFrequency );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class DisorderCmdParser : public CmdParser
{
	double disorder;
	HysteresisSimulation* pSim;
public:
	DisorderCmdParser( HysteresisSimulation* pSim );
	virtual bool AcceptCmdLine( std::istream& s );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};

class RunCmdParser : public IntCmdParser
{
	int runCnt;
	HysteresisSimulation* pSim;
public:
	RunCmdParser( HysteresisSimulation* pSim );
	virtual bool AcceptInt( int runs );
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
private:
	bool Run( int runCnt );
};


class OnSpanningParser : public ToggleCmdParser
{
	HysteresisSimulation* pSim;
public:
	OnSpanningParser( HysteresisSimulation* pSim );
	virtual bool TurnOn();
	virtual bool TurnOff();
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class CorrelationParser : public ToggleCmdParser
{
	HysteresisSimulation* pSim;
public:
	CorrelationParser( HysteresisSimulation* pSim );
	virtual bool TurnOn();
	virtual bool TurnOff();
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class AvgCorrelationParser : public ToggleCmdParser
{
	HysteresisSimulation* pSim;
public:
	AvgCorrelationParser( HysteresisSimulation* pSim );
	virtual bool TurnOn();
	virtual bool TurnOff();
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class SizeDistributionParser : public ToggleCmdParser
{
	HysteresisSimulation* pSim;
public:
	SizeDistributionParser( HysteresisSimulation* pSim );
	virtual bool TurnOn();
	virtual bool TurnOff();
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class AvgSizeDistributionParser : public ToggleCmdParser
{
	HysteresisSimulation* pSim;
public:
	AvgSizeDistributionParser( HysteresisSimulation* pSim );
	virtual bool TurnOn();
	virtual bool TurnOff();
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class ColoringParser : public ToggleCmdParser
{
	HysteresisSimulation* pSim;
public:
	ColoringParser( HysteresisSimulation* pSim );
	virtual bool TurnOn();
	virtual bool TurnOff();
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class MagnetizationParser : public ToggleCmdParser
{
	HysteresisSimulation* pSim;
public:
	MagnetizationParser( HysteresisSimulation* pSim );
	virtual bool TurnOn();
	virtual bool TurnOff();
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};


class TimeSeriesParser : public ToggleCmdParser
{
	HysteresisSimulation* pSim;
public:
	TimeSeriesParser( HysteresisSimulation* pSim );
	virtual bool TurnOn();
	virtual bool TurnOff();
	virtual void ShortHelp( std::ostream& o ) const;
	virtual void LongHelp( std::ostream& o ) const;
};

#endif //_HYSTERESIS_CMD_PARSERS_H_
