#include "DMFirstHeader.h"
#include <iostream>
#include <fstream>

#include "DiscreteAvalanche.h"
#include "HysteresisSimulation.h"
#include "DMException.h"
#include "Timer.h"
#include "ExceptionHandler.h"
#include "NetCDFReader.h"
#include "BatchTimer.h"
#include "NetCDFCheckpointer.h"
#include "SimpleTracer.h"
#include "CmdParser.h"
#include "CmdLineParser.h"
#include "HysteresisCmdStreamReader.h"
#include "HysteresisCmdParsers.h"
#include "WinProcessFactory.h"
#include "DMLastHeader.h"

using namespace std;

int main( int argc, char *argv[] )
{
	//_crtBreakAlloc=1630;

	WinProcessFactory::Create();

	// Parse cmdline arguments.
	CmdLineParser progArgs( argc, argv );

	// Create the simulation that will do the work.
	HysteresisSimulation* pSim = new HysteresisSimulation();

	// Create something to execute commands.
	HysteresisCmdStreamReader readStream( pSim );

	bool bQuit = false;

	// Try to read commands from a file.
	if ( progArgs.HaveInputFile() && !bQuit ) {
		std::ifstream inputFile( progArgs.InputFile().c_str() );
		bQuit = readStream.ReadCmdStream( inputFile );
	}

	// Try to read commands from stdin.
	if (!bQuit) {
		bQuit = readStream.ReadCmdStream( std::cin );
	}

	delete pSim;
	return 0;
}

int other()
{
	bool bFromFile = false;
	bool bBatchTimer = true;
	int iLength = 400;
	int iSeed = 1;
	int RunCount = 10;
	int RunIndex = 0;
		HysteresisSimulation* pSim;

	try {
/*		BatchTimer bTimer;
		bTimer.SetEndTimeMinutes( (int)(1) );
		SimpleTracer tracer;
		tracer.SetTraceFrequency( 300 );
		bTimer.SetTracer( &tracer );
*/
//		while ((!bBatchTimer || !bTimer.IsTimeToQuit())
//			&& iLength<100000)		
		pSim = new HysteresisSimulation();
		for (RunIndex=0; RunIndex < RunCount; RunIndex++)
		{
			Timer clock;
			if (bFromFile) {
				NetCDFReader lector("start.dat");
				pSim = dynamic_cast<HysteresisSimulation*>(lector.Get("checkpoint"));
			} else {
				//XXX problem here.
				pSim->SetSideLength( iLength );
				pSim->SetSeed( iSeed+RunIndex );
				pSim->SetDisorder( 0.7 );
			}
			//pSim->SetBatchTimer( &bTimer );
			pSim->Initialize();
			pSim->Verify();
			clock.StartClock();

			NetCDFCheckpointer cp( pSim );
			//bTimer.SetCheckpointer( &cp );
			std::cout << "Starting to run for " << iLength << " points." << std::endl;
			pSim->Run();
			clock.StopClock();
			//if (bBatchTimer && bTimer.IsTimeToQuit()) {
			//	std::cout << iLength << " killed after " << clock.TotalTime() << std::endl;
			//} else {
				std::cout << iLength << " \t" << clock.TotalTime() << std::endl;
			//}
			//pSim->SaveResults();
		//	iLength *= 2;
			++iSeed;
		}
	} catch (DMException e) {
		std::cout << e;
	}
	delete pSim;
	TRACE( "Leaks before 100 aren\'t my fault, darn it.\n");
	return 0;
}