#ifndef _AVALANCHE_LISTENER_H_
#define _AVALANCHE_LISTENER_H_

#include "DMFirstHeader.h"
#include "DiscreteAvalanche.h"

class AvalancheAlgorithm;

//! An avalanche listener is notified at the end of each avalanche.
/*! Abstract interface to observers who
	are notified at the end of each avalanche.
		*/
class AvalancheListener
{
public:
	//! This is the method called at the end of every avalanche.
	/*!
	If you derive a class from AvalancheListener, you do 
	not need to make its AvalancheCompleted member function
	virtual.  The compiler should then be able to construct
	the object without a vftable.  That is, the call to
	AvalancheCompleted can be inlined.

  @param avalanche Pointer to the subject we are observing.
  @param dField The field at which the avalanche occurred.
		*/
	virtual void AvalancheCompleted
		(
		AvalancheAlgorithm*	avalanche, 
		double				dField
		) PUREVIRTUAL;

	virtual void BeginSweep( bool bKeepData ) PUREVIRTUAL;
};

#endif //_AVALANCHE_LISTENER_H_
