#ifndef _BRUTEAVALANCHESTART_H_
#define _BRUTEAVALANCHESTART_H_

#include "DMFirstHeader.h"
#include "DiscreteAvalanche.h"
#include "SpinIndex.h"
#include "SpinLattice.h"
#include "FixedFieldAvalancheStart.h"

class FixedFieldAvalanche;

//! The brute start searches every spin in the lattice for the most likely to flip.
/*! This should test whether SortedListAvalancheStart gives the right result.
	It searches the lattice in N^2 time for the next spin to flip.

	\author ajd27
	\date 17 Aug 2000
 */
class BruteAvalancheStart : public FixedFieldAvalancheStart
{
public:
	//! Constructor
	BruteAvalancheStart();
	//! Destructor
	~BruteAvalancheStart();
	//! Tell this which algorithm it's working with before we begin.
	void SetAlgorithm( FixedFieldAvalanche* avalanche );
	//! You must call Initialize after setting the algorithm.
	void Initialize();
	//! Ask whether everything is ready to rumble.
	void Verify() const; // throw (FailedVerify);
	//! Finds the start of the next avalanche.
	AvalancheAlgorithm::Status
		FindNextStart( double* H, SpinIndex *iFirstSpin );
private:
	enum { ALGORITHM = 0x1,
		INITIALIZE = 0x2
	};
	int iVerify;
	//! Cached from AvalancheAlgorithm
	const SpinLattice* lattice;
	//! Cached from AvalancheAlgorithm
	const double *pdRandomField;
	//! Cached from AvalancheAlgorithm
	const char *pcNUp;
	//! Cached from AvalancheAlgorithm
	int Z;
	//! Cached from AvalancheAlgorithm
	int N;
};

#endif //_BRUTEAVALANCHESTART_H_
