#ifndef _DMALLOCATOR_H_
#define _DMALLOCATOR_H_

//! Allocation function to handle new on different platforms.
/*! This code exists because there aren't many good options
	for safely allocating a series of pointers.

	In Linux, operator new() by default throws std::bad_alloc and
	sets the pointer to an undetermined, nonzero state.  In Windows,
	operator new() by default sets the pointer to zero when it
	fails to allocate.  We could use operator new (std::nothrow) 
	everywhere so that the linux version acts like Windows, except
	that, when compiling in debug mode, MSVC++ throws a compiler
	error when it sees you try to delete something allocated with
	std::nothrow.

	The net result is a lot of repititious code, best summarized
	in a template, as below.

	\code
		try {
			avalanche = DMAllocate<SortedListAvalanche>();
			sweepLogic = DMAllocate<SweepLogic>();
		} catch ( std::bad_alloc ) {
			DeleteArrays();
			throw NotEnoughMemory("Class","Function",
				"Could not allocate 13kb of memory.");
		}
	\endcode


    \author ajd27
	\date 6 Feb 2001
	*/

#ifdef USE_DBGNEW
//#define new(s) ::operator new(s, _NORMAL_BLOCK, __FILE__, __LINE__)
//#define new(s) ::operator new(_NORMAL_BLOCK, __FILE__, __LINE__)
#define DMALLOCATOR_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
#define DMALLOCATOR_DELETE delete( _NORMAL_BLOCK, __FILE__, __LINE__ )
// #define new(s) new( _NORMAL_BLOCK, __FILE__, __LINE__)
//#define new new( _NORMAL_BLOCK, __FILE__, __LINE__)
#else
#define DMALLOCATOR_NEW new
#define DMALLOCATOR_DELETE delete
#endif //_DEBUG

template<class Type>
Type* DMAllocate()
{
	Type* Ptr;

	try {
		Ptr = DMALLOCATOR_NEW Type();
	} catch ( std::bad_alloc ) {
		Ptr = 0;
	}

	if (0 == Ptr ) throw std::bad_alloc();
	return Ptr;
}

//! Allocation function for arrays to handle new on different platforms.
/*! Same as other except you pass in a count of how many to allocate
	in the array.
	*/
template<class Type>
Type* DMAllocate( int Cnt )
{
	Type* Ptr;

	try {
		Ptr = DMALLOCATOR_NEW Type[Cnt];
	} catch ( std::bad_alloc ) {
		Ptr = 0;
	}

	if (0 == Ptr ) throw std::bad_alloc();
	return Ptr;
}

#endif // _DMALLOCATOR_H_