#ifndef _DISCRETEAVALANCHE_H_
#define _DISCRETEAVALANCHE_H_

/*! \mainpage Avalanches in Hysteresis

	This code simulates avalanches in the random field Ising Model
	(RFIM).  The bulk of it was written by Matt Kuntz for his
	Ph.D. thesis with Jim Sethna at Cornell University Physics.

	The current kinds of avalanches it can compute include
		- Brute force
		- Sorted list
		- Bits only
	where the bits only version using probability distributions
	to determine which spins to flip.  This dramatically reduces
	memory usage making simulations of 100,000^2 possible on
	Intel machines with 2Gb of memory.

	Features include:
		- object-oriented encapsulation of algorithms suitable to classroom use
			- Avalanche algorithms can be plugged in and out
			- Adding measurements of avalanche progress is simple.
		- gracefully saves and retrieves from disk mid-calculation (checkpointing)
		- stores spins as single bits
		- lots of assertions for your debugging pleasure

	We would be happy if this code were useful to you.
	You have permission to copy and modify this code if you keep with it
	a mention of where it came from.

	<H2>Requirements</H2>
	This code relies on <a href="../../../DMSystem/doc/html/index.html">DMSystem.</a>
	You need to compile DMSystem with NetCDF support in order to use checkpointing.

	This library compiles with
		- Windows Microsoft VC++

	There is also an interface to Python through SWIG.
	
	\author Matt Kuntz
	\author Jim Sethna
	\author Andrew Dolgert
	*/


#include "DigitalMaterial.h"

#define PUREVIRTUAL =0

#endif // _DISCRETEAVALANCHE_H_