#ifndef _GAUSSIANINITIALIZER_H_
#define _GAUSSIANINITIALIZER_H_

#include "DMFirstHeader.h"
#include <math.h>
#include "DiscreteAvalanche.h"
#include "fdlibm.h"
#include "Serializable.h"

class Rand;

	// in some compilers (MS Visual C++), M_2_SQRTPI is not defined.  If it
	// is not defined.  If it is not defined, define it.
#if !defined(M_2_SQRTPI)
#define M_2_SQRTPI 1.12837916709551257390
#endif

//! Initializes a list of doubles to a gaussian distribution.
/** Pass an initializer to an AvalancheAlgorithm
  * 
  * \author ajd27
  * \date 15 Aug 2000
  */
class GaussianInitializer : public Serializable
{
	double dDisorder;
	Rand* rand;
public:
	//! Constructor
	GaussianInitializer();
	//! Set the spread in the gaussian.
	void SetDisorder( double dDisorder );
	//! SetRand
	void SetRand( Rand* rand );
	//! This sets the value of the given random field.  It does not initialize the class.
	void Initialize( double *pdRandomField, int iLength );
	//! Return the probability distribution.
	double rho(double h) const;
	//! Return the integrated probability distribution.
	double intrhoc(double h) const;

	/*! Serializable interface */
	//@{
	virtual void Save( DMWriter *writer ) const;
	//! Call SetRand after loading.
	virtual void Load( DMReader *reader );
	virtual const std::string GetType() const;
	//@}
	
private:
	//! Cached temporary variable.
	double invSqrt2R;
};

	// rho: the probability distribution
inline double GaussianInitializer::rho(double h) const
{
	double dt = h*invSqrt2R;
	return 0.5 * M_2_SQRTPI * invSqrt2R * exp(-dt*dt);
}

	// intrhoc: the integrated probability distribution
inline double GaussianInitializer::intrhoc(double h) const
{
	return 0.5*erfc(h*invSqrt2R);
}


#endif //_GAUSSIANINITIALIZER_H_
