#include "DMFirstHeader.h"
#include <iostream>
#include "MagnetizationTracer.h"
#include "DMException.h"
#include "MagnetizationObserver.h"
#include "DMLastHeader.h"


MagnetizationTracer::MagnetizationTracer()
{
	// Set a default.
	m_traceFrequency = 300;
}

MagnetizationTracer::~MagnetizationTracer() {}

void MagnetizationTracer::SetMagnetizationObserver( MagnetizationObserver* mag )
{
	ASSERT( 0 != mag );
	if ( 0 == mag ) throw DMException( "MagnetizationTracer",
		"SetMagnetizationObserver","Cannot be a null pointer.");

	m_mag = mag;
}

void MagnetizationTracer::SetTraceFrequency( int seconds )
{
	ASSERT( seconds > 0 );
	if ( seconds <= 0 ) throw DMException("MagnetizationTracer",
		"SetTraceFrequency","The frequency should be greater than zero.");

	m_traceFrequency = seconds;
}

int MagnetizationTracer::GetNextTrace( int currentTime, int endTime )
{
	return ((currentTime/m_traceFrequency)+1)*m_traceFrequency;
}

void MagnetizationTracer::Trace( int currentTime )
{
	double currentMagnetization = m_mag->GetLastMagnetization();
	std::cout.precision(20);
	std::cout << "Time: " << currentTime/60.0
		<< " min. Magnetization: " << currentMagnetization << std::endl;
}
