#include "DMFirstHeader.h"
#include <fstream>
#include "NetCDFCheckpointer.h"
#include "DMException.h"
#include "NetCDFWriter.h"
#include "DMLastHeader.h"

NetCDFCheckpointer::NetCDFCheckpointer()
{
	toSave = 0;
	calculator.SetFrequency(120*60);
	bFlipper = true;
}

NetCDFCheckpointer::NetCDFCheckpointer( Serializable* serial )
{
	toSave = 0;
	calculator.SetFrequency(120*60);
	SetSerializable( serial );
	bFlipper = true;
}


void NetCDFCheckpointer::SetSerializable( Serializable *serial )
{
	ASSERT( 0 != serial );
	if ( 0 == serial ) throw DMException("NetCDFCheckpointer",
		"SetSerializable", "Tried to pass a null pointer for the serializable.");

	this->toSave = serial;
}

/*! The default value for the frequency if you don't call this
	is two hours.
	*/
void NetCDFCheckpointer::SetFrequency( int uiSeconds )
{
	calculator.SetFrequency(uiSeconds);
}

/*! Time is measured in seconds.  It returns negative when 
	there is no more time before the end to do a checkpoint.
	That may be time to quit.

	\param uiCurrentTime Time since time=0.
	\param uiEndTime May be zero if there is no end time
	\return Time of next checkpoint beyond current time or 
	a negative number if there are no checkpoints past the current time.
	*/
int NetCDFCheckpointer::GetNextCheckpoint( int uiCurrentTime,
													   int uiEndTime )
{
	return calculator.GetNextCheckpoint( uiCurrentTime, uiEndTime );
}

void NetCDFCheckpointer::Checkpoint( void )
{
	ASSERT( 0 != toSave );
	if (0 == toSave) throw DMException( "NetCDFCheckpointer",
		"Checkpoint","There is no object to save.");

	if (bFlipper) {
		NetCDFWriter scrivener( "checkpoint0.dat");
		scrivener.Put( toSave, "checkpoint" );
		std::ofstream emptyIt("checkpoint1.dat");
	} else {
		NetCDFWriter scrivener( "checkpoint1.dat");
		scrivener.Put( toSave, "checkpoint" );
		std::ofstream emptyIt("checkpoint0.dat");
	}
	bFlipper = !bFlipper;
}