#ifndef _NETCDF_CHECKPOINTER_H_
#define _NETCDF_CHECKPOINTER_H_

#include "DMFirstHeader.h"
#include "DiscreteAvalanche.h"
#include "Checkpointer.h"
#include "CheckpointCalculator.h"

class Serializable;

//! Concrete checkpointer to save a serializable with NetCDF.
/*! 
	\author ajd27
	\date 4 Sep 2000
	*/
class NetCDFCheckpointer : public Checkpointer
{
	CheckpointCalculator calculator; //!< Tells us when to checkpoint.
	Serializable *toSave;		//!< Whom we shall save.
	bool bFlipper;
public:
	//! Constructor.
	NetCDFCheckpointer();

	//! Constructor with serializable.
	NetCDFCheckpointer( Serializable* serial );

	//! Tell it how often to checkpoint.
	void SetFrequency( int uiSeconds );
	
	//! Tell it how often to checkpoint in minutes.
	void SetFrequencyMinutes( int iMinutes ) {
		this->SetFrequency( 60*iMinutes );
	}

	//! Tell us whom to save.
	void SetSerializable( Serializable* serial );

	//! Get the time for the next checkpoint.
	int GetNextCheckpoint( int uiCurrentTime,
		int uiEndTime );

	//! Do the checkpointing.
	void Checkpoint( void );
};

#endif //_NETCDF_CHECKPOINTER_H_

