#include "DMFirstHeader.h"
#include <fstream>
#include "OnSpanning.h"

#include "SerialBuilder.h"
#include "DMException.h"
#include "AvalancheAlgorithm.h"
#include "DMReader.h"
#include "DMWriter.h"
#include "SprintString.h"
#include "DMLastHeader.h"

const std::string sType = std::string("OnSpanning");
const SerialBuilder<OnSpanning> *_creator = new
	SerialBuilder<OnSpanning>(sType);

OnSpanning::OnSpanning( void ) : spanning(0), isLatticeSpanned(false), runIdx(-1)
{
	this->avalancheIdx = 0;
  this->isSavingAllSpans = true;
}

OnSpanning::~OnSpanning( void ) { }


void OnSpanning::SetSpanning( CheckSpanning* spanning )
{
	ASSERT( 0 != spanning );
	if ( 0 == spanning ) throw
		DMException( "OnSpanning", "SetSpanning",
		"Argument pointer should be nonzero" );

	this->spanning = spanning;
}


void OnSpanning::AvalancheCompleted
		(
		AvalancheAlgorithm*	avalanche, 
		double				dField
		)
{
	if ( spanning->IsSpanning() ) {

    if ( this->isSavingAllSpans || !this->isLatticeSpanned ) {

		  this->isLatticeSpanned = true;
	
		  SprintString filename( "spanning%i_%i", runIdx, avalancheIdx );
		  SaveConfiguration( avalanche->GetLattice(), filename );
    }

	} // else do nothing.

  this->avalancheIdx++;
}


void OnSpanning::SaveConfiguration( const SpinLattice* const lattice, const std::string& filename )
{
	SpinCoord* coords = new SpinCoord[ lattice->GetDimension() ];

	std::ofstream outfile( filename.c_str() );

  outfile << "Avalanche index: " << runIdx << std::endl;
	const SpinCoord *const dimensions = lattice->GetSideLengths();
	outfile << "Box size: ";
	outfile << dimensions[0];
	for ( int dimIdx = 1; dimIdx < lattice->GetDimension(); dimIdx++ )
		outfile << " " << dimensions[ dimIdx ];
	outfile << std::endl;

	for ( SpinIndex pos = 0; pos < lattice->GetLength(); pos++ ) {

		if ( lattice->Get(pos) == 1 ) {
			lattice->GetCoords( pos, coords );
			outfile << coords[0];
			for ( int dimIdx = 1; dimIdx< lattice->GetDimension(); dimIdx++ ) {
				outfile << " " << coords[ dimIdx ];
			}
			outfile << std::endl;
		}

	}

	delete[] coords;
}


void OnSpanning::BeginSweep( bool bKeepData )
{
	this->runIdx++;
	this->isLatticeSpanned = false;
}


void OnSpanning::Save( DMWriter* writer ) const
{
	writer->PutInt( runIdx, "numRuns" );
	writer->PutBool( isLatticeSpanned, "isSpanned" );
}


void OnSpanning::Load( DMReader* reader )
{
	runIdx = reader->GetInt( "numRuns" );
	isLatticeSpanned = reader->GetBool( "isSpanned" );
}


const std::string OnSpanning::GetType() const
{
	return sType;
}
