#include "RateSeries.h"

RateSeries::RateSeries()
{
	dTotalTime = 0;
	iTotalRuns = 0;
	dRecentRate = 0;
	dTotalRate = 0;
	dMinRate = 0;
}

RateSeries::~RateSeries()
{
}

/*! Register the current number of runs.
	@param iRuns int Number of runs since last checkin.
	@param dCurrentTime double Number of seconds since start of sim.
	*/
void RateSeries::SetDataPoint( int iRuns, double dCurrentTime )
{
	dRecentRate = iRuns/(dCurrentTime-dTotalTime);
	dTotalTime = dCurrentTime;
	iTotalRuns += iRuns;
	dTotalRate = iTotalRuns/dTotalTime;
	dMinRate = (dTotalRate<dRecentRate) ? 
		dTotalRate : dRecentRate;
}

/*! You need to call SetDataPoint with the current time before
	you can call this function.
	
	@param dTime double A time since start of this class recording.
	@return int 0 if time is less than current.
	@return int 0 if one timestep is too little to reach the time.
	@return int Best guess of number of steps to time.
	*/
int RateSeries::RunsToTime( double dTime )
{
	if (dTime < dTotalTime) return 0;
	if (iTotalRuns < 5) return 1;
	return (int)((dTime-dTotalTime)*dMinRate);
}