#include "SimScheduler.h"
#include "Timer.h"

SimScheduler::SimScheduler()
{
	timer = new Timer();
	// When in doubt, the answer is yes.
	bCheckpoint = true;
	bTrace = true;
	bTotalAllowed = false;
	bTotalGuess = false;
	dTraceFrequency = 60*5;
	dCheckpointFrequency = 60*60*4;
}

SimScheduler::~SimScheduler()
{
	delete Timer();
}

void SimScheduler::SetTimeLimit( int dTotalAllowed )
{
	this->dTotalAllowed = dTotalAllowed;
	bTotalAllowed = true;
}

void SimScheduler::SetSuggestedTime( int iTotalGuess )
{
	this->dTotalGuess = dTotalGuess;
	bTotalGuess = true;
}

void SimScheduler::StartScheduling()
{
	timer->StartClock();
}

int SimScheduler::RunsDone( int iNumberOfRuns )
{
	dCurrentTime = timer->TotalTime();
	rate.SetDataPoint( iNumberOfRuns, dCurrentTime );

	bCheckpoint = ShouldCheckpoint();
	bTrace = ShouldTrace();

	if (bCheckpoint) dLastCheckpoint = dCurrentTime;
	if (bTrace) dLastTrace = dCurrentTime;

	double dNextTrace = NextTrace();
	double dNextCheckpoint = NextCheckpoint();
	if (dNextTrace < dNextCheckpoint)
		iSuggestedRuns = RunsToTime(dNextTrace);
	else
		iSuggestedRuns = RunsToTime(dNextCheckpoint);

	if (0 == iSuggestedRuns) iSuggestedRuns += 1;
	return iSuggestedRuns;
}

bool SimScheduler::IsCheckPointTime() const
{
	return bCheckpoint;
}

bool SimScheduler::IsTraceTime() const
{
	return bTrace;
}

double SimScheduler::NextCheckpoint()
{
	double dFreq = dLastCheckpoint + dCheckpointFrequency;
	if (!bTotalAllowed) return dFreq;

	double dCPDelta = dTotalAllowed-dCurrentTime;
	if (dCPDelta > 2*60*60 ) { // 2 hours
		dSched = dTotalAllowed - 2*60*60;
	} else if (dCPDelta > 15*60 ) { // 15 minutes
		dSched = dTotalAllowed - 15*60;
	}
	if (dSched < dFreq) return dSched;
	return dFreq;
}

double SimScheduler::NextTrace()
{
	return dLastTrace + dTraceFrequency;
}

bool SimScheduler::ShouldCheckpoint()
{
	if (rate.RunsToTime(NextCheckpoint()) == 0)
		return true;
	return false;
}

bool SimScheduler::ShouldTrace()
{
	if (rate.RunsToTime(NextTrace()) == 0) return true;
	if (NextTrace()-dCurrentTime < 0.05*dTraceFrequency)
		return true;
	return false;
}