#ifndef _SIM_SCHEDULER_H_
#define _SIM_SCHEDULER_H_

#include "DiscreteAvalanche.h"
#include "RateSeries.h"

class Timer;

class SimScheduler
{
	Timer *timer;
	RateSeries rate;

	bool bCheckpoint;
	bool bTrace;

	bool bTotalAllowed;
	bool bTotalGuess;

	double dTotalAllowed;
	double dTotalGuess;

	double dTraceFrequency;
	double dCheckpointFrequency;
public:
	//! Constructor.
	SimScheduler();

	//! Destructor.
	~SimScheduler();

	//! Set the number of seconds allowed for this computation.
	/*! You don't have to set a time limit, but if you do,
		this will make sure to checkpoint before it runs out.
		*/
	void SetTimeLimit( int iTotalAllowed );

	//! If we have a feeling for how long it should take.
	/*! You don't need to set a suggested time, but it will
		give this routine a better idea of how often you want
		to write tracing statements.
		*/
	void SetSuggestedTime( int iTotalGuess );

	//! Tell it we are about to start timing runs.
	/*! You have to call this before your first run.
		It sets the start time for all calculations.
		*/
	void StartScheduling();

	//! Check in with number of runs.
	int RunsDone( int iNumberOfRuns );

	//! Ask whether it's time to checkpoint.
	bool IsCheckpointTime() const;

	//! Ask whether it's time to output trace results.
	bool IsTraceTime() const;
};

#endif //_SIM_SCHEDULER_H_
