#ifndef _SPIN_LATTICE_H_
#define _SPIN_LATTICE_H_

#include "DMFirstHeader.h"
#include "DiscreteAvalanche.h"

/*! \def POW2LATTICE
	\brief Tells the compiler to use the SpinLattice for powers of two. 

	In order for the compiler to be able to inline SpinLattice functions,
	there can be only one SpinLattice in the library, so we define the
	current version using a typedef.
	*/

#ifdef POW2LATTICE
// This message is in here because it is possible, and likely,
// to compile the console separately from the library using
// different defines so one thinks it uses Pow2SpinLattice and
// the other thinks it uses GeneralSpinLattice.
#pragma message( "  SpinLattice::Using the SpinLattice for powers of two." )
#include "Pow2SpinLattice.h"
typedef Pow2SpinLattice SpinLattice;
#else

/*! \def LATTICE2D
	\brief Tells the compiler to use the SpinLattice for two dimensions. 

	In order for the compiler to be able to inline SpinLattice functions,
	there can be only one SpinLattice in the library, so we define the
	current version using a typedef.

	This particular SpinLattice is currently deprecated because the
	version for powers of two in N dimensions is much faster than the
	two-dimensional one.
	*/

#ifdef LATTICE2D
#pragma message( "  SpinLattice::Using the SpinLattice for two dimensions." )
#include "SpinLattice2D.h"
#typedef SpinLattice2D SpinLattice;
#else

#pragma message( "  SpinLattice::Using the general SpinLattice." )
#include "GeneralSpinLattice.h"
typedef GeneralSpinLattice SpinLattice;


#endif // 2DLATTICE
#endif // POW2LATTICE

#endif //_SPIN_LATTICE_H_
