#ifndef _TIME_SERIES_H_
#define _TIME_SERIES_H_

#include "DMFirstHeader.h"
#include <vector>
#include "DiscreteAvalanche.h"
#include "AvalancheListener.h"
#include "Serializable.h"
#include "AvalancheAlgorithm.h"

//! Observes the time series of a spin avalanche.
class TimeSeries : public AvalancheListener, public Serializable
{
	std::vector<double> H;
	std::vector<int> size;
	int lastM;
	AvalancheAlgorithm* avalanche;
public:	
	//! Constructor.
	TimeSeries();

	//! Give this a pointer to the AvalancheAlgorithm.
	void SetAlgorithm( AvalancheAlgorithm* avalanche );

	//! Call this after setup and before using the class.
	void Initialize();

	//! Call this before doing a sweep.
	void BeginSweep( bool bKeepData = false );

	//! Called at the end of an avalanche.
	void AvalancheCompleted( AvalancheAlgorithm* avalanche,
							double h);

	/*! Serializable interface */
	//@{
	void Save( DMWriter *writer ) const;
	void Load( DMReader *reader );
	const std::string GetType() const;
	//@}

	friend std::ostream& operator<<( std::ostream& s, const TimeSeries& r );
private:
	void DeleteArrays();	
	bool bSerialized;

	enum { VERIFY_ALGORITHM=0x1 };
	int iVerify;
};

extern std::ostream& operator<<( std::ostream& s, const TimeSeries& r );


inline void
TimeSeries::AvalancheCompleted( AvalancheAlgorithm* avalanche,
										   double h)
{
	int M = avalanche->GetM();
	size.push_back((M - lastM) / 2);
	H.push_back(h);
	lastM = M;
}

#endif //_TIME_SERIES_H_
