#ifndef _WALL_CLOCK_TIMER_H_
#define _WALL_CLOCK_TIMER_H_

//! Timer which measures seconds not accounting for thread times.
/*! Most of our timers need to be very precise and measure thread times,
	but a wall clock time is useful for batch timing and tracing of a program.
	It is much more low-tech, needing only to return times in seconds (that 
	should be enough for an unsigned int to represent a year, if I have it 
	right.  (Pi times 10^7 seconds in a year.)

	\author ajd27
	\date 4 Sep 2000
	*/
class WallClockTimer
{
public:
	//! When you call start, you set time zero.
	virtual void Start() = 0;
	//! Returns time in seconds since start.
	virtual unsigned int GetTime() = 0;
};

#endif //_WALL_CLOCK_TIMER_H_
