#ifndef _WIN_RES_GUARD_H_
#define _WIN_RES_GUARD_H_

#include "DMFirstHeader.h"
#include <windows.h>
#include "DiscreteAvalanche.h"

//! A general critical section for threaded apps.
/*!	Taken from Richter.
	*/
class WinResGuard
{
public:
	WinResGuard() { m_lGrdCnt = 0; InitializeCriticalSection( &m_cs ); }
	~WinResGuard() { DeleteCriticalSection( &m_cs ); }

	BOOL IsGuarded() const { return( m_lGrdCnt > 0) ; }

public:
	class WinGuard
	{
	public:
		WinGuard( WinResGuard& rg ) : m_rg(rg) { m_rg.Guard(); };
		~WinGuard() { m_rg.Unguard(); }

	private:
		WinResGuard& m_rg;
	};

private:
	void Guard() { EnterCriticalSection(&m_cs); m_lGrdCnt++; }
	void Unguard() { m_lGrdCnt--; LeaveCriticalSection(&m_cs); }

	friend class WinResGuard::WinGuard;

private:
	CRITICAL_SECTION m_cs;
	long m_lGrdCnt;
};

#endif //_WIN_RES_GUARD_H_
