#include "HysteresisCanvas.h"
#include "EventID.h"


BEGIN_EVENT_TABLE(HysteresisCanvas, wxWindow)
	EVT_PAINT			(	HysteresisCanvas::OnPaint)
	EVT_SCROLLWIN		(	HysteresisCanvas::OnScroll)
	EVT_SIZE			(	HysteresisCanvas::OnSize)
	EVT_ERASE_BACKGROUND(	HysteresisCanvas::OnEraseBackground)
END_EVENT_TABLE()


const int HysteresisCanvas::m_ColorCnt = 216;

HysteresisCanvas::HysteresisCanvas
	(
	const ColoringObserver* coloring,
	wxWindow* parent,
	const wxWindowID id,
	const wxPoint& pos,
	const wxSize& size
	)
	: wxScrolledWindow(parent, id, pos, size, wxSUNKEN_BORDER | wxTAB_TRAVERSAL )
{
	m_Coloring = coloring;
	m_CellSize = 4;
	m_scBoardX = coloring->GetBounds()[0];
	m_scBoardY = coloring->GetBounds()[1];;

	// Set up colors to netscape pallette
	wxColour ColorTable[ m_ColorCnt ];
	// Why won't this loop work if r,g,b are of type unsigned char?
	int i=0;
	for(int r=0;r<256;r+=0x33)
	{
		for(int g=0;g<256;g+=0x33)
		{
			for(int b=0;b<256;b+=0x33)
			{
				ColorTable[i].Set(r,g,b);
				i++;
			}
		}
	}
	ASSERT( i == m_ColorCnt );

	Colors.resize( m_ColorCnt );

	int ColorIdx;
	for ( ColorIdx = 0; ColorIdx < m_ColorCnt; ColorIdx++ ) {
		Colors[ColorIdx].pen = new wxPen( ColorTable[i], 1, wxSOLID );
		Colors[ColorIdx].pen->SetColour( ColorTable[ ColorIdx ] );
		Colors[ColorIdx].brush = new wxBrush( ColorTable[i], wxSOLID );
		Colors[ColorIdx].brush->SetColour( ColorTable[ ColorIdx ] );
	}

	SetBackgroundColour( *wxWHITE );
}


HysteresisCanvas::~HysteresisCanvas()
{
	int ColorIdx;
	for ( ColorIdx = 0; ColorIdx < m_ColorCnt; ColorIdx++ ) {
		delete Colors[ColorIdx].pen;
		delete Colors[ColorIdx].brush;
	}
}


/*!
	This routine should do something.
	*/
void HysteresisCanvas::Recenter( wxInt32 i, wxInt32 j )
{
	// Redraw everything.
	Refresh( FALSE );
}


void HysteresisCanvas::SetCellSize( int cellsize )
{
	m_CellSize = cellsize;

	SetScrollbars( m_CellSize, m_CellSize, m_scBoardX, m_scBoardY );

	Refresh( FALSE );
}


void HysteresisCanvas::DrawCell( wxInt32 i, wxInt32 j, int Color )
{
	wxClientDC dc( this );
	
	if ( (Color % 2) == 0 ) {
		dc.SetPen( *wxBLACK_PEN );
		dc.SetBrush( *wxBLACK_BRUSH );
	} else {
		dc.SetPen( *wxWHITE_PEN );
		dc.SetBrush( *wxWHITE_BRUSH );
	}

	dc.BeginDrawing();
	DrawCell( i, j, dc );
	dc.EndDrawing();
}


void HysteresisCanvas::DrawCell( wxInt32 i, wxInt32 j, wxDC &dc )
{
	wxCoord x = CellToX( i );
	wxCoord y = CellToY( j );

	if ( 1 == m_CellSize ) {
		dc.DrawPoint( x, y );
	} else {
		dc.DrawRectangle( x, y, m_CellSize, m_CellSize );
	}
}


void HysteresisCanvas::OnPaint( wxPaintEvent& event )
{
	wxPaintDC dc(this);
	PrepareDC( dc );

	wxRect rect = GetUpdateRegion().GetBox();
	wxCoord x, y, w, h;

	// Find damaged area.
	x = rect.GetX();
	y = rect.GetY();
	w = rect.GetWidth();
	h = rect.GetHeight();

	int scXIdx = XToCell( x );
	int scYIdx = YToCell( y );
	int scXCnt = XToCell( x+w-1 );
	int scYCnt = YToCell( y+h-1 );
	
	scXCnt = m_scBoardX;
	scYCnt = m_scBoardY;

	dc.BeginDrawing();

	int coords[2];

	// Draw all cells.
	for ( scXIdx = 0; scXIdx<scXCnt; scXIdx++ ) {
		coords[0] = scXIdx;

		for ( scYIdx = 0; scYIdx<scYCnt; scYIdx++ ) {
			coords[1] = scYIdx;
			int color = m_Coloring->GetColor( coords ) % m_ColorCnt;
			//int color = (scXIdx+scYIdx) % m_ColorCnt;

			dc.SetPen( *(Colors[color].pen) );
			dc.SetBrush( *(Colors[color].brush) );

			x = CellToX( scXIdx );
			y = CellToY( scYIdx );
			dc.DrawRectangle( x, y, m_CellSize, m_CellSize );
		}
	}

	dc.EndDrawing();
}


void HysteresisCanvas::OnEraseBackground(wxEraseEvent& WXUNUSED(event))
{
    // do nothing. I just don't want the background to be erased, you know.
}


