from wxPython.wx import *
from wxPython.lib import vtk

class Cubes:
	def __init__(self, frame, filename):
		self.voi = None
		self.scaleFactor = 0.7
		self.threshold = None
		self.outline = None
		self.obs = None

		self.ren = vtk.vtkRenderer()
		self.win = vtk.wxVTKRenderWindow(frame,-1)
		self.win.GetRenderWindow().AddRenderer(self.ren)
		self.win.GetPicker().SetEndPickMethod( self.GetLocation )
		
		self._ReadFromFile( filename )
		self.CreateDefaults()
		self._SetThreshold([1,100000])
		self.HookTogether()
		self.ren.AddActor(self.spinsActor)
		self._CreateBoundingBox()

	def SetPickObserver(self,obs):
		self.obs = obs

	def GetLocation(self):
		picker = self.win.GetPicker()
		print "positions: ",picker.GetPickedPositions()
		print "PCCoords: ", picker.GetPCoords()
		print "Cell ID: ", picker.GetCellId()
		print "Sub ID: ", picker.GetSubId()
		pos = picker.GetPickPosition()
		print "Pick position: ", pos
		structpoints = self.reader.GetOutput()
		pt = structpoints.FindPoint( pos[0],pos[1],pos[2] )
		data = structpoints.GetPointData()
		if ( pt > 0 ):
			aval = data.GetScalars().GetScalar( pt )
			print "avalanche number: ", int(aval)
			if (self.obs):
				self.obs.Pick( int(aval) )

	def ReadFromFile(self,filename):
		self._ReadFromFile( filename )
		self.HookTogether()

	def _ReadFromFile(self,filename):
		reader = vtk.vtkStructuredPointsReader()
		print "filename: ", filename
		reader.SetFileName( filename )
		reader.OpenVTKFile()
		reader.Update()
		self.ptCnt = reader.GetOutput().GetNumberOfPoints()
		print "Number of points: ", self.ptCnt
		minmax = reader.GetOutput().GetPointData().GetScalars().GetRange()
		self.dimensions = reader.GetOutput().GetDimensions()
		if (self.outline):
			d = self.dimensions
			self.outline.SetBounds( [-0.5,d[0]-0.5,-0.5,d[1]-0.5,-0.5,d[2]-0.5] )
		self.maxScalar = minmax[1]
		self.reader = reader

	def GetNumberOfAvalanches(self):
		return minmax[1]

	def GetDimensions(self):
		return self.dimensions
		
	def SetVOI(self,extents):
		if ( extents ):
			self.voiLimits = extents
			self.voi = vtk.vtkExtractVOI()
			self.voi.SetVOI( extents )
		else:
			self.voi = None
		self.HookTogether()

	def SetSlice(self, ijk, slice):
		d = self.dimensions
		extents = [0,d[0]+1,0,d[1]+1,0,d[2]+1]
		extents[2*ijk] = slice
		extents[2*ijk+1] = slice
		self.SetVOI( extents )

	def SetScaleFactor(self,scale):
		self.scaleFactor = scale
		self.spins.SetScaleFactor( self.scaleFactor )

	def _SetThreshold(self,minmax):
		if (minmax):
			self.thresholdLimits = minmax
			threshold = vtk.vtkThresholdPoints()
			threshold.ThresholdBetween( minmax[0],minmax[1] )
			self.threshold = threshold
			if (minmax[1]>self.maxScalar): max = self.maxScalar
			else: max = minmax[1]
			self.spinsMapper.SetScalarRange( minmax[0], max )
		else:
			self.threshold = None
			self.spinsMapper.SetScalarRange( 1, self.maxScalar )

	def SetThreshold(self,minmax):
		self._SetThreshold(minmax)
		self.HookTogether()

	def _CreateBoundingBox(self):
		self.outline = vtk.vtkOutlineSource()
		d = self.dimensions
		self.outline.SetBounds( [-0.5,d[0]-0.5,-0.5,d[1]-0.5,-0.5,d[2]-0.5] )
		outlineMapper = vtk.vtkPolyDataMapper()
		outlineMapper.SetInput(self.outline.GetOutput())
		self.outlineActor = vtk.vtkActor()
		self.outlineActor.SetMapper( outlineMapper )
		self.ren.AddActor( self.outlineActor )

	def CreateBoundingBox(self):
		if (not self.outline):
			self._CreateBoundingBox()

	def DestroyBoundingBox(self):
		if (self.outline):
			self.ren.RemoveActor( self.outlineActor )
			self.outlineActor = None
		self.outline = None

	def CreateDefaults(self):
		cubeSource = vtk.vtkCubeSource()
		self.spins = vtk.vtkGlyph3D()
		self.spins.SetSource( cubeSource.GetOutput() )
		self.spins.SetScaleModeToDataScalingOff()
		self.spins.SetScaleFactor( self.scaleFactor )
		lut = vtk.vtkLookupTable()
		lut.SetNumberOfColors(256)
		lut.SetHueRange( 0, 0.667 )
		lut.Build()
		spinsMapper = vtk.vtkPolyDataMapper()
		spinsMapper.SetLookupTable( lut )
		self.spinsMapper = spinsMapper
		print "max scalar", self.maxScalar
		self.spinsMapper.SetScalarRange( 1, self.maxScalar )
		self.spinsActor = vtk.vtkActor()

	def HookTogether(self):
		output = self.reader.GetOutput()
		if (self.voi):
			self.voi.SetInput( output )
			output = self.voi.GetOutput()
		if (self.threshold):
			self.threshold.SetInput( output )
			output = self.threshold.GetOutput()
		self.spins.SetInput( output )
		self.spinsMapper.SetInput(self.spins.GetOutput())
		self.spinsActor.SetMapper( self.spinsMapper )

	def Render(self):
		self.win.Render()

class Cone:
	def __init__(self, frame):
		self.cone = vtk.vtkConeSource()
		self.cone.SetResolution(15)
		self.coneMapper = vtk.vtkPolyDataMapper()
		self.coneMapper.SetInput(self.cone.GetOutput())
		self.coneActor = vtk.vtkActor()
		self.coneActor.SetMapper(self.coneMapper)
		self.ren = vtk.vtkRenderer()
		self.win = vtk.wxVTKRenderWindow(frame,-1)
		self.win.GetRenderWindow().AddRenderer(self.ren)
		self.ren.AddActor(self.coneActor)

	def SetResolution(self,res):
		self.cone.SetResolution(int(res))
		self.win.Render()

class MainEntryPanel(wxPanel):
	def __init__(self,parent):
		wxPanel.__init__(self,parent,-1)
		self.resolutionCtrl = wxTextCtrl(self,27,"15",
			wxDLG_PNT(self,wxPoint(4,20)),wxDefaultSize)
		EVT_TEXT_ENTER(self, 27, self.OnText)

		self.render = wxButton(self,10,"render",
			wxDLG_PNT(self,wxPoint(70,70)),wxDefaultSize)
		EVT_BUTTON(self,10,self.OnRenderButton)

		self.radiobox = wxRadioBox(self,20,"Slice direction",
			wxDLG_PNT(self,wxPoint(4,40)),wxDefaultSize,
			['i','j','k'],3,wxRA_SPECIFY_COLS)

		self.sliceCB = wxCheckBox(self,40,"Choose slice",
			wxDLG_PNT(self,wxPoint(4,10)),wxDefaultSize)
			
		#EVT_RADIOBOX(self, 20, self.OnRadio)
		self.avalCB = wxCheckBox(self,40,"Choose avalanches",
			wxDLG_PNT(self,wxPoint(80,10)),wxDefaultSize)
		self.avalStartCtrl = wxTextCtrl(self,27,"1",
			wxDLG_PNT(self,wxPoint(80,20)),wxDefaultSize)
		self.avalEndCtrl = wxTextCtrl(self,27,"100",
			wxDLG_PNT(self,wxPoint(80,30)),wxDefaultSize)

		self.outlineCB = wxCheckBox(self,50,"Outline",
			wxDLG_PNT(self,wxPoint(4,70)),wxDefaultSize)
		self.outlineCB.SetValue(1)

	def Pick(self, avalanche):
		self.avalStartCtrl.SetValue( str(avalanche) )
		self.avalEndCtrl.SetValue( str(avalanche) )

	def OnText(self, event):
		res = self.resolutionCtrl.GetValue()
		#self.cone.SetResolution(res)

	def OnRenderButton(self,event):
		if ( self.sliceCB.GetValue() ):
			res = self.resolutionCtrl.GetValue()
			self.sliceDirection = self.radiobox.GetSelection()
			self.cubes.SetSlice( self.sliceDirection, int(res))
		else:
			self.cubes.SetVOI( None )
			
		if ( self.avalCB.GetValue() ):
			start = self.avalStartCtrl.GetValue()
			end = self.avalEndCtrl.GetValue()
			self.cubes.SetThreshold( [int(start), int(end)] )
		else:
			self.cubes.SetThreshold( [1, 10000] )

		if ( self.outlineCB.GetValue() ):
			print 'creating bounding box'
			self.cubes.CreateBoundingBox()
		else:
			print 'destroying bounding box'
			self.cubes.DestroyBoundingBox()
			
		self.cubes.Render()

	def SetCubes(self,cubes):
		self.cubes = cubes
		self.cubes.SetPickObserver(self)

class MainColoringFrame(wxFrame):
	def __init__(self, parent, id, title):
		wxFrame.__init__(self, parent, id, title,
	                         wxPoint(100,100),wxSize(320,200))

		# Associate some events with methods of this class
		EVT_SIZE(self, self.OnSize)

		self.panel = MainEntryPanel(self)
		#wxStaticText(panel, -1, "Size:",
		#	wxDLG_PNT(panel, wxPoint(4,4)),wxDefaultSize)

	def OnCloseWindow(self, event):
		self.Destroy()

	def OnSize(self, event):
		size = event.GetSize()
		event.Skip()

	def SetCubes(self,cubes):
		self.panel.SetCubes(cubes)


class ColoringApp(wxApp):

	def OnInit(self):
		frame = MainColoringFrame(NULL,-1,"Avalanche Coloring")
		frame.Show(true)

		frame2 = wxFrame(frame, -1, "VTK View",size=(450,300),
			style=wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE)
		cubes = Cubes(frame2, "ColoringL12_R3.S2")
		frame2.Show(true)
		frame.otherWin = frame2

		frame.SetCubes( cubes )
		
		self.SetTopWindow(frame)
		return true

app = ColoringApp(0)
app.MainLoop()

