#include "DMFirstHeader.h"
#include <iostream>
#include "DigitalMaterial.h"
#include "MDMain.h"

#include "DMLastHeader.h"


using namespace std;

void main(int argc, char **argv)
{
try
	{
#ifdef WIN32
	WinProcessFactory::Create();
#else
	UnixProcessFactory::Create();
#endif

#if DMPARALLEL == 1
	MPI_Init(&argc, &argv);

	//MPIProtocol protocol;
	//dmProtocol = &protocol;

	ProtocolFactory::Instance()->SetProtocol(ProtocolFactory::World);
#endif	
	int nProcs = dmProtocol->GetNumberOfProcessors();
	int myProc = dmProtocol->GetProcessorNumber();

	cout.precision(12);
	//DynamicListOfAtoms *dyn = new DynamicListOfAtoms(10);
	//cout << "Dyn: " << dyn->GetNumber() << endl;
	
	// get a filename

	// Create a CheckPointRestarter using the filename.

	CheckPointRestarter restarter("siliconCube.00006");

	// Call recreate
	restarter.Recreate();

	DynamicListOfAtoms *atoms = dynamic_cast<DynamicListOfAtoms*> (restarter.GetListOfAtoms());
	StillingerWeberPotential *potential = dynamic_cast<StillingerWeberPotential*> (restarter.GetPotential());
	LangevinAtomsMover *mover = dynamic_cast<LangevinAtomsMover*> (restarter.GetMover());
	CellNeighborList *neighborLocator = dynamic_cast<CellNeighborList*> (restarter.GetNeighborLocator());
	SimplePeriodicBoundaryConditions *pbc = dynamic_cast<SimplePeriodicBoundaryConditions*> (restarter.GetBoundaryConditions());
	
	int nMajorSteps = 3;
	for ( int step = 0; step < nMajorSteps; ++step )
	{
		mover->Move(atoms);
		double potentialEnergy = potential->CalculateEnergy(atoms);
		double kineticEnergy = atoms->GetKineticEnergy();
		if( 0 == myProc )
		{
			cout << "Potential Energy = " << potentialEnergy << endl;
			cout << "Kinetic Energy = " << kineticEnergy << endl;
			cout << "Total Energy = " << kineticEnergy + potentialEnergy << endl;
		}

	}

	//  attach a Plot observer

	// Run and plot energy



	delete atoms;
	delete pbc;
	delete neighborLocator;
	delete potential;

	}
	catch (DMException e){
		std::cout << e;}

#if DMPARALLEL == 1
	MPI_Finalize();
#endif
}
