// MDMain.cpp : main loop for Unix, Windows Console applications

#include <iostream>
#include <math.h>
#include "MDMain.h"
#include "LatticeStatics.h"

#ifdef _DEBUG
// XXX #define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Main loop: sets up and controls the simulation
// The software is designed so that this can be done in an interpreted, 
// scripting language

void main(int argv, char **argc)
{
	// 6.82 Bohr for each unit cell in copper
  //	FCCLattice lattice(6.82);
        CutLennardJonesPotential potential;
        TriangularLattice lattice(potential.GetLengthScale());

	double rot[DIMENSION][DIMENSION] = {{0,1},{-1,0}};
	lattice.RotateLatticeVectors(rot);

	GeneralLattice stretchedLattice;
	double stretchedVectors[DIMENSION][DIMENSION];

	for(int vec=0; vec<DIMENSION; vec++) stretchedVectors[vec][0] = lattice.GetLatticeVector(vec,0);
	for(int vec=0; vec<DIMENSION; vec++) stretchedVectors[vec][1] = lattice.GetLatticeVector(vec,1)*1.05;
	stretchedLattice.SetLatticeVectors(stretchedVectors);

	double shift[DIMENSION] = {0.1,0.2};
	stretchedLattice.SetOrigin(shift);

	DynamicListOfAtoms atoms0, atoms1, atoms2;
	ListOfTypedAtoms atoms;
	
	double rectSize2[DIMENSION], rectSize1[DIMENSION] = {80,80};
	double rectCenter2[DIMENSION], rectCenter1[DIMENSION] = {20,30};
	RectangularClusterInitializer rect(rectSize1,&stretchedLattice);
	rect.SetCenter(rectCenter1);
	rect.Create(&atoms0);

	
	rectSize2[0] = rectSize1[0] ; rectSize2[1] = 5;
	rect.SetRectangleSize(rectSize2);
	rectCenter2[0] = rectCenter1[0]; rectCenter2[1] = rectCenter1[1] + (rectSize2[1] + rectSize1[1])/2.;
	rect.SetCenter(rectCenter2);
	rect.Create(&atoms1);

	rectCenter2[0] = rectCenter1[0]; rectCenter2[1] = rectCenter1[1] - (rectSize2[1] + rectSize1[1])/2.;
	rect.SetCenter(rectCenter2);
	rect.Create(&atoms2);

	double tipLocation[DIMENSION] = {rectCenter1[0] - rectSize1[0]/2. + 15,rectCenter1[1]};
	NotchMaker notch(PI/12., tipLocation, 0);
	notch.Transform(&atoms0);

	VerletAtomsMover verlet(&potential, 0.01, 100);

	AtomType universalAtom;

	double velocity[DIMENSION] = {0,0.1};
	UniformlyMovingBody umb1(atoms1.GetCartesianPositions(), atoms1.GetNumber(), velocity, &verlet);
	atoms1.AddConstraint(&umb1);

	velocity[1] = -velocity[1];
	UniformlyMovingBody umb2(atoms2.GetCartesianPositions(), atoms2.GetNumber(), velocity, &verlet);
	atoms2.AddConstraint(&umb2);

	atoms.AddNewType(&atoms0, &universalAtom);
	atoms.AddNewType(&atoms1, &universalAtom);
	atoms.AddNewType(&atoms2, &universalAtom);

	NeighborLocator *neighborLocator = new CellNeighborLocator(potential.GetCutoffDistance(),1.2*potential.GetCutoffDistance());

	atoms.SetNeighborLocator(neighborLocator);

	// Thermalize Velocities?

	// Attach observer

	//	PlotAtomsObserver plot(60.0, 60.0, 3);
	//	atoms.Attach(&plot);

	double t=0;
	atoms.Notify();

	for (int i=0; i<10; i++)
	//for (int i = 0; i < 100; i++)
	{
		verlet.Move(&atoms);
		atoms.Notify();
		cout << potential.CalculateEnergy(&atoms) + atoms.GetKineticEnergy()<< endl;
	}
}
