%{
#include "AtomicSurfaceInitializer.h"
#include "ClusterInitializer.h"
#include "PolyClusterInitializer.h"
#include "RectangularClusterInitializer.h"
#include "SphericalClusterInitializer.h"
#include "CompositeClusterInitializer.h"
#include "CylindricalClusterInitializer.h"
#include "ParallelepipedClusterInitializer.h"
%}

%nodefault AtomsInitializer;
class AtomsInitializer 
{
public:
	void Create(ListOfAtoms *atoms);
};

%nodefault ClusterInitializer;
class ClusterInitializer:public AtomsInitializer
{
public:
	void SetCenter(double clusterCenter[DIMENSION]);
	void SetBravaisLattice(BravaisLatticeWithBasis *bravais);
};

%include "AtomicSurfaceInitializer.h"
%include "RectangularClusterInitializer.h"
%include "SphericalClusterInitializer.h"

%include "CylindricalClusterInitializer.h"


class PolyClusterInitializer: public ClusterInitializer
{
public:
%extend
{
	PolyClusterInitializer(PyObject *pyCorners, BravaisLatticeWithBasis *lattice)
	{ // pyCorners are numPy arrays containing the corners.
		if(PyArray_Check(pyCorners))
		{
			PyArrayObject *o = (PyArrayObject *)pyCorners;
			int nCorners = o->dimensions[0];
			if (o->dimensions[1] != DIMENSION) std::cerr << " != DIMENSION !!!" << std::endl;
			if(o->descr->type_num == PyArray_DOUBLE)
			{
				double (*cornersData)[DIMENSION] = (double (*)[DIMENSION]) (o->data);
				ArrayDoubleVector corners(nCorners);
				for(int i=0; i<nCorners; ++i)
					for(int coord=0; coord<DIMENSION; ++coord)
						corners[i][coord] = cornersData[i][coord];
				PolyClusterInitializer *init = new PolyClusterInitializer(&corners,lattice);
				return init;
			}
			else
			{
				PyErr_SetString(PyExc_TypeError,"need a double NumPy Array");
				return NULL;
			}
		}
		else
		{
			PyErr_SetString(PyExc_TypeError,"not a NumPy Array");
			return NULL;
		}
	}
}
};

%include "CompositeClusterInitializer.h"
%include "ParallelepipedClusterInitializer.h"
