// RectangularClusterInitializer.cpp: implementation of the RectangularClusterInitializer class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"
#include "RectangularClusterInitializer.h"

#include "BravaisLatticeWithBasis.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

RectangularClusterInitializer::RectangularClusterInitializer(double rectangleSize[DIMENSION], BravaisLatticeWithBasis *bravais) : ClusterInitializer(bravais)
{
	SetRectangleSize(rectangleSize);
}


RectangularClusterInitializer::~RectangularClusterInitializer()
{
	// cout << "RectangularClusterInitializer deleted" << endl;
}

void RectangularClusterInitializer::SetRectangleSize(const double rectangleSize[DIMENSION])
{
	int coord;
    for(coord=0; coord<DIMENSION;coord++) this->rectangleSize[coord] = rectangleSize[coord];
	// XXX JPS added
	SetMaxSize();
}

int RectangularClusterInitializer::SetMaxSize()
{
 	int i;
	double volumeOfRectangle=1.;
	double corner[DIMENSION], ind[DIMENSION], maxi[DIMENSION], mini[DIMENSION];
	for(int index=0; index<pow(2,DIMENSION); index++)
	{
		int itmp = index;
		for(i=0; i<DIMENSION; i++) 
		{
			corner[i] = clusterCenter[i] + rectangleSize[i]/2.*(double) (itmp % 2 == 0 ? -1:1);
			itmp /= 2;
		}
		bravais->LatticeCoordinatesFromCartesianCoordinates(ind,corner);
		for(i=0; i<DIMENSION; i++)
		{

			if((index == 0) || (maxi[i] < ind[i])) maxi[i] = ind[i];
			if((index == 0) || (mini[i] > ind[i])) mini[i] = ind[i];
		}
	}
	for(i=0; i<DIMENSION; i++) 
	{
		volumeOfRectangle *= rectangleSize[i] + 2*bravais->GetLatticeSpacing(); // extend the rectangle 
		maxSize[i] = (int)maxi[i] + 1;
		minSize[i] = (int)mini[i] - 1;
	}
	return int(volumeOfRectangle/bravais->GetVolumeOfCell()*bravais->GetNAtomsPerCell());
}


bool RectangularClusterInitializer::Inside(double X[DIMENSION])
{
	/* Doesn't check if near boundary!
        bool in=true;
        for(int coord=0; coord<DIMENSION;coord++) in = in && (fabs(X[coord]) <= rectangleSize[coord]/2.);
        return in;
	*/
	// Return outside if near negative boundaries, inside if near positive boundaries

	bool in = true;
	for (int coord=0; coord<DIMENSION; coord++)
	{
		double smallFraction = 1e-14;

		if ((rectangleSize[coord]!=0) && 
			(fabs((fabs(X[coord]-clusterCenter[coord])- 0.5 * rectangleSize[coord])/rectangleSize[coord])
				< smallFraction))
		{
			// Near boundary
			if (X[coord] - clusterCenter[coord] < 0) in = false;
		}
		else
		{
			in = in && (fabs(X[coord]-clusterCenter[coord]) <= 0.5 * rectangleSize[coord]);
		}
	}
	return in;
}
