// SphericalClusterInitializer.cpp: implementation of the SphericalClusterInitializer class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"
#include "SphericalClusterInitializer.h"

#include "BravaisLatticeWithBasis.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SphericalClusterInitializer::SphericalClusterInitializer(double radius, BravaisLatticeWithBasis *bravais) : ClusterInitializer(bravais)
{	
	SetRadius(radius);
}

SphericalClusterInitializer::~SphericalClusterInitializer()
{

}

void SphericalClusterInitializer::SetRadius(double radius)
{
	this->radius = radius;
	// XXX JPS Added
	SetMaxSize();
}

int SphericalClusterInitializer::SetMaxSize()
{
	// Calculate maxsize[DIMENSION], the maximal value of each coordinates of the points inside a 
	// sphere of given radius. After some calculation one arrives at the simple following result.
	    int coord;
        double volumeOfSphere = 1.;
		double extRadius;  // extRadius is an extended radius, used to estimate how many cells are enclosed in the sphere
		extRadius = radius + DIMENSION*pow(bravais->GetVolumeOfCell(),1/(double)DIMENSION);
        double latticevect[DIMENSION], cartvect[DIMENSION], temp[DIMENSION];
        for(coord=0;coord<DIMENSION;coord++) temp[coord] = 0.;
	    for(int k=0; k<DIMENSION; k++) 
		{
	      for(coord=0;coord<DIMENSION;coord++) cartvect[coord] = (coord==k ? 1. : 0.);
	      bravais->LatticeCoordinatesFromCartesianCoordinates(latticevect,cartvect);
	      for(coord=0;coord<DIMENSION;coord++) temp[coord] += latticevect[coord]*latticevect[coord];
		}
	    for(coord=DIMENSION; coord>=2;coord-=2) volumeOfSphere *= extRadius*extRadius*2.*PI/(double)coord ;
	    if(coord==1) volumeOfSphere *= 2.*extRadius;
		for(coord=0;coord<DIMENSION;coord++)
		{
			maxSize[coord] = int(sqrt(temp[coord])*radius) + 1;
			minSize[coord] = -maxSize[coord];
		}
		double shift[DIMENSION];
		bravais->LatticeCoordinatesFromCartesianCoordinates(shift, clusterCenter);
		for(coord=0; coord<DIMENSION; coord++) 
		{
			maxSize[coord] += (int) shift[coord];
			minSize[coord] += (int) shift[coord];
		}
		return int(volumeOfSphere*bravais->GetNAtomsPerCell()/bravais->GetVolumeOfCell());
}

bool SphericalClusterInitializer::Inside(double X[DIMENSION])
{
	double r2=0, tmp;
	for (int coord=0; coord<DIMENSION; coord++)
	{
	      tmp = X[coord] - clusterCenter[coord];
	      r2 += tmp*tmp;
	}
	return (r2 < radius*radius);
}
