// AtomsMover.h: interface for the AtomsMover class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ATOMSMOVER_H
#define ATOMSMOVER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "Serializable.h"
#include "AttributeError.h"
#include "Potential.h"

// Strategy design pattern: many different Movers act interchangibly


class ListOfAtoms;

#include "DMArray.h"

class AtomsMover : public Serializable
{
public:
	AtomsMover() {}
	virtual ~AtomsMover() {}
	virtual void Move(ListOfAtoms *atoms)=0;
	void SetPotential(Potential *potential) 
	{
		this->potential = potential;
	}
	// Time step only works for some AtomsMovers
	virtual double GetDt() const {throw AttributeError("AtomsMover","GetDt");}
	virtual void SetDt(double dt) {throw AttributeError("AtomsMover","SetDt");}

	virtual double GetTime() const {throw AttributeError("AtomsMover","GetTime");}
	virtual void SetTime(double time=0.) {
		throw AttributeError("Potential","SetTime");
	}


	void Save(DMWriter *writer) const {}
	void Load(DMReader *reader) {}	

protected:
	Potential *potential;
	static const std::string className;
};


#endif // !defined(ATOMSMOVER_H)
