// PredictorCorrectorAtomsMover.h: interface for the PredictorCorrectorAtomsMover class.
//
//////////////////////////////////////////////////////////////////////

#ifndef PREDICTORCORRECTORATOMSMOVER_H
#define PREDICTORCORRECTORATOMSMOVER_H

#include "DigitalMaterial.h"
#include "VerletAtomsMover.h"
#include "Potential.h"

#define N_VALUES 4 // for 4-value predictor-corrector (could also be 5)

//! PredictorCorrectorAtoms mover integrates the equations of motion using the predictor-corrector algorithm

/** \ingroup moleculardynamics
 *
 * For second order ODEs where rhs may or may not have first derivatives (for
 * 4-value) or for secind order ODEs where rhs must not have first derivatives
 * (for 5- or 6-value). Note that it is not hard to change the coefficients to
 * allow the 5-value and 5-value ones to work with first derivatives on the
 * RHS. See Allen and Tildesley Appendix E ("Computer Simulation of Liquids",
 *  Oxford 1986).
 */


class PredictorCorrectorAtomsMover : public VerletAtomsMover  
{
public:
	PredictorCorrectorAtomsMover(Potential *potential = 0, double dt = 0.01, int nSteps = 1);
	virtual ~PredictorCorrectorAtomsMover();
	void Move(ListOfAtoms *atoms);

	void CalculateCoefficients();

#ifndef SWIG
	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	const std::string &GetType() const
		{
			return className;
		}

protected:
	void Allocate(const DMArray<double, DIMENSION> *array);
	double s[N_VALUES];
	double coeff[N_VALUES];
	DMArray<double, DIMENSION> *secondDerivs;
	DMArray<double, DIMENSION> *thirdDerivs;
	DMArray<double, DIMENSION> *fourthDerivs;
	DMArray<double, DIMENSION> *fifthDerivs;
	DMArray<double, DIMENSION> *corrections;

	static const std::string className;
#endif // SWIG
};

#endif // !defined(PREDICTORCORRECTORATOMSMOVER_H)
