// QuickMinAllCoordinates.cpp: implementation of the QuickMinAllCoordinates class.
//
//////////////////////////////////////////////////////////////////////

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"

#include "QuickMinAllCoordinates.h"
#include "ListOfAtoms.h"
#include "DMArray.h"
#include "SerialBuilder.h"
#include "Constraint.h"

#include "DMLastHeader.h"


using namespace std;


static SerialBuilder<QuickMinAllCoordinates> *_QMPcreator = 
    new SerialBuilder<QuickMinAllCoordinates>("QuickMinAllCoordinates");

const string QuickMinAllCoordinates::className = string("QuickMinAllCoordinates");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


void QuickMinAllCoordinates::AdjustVelocities(const DMArray<double, DIMENSION> *forces, const DMArray<double, DIMENSION> *velocities)
{
	int coord;
	double vDotF, fSq, fSqInv;
	double *veloCurr, *forcesCurr, *tVeloCurr;
	// This works atom by atom.
	vDotF = 0;
	fSq = 0;

	DMArrayIterator<double, DIMENSION> forcesIter(forces), veloIter(velocities), tVeloIter(adjustedVelocities);
	for(forcesIter.Init(), veloIter.Init(); !forcesIter.IsDone(); 
		forcesIter.Next(), veloIter.Next())
	{
		veloCurr = veloIter.Current();
		forcesCurr = forcesIter.Current();	

		for( coord = 0; coord < DIMENSION; ++coord )
		{
			vDotF += veloCurr[coord]* forcesCurr[coord];
			fSq += forcesCurr[coord] * forcesCurr[coord];
		}
	}
	
	if(vDotF < 0)
	{
		adjustedVelocities->Zero();
	}
	else if(fSq > 1.0e-10)
	{
		fSqInv = 1./fSq;
		for(forcesIter.Init(), tVeloIter.Init(); !forcesIter.IsDone(); 
			forcesIter.Next(), tVeloIter.Next())
		{
			tVeloCurr = tVeloIter.Current();
			forcesCurr = forcesIter.Current();
			for(coord=0;coord<DIMENSION;coord++)
				tVeloCurr[coord] = forcesCurr[coord]*vDotF*fSqInv;		
		}
	}
}
