// QuickMinProjection.cpp: implementation of the QuickMinProjection class.
//
//////////////////////////////////////////////////////////////////////

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"

#include "QuickMinProjection.h"
#include "ListOfAtoms.h"
#include "DMArray.h"
#include "SerialBuilder.h"
#include "Constraint.h"

#include "DMLastHeader.h"


using namespace std;


static SerialBuilder<QuickMinProjection> *_QMPcreator = 
    new SerialBuilder<QuickMinProjection>("QuickMinProjection");

const string QuickMinProjection::className = string("QuickMinProjection");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


void QuickMinProjection::AdjustVelocities(const DMArray<double, DIMENSION> *forces, const DMArray<double, DIMENSION> *velocities)
{
	// This works atom by atom.
	int coord;
	double vDotF, fSq, fSqInv;
	double *veloCurr, *forcesCurr, *tVeloCurr;

	DMArrayIterator<double, DIMENSION> forcesIter(forces), veloIter(velocities), tVeloIter(adjustedVelocities);
	for(forcesIter.Init(), veloIter.Init(), tVeloIter.Init(); !forcesIter.IsDone(); 
		forcesIter.Next(), veloIter.Next(), tVeloIter.Next())
	{
		vDotF = 0;
		fSq = 0;
		veloCurr = veloIter.Current();
		tVeloCurr = tVeloIter.Current();
		forcesCurr = forcesIter.Current();
		for (coord=0; coord<DIMENSION; coord++)
		{
			vDotF += veloCurr[coord]* forcesCurr[coord];
			fSq += forcesCurr[coord] * forcesCurr[coord];
		}
		if(vDotF < 0)
			for(coord=0;coord<DIMENSION;coord++) tVeloCurr[coord] = 0.; 
		else if(fSq > 1.0e-10)
		{
			fSqInv = 1.0/fSq;
			for(coord=0;coord<DIMENSION;coord++)
			    tVeloCurr[coord] = forcesCurr[coord] * vDotF * fSqInv;		
		}
	}
	
}
