// QuickMinProjection.h: interface for the QuickMinProjection class.
//
//////////////////////////////////////////////////////////////////////

#ifndef QUICKMINPROJECTION_H
#define QUICKMINPROJECTION_H

#include "DigitalMaterial.h"
#include "QuickMinAtomsMover.h"
#include "Potential.h"

//! Variation of QuickMin with force-projection when dot-product is positive


/*!
  This version of QuickMin operates like QuickMinAtomsMover, from which it is 
  derived, in that it goes atom-by-atom, but as well as zeroing velocities
  which have negative dot-product with forces, for those atoms with positive
  dot-products, it projects velocities along the force.


*/

class QuickMinProjection : public QuickMinAtomsMover 
{
public:
	QuickMinProjection(Potential *potential = 0,double dt = 1.,double tolerance = 1.e-6) : QuickMinAtomsMover(potential, dt, tolerance) {}
protected:
	void AdjustVelocities(const DMArray<double, DIMENSION> *forces, const DMArray<double, DIMENSION> *velocities);

private:
	static const std::string className;
};

#endif //  QUICKMINPROJECTION_H
