// DTUPeriodicBoundaryConditions.cpp: implementation of the DTUPeriodicBoundaryConditions class.
//
//////////////////////////////////////////////////////////////////////



#ifdef _WINDOWS
#include "stdafx.h"
#include "MD.h"
#endif _WINDOWS


#include "DTUPeriodicBoundaryConditions.h"
#include "blas.h"

#ifdef _WINDOWS
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
#endif _WINDOWS


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

DTUPeriodicBoundaryConditions::DTUPeriodicBoundaryConditions()
{
	for(int coord=0;coord<DIMENSION;coord++)
	{
		length[coord] = invLength[coord] = 1.0; // default; change using SetLength
	}
	assureposi = ASSUREPOSI;
	assureposihalf = ASSUREPOSI + 0.5;
}

DTUPeriodicBoundaryConditions::~DTUPeriodicBoundaryConditions()
{

}

BoundaryConditions *DTUPeriodicBoundaryConditions::Copy() const
{
  PeriodicBoundaryConditions *newPBC = new DTUPeriodicBoundaryConditions();
  newPBC->SetLength(length);
  return newPBC;
}

void DTUPeriodicBoundaryConditions::EnforceBoundaryConditions(DMArray *positions) const
{
  int coord,subArray,nSubArrays=positions->GetShape().size(),nAtomsInSubArray;
  double (*subArrayPtr)[DIMENSION];
  
  // convert to scaled coordinates:
  for(subArray=0;subArray<nSubArrays;++subArray)
    {
      nAtomsInSubArray = positions->GetShape()[subArray];
      subArrayPtr = positions->GetPtrsList()[subArray]; 
      if(nAtomsInSubArray > 0)
	{
	  for(coord=0;coord<DIMENSION;coord++)
	    cblas_dscal(nAtomsInSubArray,invLength[coord],((double*)subArrayPtr)+coord,DIMENSION);
	}
    }

 DMArrayIterator posIter(positions);
 for(posIter.Init(); !posIter.IsDone(); posIter.Next())
   {
     double *atomPos = posIter.Current();
     for(coord=0;coord<DIMENSION;coord++)
       atomPos[coord] = atomPos[coord] - (((int) (atomPos[coord] + assureposi)) - assureposi);
   }

 // convert to real-space coordinates
 for(subArray=0;subArray<nSubArrays;++subArray)
   {
     nAtomsInSubArray = positions->GetShape()[subArray];
     subArrayPtr = positions->GetPtrsList()[subArray]; 
     if(nAtomsInSubArray > 0)
       {
	 for(coord=0;coord<DIMENSION;coord++)      
	   cblas_dscal(nAtomsInSubArray,length[coord],((double*)subArrayPtr)+coord,DIMENSION);
       }
   }
}


void DTUPeriodicBoundaryConditions::EnforceBoundaryConditions(double *position) const
{
  for(int coord=0;coord<DIMENSION;coord++)  
    {
      position[coord] *= invLength[coord];
      position[coord] = position[coord] - (((int) (position[coord] + assureposi)) - assureposi);
      position[coord] *= length[coord];
    }
}


void DTUPeriodicBoundaryConditions::DifferenceBoundaryConditions(double (*displacements)[DIMENSION],int nAtoms) const
{

// XXX Needed to prevent weird BLAS bug in a future call to cblas_daxpy in IncrementCartesianVelocities!!!
  if (nAtoms == 0) return;

  int i,coord;
  double magic = 6755399441055744.0;

  // convert to scaled coordinates:
  for(coord=0;coord<DIMENSION;coord++)
    cblas_dscal(nAtoms,invLength[coord],((double*)displacements)+coord,DIMENSION);
  


  for(i=0;i<nAtoms;i++)
    for(coord=0;coord<DIMENSION;coord++)
      //  displacements[i][coord] = displacements[i][coord] - (((int) (displacements[i][coord] + assureposihalf)) - assureposi);
      {
	double t1 = displacements[i][coord] + magic;
	double t2 = t1 - magic;
	displacements[i][coord] -= t2;
      }
	// use assureposihalf for differences

  // convert to real-space coordinates
  for(coord=0;coord<DIMENSION;coord++)      
    cblas_dscal(nAtoms,length[coord],((double*)displacements)+coord,DIMENSION);
  
}
