// PeriodicBoundaryConditions.h: interface for the PeriodicBoundaryConditions class.
//
//////////////////////////////////////////////////////////////////////

#ifndef PERIODICBOUNDARYCONDITIONS_H__
#define PERIODICBOUNDARYCONDITIONS_H__

#include "BoundaryConditions.h"

#include "DMArray.h"
#include "DMWriter.h"
#include "DMReader.h"
#include "DMException.h"

class PeriodicBoundaryConditions : public BoundaryConditions
{
public:
	// overriding BoundaryConditions methods
	PeriodicBoundaryConditions() {}
	virtual ~PeriodicBoundaryConditions(){}
	virtual void EnforceBoundaryConditions(DMArray<double, DIMENSION> *positions) const =0;
	virtual void DifferenceBoundaryConditions(DMArray<double, DIMENSION> *positions) const=0;
	virtual void DifferenceBoundaryConditions(double (*displacements)[DIMENSION],int nAtoms) const=0;
	virtual void EnforceBoundaryConditions(double *position) const =0;
	// Interface special to PeriodicBoundaryConditions
	virtual void SetLength(const double l[DIMENSION]) = 0;
	double GetLength(int coord) const {return length[coord];}
	const double *GetLength() const {return (const double *)length;}

	virtual void Save(DMWriter *writer) const 
	{
		std::vector<int> dim;
		dim.push_back(DIMENSION);
		writer->PutDoubleArray(length, dim, "length");
	}
	virtual void Load(DMReader *reader)
	{
		std::vector<int> dim;
		double *data = reader->GetDoubleArray(dim, "length");
		if((dim.size() != 1) || (dim[0] != DIMENSION)) 
			throw DMException("PeriodicBoundaryConditions", "Load", "Problem while reading length");
		for(int coord=0; coord<DIMENSION; ++coord) length[coord] = data[coord];
		delete data;
	}

protected:
	double length[DIMENSION];
};


#endif // !defined(PERIODICBOUNDARYCONDITIONS_H__)
