// SimplePeriodicBoundaryConditions.h: interface for the SimplePeriodicBoundaryConditions class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SIMPLEPERIODICBOUNDARYCONDITIONS_H
#define SIMPLEPERIODICBOUNDARYCONDITIONS_H

#include "PeriodicBoundaryConditions.h"

#include "DigitalMaterial.h"

#include "DMArray.h"

class SimplePeriodicBoundaryConditions : public PeriodicBoundaryConditions  
{
public:
	// interface overridden from BoundaryConditions
	SimplePeriodicBoundaryConditions();
	virtual ~SimplePeriodicBoundaryConditions();

	void EnforceBoundaryConditions(DMArray<double, DIMENSION> *positions) const ;
#ifndef SWIG
	SimplePeriodicBoundaryConditions(const double l[DIMENSION]);
	void DifferenceBoundaryConditions(double (*displacements)[DIMENSION],int nAtoms) const;
	void EnforceBoundaryConditions(double *position) const;
	void DifferenceBoundaryConditions(DMArray<double, DIMENSION> *displacements) const;

	void Save(DMWriter *writer) const {PeriodicBoundaryConditions::Save(writer);}
	void Load(DMReader *reader) {PeriodicBoundaryConditions::Load(reader);}
	//! Get the string name of the class for saving or loading.
#endif // SWIG
	const std::string &GetType() const 
	{
		return className;	
	}

	
	
	// interface overridden from PeriodicBoundaryConditions
	void SetLength(const double l[DIMENSION])
	{
		// somewhere it should be checked that the length of the box is not too 
		// small compared with the cutoffs used in the simulation (maybe at outer level)
		for (int coord = 0; coord < DIMENSION; coord++)
		{
			length[coord] = l[coord];
		}
	} 
	BoundaryConditions *Copy() const;
private:
	static const std::string className;
};

#endif // !defined(SIMPLEPERIODICBOUNDARYCONDITIONS_H)
