#include "DMFirstHeader.h"

#include "DigitalMaterial.h"

#if DIMENSION == 2

#include "2DLattices.h"
#include "SerialBuilder.h"
#include "DMWriter.h"
#include "DMReader.h"

#include "DMLastHeader.h"


static SerialBuilder<SquareLattice> *_SquareLatticecreator = 
    new SerialBuilder<SquareLattice>("SquareLattice");

static SerialBuilder<TriangularLattice> *_TriangularLatticecreator = 
    new SerialBuilder<TriangularLattice>("TriangularLattice");


const std::string SquareLattice::className = std::string("SquareLattice");

const std::string TriangularLattice::className = std::string("TriangularLattice");

///////////////////  SQUARE LATTICE  /////////////////////////



SquareLattice::SquareLattice(double a)
{
	latticeConstant = a;
    for(int vec=0; vec<DIMENSION; vec++)
    {
        for(int coord=0; coord<DIMENSION;coord++)
            latticeVectors[vec][coord] = (vec==coord? a : 0.);
    }
    UpdateMemberVariables();
}

void SquareLattice::SetLatticeConstant(double a)
{
    double factor = a/latticeConstant;
    for(int vec=0; vec<2; vec++)
    {
        for(int coord=0; coord<DIMENSION; coord++)
            latticeVectors[vec][coord] *= factor;
    }
    latticeConstant = a;
    UpdateMemberVariables();
}


///////////////////  TRIANGULAR LATTICE  /////////////////////////

TriangularLattice::TriangularLattice(double a)
{
	latticeConstant = a;
    latticeVectors[0][0] = a;
    latticeVectors[0][1] = 0.;
    latticeVectors[1][0] = 0.5*a;
    latticeVectors[1][1] = 0.5*sqrt(3.)*a;
    UpdateMemberVariables();
}

void TriangularLattice::SetLatticeConstant(double a)
{
    double factor = a/latticeConstant;
    for(int vec=0; vec<2; vec++)
    {
        for(int coord=0; coord<DIMENSION; coord++)
            latticeVectors[vec][coord] *= factor;
    }
    latticeConstant = a;
    UpdateMemberVariables();
}

#endif // DIMENSION == 2
