// FixedBody.cpp: implementation of the FixedBody class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"

#include "FixedBody.h"
#include "DMArray.h"

#include "DMLastHeader.h"


const std::string FixedBody::className = std::string("FixedBody");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////



FixedBody::FixedBody(const DMArray<double, DIMENSION> *positions)
{
	fixedPositions = 0;
	SetFixedPositions(positions);
	for(int i = 0; i < DIMENSION; i++)
		totalForce[i] = 0.0;
}

FixedBody::~FixedBody()
{
	if(fixedPositions) delete fixedPositions;
}

Constraint *FixedBody::Copy() const
{
	return new FixedBody(fixedPositions);
}

void FixedBody::SetFixedPositions(const DMArray<double, DIMENSION> *positions)
{
	if(fixedPositions) delete fixedPositions;
	fixedPositions = new DMArray<double, DIMENSION> (positions);
	fixedPositions->Copy(positions);
}

void FixedBody::AdjustPositions(DMArray<double, DIMENSION> *positions)
{
	if(!(positions->GetShape() == fixedPositions->GetShape()))
		throw DMException("FixedBody","AdjustPositions","Shape of positions differs from that of fixedPositions");
	ASSERT(positions->GetShape() == fixedPositions->GetShape());
	positions->Copy(fixedPositions);
}

void FixedBody::AdjustForces(DMArray<double, DIMENSION> *forces)
{
	forces->Sum(totalForce);
	forces->Zero();
}

double FixedBody::GetTotalForce(int coord)
{
	return totalForce[coord];
}

int FixedBody::NumberGeneralizedCoordinates()
{
	return 0;
}

void FixedBody::SetGeneralizedCoordinates(double *generalizedCoordinates)
{

}


void FixedBody::GetGeneralizedCoordinates(double *generalizedCoordinates)
{

}


void FixedBody::IncrementGeneralizedCoordinate(int n,double inc)
{
	
}


void FixedBody::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces)
{

}

void FixedBody::DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION])
{

}
