// FixedInPlane.cpp: implementation of the FixedInPlane class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"

#include "FixedInPlane.h"
#include "DMArray.h"

#include "DMLastHeader.h"
#include "ListOfAtoms.h"


const std::string FixedInPlane::className = std::string("FixedInPlane");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////



FixedInPlane::FixedInPlane(const DMArray<double, DIMENSION> *positions, const double normal[DIMENSION])
{
	fixedPositions = 0;
	SetFixedPositions(positions);
	for(int i = 0; i < DIMENSION; i++)
		totalForce[i] = 0.0;
	SetNormal(normal);
}

FixedInPlane::~FixedInPlane()
{
	if(fixedPositions) delete fixedPositions;
}

Constraint *FixedInPlane::Copy() const
{
	return new FixedInPlane(fixedPositions,fixedPlaneNormal);
}

void FixedInPlane::SetFixedPositions(const DMArray<double, DIMENSION> *positions)
{
	if(fixedPositions) delete fixedPositions;
	fixedPositions = new DMArray<double, DIMENSION> (positions);
	fixedPositions->Copy(positions);
}

void FixedInPlane::SetNormal(const double normal[DIMENSION])
{
	for(int coord=0;coord<DIMENSION;coord++)
		fixedPlaneNormal[coord] = normal[coord];
}

void FixedInPlane::AdjustPositions(DMArray<double, DIMENSION> *positions)
{
	double disp, newDisp, dot;


	if(!(positions->GetShape() == fixedPositions->GetShape()))
		throw DMException("FixedInPlane","AdjustPositions","Shape of positions differs from that of fixedPositions");
	ASSERT(positions->GetShape() == fixedPositions->GetShape());
	for(int i=0; i < atoms->GetNumber(); i++){
		dot = 0;
		for(int coord=0;coord<DIMENSION;coord++){
			disp = (*positions)[i][coord] - (*fixedPositions)[i][coord];
			dot += disp*fixedPlaneNormal[coord];
		}
		for(int coord=0;coord<DIMENSION;coord++){
			disp = (*positions)[i][coord] - (*fixedPositions)[i][coord];
			newDisp = disp - dot * fixedPlaneNormal[coord]; 
			(*positions)[i][coord] = (*fixedPositions)[i][coord] + newDisp;
		}
	}	  
}

void FixedInPlane::AdjustForces(DMArray<double, DIMENSION> *forces)
{
	double dot;


	forces->Sum(totalForce);
	for(int i=0; i < atoms->GetNumber(); i++){
		for(int coord=0;coord<DIMENSION;coord++){
			dot += (*forces)[i][coord]*fixedPlaneNormal[coord];
		}
		for(int coord=0;coord<DIMENSION;coord++){
			(*forces)[i][coord] -= dot*fixedPlaneNormal[coord];
		}
	}
}

double FixedInPlane::GetTotalForce(int coord)
{
	return totalForce[coord];
}

int FixedInPlane::NumberGeneralizedCoordinates()
{
	return 0;
}

void FixedInPlane::SetGeneralizedCoordinates(double *generalizedCoordinates)
{

}


void FixedInPlane::GetGeneralizedCoordinates(double *generalizedCoordinates)
{

}


void FixedInPlane::IncrementGeneralizedCoordinate(int n,double inc)
{
	
}


void FixedInPlane::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces)
{

}

void FixedInPlane::DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION])
{

}
