// FixedInPlane.h: interface for the FixedInPlane class.
//
//////////////////////////////////////////////////////////////////////

#ifndef FIXEDINPLANE_H
#define FIXEDINPLANE_H

#include "DigitalMaterial.h"
#include "Constraint.h"
#include "FixedBody.h"

class FixedInPlane : public Constraint
{
public:
	FixedInPlane(const DMArray<double, DIMENSION> *positions, const double normal[DIMENSION]);
        ~FixedInPlane();
	Constraint *Copy() const;

	void SetFixedPositions(const DMArray<double, DIMENSION> *positions);
	void SetNormal(const double normal[DIMENSION]);
	void AdjustPositions(DMArray<double, DIMENSION> *positions);
	void AdjustForces(DMArray<double, DIMENSION> *forces);
	double GetTotalForce(int coord);

	int NumberGeneralizedCoordinates();
	// these all do nothing, since there are no generalized coordinates
	// we don't want them to throw errors because they will be involved
	// in loops over branches, in which case they will be passed over
	// and the arrays wfilled in/used by the next leaf/branch which has
	// actual degrees of freedom
	void SetGeneralizedCoordinates(double *generalizedCoordinates) ;
    void GetGeneralizedCoordinates(double *generalizedCoordinates); 
	void IncrementGeneralizedCoordinate(int n,double inc);
#ifndef SWIG
    void CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces);
    void DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION]);
#endif // SWIG
	const std::string &GetType() const
	{
		return className; 
	}


protected:
	DMArray<double, DIMENSION> *fixedPositions;
	double fixedPlaneNormal[DIMENSION];
	static const std::string className;
	double totalForce[DIMENSION];
};



#endif // !defined(PARTIALFIXEDBODY_H)
