// PartialFixedCOM.h: interface for the PartialFixedCOM class.
//
//////////////////////////////////////////////////////////////////////

#ifndef PARTIALFIXEDCOM_H
#define PARTIALFIXEDCOM_H

#include "DigitalMaterial.h"
#include "Constraint.h"

//! fixed COM: number of DOFS is DIMENSION*(nAtoms-1)

/*!
  
  \author Thierry Cretegny

 */

class PartialFixedCOM: public Constraint
{
public:
	PartialFixedCOM(bool fixedDir[DIMENSION], const DMArray<double, DIMENSION> *initialPositions);
	PartialFixedCOM(bool fixedDir[DIMENSION], const double center[DIMENSION]);
	
	void AdjustPositions(DMArray<double, DIMENSION> *positions);
	void SetCenter(const double center[DIMENSION]);
	const double GetCenter(int coord) const { return center[coord]; }

	// GeneralizedCoordinate functions: most throw errors right now (March 01)
	int NumberGeneralizedCoordinates();
	void SetGeneralizedCoordinates(double *generalizedCoordinates) ;
    void GetGeneralizedCoordinates(double *generalizedCoordinates); 
	void IncrementGeneralizedCoordinate(int n,double inc);
    void CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces);
#ifndef SWIG
    void DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION]);
#endif //SWIG

	const std::string &GetType() const
	{
		return className;
	}

private:
	double center[DIMENSION];
	bool fixedDir[DIMENSION];
	int nFixed;
	static const std::string className;
};

#endif // !defined(PARTIALFIXEDCOM_H)
