/*!

	\class ArrayDoubleVector
	\fn ArrayDoubleDIM* GetLeaf( int n )
	\brief Get the next leaf from the ArrayDoubleDIM in a depth-first search.

	\fn DoubleVector __getitem__( int n )
	\brief Permits access of the form array[13][2] with gets and sets.
	This function relies on a helper class which represents a row of the
	array.

	\fn PyObject* __getslice__(int i, int j)
	\brief Returns a NumPy array when you ask for vector[3:7].
	The array is a vector where the second dimension is DIMENSION.

*/

%{
#include "LeafToLeafIterator.h"
#include "DMArray.h"
#include "NotEnoughMemory.h"

typedef DMArray<double, DIMENSION> ArrayDoubleVector;
typedef DMArray<int, DIMENSION> ArrayIntVector;
typedef DMArrayIterator<double, DIMENSION> ArrayDoubleVectorIterator;
typedef DMArrayIterator<int, DIMENSION> ArrayIntVectorIterator;
typedef DMArray<double> ArrayDouble;
typedef DMArray<int> ArrayInt;

typedef LeafToLeafIterator<ArrayDoubleVector> ArrayDoubleVectorLeafToLeaf;

%}

# This allows the SWIG type-checking mechanism to recognize
# that a _ArrayDoubleVector_p can be a _DMArray<_double,_DIMENSION_>_p.
%typemap(python,in) DMArray<double, 3> *, const DMArray<double, 3> *
{
    if ($input)
    {
        if ($input == Py_None) { $1 = NULL; }
        else    
		{
	// the following is for swig 1.3.16
            if ((SWIG_ConvertPtr($input,(void **) &$1,SWIGTYPE_p_DMArrayTdouble_3_t,1) == -1) &&
                (SWIG_ConvertPtr($input,(void **) &$1,SWIGTYPE_p_ArrayDoubleVector,1) == -1))

	// the following is for swig 1.3.6 
	//if ((SWIG_ConvertPtr($input,(void **) &$1,SWIGTYPE_p_DMArray_double_3_,1) == -1) &&
         //  (SWIG_ConvertPtr($input,(void **) &$1,SWIGTYPE_p_ArrayDoubleVector,1) == -1))
            {
                PyErr_SetString(PyExc_TypeError,"Type error. Expected _DMArray<_double,DIMENSION_>_p or _ArrayDoubleVector_p.");
                return NULL;
            }
			else PyErr_Clear(); // if the 1st SWIG_ConvertPtr has failed, the error string has been set
        }
    }
}

%typemap(python,in) DMArray<double, 2> *, const DMArray<double, 2> *
{
    if ($input)
    {
        if ($input == Py_None) { $1 = NULL; }
        else
        {
		// the following is for swig 1.3.16
             if ((SWIG_ConvertPtr($input,(void **) &$1,SWIGTYPE_p_DMArrayTdouble_2_t,1) == -1) &&
                 (SWIG_ConvertPtr($input,(void **) &$1,SWIGTYPE_p_ArrayDoubleVector,1) == -1)) 
		// the following is for swig 1.3.6		
	  //if ((SWIG_ConvertPtr($input,(void **) &$1,SWIGTYPE_p_DMArray_double_2_,1) == -1) &&
        //	(SWIG_ConvertPtr($input,(void **) &$1,SWIGTYPE_p_ArrayDoubleVector,1) == -1)) 

            {
                PyErr_SetString(PyExc_TypeError,"Type error. Expected _DMArray<_double,DIMENSION_>_p or _ArrayDoubleVector_p.");
                return NULL;
            }
            else PyErr_Clear(); // if the 1st SWIG_ConvertPtr has failed, the error string has been set
        }
    }
}

%typemap(python,out) DMArray_AnyType::siblingContainer* {
	char str[200];
	DMArray_AnyType::siblingContainer::const_iterator it = $1->begin();
	int size = $input->size();
	
	PyObject *strObject;
	int i = 0;
	$result = PyList_New( size );
	while (it != $1->end()) {
		SWIG_MakePtr( str, (char *) *it, "_ArrayDoubleVector_p" );
		strObject = Py_BuildValue("s", str);
		PyList_SetItem( $result, i, strObject );
		++it; ++i;
	}
}

%inline %{
	struct DoubleVector {
		double* data;
		double __getitem__(int i) {
			return data[i];
		};
		void __setitem__(int i, double val) {
			data[i] = val;
		};
	};
%}


class ArrayDoubleVector : public Serializable
{
public:
	ArrayDoubleVector( int n = 0 );
	~ArrayDoubleVector();
	void Copy( const ArrayDoubleVector * );
	void Clear( );
	void AddElementsAtEnd( int n );
	void Set(const double[DIMENSION]);
	void Zero();
	void Increment( const double[DIMENSION] );
	void RemoveBranch( ArrayDoubleVector *toRemove );
	%name(ConstSubArray) const ArrayDoubleVector* GetBranch( int i ) const;
	%name(__len__) int Size() const;
	void Merge( ArrayDoubleVector *array );
	void Attach( ArrayDoubleVector *array );
	const DMArray_AnyType::siblingContainer& GetSiblings();
	const Shape &GetShape();
	  // The Tree interface is inherited from a template class, and must be added manually
	int NumberOfBranches();
	int NumberOfLeaves();
	bool IsLeaf();
	ArrayDoubleVector *GetBranch(int i);
	ArrayDoubleVector *GetLeaf(int i);
	%extend
	{
		// Python doesn't allow two types of constructors
		// so we need to disguise the one to copy the shape of
		// the array it is called on
		ArrayDoubleVector *ConstructFromArray(ArrayDoubleVector *array)
		{ return new ArrayDoubleVector(array);}

		const ArrayDoubleVector *GetExtended() const
		{ return self->GetSynchronizationKit()->GetExtended();}
		ArrayDoubleVector *GetGhosts()
		{ return self->GetSynchronizationKit()->GetGhosts();}

		void Synchronize()
		{ self->GetSynchronizationKit()->Synchronize();}

		PyObject *Average()
		{
			double avg[DIMENSION];
			self->Average(avg);
			int dims = DIMENSION;
			PyObject *o = PyArray_FromDims(1, &dims, 'd');
			char *ptr = ((PyArrayObject *)o)->data;
			memcpy(ptr, avg, DIMENSION*sizeof(double));
			return o;
		}

		int GetNumberOfSiblings(void) const {
			return self->GetSiblings().size();
		}
		ArrayDoubleVector* GetSibling(int i) {
			const DMArray_AnyType::siblingContainer& sibs = self->GetSiblings();
			DMArray_AnyType::siblingContainer::const_iterator it =
				sibs.begin();
			int j=0;
			while ((j<i) && (it!=sibs.end())) {
				++it; ++j;
			}
			if (it==sibs.end()) return 0;
			ArrayDoubleVector *arr = dynamic_cast<ArrayDoubleVector *>(*it);
			if(!arr) throw DMException("ArrayDoubleVector", "GetSibling", "not the right type");
			return arr;
		}

		DoubleVector __getitem__( int n ) {
			DoubleVector r;
			if ((n < 0) || (n >= self->Size()))
			    throw IndexError("ArrayDoubleVector", "__getitem__", "Index out of range.");
			r.data = (*self)[n];
			return r;
		}

		PyObject *__getslice__(int i, int j)
		{
			int dims[2], len;
			int n;
			int length;
			length = self->Size();
			if (i < 0)
			    i = length + 1 - i;
			else if (i > length)
			    i = length;
			if (j < 0)
			    j = length + 1 - j;
			else if (j > length)
			    j = length;
			dims[0] = j-i; dims[1] = DIMENSION;
			PyObject *o = PyArray_FromDims(2, dims, 'd'); //PyArray_FLOAT);
			if (!o) 
			{
			    throw NotEnoughMemory("ArrayDoubleVector", "__getslice__",
						  "result of __getslice__");
			}
			char *ptr = ((PyArrayObject *)o)->data;
			DMArrayIterator<double, DIMENSION> iter(self);
			len = DIMENSION*sizeof(double)/sizeof(char);
			n = 0;
			for(iter.Init(); !iter.IsDone(); iter.Next())
			{
				if ( (n>=i) && (n<j) ) {
					memcpy(ptr, iter.Current(), len);
					ptr += len;
				}
				++n;
			}
			return o;
		}

		PyObject* __setslice__(int i, int j, PyObject* s)
		{
			PyArrayObject* o;
			int len, n;
			char msg[200];
			int length;
			length = self->Size();
			if (i < 0)
			    i = length + 1 - i;
			else if (i > length)
			    i = length;
			if (j < 0)
			    j = length + 1 - j;
			else if (j > length)
			    j = length;
			if (PyArray_Check(s)) {
				o = (PyArrayObject*) s;
				if ( (2==o->nd) && (DIMENSION==o->dimensions[1]) &&
				                (j-i==o->dimensions[0]) &&
						(PyArray_DOUBLE == o->descr->type_num) ) {
					char *ptr = ((PyArrayObject *)o)->data;
					DMArrayIterator<double, DIMENSION> iter(self);
					len = DIMENSION*sizeof(double)/sizeof(char);
					n = 0;
					for (iter.Init(); !iter.IsDone(); iter.Next())
					{
						if ( (n>=i) && (n<j) ) {
							memcpy(iter.Current(), ptr, len);
							ptr += len;
						}
						++n;
					}
				} else {
				    /* Wrong dimensions and type is a ValueError (not logical!) */
					if (2 != o->nd)
						PyErr_SetString(PyExc_ValueError,
								"Must be a 2d array.");
					else if (o->dimensions[o->nd-1] != DIMENSION) {
						sprintf(msg,
								"This array must be of 2nd dimension %d.",
								DIMENSION);
						PyErr_SetString(PyExc_ValueError, msg);
					} else if (j-i!=o->dimensions[0]) {
					        PyErr_SetString(PyExc_ValueError,
								"Cannot change length of array in __setslice__.");
					} else if (PyArray_DOUBLE != o->descr->type_num) {
						PyErr_SetString(PyExc_ValueError,
								"It needs to be an array of doubles.");
					}
					return NULL;
				}
			} else {
				PyErr_SetString(PyExc_TypeError,"accepts only arrays");
				return NULL;
			}
			Py_INCREF(Py_None);
			return Py_None;
		}
	};
};

class ArrayDoubleVectorIterator
{
public:
	ArrayDoubleVectorIterator(const ArrayDoubleVector *dmArray);
	void Init();
	void Next();
	bool IsDone() const;
	int Size() const;
	%extend
	{
		double Current(int coord) const {return self->Current()[coord];}
	}
};

class ArrayDoubleVectorLeafToLeaf
{
public:
	ArrayDoubleVectorLeafToLeaf( ArrayDoubleVector* head );
	void Next( );
	bool Done( );
	ArrayDoubleVector* Current( );
};

class ArrayDouble
{
public:
	ArrayDouble( int n = 0 );
	~ArrayDouble();
	void Copy( const ArrayDouble * );
	void Clear( );
	void AddElementsAtEnd( int n );
	void Set(const double[DIMENSION]);
	void Zero();
	void Increment( const double[DIMENSION] );
	void RemoveBranch( ArrayDouble *toRemove );
	%name(ConstSubArray) const ArrayDouble* GetBranch( int i ) const;
	%name(__len__) int Size() const;
	void Merge( ArrayDouble *array );
	void Attach( ArrayDouble *array );
	const DMArray_AnyType::siblingContainer& GetSiblings();
	const Shape &GetShape();
	  // The Tree interface is inherited from a template class, and must be added manually
	int NumberOfBranches();
	int NumberOfLeaves();
	bool IsLeaf();
	ArrayDouble *GetBranch(int i);
	ArrayDouble *GetLeaf(int i);
	%extend
	{

		int GetNumberOfSiblings(void) const {
			return self->GetSiblings().size();
		}

		double __getitem__( int n ) {
			if ((n < 0) || (n >= self->Size()))
			    throw IndexError("ArrayDouble", "__getitem__", "Index out of range.");
			return (*self)[n];
		}

		PyObject *__getslice__(int i, int j)
		{
			int dims[1], len;
			int n;
			int length;
			length = self->Size();
			if (i < 0)
			    i = length + 1 - i;
			else if (i > length)
			    i = length;
			if (j < 0)
			    j = length + 1 - j;
			else if (j > length)
			    j = length;
			dims[0] = j-i;
			PyObject *o = PyArray_FromDims(1, dims, 'd'); //PyArray_FLOAT);
			if (!o) 
			{
			    throw NotEnoughMemory("ArrayDouble", "__getslice__",
						  "result of __getslice__");
			}
			double *ptr = (double *) ((PyArrayObject *)o)->data;
			DMArrayIterator<double> iter(self);
			len = sizeof(double)/sizeof(char);
			n = 0;
			for(iter.Init(); !iter.IsDone(); iter.Next())
			{
				if ( (n>=i) && (n<j) ) {
					*(ptr++) = iter.Current();
				}
				++n;
			}
			return o;
		}

		PyObject* __setslice__(int i, int j, PyObject* s)
		{
			PyArrayObject* o;
			int len, n;
			char msg[200];
			int length;
			length = self->Size();
			if (i < 0)
			    i = length + 1 - i;
			else if (i > length)
			    i = length;
			if (j < 0)
			    j = length + 1 - j;
			else if (j > length)
			    j = length;
			if (PyArray_Check(s)) {
				o = (PyArrayObject*) s;
				if ( (1==o->nd) && (j-i==o->dimensions[0]) &&
						(PyArray_DOUBLE == o->descr->type_num) ) {
					double *ptr = (double *) ((PyArrayObject *)o)->data;
					DMArrayIterator<double> iter(self);
					len = sizeof(double)/sizeof(char);
					n = 0;
					for (iter.Init(); !iter.IsDone(); iter.Next())
					{
						if ( (n>=i) && (n<j) ) {
							iter.Current() = *(ptr++);
						}
						++n;
					}
				} else {
				    /* Wrong dimensions and type is a ValueError (not logical!) */
					if (1 != o->nd)
						PyErr_SetString(PyExc_ValueError,
								"Must be a 1d array.");
					else if (j-i!=o->dimensions[0]) {
					        PyErr_SetString(PyExc_ValueError,
								"Cannot change length of array in __setslice__.");
					} else if (PyArray_DOUBLE != o->descr->type_num) {
						PyErr_SetString(PyExc_ValueError,
								"It needs to be an array of doubles.");
					}
					return NULL;
				}
			} else {
				PyErr_SetString(PyExc_TypeError,"accepts only arrays");
				return NULL;
			}
			Py_INCREF(Py_None);
			return Py_None;
		}
	};
};

