//! Leaf-to-leaf iterator for objects with tree structure.
/*! Designed for DMArrays.  It works for anything which has the
  following methods.
  \begincode
  int NumberOfBranches();
  bool IsLeaf();
  Type* GetBranch(int i);
  \endcode
  \author ajd27
  \date 4 October 2000
 */
#include "DMFirstHeader.h"
#include <stack>
#include "DigitalMaterial.h"

template <class Type>
class LeafToLeafIterator
{
	Type* head;
	//! The index of the current leaf.
	int leaf;
	//! The number of leaves in the tree.
	int nLeaves;
 public:
	LeafToLeafIterator( Type* head )
	{
		this->head = head;
		Init(); //  for compatibility with previous version. Should maybe be removed in the long run.
	}
	void Init()
	{
		leaf	   = 0;
		nLeaves    = head->NumberOfLeaves();
	}
	void Next()
	{
		++leaf;
	}
	Type *Current()
	{
		return head->GetLeaf(leaf);
	}
	bool Done() { return leaf >= nLeaves; }
};
