#ifndef _SYNCHRONIZATION_TOKEN_H_
#define _SYNCHRONIZATION_TOKEN_H_

#include <set>
#include "DigitalMaterial.h"

//! Stores relationship among sibling DMArrays.
/*!	We want arrays to be siblings for the sake of network
	migration of several arrays at the same time.  This token
	stores the arrays which have asked to be joined together.

	This object is guaranteed to have a pointer to at least one
	array at all times.  If you try to remove the last one, it
	will fail, so that it always has at least a pointer to 
	the array that owns it.

	When deleting a DMArray, the destructor needs to see whether
	it should free the memory associated with this object.
	\begincode
	if (token->GetSiblings().size()==1)
		free token;
	else
		token->RemoveSibling(this);
	\endcode

	\author ajd27
	\date 29 August 2000
	*/
class DMArray_AnyType;

class SynchronizationToken
{
	typedef DMArray_AnyType::siblingContainer arrayVector;
	arrayVector siblings; //! Siblings are the arrays joined together.
public:
	//! Constructor to record this single array making the token.
	SynchronizationToken( DMArray_AnyType*const array );
	//! Destructor
	~SynchronizationToken( );
	//! Add an array to the list of arrays.
	bool AddSibling( DMArray_AnyType*const array );
	//! Remove an array from the list
	bool RemoveSibling( DMArray_AnyType*const array );
	//! Find out what siblings this token has.
	const arrayVector& GetSiblings( ) const;
#ifndef NDEBUG
	bool Contains( DMArray_AnyType*const array ) const;
#endif
};

#endif //_SYNCHRONIZATION_TOKEN_H_
