// CheckPointObserver.h: interface for the CheckPointObserver class.
//
//////////////////////////////////////////////////////////////////////

#ifndef CHECKPOINTOBSERVER_H
#define CHECKPOINTOBSERVER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include <string>
#include "ListOfAtomsObserver.h"

class DMWriter;
class Potential;
class AtomsMover;

//! An observer which outputs the LOA to a file whenever notify/update is called

/*!

  This is an observer which, whenever Update is called, increments a counter.
  When the counter is a multiple of the member variable countInterval the LOA
  and its boundary conditions are saved to disk, using the NetCDF format.

  
  \author nbailey
  \date 7/10/2001
  \ingroup moleculardynamics

 */

class CheckPointObserver : public ListOfAtomsObserver
{
public:
	CheckPointObserver(Potential *potential, AtomsMover *mover, std::string baseFileName, bool oneFile, int notifyLevel = 0);
	void Update(ListOfAtoms *atoms);
	virtual ~CheckPointObserver();
	void ResetCounter(int count = 0) { counter = count; }
	std::string GetLastFileName() { return lastFileName; }

private:
	DMWriter *writer;
	Potential *potential;
	AtomsMover *mover;
	int counter;
	bool oneFile;
	std::string baseFileName;
	std::string lastFileName;
};

#endif // !defined(CHECKPOINTOBSERVER_H)
