// CheckPointRestarter.h: interface for the CheckPointRestarter class.
//
//////////////////////////////////////////////////////////////////////

#ifndef CHECKPOINTRESTARTER_H
#define CHECKPOINTRESTARTER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "DMReader.h"
#include <string>


class DMWriter;
class Potential;
class AtomsMover;
class ListOfAtoms;
class NeighborLocator;
class BoundaryConditions;

//! A class which can restart a simulation given the filename corresponding to a particular checkpoint



/*!

  
  \author nbailey
  \date 8/1/2001
  \ingroup moleculardynamics

 */

class CheckPointRestarter 
{
public:
	CheckPointRestarter(std::string fileName);
	void Update(ListOfAtoms *atoms);
	virtual ~CheckPointRestarter();

	//! Recreate reads creates the object from the information in the file and attaches them to each other (sets the various pointers)
	void Recreate();
	
	//! Having called Recreate(), the actual objects can be obtained by the following functions
	Potential *GetPotential() { return potential; }
	AtomsMover *GetMover() { return mover; }
	ListOfAtoms *GetListOfAtoms () { return atoms; }
	BoundaryConditions *GetBoundaryConditions() { return boundaryConditions; }
	NeighborLocator *GetNeighborLocator() { return neighborLocator; }

private:
	DMReader *reader;
	ListOfAtoms *atoms;
	BoundaryConditions *boundaryConditions;
	NeighborLocator *neighborLocator;
	Potential *potential;
	AtomsMover *mover;
};

#endif // !defined(CHECKPOINTRESTARTER_H)
