// CrackPlaneObserver.h: interface for the CrackPlaneObserver class.
//
//////////////////////////////////////////////////////////////////////

#ifndef CRACKPLANEOBSERVER_H
#define CRACKPLANEOBSERVER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"

#include "ListOfAtomsObserver.h"
#include <vector>

class CrackPlaneObserver : public ListOfAtomsObserver
{
public:
	CrackPlaneObserver(double planePoint[DIMENSION], double planeNormal[DIMENSION], double cutoffDistance, int notifyLevel=0);
	virtual ~CrackPlaneObserver();
	void MakePairList(ListOfAtoms *atoms);
	void Update(ListOfAtoms *atoms);



	double GetMaxIncrease() { return maxIncrease; }
	double GetMeanIncrease() { return meanIncrease; }
	// returns perpendicular distance from plane with a minus if on the opposite side to the positive of the normal
	double GetDistanceFromPlane(double position[DIMENSION]) const;
	

	const double *GetPlanePoint() const { return planePoint; }
	void SetPlanePoint(double planePoint[DIMENSION]);
	const double *GetPlaneNormal() const { return planeNormal; }
	void SetPlaneNormal(double planeNormal[DIMENSION]); 

	const double GetCutoffDistance() const { return cutoffDistance; }
	void SetCutoffDistance(double cutoffDistance) { this->cutoffDistance = cutoffDistance; } 

#ifndef SWIG
	const std::vector<double> &GetPairSeparations() const { return pairSeparations; }
	const std::vector<std::pair<int, int> > &GetPairList() const { return pairList; } 

private:
	double planePoint[DIMENSION];
	double planeNormal[DIMENSION];
	double cutoffDistance;

	double maxIncrease;
	double meanIncrease;
	
	std::vector<std::pair<int, int> > pairList;
	std::vector<double> pairSeparations;
	std::vector<double> initialSeparations;
#endif // SWIG
};

#endif // !defined(CRACKPLANEOBSERVER_H)
