// EnergyObserver.h: interface for the EnergyObserver class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ENERGYOBSERVER_H
#define ENERGYOBSERVER_H

#include "DigitalMaterial.h"

#include "ListOfAtomsObserver.h"

class Potential;

class EnergyObserver : public ListOfAtomsObserver
{
public:
	void SetNSampled(int nSampled);
	int GetNSampled() const;
	void Reset();
	void SetPotential(Potential *potential);
	void Update(ListOfAtoms *atoms);
	EnergyObserver(Potential *potential, int notifyLevel = 0);
	virtual ~EnergyObserver();

	double GetEnergy() const {return kineticEnergy+potentialEnergy;}
	double GetKineticEnergy() const {return kineticEnergy;}
	double GetPotentialEnergy() const {return potentialEnergy;}
	double GetMeanEnergy() const {
		if (nSampled == 0) return 0.;
		else return (summedKineticEnergy+summedPotentialEnergy)/nSampled;}
	double GetMeanKineticEnergy() const {
		if (nSampled == 0) return 0.;
		else return summedKineticEnergy/nSampled;}
	double GetMeanPotentialEnergy() const {
		if (nSampled == 0) return 0.;
		else return summedPotentialEnergy/nSampled;}
	double GetKineticTemperature() const;
	double GetMeanSquaredEnergyFluctuation() const;
	double GetMeanSquaredKineticEnergyFluctuation() const;
	double GetKineticSpecificHeat() const;

private:
	Potential * potential;
	int nSampled;
	int nAtoms;
	double kineticEnergy;
	double summedKineticEnergy;
	double summedSquaredKineticEnergy;
	double potentialEnergy;
	double summedPotentialEnergy;
	double summedSquaredEnergy;

};

#endif // !defined(ENERGYOBSERVER_H)
