%{
#include "CheckPointObserver.h"
#include "ColorMethod.h"
#include "CrackPlaneObserver.h"
#include "EnergyObserver.h"
#include "LocalForceObserver.h"
#include "PythonLOAObserver.h"
#include "StressIntensityObserver.h"
%}


%nodefault ListOfAtomsObserver;
class ListOfAtomsObserver{};

%nodefault ColorMethod;
class ColorMethod 
{
public:
    %extend
    {
        double __call__(ListOfAtoms *atoms,int i) {return (*self)(atoms,i);}
    }
};

class EnergyColorMethod : public ColorMethod
{
public:
	EnergyColorMethod(Potential *potential);
};

class StressColorMethod : public ColorMethod
{
public:
	StressColorMethod(Potential *potential);
	void SetComponent(int comp);
};

class ForceColorMethod : public ColorMethod
{
public:
    ForceColorMethod(Potential *potential);
    void SetComponent(int comp);
};


class CoordinationColorMethod : public ColorMethod
{
public:
    CoordinationColorMethod();
};

class CentroSymmetryColorMethod : public ColorMethod
{
public:
    CentroSymmetryColorMethod(BravaisLatticeWithBasis *undeformedLattice, 
        int numberNearestNeighbors, ArrayDoubleVector *nearestNeighborVectors);
};

%include "CheckPointObserver.h"
%include "CrackPlaneObserver.h"

%extend CrackPlaneObserver
{
	PyObject *GetPairDistances()
	{
		const std::vector<double > &pairDist = self->GetPairSeparations();
		int size = pairDist.size();
		PyObject *pResult = PyList_New(0);
		for(int idx = 0; idx < size; ++idx )
		{
		PyList_Append(pResult, PyFloat_FromDouble(pairDist[idx]) );
		}
	return pResult;
	}
} // end extend


%include "EnergyObserver.h"
%include "LocalForceObserver.h"
%include "PythonLOAObserver.h"
%include "StressIntensityObserver.h"
