// PythonLOAObserver.cpp: implementation of the PythonLOAObserver class.
//
//////////////////////////////////////////////////////////////////////

#include "PythonLOAObserver.h"
#include <assert.h>
#include "DMException.h"
#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PythonLOAObserver::PythonLOAObserver(PyObject *updateMethod, PyObject *exitMethod, int notifyLevel) : ListOfAtomsObserver(notifyLevel)
{
    // Store a reference to the Python ListOfAtomsObserver routine.
    assert(0 != updateMethod);
    Py_INCREF(updateMethod);  
    this->updateMethod = updateMethod;
	this->exitMethod = exitMethod;
	if(exitMethod) Py_INCREF(exitMethod); 
}

PythonLOAObserver::~PythonLOAObserver()
{
    Py_XDECREF(updateMethod);
	if(exitMethod)
	{
	    PyObject *result = PyEval_CallObject(exitMethod, Py_BuildValue("()"));
	    Py_XDECREF(result);
	}
}


void PythonLOAObserver::Update(ListOfAtoms *atoms)
{
	this->atoms = atoms;
    PyObject *args;
    PyObject *result;
    assert(0 != updateMethod);
	args = Py_BuildValue("()");
    result = PyEval_CallObject(updateMethod, args);
    Py_DECREF(args);
    if (result == 0) 
    {
	// An exception occured in the Python code.
		throw DMException("PythonLOAObserver", "Update",
			  "Error in the Python LOAObserver' update Method.");
    }
    Py_XDECREF(result);
}
