// StressIntensityObserver.h: interface for the StressIntensityObserver class.
//
//////////////////////////////////////////////////////////////////////

#ifndef STRESSINTENSITYOBSERVER_H
#define STRESSINTENSITYOBSERVER_H

#include "DigitalMaterial.h"
#include "ListOfAtomsObserver.h"
#include "DMArray.h"

class Potential;
class NeighborLocator;
class BravaisLatticeWithBasis;
class NotchFieldDisplacer;

class StressIntensityObserver : public ListOfAtomsObserver
{
public:
	void SetPotential(Potential *potential);
	void SetNeighborLocator(NeighborLocator *neighborLocator);
	void SetLattice(BravaisLatticeWithBasis *lattice);
	void SetPathCenter(double pathCenter[DIMENSION]);
	void SetPathRadius(double pathRadius);

	void Update(ListOfAtoms *atoms);
	StressIntensityObserver(Potential *potential, NeighborLocator *neighborLocator, BravaisLatticeWithBasis *lattice, NotchFieldDisplacer *comlimentaryField, double pathCenter[DIMENSION], double pathRadius, double heightTolerance, const DMArray<double, DIMENSION> *undeformedPositions, int notifyLevel = 0);
	virtual ~StressIntensityObserver();

	double GetH_Integral() {return H_integral;}

private:
	Potential * potential;
	NeighborLocator *neighborLocator;
	BravaisLatticeWithBasis *lattice;
	NotchFieldDisplacer *complimentaryField;
	double pathCenter[DIMENSION];
	double pathRadius;
	double heightTolerance;
	double H_integral;

	int stresscomp[DIMENSION][DIMENSION];
	const DMArray<double, DIMENSION> *undeformedPositions;
};

#endif // !defined(STRESSINTENSITYOBSERVER_H)
