// AbstractDomainSubdivision.h
//
// Two related tasks:
//   1: Decide on which processor an atom should reside.
//   2: Decide which processors need to know about an atom that resides on
//      this processor.
#ifndef DOMAINSUBDIVISION_H
#define DOMAINSUBDIVISION_H

#include "DMFirstHeader.h"
#include <vector>

#include "DigitalMaterial.h"

template<class Type, int N> class DMArray;
class Index;

class DomainSubdivision 
{
public:
	DomainSubdivision(const int cpuLayout[DIMENSION],
							  int myDomain);
	virtual ~DomainSubdivision() {};
	//! Ask the domain subdivision to adjust its parameters according to these positions
	virtual void Update(const DMArray<double, DIMENSION> *positions) {};
	virtual int WhichDomain(const double point[]) const = 0; 
	virtual bool PointNearDomain(const double point[DIMENSION], int domain) const = 0;
	virtual void FindPointsNearDomain(const DMArray<double, DIMENSION> *positions,
									  std::vector<Index> *interestingAtomsToSend) const = 0;
protected:
	int numberOfProcs;
	int myDomain;
	int cpuLayout[DIMENSION];
};


#endif // !ndef ABSRACTDOMAINSUBDIVISION_H
