// HomogeneousDomainSubdivision.h: interface for the HomogeneousDomainSubdivision class.
//
//////////////////////////////////////////////////////////////////////

#ifndef HOMOGENEOUSDOMAINSUBDIVISION_H
#define HOMOGENEOUSDOMAINSUBDIVISION_H

#include "DMFirstHeader.h"
#include <vector>

#include "DigitalMaterial.h"
#include "DomainSubdivision.h"

#include "TinyVector.h"

//! A simple subdivision, where the processors' domain is a homogeneous subdivision of a rectangular box.
/*!
Each processor's domain is a rectangle and we have periodic boundary conditions to the domains.

*/

class HomogeneousDomainSubdivision : public DomainSubdivision  
{
public:
	HomogeneousDomainSubdivision(const int cpuLayout[DIMENSION],
								 int myDomain,
								 double bufferLayer);
	virtual ~HomogeneousDomainSubdivision();
	virtual void Update(const DMArray<double, DIMENSION> *positions) ;

	int WhichDomain(const double point[DIMENSION]) const;
	bool PointNearDomain(const double point[DIMENSION], int domain) const;
#ifndef SWIG
	void FindPointsNearDomain(const DMArray<double, DIMENSION> *positions,
							  std::vector<Index> *interestingAtomsToSend) const;
private:
	double lowerCorner[DIMENSION];

	void SetBoundingBox(const DMArray<double, DIMENSION> *positions);
	double myLowerCorner[DIMENSION];
	double upperCorner[DIMENSION];
	double myUpperCorner[DIMENSION];
	double cpuBoxSize[DIMENSION];

	std::vector<int> neighborDomains;
	std::vector<TinyVector<int, DIMENSION> > differenceCpuCoordNeighborDomain;
	double bufferLayer;
	int CpuCoordToDomain(const int cpuCoord[DIMENSION]) const;
	void DomainToCpuCoord(int domain, int cpuCoord[DIMENSION]) const;
#endif //SWIG
};

#endif // !defined HOMOGENEOUSDOMAINSUBDIVISION_H
