// Migrator.h: interface for the Migrator class.
//
//////////////////////////////////////////////////////////////////////

#ifndef MIGRATOR_H
#define MIGRATOR_H

#include "DigitalMaterial.h"
#include "DMProtocol.h"

class DomainSubdivision;
template<class Type, int N> class DMArray;

//! A class whose responsibility is to move atoms and there related data among processors.
/*!
    Each time the Migrate function is called, we check that the atoms be have on our 
    processor are located in our domain. If not, send them to the right processor. This
    communicates not only the positions, but also all their related siblings.

    Note that siblings of the branches of the positions array may have to be migrated too. 
    This is why in the Migrate function we loop over the leaves: we're sure we do not miss 
    any sibling.
*/
class Migrator
{
public:
    Migrator(DomainSubdivision *domain,
             DMArray<double, DIMENSION> *positions = 0, 
             DMProtocol *protocol = dmProtocol);
    virtual ~Migrator();
    void SetPositionsArray(DMArray<double, DIMENSION> *positions)
        {this->positions = positions;}
    void Migrate();
private:
    DomainSubdivision *domain;
    DMProtocol *protocol;
    //! The buffers used for the communiation
    /*! We prefer to keep the buffers, because otherwise we have to 
    reallocate each time we Migrate. If their allocated size is too
    small, then this leads to a lot of unnecessary communication */
    DMProtocol::Buffers *buffers;
    DMArray<double, DIMENSION> *positions;
    //! the number of processors
    int nProcs;
    //! the index of my processor
    int myProc;
};

#endif // !defined MIGRATOR_H
