// ParallelAtomsInitializer.cpp: implementation of the ParallelAtomsInitializer class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"


#include "ListOfAtoms.h"
#include "ParallelAtomsInitializer.h"
#include "DMProtocol.h"
#include "ProtocolFactory.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ParallelAtomsInitializer::ParallelAtomsInitializer(AtomsInitializer *initializer)
{
    this->initializer = initializer;
}

void ParallelAtomsInitializer::Create(ListOfAtoms *atoms)
{
    const int myProc = dmProtocol->GetProcessorNumber();
    const int nProcs = dmProtocol->GetNumberOfProcessors();
    DMProtocol::Buffers *buffers = dmProtocol->NewBuffers();
    if(myProc == 0)
    {
        ProtocolFactory::Instance()->SetProtocol(ProtocolFactory::Local);
        initializer->Create(atoms);
        ListOfAtoms *tmp = atoms->Copy(); // we may invent another mechanism.
        for(int leaf = 0; leaf < tmp->NumberOfLeaves(); ++leaf)
            tmp->GetLeaf(leaf)->SetShape(0); // empty the leaves before sending them
        for(int proc=1; proc < nProcs; ++proc)
            tmp->Save(buffers->GetSendBuffer(proc));
        delete tmp;
        ProtocolFactory::Instance()->Reset();
    }

    buffers->AllToAll();
    
    if(myProc != 0)
    {
        atoms->Load(buffers->GetRecvBuffer(0));
    }
    delete buffers;
}
