// ParallelPotential.cpp: implementation of the ParallelPotential class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"
#include "DMArray.h"
#include "ParallelPotential.h"
#include "DMProtocol.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ParallelPotential::ParallelPotential(Potential *potential)
{
    this->potential = potential;
}

double ParallelPotential::CalculateEnergy(ListOfAtoms *atoms)
{
    double totalEnergy;
    double myEnergy = potential->CalculateEnergy(atoms);
    dmProtocol->AllReduce(&myEnergy, &totalEnergy, 1, DMProtocol::SUM);
    return totalEnergy;
}

void ParallelPotential::CalculateForces (ListOfAtoms *atoms,
                                         DMArray<double, DIMENSION> *forces)
{
    forces->GetSynchronizationKit()->Init();
    potential->CalculateForces(atoms, forces->GetSynchronizationKit()->GetExtended());
}

void ParallelPotential::CalculateForcesBetweenTypes(ListOfAtoms *atoms,
                          DMArray<double, DIMENSION> *forces, int type1, int type2)
{
    forces->GetSynchronizationKit()->Init();
    potential->CalculateForcesBetweenTypes(atoms, 
        forces->GetSynchronizationKit()->GetExtended(), type1, type2);
}

double ParallelPotential::CalculateEnergyAndForces(ListOfAtoms *atoms, 
                                                   DMArray<double, DIMENSION> *forces)
{
    forces->GetSynchronizationKit()->Init();
    double totalEnergy;
    double myEnergy = potential->CalculateEnergyAndForces(atoms, 
        forces->GetSynchronizationKit()->GetExtended());
    dmProtocol->AllReduce(&myEnergy, &totalEnergy, 1, DMProtocol::SUM);
    return totalEnergy;
}

double ParallelPotential::CalculateAtomicEnergy(ListOfAtoms *atoms, int i)
{
    return potential->CalculateAtomicEnergy(atoms, i);
}