// Synchronizer.h: interface for the Synchronizer class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SYNCHRONIZER_H
#define SYNCHRONIZER_H

#include "DMFirstHeader.h"
#include <vector>

#include "DigitalMaterial.h"
#include "DMProtocol.h"
#include "DMArray.h"
class DomainSubdivision;

class Synchronizer
{
public:
    Synchronizer(DMArray<double, DIMENSION> *positions=0, DMProtocol *protocol = dmProtocol);
    virtual ~Synchronizer();
    void SetPositionsArray(DMArray<double, DIMENSION> *positions);
    void Rebuild(const DomainSubdivision *domain);
    void Synchronize(DMArray_AnyType *array) const;
    const Shape &GetGhostsShape() const {return ghostShape;}
private:
    //! The message passing protocol
    DMProtocol *protocol;
    //! The buffers we use for the communication.(See comment in Migrator).
    DMProtocol::Buffers *buffers;
    //! The number of precessors
    int nProcs;
    //! The index of my process
    int myProc;
    //! The positions array that says which atoms are to be synchronized.
    DMArray<double, DIMENSION> *positions;
    //! The number of leaves of the positions array
    int numberOfLeaves;
    //! we store the index of the atoms  we have to send in vectors (one per processor).
    std::vector<Index> *interestingAtomsToSend;
    //! we store the index of the ghost atoms that are received in the receiveBuffers (one per processor).
    std::vector<Index> *interestingAtomsToRecv;
    //! the size of the leaves of the array that has to be sent (one per processor per leaf)
    std::vector<int> *sizeOfLeavesToSend;
    //! the size of the leaves of the array that has to be received (one per processor per leaf)
    std::vector<int> *sizeOfLeavesToRecv;
    //! The length of each leaves of the ghost array
    std::vector<int> sizeOfGhostLeaves;
    //! The shape of the ghosts
    Shape ghostShape;
    //! the size of the array that has to be sent (one per processor per leaf)
    int *sizeToSend;
    //! the size of the array that has to be received (one per processor per leaf)
    int *sizeToRecv;
};

#endif // !defined SYNCHRONIZER_H
