// Composite Potential
//
//////////////////////////////////////////////////////////////////////
#include "DigitalMaterial.h"
#include <iostream>
#include "CompositePotential.h"

#include "ListOfAtoms.h"
#include "DMArray.h"
#include "IndexError.h"
#include "NotEnoughMemory.h"
#include "NeighborLocator.h"
#include "SerialBuilder.h"
#include "BoundaryConditions.h"

#include "DMLastHeader.h"

using namespace std;

#define LEN_NBR_ARRAYS 1500


static SerialBuilder<CompositePotential> *_Gcreator = 
    new SerialBuilder<CompositePotential>("CompositePotential");

const string CompositePotential::className = string("CompositePotential");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CompositePotential::CompositePotential(Potential *potential1, Potential *potential2) 
{
	this->potential1 = potential1;
	this->potential2 = potential2;
	forces2 = 0;
}

CompositePotential::~CompositePotential()
{
	if (forces2) delete forces2;
}
 
void CompositePotential::CalculateForces(ListOfAtoms *atoms, DMArray<double, DIMENSION> *forces)
{

	if (!(forces2 && (forces2->GetShape() == atoms->GetShape())))
		Allocate(atoms->GetCartesianPositions());
	potential1->CalculateForces(atoms,forces);
	potential2->CalculateForces(atoms,forces2);
	forces->Axpy(forces2,1);

}

double CompositePotential::CalculateEnergy(ListOfAtoms *atoms)
{
	return (potential1->CalculateEnergy(atoms)) + (potential2->CalculateEnergy(atoms));
}

double CompositePotential::GetLengthScale() const 
{
	return min(potential1->GetLengthScale(),potential2->GetLengthScale());
}
	
double CompositePotential::GetEnergyCurvatureScale() const
{
	return min(potential1->GetEnergyCurvatureScale(),potential2->GetEnergyCurvatureScale());
}

// double CompositePotential::GetInnerCutoffDistance() const
// {
// 	return min(potential1->GetInnerCutoffDistance(),potential2->GetInnerCutoffDistance());
// }

double CompositePotential::CalculateAtomicEnergy(ListOfAtoms *atoms, int i)
{
	return (potential1->CalculateAtomicEnergy(atoms,i)) + (potential2->CalculateAtomicEnergy(atoms,i));
}

void CompositePotential::Save(DMWriter *writer) const
{
	potential1->Save(writer);
	potential2->Save(writer);
}

void CompositePotential::Load(DMReader *reader)
{
	potential1->Load(reader);
	potential2->Load(reader);
}

void CompositePotential::Allocate(const DMArray<double, DIMENSION> *array)
{
	if (forces2) delete forces2;
	forces2 = new DMArray<double, DIMENSION>(array);
}
