// Gravity Potential
//
//////////////////////////////////////////////////////////////////////
#include "DigitalMaterial.h"
#include <iostream>
#include "GravityPotential.h"

#include "ListOfAtoms.h"
#include "DMArray.h"
#include "IndexError.h"
#include "NotEnoughMemory.h"
#include "NeighborLocator.h"
#include "SerialBuilder.h"
#include "BoundaryConditions.h"

#include "DMLastHeader.h"

using namespace std;

#define LEN_NBR_ARRAYS 1500


static SerialBuilder<GravityPotential> *_Gcreator = 
    new SerialBuilder<GravityPotential>("GravityPotential");

const string GravityPotential::className = string("GravityPotential");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


GravityPotential::GravityPotential() 
{
	g = 0.01;
	direction[0]=0;
	direction[1]=-1;
	if(DIMENSION==3) direction[2]=0;	// Direction for gravity
}


void GravityPotential::CalculateForces(ListOfAtoms *atoms, DMArray<double, DIMENSION> *forces)
{
	double grav[DIMENSION];
	for(int coord=0; coord<DIMENSION; coord++) 
		grav[coord] = g * direction[coord];
	forces->Set(grav);
}
	

void GravityPotential::SetG(double g) {this->g = g;}

void GravityPotential::SetDirection(double direction[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; ++coord)
		this->direction[coord]=direction[coord];	// Direction for gravity
}

double GravityPotential::CalculateEnergy(ListOfAtoms *atoms)
{
	double energy = 0;
	for (int atom=0; atom < atoms->GetNumber(); atom++)
	{	
		for(int coord=0; coord<DIMENSION; coord++) 
			energy += g * direction[coord] * atoms->GetCartesianPosition(atom, coord);
	}
	return energy;
}

