// AtomsLocator.cpp: implementation of the AtomsLocator class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"
#include "AtomsLocator.h"
#include "ListOfAtoms.h"

#include "DMLastHeader.h"


/*!
  In the base class we implement the most direct, but possibly not the most
  efficient, method of finding atoms in the specified region: iterate through
  all positions and use the Inside function to identify the ones we keep.


 */

const std::vector<int> &AtomsLocator::GetAtomNumbers(ListOfAtoms *atoms)
{
	listOfAtomNumbers.clear();

	int nAtoms = atoms->GetNumber(),atomIdx;
	DMArrayIterator<double, DIMENSION> posIter(atoms->GetCartesianPositions());
	

	for( posIter.Init(), atomIdx = 0; !posIter.IsDone(); posIter.Next(), ++atomIdx)
	{
		double *position = posIter.Current();
		
		if (Inside(position))
			listOfAtomNumbers.push_back(atomIdx);
	}

	return listOfAtomNumbers;
}
