// DislocationMaker.h: interface for the DislocationMaker class.
//
//////////////////////////////////////////////////////////////////////

#ifndef DISLOCATIONMAKER_H
#define DISLOCATIONMAKER_H

#include "DMFirstHeader.h"
#include <math.h>
#include "DigitalMaterial.h"
#include <assert.h>


#include "Quasi2DElasticField.h"
#include "ListOfAtoms.h"



class DislocationMaker : public Quasi2DElasticField
{
public:
	DislocationMaker(const double burgers[DIMENSION],const double direction[DIMENSION], double center[DIMENSION],const double nu,double thetaLowerCutoff=-PI);
	virtual ~DislocationMaker();




	// Gets and Sets for array member variables
	void SetBurgers(const double burgers[DIMENSION]); 
	const double *GetBurgers() const {return (const double *)burgers;}

	void SetDirection(const double direction[DIMENSION]); 
	const double *GetDirection() const {return (const double *)direction;}
	void SetCenter(const double center[DIMENSION]); 
	const double *GetCenter() const {return (const double *)center;}
	// Gets and Sets for nu
	void SetNu(double nu) {this->nu = nu;}
	double GetNu() const {return nu;}

#ifndef SWIG
	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);
	void GradientWRTPosition(const double position[DIMENSION],  double gradientWRTPosition[DIMENSION][DIMENSION]);


protected:
	double burgers[DIMENSION];
	double nu;
	double edge;
	double edge_over_2PI;
	double nu_factor;
	double screw;
	double edgeComponent[DIMENSION];
	virtual void UpdateMemberVariables();
#endif //SWIG
};

#endif // !defined(DISLOCATIONMAKER_H)
