// ExternalStress.h: interface for the ExternalStress class.
//
//////////////////////////////////////////////////////////////////////

#ifndef EXTERNALSTRESS_H
#define EXTERNALSTRESS_H

#include "DMFirstHeader.h"
#include <vector>

#include "ElasticFieldTransformer.h"
#define N_CPTS (DIMENSION*(DIMENSION+1)/2) // number of independent stress or strain components

/**
 * \author Nick Bailey
 * \date October 2000
 * \ingroup moleculardynamics
**/

class ListOfAtoms;

class ExternalStress : public ElasticFieldTransformer  
{
public:
	ExternalStress(double *C,double *origin,ListOfAtoms *undeformed = NULL);
	virtual ~ExternalStress();
	void SetStress(double stress[DIMENSION][DIMENSION]);
	const double GetStress(int a, int b) { return stress[a][b]; }

	void SetStrain(double strain[DIMENSION][DIMENSION]);
	const double GetStrain(int a, int b) { return strain[a][b]; }


	void SetUndeformed(ListOfAtoms *undeformed) {this->undeformed = undeformed;}
	void SetOrigin(double *origin)
	  { for(int coord=0;coord<DIMENSION;++coord) this->origin[coord] = origin[coord];}
	const double *GetOrigin() const {return origin; }
	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);
	void GradientWRTPosition(const double position[DIMENSION], double gradientWRTPosition[DIMENSION][DIMENSION]);

private:
	double stress[DIMENSION][DIMENSION];
	double strain[DIMENSION][DIMENSION];
	double origin[DIMENSION]; // in principle not necessary but important if I'm applying this transformer to the boundary atoms and not the central atoms---a large shift in origin would be bad!
	ListOfAtoms *undeformed;
	double C[3]; // assuming cubic symmetry: 3 elastic constants (C11, C12, C44)	
};

#endif // !defined(EXTERNALSTRESS_H)
