// LinearPositionTransformer.cpp: implementation of the LinearPositionTransformer class.
//
//////////////////////////////////////////////////////////////////////
#include "DigitalMaterial.h"

#include "LinearPositionTransformer.h"
#include "DMArray.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


LinearPositionTransformer::LinearPositionTransformer(const double transformationMatrix[DIMENSION][DIMENSION])
{
	for(int i=0;i<DIMENSION;++i)
		for(int j=0;j<DIMENSION;++j)
			this->transformationMatrix[i][j] = transformationMatrix[i][j];
	
	for(int coord = 0; coord < DIMENSION; ++coord)
		center[coord] = 0.;
}


LinearPositionTransformer::~LinearPositionTransformer()
{


}

void LinearPositionTransformer::ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION])
{
	for( int coord1 = 0; coord1 < DIMENSION; ++coord1 )
		for( int coord2 = 0; coord2 < DIMENSION; ++coord2 )
			displacement[coord1] += transformationMatrix[coord1][coord2] * (position[coord2] - center[coord2]);
	
}


void LinearPositionTransformer::GradientWRTPosition(const double position[DIMENSION],double gradientWRTPosition[DIMENSION][DIMENSION])
{
	for( int coord1 = 0; coord1 < DIMENSION; ++coord1 )
	  for( int coord2 = 0; coord2 < DIMENSION; ++coord2 )
		  gradientWRTPosition[coord1][coord2] = transformationMatrix[coord1][coord2];
}



void LinearPositionTransformer::SetTransformationMatrix(const double transformationMatrix[DIMENSION][DIMENSION])
{
	for(int i=0;i<DIMENSION;++i)
		for(int j=0;j<DIMENSION;++j)
			this->transformationMatrix[i][j] = transformationMatrix[i][j];
}
