// LinearPositionTransformer.h: interface for the LinearPositionTransformer class.
//
//////////////////////////////////////////////////////////////////////

#ifndef LINEARPOSITIONTRANSFORMER_H
#define LINEARPOSITIONTRANSFORMER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include <math.h>
#include <assert.h>

#include "ElasticFieldTransformer.h"
#include "ListOfAtoms.h"

//! Applies a given linear transformation to the coordinates of each atom in the ListOfAtoms

/* \ingroup moleculardynamics
 
 Takes a DIMENSIONxDIMENSION matrix, and operates on the coordinates of
 each atom in turn (using an iterator), giving the displacement, which is then 
 added to the position. This is different to how the class was originally
 implemented--then the given matrix was to multiply the position giving the
 new position rather than the change in position. You can think of it as this:
 Before, the passed in matrix was the deformation gradient. Now it is the 
 strain matrix (in the context of infinitesimal transformations)

 Can be used to scale the positions etc. 
 
 /author Nicholas Bailey
 
 */


class LinearPositionTransformer : public ElasticFieldTransformer
{
public:
	LinearPositionTransformer(const double transformationMatrix[DIMENSION][DIMENSION]);
	virtual ~LinearPositionTransformer();
	
	void SetTransformationMatrix(const double transformationMatrix[DIMENSION][DIMENSION]);
	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);

	void GradientWRTPosition(const double position[DIMENSION],double gradientWRTPosition[DIMENSION][DIMENSION]);
  

	void SetCenter(double center[DIMENSION]){for (int coord = 0; coord < DIMENSION; ++coord) this->center[coord] = center[coord];}
	const double *GetCenter() { return center; }

private:
	double transformationMatrix[DIMENSION][DIMENSION];
	double center[DIMENSION];
};

#endif // !defined(LINEARPOSITIONTRANSFORMER_H)
