// NotchMaker.cpp: implementation of the NotchMaker class.
//
//////////////////////////////////////////////////////////////////////
#include "DigitalMaterial.h"
#include "NotchMaker.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

NotchMaker::NotchMaker(double openingAngle, double tipLocation[DIMENSION], double dirAngle)
{
	this->openingAngle = openingAngle;
	for(int coord=0; coord<DIMENSION; coord++) this->tipLocation[coord] = tipLocation[coord];
	this->dirAngle = dirAngle;
	UpdateMemberVariables();
}

NotchMaker::~NotchMaker()
{

}

void NotchMaker::SetDirAngle(double dirAngle)
{
	this->dirAngle = dirAngle;
	UpdateMemberVariables();
}

double NotchMaker::GetDirAngle() const
{
	return dirAngle;
}

void NotchMaker::SetOpeningAngle(double openingAngle)
{
	this->openingAngle = openingAngle;
	UpdateMemberVariables();
}

double NotchMaker::GetOpeningAngle() const
{
	return openingAngle;
}

void NotchMaker::SetTipLocation(const double tipLocation[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; coord++) this->tipLocation[coord] = tipLocation[coord];
}

void NotchMaker::UpdateMemberVariables()
{
	normal1[0] = -sin(dirAngle-openingAngle/2.);
	normal1[1] =  cos(dirAngle-openingAngle/2.);
	normal2[0] =  sin(dirAngle+openingAngle/2.);
	normal2[1] = -cos(dirAngle+openingAngle/2.);
}

bool NotchMaker::Inside(const double X[DIMENSION])
{
	bool inside;
	inside = ( ((X[0]-tipLocation[0])*normal1[0] + (X[1]-tipLocation[1])*normal1[1] >= 0.) ||
		   ((X[0]-tipLocation[0])*normal2[0] + (X[1]-tipLocation[1])*normal2[1] >= 0.)   )	;
	return inside;
}

