// SphereAtomsLocator.cpp: implementation of the SphereAtomsLocator class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"
#include "SphereAtomsLocator.h"
#include "ListOfAtoms.h"
#include "NeighborLocator.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SphereAtomsLocator::SphereAtomsLocator(double center[DIMENSION], double radius)
{
	for( int coord = 0; coord < DIMENSION; ++coord )
		this->center[coord] = center[coord];
	this->radius = radius;

	neighbors       = 0;
	displacements   = 0;
	displacementsSq = 0;
}

SphereAtomsLocator::~SphereAtomsLocator()
{

}

void SphereAtomsLocator::AllocateForNeighborLocator(int nSpaces)
{
// nSpaces is playing the role of EMTPotential's arrayLim here
	this->nSpaces = nSpaces;
	
	if(neighbors) 
	{	
		delete [] neighbors;
		delete [] displacements;
		delete [] displacementsSq;
	}

	if(nSpaces > 0)
	{
		neighbors       = new (std::nothrow) int[nSpaces];
		displacements   = new (std::nothrow) double[nSpaces][DIMENSION];
		displacementsSq = new (std::nothrow) double[nSpaces];
		
		ASSERT(neighbors);
		ASSERT(displacements);
		ASSERT(displacementsSq);
	}
	else
	{
		neighbors       = 0;
		displacements   = 0;
		displacementsSq = 0;
	}
}



const std::vector<int> &SphereAtomsLocator::GetAtomNumbers(ListOfAtoms *atoms)
{
	listOfAtomNumbers.clear();
	
	int  numNeighbors, spacesLeftInArrays;
    bool arrayNotTooFull;
	int nAtoms = atoms->GetNumber();
	AllocateForNeighborLocator(nAtoms);	
	NeighborLocator *neighborLocator = atoms->GetNeighborLocator();


	spacesLeftInArrays = nSpaces;     
	arrayNotTooFull = neighborLocator->Neighbors(center, neighbors, numNeighbors, displacements, displacementsSq, spacesLeftInArrays, radius);
	
	while(!arrayNotTooFull)
	{
		AllocateForNeighborLocator(2*nSpaces);
		spacesLeftInArrays = nSpaces; // nSpaces has been doubled
		arrayNotTooFull = neighborLocator->Neighbors(center,neighbors, numNeighbors, displacements, displacementsSq, spacesLeftInArrays, radius);
	}
	
	for(int i = 0; i < numNeighbors; ++i)
	{
		listOfAtomNumbers.push_back(neighbors[i]);
	}

	return listOfAtomNumbers;
}

bool SphereAtomsLocator::Inside(const double *position)
{
	double rSq = 0.;
	for( int coord = 0; coord < DIMENSION; ++coord )
	{
		double diff = position[coord] - center[coord];
		rSq += diff * diff;
	}

	return (rSq < radius * radius);
}
