// UndercoordinatedPruner.cpp: implementation of the UndercoordinatedPruner class.
//
//////////////////////////////////////////////////////////////////////

#include "DMFirstHeader.h"
#include <vector>
#include <set>

#include "DigitalMaterial.h"

#include "UndercoordinatedPruner.h"
#include "CellNeighborList.h"
#include "ListOfAtoms.h"
#include "BoundaryConditions.h"
#include "DMException.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

UndercoordinatedPruner::UndercoordinatedPruner(double cutoff, int minCoordination):cutoff(cutoff), minCoordination(minCoordination)
{

}

UndercoordinatedPruner::~UndercoordinatedPruner()
{
}

void UndercoordinatedPruner::SetCutoff(double cutoff)
{
	this->cutoff = cutoff;
}


void UndercoordinatedPruner::Transform(ListOfAtoms *atoms)
{


	const BoundaryConditions *bc = atoms->GetBoundaryConditions();
	int i;
	double drift = 0.05*cutoff;
	const DMArray<double, DIMENSION> *positions = atoms->GetCartesianPositions();
	const DMArray<double, DIMENSION> *velocities = atoms->GetCartesianVelocities();
	int nAtoms = atoms->GetNumber();
	NeighborLocator *tempNList = new CellNeighborList(cutoff, drift);
	tempNList->SetBoundaryConditions(atoms->GetBoundaryConditions());
	tempNList->SetPositionArray(atoms->GetCartesianPositions());
	int spaceLeftInArray, numberNeighbors;
	int *neighbors = new int[nAtoms];
	double (*displacement)[DIMENSION] = new double [nAtoms][DIMENSION];
	double *distSQ = new double[nAtoms];
	
	tempNList->UpdatePosition();
	atomsRemoved.clear();

	for(i=0;i<nAtoms;i++)
	{
		spaceLeftInArray = nAtoms;
		
		bool ok = tempNList->Neighbors(i, neighbors, numberNeighbors, displacement, distSQ, spaceLeftInArray);
		assert(ok);
		if(numberNeighbors < minCoordination)
			atomsRemoved.insert(i);

	}
	atoms->RemoveAtoms(atomsRemoved);

	delete [] neighbors;
	delete [] displacement;
	delete [] distSQ;
	delete tempNList;

}
