# -*- makefile -*-
# Processing
# CXX is C++ files.
CFILES = $(filter %.c, $(ALLFILES))
CXXFILES = $(filter %.cpp, $(ALLFILES))

COBJECTS = $(CFILES:%.c=$(ARCH)/$(OBJDIR)/%.o)
CXXOBJECTS = $(CXXFILES:%.cpp=$(ARCH)/$(OBJDIR)/%.o)

# Dependencies get their own variable so we don't try to include
# an empty list of dependencies.
DFILES = $(CFILES:%.c=deps/%.d) $(CXXFILES:%.cpp=deps/%.d) 

CFLAGS += $(OPTS)

.PHONY: ALL clean $(SUBDIRS)

ECHO =@# set to @ to hide everything.  Unset to see it all.

# You have to make the library before the subdirectories or the
# $(ARCH)  $(OBJDIR) subdirectory won't exist.
ALL: $(LIBRARY) $(SUBDIRS)

# If you don't like this funny target, check out the CLEAN target below.
# $@ is the target subdirectory.
$(SUBDIRS):
	$(ECHO)(cd $@; $(MAKE) ALL)


# Ar only the updated objects. $? are files among $(OBJECTS) newer than $(LIBRARY).
$(LIBRARY): $(COBJECTS) $(CXXOBJECTS)
	$(ECHO)echo Archiving $?
	$(ECHO)-mkdir -p $(ARCH)
	$(ECHO)-mkdir -p $(ARCH)/$(OBJDIR)
	$(ECHO)-$(AR) -r $@ $?

deps/%.d: %.cpp
	$(ECHO)echo Making dependencies for $<
	$(ECHO)-mkdir -p deps
	$(ECHO)$(CPP) $(CFLAGS) $(INCLUDE) $< \
	| sed 's/\($*\)\.o[ :]*/$(ARCH)\/obj2D\/\1.o $(ARCH)\/obj3D\/\1.o deps\/$(notdir $@) : /g' > $@
#	$(ECHO)$(SHELL) -ec '$(CPP) $(CFLAGS) $(INCLUDE) $< \
#	| sed '\''s/\($*\)\.o[ :]*/\1.o $(notdir $@) : /g'\'' > $@'

deps/%.d: %.c
	$(ECHO)echo Making dependencies for $<
	$(ECHO)-mkdir -p deps
	$(ECHO)$(CPP) $(CFLAGS) $(INCLUDE) $< \
	| sed 's/\($*\)\.o[ :]*/$(ARCH)\/obj2D\/\1.o $(ARCH)\/obj3D\/\1.o deps\/$(notdir $@) : /g' > $@

ifneq ($(DFILES),)
include $(DFILES)
endif

$(ARCH)/$(OBJDIR)/%.o: %.cpp
	$(ECHO)echo Compiling $<
	$(ECHO)-mkdir -p $(ARCH)
	$(ECHO)-mkdir -p $(ARCH)/$(OBJDIR)
	$(ECHO)$(CXX) $(CFLAGS) -c $< $(INCLUDE) -o $@

$(ARCH)/$(OBJDIR)/%.o: %.c
	$(ECHO)echo Compiling $<
	$(ECHO)-mkdir -p $(ARCH)
	$(ECHO)-mkdir -p $(ARCH)/$(OBJDIR)
	$(ECHO)$(CC) $(CFLAGS) -c $< $(INCLUDE) -o $@

clean::
	-rm -rf deps
	-rm -rf $(ARCH)

clean::
	$(ECHO)$(foreach dir, $(SUBDIRS), (cd $(dir); $(MAKE) clean );)

distclean:: clean
	-rm -rf $(ARCH) deps

distclean::
	$(foreach dir, $(SUBDIRS), $(MAKE) -C $(dir) distclean; )