// PlotAtomsObserver.h: interface for the PlotAtomsObserver class.
//
//////////////////////////////////////////////////////////////////////



#ifndef PLOTATOMSOBSERVER_H
#define PLOTATOMSOBSERVER_H


#include <stdio.h> // for FILE
#include <map>

#include "ListOfAtomsObserver.h"
#include "ColorMethod.h"


//! Uses LASSPTools PlotAtoms to plot ListOfAtoms.
/*! The PlotAtomsObserver class is from LASSPTools.  It is 
  an observer of ListOfAtoms.

  It was written to use stdiostream, but that has been removed because
  it can conflict with standard iostream libraries.  It now uses
  regular stdlib library functions.
*/
class PlotAtomsObserver : public ListOfAtomsObserver  
{
//#ifndef SWIG
public:
	//! Constructor.
	PlotAtomsObserver(double xMin, double xMax, double yMin, double yMax, int radius, int defaultColor = 0, ColorMethod *colorMethod=0);
	//! Destructor.
	virtual ~PlotAtomsObserver();
	//! Called by the subject to tell us when atoms move.
	void Update(ListOfAtoms *atoms);
	//! Alternative syntax for Update().
	void Plot(ListOfAtoms *atoms);

	void SetColorScale ( double colorScale ) { this->colorScale = colorScale; colorScaleSetByUser = true; }
	void UnSetColorScale() { colorScaleSetByUser = false; }
	double GetColorScale() { return colorScale; }

	void SetColor(int leaf,int color)
		{
			colorMap.insert(std::make_pair(leaf,color));
		}
	
	void SetLengthScale ( double lengthScale ) { this->lengthScale = lengthScale; UpdateRotateScale(); }
	double GetLengthScale() { return lengthScale; }

	void SetRotationMatrix(double rotationMatrix[DIMENSION][DIMENSION]);

//#endif // SWIG

protected:
#ifndef SWIG
	void UpdateRotateScale();
	FILE*	        pipe_file; //!< Pipe to PlotAtoms.
	ColorMethod*    colorMethod;
	bool colorScaleSetByUser;
	double colorScale;
	std::map<int, int> colorMap; // maps leaf index to colour index
	int nColors;

	double lengthScale;
	double rotationMatrix[DIMENSION][DIMENSION];
	double rotateScaleMatrix[DIMENSION][DIMENSION];
	
#endif // SWIG
};

#endif // !defined(PLOTATOMSOBSERVER_H)

