// MDDoc.cpp : implementation of the CMDDoc class
//

#include "stdafx.h"
#include "MD.h"

#include "MDDoc.h"

#include "MDThread.h"
#include "Thread.h"
#include "DialogBuilder.h"	// My dialog builder class: JPS
//#include "vtkPropSource.h"
#include "MDComponentSelector.h"
#include "ChangeParameterMenu.h"
#include "TransformerMenu.h"
//#include "vtkSphereFilter.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMDDoc

IMPLEMENT_DYNCREATE(CMDDoc, CDocument)

BEGIN_MESSAGE_MAP(CMDDoc, CDocument)
	//{{AFX_MSG_MAP(CMDDoc)
	ON_COMMAND(ID_CONTROL_PAUSERESUME, OnControlPauseresume)
	//}}AFX_MSG_MAP

	// Digital Material
	ON_COMMAND_RANGE(0x9000,0x9fff,OnMenuSelect)
	ON_UPDATE_COMMAND_UI_RANGE (0x9000,0x9fff,OnUpdateComponentUI)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CMDDoc construction/destruction

CMDDoc::CMDDoc()
{
	// TODO: add one-time construction code here
//	vtkCoordSource = 0;
	componentSelector = 0;
	changeParameterMenu = 0;
	transformerMenu = 0;
	simBody = 0;
	simThread = 0;
}

CMDDoc::~CMDDoc()
{
	if (simThread) delete simThread;
	if (simBody) delete simBody;
	if (componentSelector) delete componentSelector;
	if (changeParameterMenu) delete changeParameterMenu;
	if (transformerMenu) delete transformerMenu;
}

Thread *const CMDDoc::GetSimThread()
{
	return simThread;
}

MDComponentSelector *const CMDDoc::GetComponentSelector()
{
	return componentSelector;
}

ChangeParameterMenu *const CMDDoc::GetChangeParameterMenu()
{
	return changeParameterMenu;
}

TransformerMenu *const CMDDoc::GetTransformerMenu()
{
	return transformerMenu;
}


BOOL CMDDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)
	if (componentSelector) delete componentSelector;
	if (changeParameterMenu) delete changeParameterMenu;
	if (transformerMenu) delete transformerMenu;
	if (simBody) delete simBody;
	if (simThread) delete simThread;

	simBody = new MDThread(&sim);
	simThread = new Thread(simBody);

	componentSelector = new MDComponentSelector(&sim,simThread);
	changeParameterMenu = new ChangeParameterMenu(&sim,simThread);
	transformerMenu = new TransformerMenu(&sim,simThread);

	int ret = simThread->Start();
	if (ret != Thread::SUCCESS) return FALSE;

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CMDDoc serialization

void CMDDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMDDoc diagnostics

#ifdef _DEBUG
void CMDDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CMDDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMDDoc commands

/* //XXX VTK
vtkPropSource* CMDDoc::GetVTKPropSource(void)
{
	if (vtkCoordSource==0) {
		vtkCoordSource = new vtkSphereFilter(&sim);
	}
	return vtkCoordSource;
}
*/

void CMDDoc::OnMenuSelect(UINT nID)
{
	componentSelector->OnSelect(nID);
	changeParameterMenu->OnSelect(nID);
	transformerMenu->OnSelect(nID);
}

void CMDDoc::OnUpdateComponentUI (CCmdUI* pCmdUI)
{
	componentSelector->OnUpdateComponentUI(pCmdUI);
	changeParameterMenu->OnUpdateComponentUI(pCmdUI);
	transformerMenu->OnUpdateComponentUI(pCmdUI);
}

void CMDDoc::OnControlPauseresume() 
{
	simThread->Toggle();
}


void CMDDoc::OnCloseDocument() 
{
	int ret = -1;
	if (simThread) {
		ret = simThread->Stop();
	}
	CDocument::OnCloseDocument();
}
