// ChangeParameterMenu.cpp: implementation of the ChangeParameterMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MD.h"
#include "MDDoc.h"
#include "ChangeParameterMenu.h"
#include "CommandIDManager.h"
#include "DialogBuilder.h"
#include "Thread.h"

#include <algorithm>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ChangeParameterMenu::ChangeParameterMenu(MDSimulation *sim, Thread *simThread)
{
	// "File Edit ..." menu at top of screen does not belong to the view, but to MainFrame
	// MainFrame, at least for this program, is the parent of the CMDView class

	pSim = sim;

	pSimThread = simThread;
	selectMenu = 0;
}

ChangeParameterMenu::~ChangeParameterMenu()
{
	CommandToComponentMap::const_iterator begin = menuMap.begin();
	CommandToComponentMap::const_iterator end = menuMap.end();
	CommandIDManager* idmanager = CommandIDManager::Instance();
	while (begin != end) {
		idmanager->ReleaseCommandID(begin->first);
		begin++;
	}
}

HMENU ChangeParameterMenu::GetMenu()
{
	if (selectMenu==0) selectMenu = this->MakeComponentList(pSim);
	return selectMenu;
}

// Responds to windows command
void ChangeParameterMenu::OnSelect(UINT idd)
{
	// Should catch if idd is not in menuMap.
	CommandToComponentMap::const_iterator found = menuMap.find(idd);
	if (found==menuMap.end()) return;
	ComponentIdentifier index = found->second;
	Introspective *pIntrospective = pSim->listOfLists[index.iBaseType]->GetCurrent();
	if (pIntrospective) {
		// Pause thread before changing sim!
		DialogBuilder params(pSim,pSimThread);
		params.DoModal_DM(pIntrospective);
	} // else show a None to Modify dialog.
}

// responds to windows update command
void ChangeParameterMenu::OnUpdateComponentUI (CCmdUI* pCmdUI)
{
	return;
//	pCmdUI->SetRadio(this->IsSelected(pCmdUI->m_nID));
}

// Private methods
// overrideable
bool ChangeParameterMenu::IsSelected(UINT idd) const
{
	return false;
}

HMENU ChangeParameterMenu::MakeComponentList(MDSimulation *sim)
{
	// empty the current list of IDs.
	this->menuMap.clear();

	int nLists = pSim->listOfLists.size();

	ComponentIdentifier index;

	CMenu wholeMenu;
	wholeMenu.CreateMenu();
	for (int componentList = 0; componentList < pSim->listOfLists.size(); componentList++)
	{
		DM_ComponentList * pList = pSim->listOfLists[componentList];
		if (!pList->SelectMany()) {
			index.iBaseType = componentList;
			
			int idd_current_user_number = CommandIDManager::Instance()->GetCommandID();
			this->menuMap[idd_current_user_number] = index;
			
			std::string sEllipseName = pList->GetName();
			sEllipseName.append(std::string("..."));
			wholeMenu.AppendMenu(MF_STRING|MF_UNCHECKED,
				idd_current_user_number,_T(sEllipseName.c_str()) );
		}
	}
	return wholeMenu.Detach();
}

