/* IDResourceManager.cpp
 * Drew Dolgert 19 Feb 2000
 * For a class like this, the implementation is the fun part.
 * This class stores available IDs as a list of pairs of (min,max).
 * When you request one, it increments the first min value.  When you
 * return one, it has to see if the ID returned goes into an
 * existing pair or is its own pair (ret,ret) to insert in the list.
 * The algorithm was written with singly-linked lists in mind.
 * It would be fun to make this algorithm prettier.
 */
#include "IDResourceManager.h"
#include <algorithm>


IDResourceManager::IDResourceManager(int IDMin, int IDMax)
{
	this->IDMin = IDMin; this->IDMax = IDMax;
	freeIDs.push_back(intpair(IDMin,IDMax));
}

IDResourceManager::~IDResourceManager()
{
}


int IDResourceManager::GetID()
{
	if (freeIDs.empty()) return -1;
	IDList::iterator start = freeIDs.begin();
	int ret = start->first;
	if (start->first==start->second) {
		freeIDs.erase(start);
	} else
		start->first++;
	return ret;
}


void IDResourceManager::ReleaseID(int ID)
{
	if ((ID<IDMax) || (ID>IDMin)) return;
	// This finds the range for which the returned ID is less than one past the highest
	// number.  That means that either the ID adds to the top of the range, or it adds
	// to the bottom of the range, or it is its own range below.
	IDList::iterator closeRange = std::find_if(freeIDs.begin(),freeIDs.end(),CouldJoinRange(ID));
	if (closeRange==freeIDs.end()) return;  // not ours to begin with.
	if (ID<closeRange->first-1) {
		freeIDs.insert(closeRange,intpair(ID,ID));
	} else if (ID==closeRange->first-1) {
		closeRange->first -= 1;
	} else if (ID==closeRange->second+1) {
		// XXX I wanted to say nextRange = closeRange+1.
		IDList::iterator lastRange = closeRange;
		closeRange++;
		IDList::iterator nextRange = closeRange;
		closeRange = lastRange;
		if (nextRange->first==ID+1) {
			closeRange->second = nextRange->second;
			freeIDs.erase(nextRange);
		} else {
			closeRange->second +=1;
		}
	} else {
		// trying to return an ID in the range.  Redundant, but not a good sign.
	}
}


