// MDComponentSelector.cpp: implementation of the MDComponentSelector class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MD.h"
#include "MDComponentSelector.h"
#include "MDDoc.h"
#include "CommandIDManager.h"
#include <algorithm>
#include "Thread.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MDComponentSelector::MDComponentSelector(MDSimulation *sim, Thread *simThread)
{
	// "File Edit ..." menu at top of screen does not belong to the view, but to MainFrame
	// MainFrame, at least for this program, is the parent of the CMDView class

	pSim = sim;

	pSimThread = simThread;
	selectMenu = 0;
}

MDComponentSelector::~MDComponentSelector()
{
	CommandToComponentMap::const_iterator begin = menuMap.begin();
	CommandToComponentMap::const_iterator end = menuMap.end();
	CommandIDManager* idmanager = CommandIDManager::Instance();
	while (begin != end) {
		idmanager->ReleaseCommandID(begin->first);
		begin++;
	}
}

HMENU MDComponentSelector::GetMenu()
{
	if (selectMenu==0) selectMenu = this->MakeComponentList(pSim);
	return selectMenu;
}

// Responds to windows command
void MDComponentSelector::OnSelect(UINT idd)
{
	// Should catch if idd is not in menuMap.
	CommandToComponentMap::const_iterator found = menuMap.find(idd);
	if (found==menuMap.end()) return;
	ComponentIdentifier index = found->second;
	// Pause thread before changing sim!
	int ret = pSimThread->Pause();	// Wait for thread to pause
	bool wasRunning = !(ret==Thread::ALREADY_PAUSED);
	DM_ComponentList *pList = pSim->listOfLists[index.iBaseType];
	if (pList->IsCurrentNumber(index.iType))
		pList->UnsetCurrentNumber(index.iType);
	else
		pList->SetCurrentNumber(index.iType);
	if (wasRunning) pSimThread->Resume();	// Resume thread
}

// responds to windows update command
void MDComponentSelector::OnUpdateComponentUI (CCmdUI* pCmdUI)
{
	CommandToComponentMap::const_iterator found = menuMap.find(pCmdUI->m_nID);
	if (found==menuMap.end()) return;
	DM_ComponentList *pList = pSim->listOfLists[found->second.iBaseType];
	if (pList->SelectMany())
		pCmdUI->SetCheck(this->IsSelected(found->second));
	else
		pCmdUI->SetRadio(this->IsSelected(found->second));
}

// Private methods
// overrideable
bool MDComponentSelector::IsSelected(const ComponentIdentifier& index) const
{
	return (pSim->listOfLists[index.iBaseType]->IsCurrentNumber(index.iType));
}

HMENU MDComponentSelector::MakeComponentList(MDSimulation *sim)
{
	// empty the current list of IDs.
	this->menuMap.clear();

	int nLists = pSim->listOfLists.size();

	ComponentIdentifier index;

	CMenu wholeMenu;
	wholeMenu.CreateMenu();
	for (int componentList = 0; componentList < pSim->listOfLists.size(); componentList++)
	{
		DM_ComponentList * pList = pSim->listOfLists[componentList];
		index.iBaseType = componentList;

		CMenu menu;
		menu.CreateMenu();

		for (int item=0; item < pList->size(); item++)
		{
			index.iType = item;
			int idd_current_user_number = CommandIDManager::Instance()->GetCommandID();
			this->menuMap[idd_current_user_number] = index;

			menu.AppendMenu(MF_STRING|MF_UNCHECKED,
				idd_current_user_number,_T(pList->at(item)->GetName().c_str()) );
		}

		if (pList->size()==0) {
			menu.AppendMenu(MF_STRING|MF_UNCHECKED|MF_DISABLED,
				0,_T("None") );
		}

		wholeMenu.AppendMenu(MF_POPUP, (UINT) menu.Detach(),_T( pList->GetName().c_str() ) );
	}
	return wholeMenu.Detach();
}

