from DigitalMaterial.PX.Geometry.Common \
     import Points, Vectors, GeoFmt, GeoFmtIO, qhull, PointSetSource
from DigitalMaterial.PX.Tools.Utils.SystemCall import SystemCall
from DigitalMaterial.PX.Tools.Utils.FileIO import *
import Numeric
import string
import tempfile
import UserDict, UserList
Vector = Vectors.Vector

qh_INFINITE_VERTEX = 0

# -------------------------------------------------------------------------

class VoronoiInterface(qhull.qhull_GeomElement):
    
    def __init__(self, inputSites=(), vertices=[], hyperplane=None):
        self.inputSites = inputSites
        self.vertices = vertices
        self.hyperplane = hyperplane

    def ID(self):
        return self.inputSites

    def __repr__(self):
        s = 'VoronoiInterface( %s ): ' % (self.ID(),)
        s = s + 'vertices = %s' % (self.vertices)
        return s

    def __cmp__(self, other):
        status = 0
        status = status + cmp(self.inputSites, other.inputSites)
        status = status + cmp(self.vertices,  other.vertices)
        return status


class VoronoiEdge(qhull.qhull_GeomElement):
    
    def __init__(self, vertices=[]):
        self.vertices = vertices

    def ID(self):
        return tuple(self.vertices)

    def __repr__(self):
        s = 'VoronoiEdge( %s ): ' % (self.ID(),)
        s = s + 'vertices = %s' % self.vertices
        return s


class VoronoiRegion(qhull.qhull_GeomElement):

    def __init__(self, inputSite=None, vertices=[]):
        self.inputSite = inputSite
        self.vertices  = vertices

    def ID(self):
        return self.inputSite
        
    def __repr__(self):
        s = 'VoronoiRegion( %s ): ' % (self.ID(),)
        s = s + 'vertices = %s' % (self.vertices)
        return s

    def __cmp__(self, other):
        status = 0
        status = status + cmp(self.inputSite, other.inputSite)
        status = status + cmp(self.vertices,  other.vertices)
        return status
        


# -------------------------------------------------------------------------

class QhullVoronoiTesselation:

    def __repr__(self):
        s = 'QhullVoronoiTesselation (%d regions, %d interfaces, %d vertices)'\
            % (self.NumberOfRegions(), self.NumberOfInterfaces(), \
               self.NumberOfVertices())
        return s

    def NumberOfVertices(self):
        return len(self.VertexSet)

    def Vertex(self, index):
        return self.VertexSet[index]

    def NumberOfRegions(self):
        return len(self.RegionSet)      

    def Regions(self):
        return self.RegionSet.values()

    def region(self, index):
        return self.RegionSet[index]

    def NumberOfInterfaces(self):
        return len(self.InterfaceSet)

    def Interfaces(self):
        return self.InterfaceSet.values()

    def interface(self, index):
        return self.InterfaceSet[index]

    def NumberOfEdges(self):
        return len(self.EdgeSet)

    def Edges(self):
        return self.EdgeSet.values()

    def edge(self, index):
        return self.EdgeSet[index]



# ------------------------------------------------------------------------
# Builders
# ------------------------------------------------------------------------

class VoronoiBuilder:

    def __init__(self):
        pass

    def GetVoronoiDiagram(self):
        pass


class UnclippedQhullVoronoiBuilder(VoronoiBuilder):

    def __init__(self, pointSet, clippingGeometry=None):
        self.voronoi = QhullVoronoiTesselation()

        self.voronoi.qhull_options = 'v o Fv'
        self.voronoi.qhull = qhull.qhullGenerator(self.voronoi.qhull_options)
        self.voronoi.qhull.SetPointSet(pointSet)
        self.voronoi.qhull.generate()
        self.voronoi.infile = self.voronoi.qhull.outfile

        self.clippingGeometry = clippingGeometry

        f = open(self.voronoi.infile, 'r')
        self.__parse_o(f)
        self.__parse_Fv_v(f)
        f.close()


    def __parse_o(self, fp):

        self.voronoi.dim = parse_atoi(fp)
        self.voronoi._nvertices, self.voronoi._nregions, ignore = \
                                 parse_map_atoi(fp)

        self.voronoi.VertexSet = qhull.ElementDict({})
        self.voronoi.RegionSet = qhull.ElementDict({})

        if self.clippingGeometry is None:
            for n in range(self.voronoi._nvertices):
                p = Vector(parse_map_atof(fp))
                self.voronoi.VertexSet[n] = p

            for n in range(self.voronoi._nregions):
                line = parse_map_atoi(fp)
                nvertices = line[0]
                vertices = line[1:]
                reg = VoronoiRegion(n, vertices)
                self.voronoi.RegionSet[n] = reg

        else:
            for n in range(self.voronoi._nvertices):
                p = Vector(parse_map_atof(fp))
                if self.clippingGeometry.isInside(p):
                    self.voronoi.VertexSet[n] = p

            for n in range(self.voronoi._nregions):
                line = parse_map_atoi(fp)
                nvertices = line[0]
                vertices = line[1:]
                isInside = 1
                if 0 in map(self.voronoi.VertexSet.has_key, vertices):
                    continue
                else:
                    reg = VoronoiRegion(n, vertices)
                    self.voronoi.RegionSet[n] = reg



    def __parse_Fv_v(self, fp):

        self.voronoi._ninterfaces = parse_atoi(fp)
        self.voronoi.InterfaceSet = qhull.ElementDict({})

        if self.clippingGeometry is None:

            for n in range(self.voronoi._ninterfaces):
                line = string.split(fp.readline())
                nindices   = string.atoi(line[0])
                inputSites = tuple(map(string.atoi,line[1:3]))
                vertices   = map(string.atoi,line[3:])
                inter = VoronoiInterface(inputSites, vertices)
                self.voronoi.InterfaceSet[inputSites] = inter

        else:

            for n in range(self.voronoi._ninterfaces):
                line = string.split(fp.readline())
                nindices   = string.atoi(line[0])
                inputSites = tuple(map(string.atoi,line[1:3]))
                vertices   = map(string.atoi,line[3:])
                if 0 in map(self.voronoi.VertexSet.has_key, vertices):
                    continue
                else:
                    inter = VoronoiInterface(inputSites, vertices)
                    self.voronoi.InterfaceSet[inputSites] = inter


    def GetVoronoiDiagram(self):
        return self.voronoi
    

class ClippedQhullVoronoiBuilder(VoronoiBuilder):

    def __init__(self, input_voronoi, clippingGeometry):

        self.numRegions = len(input_voronoi.qhull.pointSet)
        extended_pointSet = \
                          GetExtendedPointSet(input_voronoi, clippingGeometry)
        
        builder = UnclippedQhullVoronoiBuilder(extended_pointSet, clippingGeometry)
        voronoi = builder.GetVoronoiDiagram()
        self.voronoi = self._cleanup(voronoi, clippingGeometry)

    def _cleanup(self, voronoi, clippingGeometry):
        #RemoveExternalVoronoiElements(voronoi, self.numRegions, 
        #clippingGeometry)
        PackVoronoiVertices(voronoi)
        OrientInterfaceNormals(voronoi)
        return voronoi


    def GetVoronoiDiagram(self):
        return self.voronoi
    

class QhullVoronoiBuilder(VoronoiBuilder):

    def __init__(self, pointSetSource=None, componentGeometry=None):

        self.pointSetSource = pointSetSource
        self.pointSet = self.pointSetSource.GetOutput()
        self.componentGeometry = componentGeometry

        builder = UnclippedQhullVoronoiBuilder(self.pointSet)
        voronoi = builder.GetVoronoiDiagram()
        builder = ClippedQhullVoronoiBuilder(voronoi, self.componentGeometry)
        self.voronoi = builder.GetVoronoiDiagram()


    def GetVoronoiDiagram(self):
        return self.voronoi
    




# utility functions

def GetExtendedPointSet(voronoi, clippingGeometry=None):

    if clippingGeometry is None: return voronoi
        
    externalVertices  = qhull.PointsOutside(voronoi.VertexSet, 
                                            clippingGeometry)
    boundaryRegionIDs = FindBoundaryRegionIDs(voronoi, externalVertices)
    clippingFacets    = FindClippingFacets(voronoi, boundaryRegionIDs,
                                           clippingGeometry)
    reflectedPointSet = FindReflectedPointSet(voronoi, boundaryRegionIDs, 
                                              clippingFacets, clippingGeometry)
    extended_pointSet = Points.PointSet(voronoi.qhull.pointSet.data)
    extended_pointSet.extend(reflectedPointSet)

    return extended_pointSet


def FindBoundaryRegionIDs(voronoi, extV):
    boundaryIDs = []
    for f in voronoi.RegionSet.values():
        iS = f.inputSite
        isboundaryID = 0
        for v in f.vertices:
            if extV.count(v) > 0: 
                isboundaryID=1
                break
        if isboundaryID:
            if boundaryIDs.count(iS) == 0:
                boundaryIDs.append(iS)

    return boundaryIDs


def FindClippingFacets(voronoi, boundaryIDs, clippingGeometry):

    clippingFacets = {}
    for s in boundaryIDs:
        clippingFacets[s] = range(clippingGeometry.NumberOfFacets())
    return clippingFacets


def FindReflectedPointSet(voronoi, regionIDs, clippingFacets, 
                          clippingGeometry):
    reflectedPoints = Points.PointSet([])
    np = 0
    for nv in regionIDs:
        for nhyp in clippingFacets[nv]:
            hyp = clippingGeometry.normalSet[nhyp]
            ip = voronoi.qhull.pointSet[nv]
            rp = qhull.pointReflect(ip, hyp)
            reflectedPoints.append(rp)
            np = np+1
    return reflectedPoints



def FindProjectedPointSet(voronoi, regionIDs, clippingGeometry):
    projectedPoints = Points.PointSet([])
    np = 0
    for nv in regionIDs:
        for nhyp in range(clippingGeometry.NumberOfFacets()):
            hyp = clippingGeometry.normalSet[nhyp]
            ip = voronoi.qhull.pointSet[nv]
            pp = qhull.pointProject(ip, hyp)
            projectedPoints.append(pp)
            np = np+1
    return projectedPoints


def LocateVoronoiRegion(voronoi, point):
    """return which Voronoi region the specified point is in"""

    distances = []
    for np in range(len(voronoi.qhull.pointSet)):
        diff = point-voronoi.qhull.pointSet[np]
        dist = diff.length()
        distances.append(dist)

    min_distance = min(distances)
    reg_index = distances.index(min_distance)
    return reg_index

def RegionCenter(voronoi, regionID):
    vertices = voronoi.RegionSet[regionID].vertices
    sum = Vector([0.,0.,0.])
    for v in vertices:
        sum = sum + voronoi.VertexSet[v]
    center = sum/len(vertices)
    return (center)


def _swapInterfaceID(voronoi, ID):
    inter = voronoi.InterfaceSet[ID]
    swap = (ID[1], ID[0])
    del voronoi.InterfaceSet[ID]
    voronoi.InterfaceSet[swap] = inter

def OrientInterfaceNormals(voronoi):
    """define interface normals so that loop described by interface vertices
    points (via right-hand-rule) toward the region associated with the 
    first index in the interface ID; ID indices are swapped if necessary
    to achieve this consistency"""

    for ID, inter in voronoi.InterfaceSet.items():
        v0,v1,v2 = inter.vertices[0:3]
        V01 = voronoi.VertexSet[v1]-voronoi.VertexSet[v0]
        V02 = voronoi.VertexSet[v2]-voronoi.VertexSet[v0]        
        crossProduct = V01.cross(V02)

        rID = ID[0]
        vecToCenter = voronoi.qhull.pointSet[rID] - voronoi.VertexSet[v0]
        dotProduct = crossProduct*vecToCenter
        if dotProduct < 0:
            _swapInterfaceID(voronoi, ID)

    if 0:
        for ID, inter in voronoi.InterfaceSet.items():
            v0,v1,v2 = inter.vertices[0:3]
            V01 = voronoi.VertexSet[v1]-voronoi.VertexSet[v0]
            V02 = voronoi.VertexSet[v2]-voronoi.VertexSet[v0]        
            normal = V01.cross(V02)
            dotProduct = normal*voronoi.VertexSet[v0]
            if dotProduct < 0:
                inter.vertices.reverse()

def RemoveExternalVoronoiElements(voronoi, numRegions, clippingGeometry):

    removalVertices = qhull.PointsOutside(voronoi.VertexSet, clippingGeometry)

    for reg in voronoi.RegionSet.values():
        iS = reg.inputSite
        if iS >= numRegions:
            del voronoi.RegionSet[iS]

    for inter in voronoi.InterfaceSet.values():
        iS0 = inter.inputSites[0]
        iS1 = inter.inputSites[1]
        if iS0 >= numRegions and iS1 >= numRegions:
            del voronoi.InterfaceSet[(iS0, iS1)]

    for nv in removalVertices:
        del voronoi.VertexSet[nv]

    if 0:
        for ID, p in voronoi.qhull.pointSet.items():
            if ID > numRegions:
                del voronoi.qhull.pointSet[ID]


def PackVoronoiVertices(voronoi):

    import copy
    vmap = qhull.ElementDict({})
    vertices = qhull.ElementDict({})

    np = 0
    for ID, v in voronoi.VertexSet.items():
        vmap[ID] = np
        vertices[np] = v
        np = np + 1
    voronoi.VertexSet = qhull.ElementDict(vertices)

#    for e in voronoi.EdgeSet.values():
#       for nv in range(len(e.vertices)):
#           e.vertices[nv] = vmap[e.vertices[nv]]

    for i in voronoi.InterfaceSet.values():
        for nv in range(len(i.vertices)):
            i.vertices[nv] = vmap[i.vertices[nv]]

    for r in voronoi.RegionSet.values():
        for nv in range(len(r.vertices)):
            r.vertices[nv] = vmap[r.vertices[nv]]



def VoronoiDiagramToPolyhedralTesselation(voronoi):

    import Tesselation
    polyt = Tesselation.PolyhedralTesselation()

    polyt._vertices = {}
    for ID, v in voronoi.VertexSet.items():
        polyt._vertices[ID] = v

    polyt._interfaces = {}
    for ID, inter in voronoi.InterfaceSet.items():
        polyt._interfaces[ID] = \
             Tesselation.PolyhedralInterface(polyt, ID, inter.vertices)

    polyt._regions = {}
    interfaceLists = {}
    for iID in voronoi.InterfaceSet.keys():
        i0, i1 = iID
        if not interfaceLists.has_key(i0):
            interfaceLists[i0] = []
        if not interfaceLists.has_key(i1):
            interfaceLists[i1] = []
        interfaceLists[i0].append(iID)
        interfaceLists[i1].append(iID)
    for rID in voronoi.RegionSet.keys():
        polyt._regions[rID] = \
             Tesselation.PolyhedralRegion(polyt, rID, interfaceLists[rID])

    return polyt


if __name__ == '__main__':

    from DigitalMaterial.PX.Geometry.Models import qhullModels

    #source = PointSetSource.RboxGenerator()
    source = PointSetSource.UniformPointSetGenerator()
    #comp = qhullModels.UnitCube(3)
    comp = qhullModels.UnitDiamond(3)
    comp = qhullModels.HexagonalCylinder(3)
    source.SetComponentGeometry(comp)
    source.SetNumberOfPoints(10)
    builder = QhullVoronoiBuilder(pointSetSource=source,
                                  componentGeometry=comp)
    voronoi = builder.GetVoronoiDiagram()



    
