"""Module containing various functions to interrogate a polycrystal"""

from DigitalMaterial.PX.Structures.Polycrystals \
     import Polycrystal, PolycrystalSource
from DigitalMaterial.PX.Tools.Viz import PXView
import Gnuplot

# -----------------------------------------------------------------------


def demo(ngrains=100):
    px = PolycrystalSource.BuildUniformGrainsWithUniformTexture(ngrains)
    px.Store()

    viewer = PXView.PolycrystalViewer()
    viewer.SetPolycrystal(px)
    viewer.GetSurfaceGrains()
    viewer.Display()

    px.ComputeAngularMODF()
    modf = px.GetAngularMODF()
    plotter = Gnuplot.Gnuplot()
    plotter('set data style linespoints')
    plotter.plot((modf,))

    return px, viewer, plotter

def cutdemo(ngrains=100):
    px = PolycrystalSource.BuildUniformGrainsWithUniformTexture(ngrains)
    px.Store()

    viewer = PXView.PolycrystalViewer()
    viewer.SetPolycrystal(px)

    import Cut

    knife = Cut.PolycrystalCutter(px)
    knife.Cut((0.,1.,0.), 0.)
    viewer.GetPlanarCut(knife)
    viewer.Display()

    plotter = None

    return px, viewer, plotter

# ------------------------------------------------------------------------

# miscellaneous interrogation filters

def SelectGrains(polycrystal, selectionFunction):
    selected = []
    for gID in polycrystal.GrainIDs():
        if selectionFunction(polycrystal, gID):
            selected.append(gID)
    return selected

def SelectGrainBoundaries(polycrystal, selectionFunction):
    selected = []
    for iID in polycrystal.GrainBoundaryIDs():
        if selectionFunction(polycrystal, iID):
            selected.append(iID)
    return selected

def BuildGrainSet(polycrystal, selectionFunction):
    indexSet = SelectGrains(polycrystal, selectionFunction)
    return GrainSet(polycrystal, indexSet)

def BuildInterfaceSet(polycrystal, selectionFunction):
    indexSet = SelectGrainBoundaries(polycrystal, selectionFunction)
    return InterfaceSet(polycrystal, indexSet)

# ------------------------------------------------------------------------

if __name__ == '__main__':

    import sys
    if len(sys.argv) > 1:
        ngrains = sys.argv[1]
    else:
        ngrains = 100

    px, viewer, plotter = cutdemo(ngrains)
