#
# 
#
from DigitalMaterial.PX.Structures.Polycrystals import PolycrystalSource
from DigitalMaterial.PX.Geometry.Common.GeomShape3D import Line, Polygon
import Numeric

def PolyhedronRayShoot(poly, rays, hBin = 10):
    """Given polyhedra and shooting rays, the routine return a set of 
       non-duplicate segment lengths (grain sizes) and histogram"""
    tessellation = poly.geometry
    intrinsic_dimension = tessellation.dimension()
    if intrinsic_dimension != 3:
        raise ValueError, "geoPoly3D only support for dimension=3"
    interscPointSet = []
    for ray in rays:
        interscPoints = []
        for f in tessellation._interfaces.values():
            polyPoints = []
            for i in range(len(f.vertices)):
                polyPoints.append(tessellation._vertices[f.vertices[i]])
            pg = Polygon(polyPoints)
            point = pg.lineIntersection(ray.geoLine)
            if (point is not None and point not in interscPoints):
                interscPoints.append(point)
        #sort according to head and tail of the line
        if (interscPoints != []):
            interscPoints = IntersecPointsSort(interscPoints, ray)
        interscPointSet.append(interscPoints)
    sizeSet = GrainSizes(interscPointSet)       
    #return sizeSet, SizeDistribution(sizeSet, hBin)
    return GrainSizes(interscPointSet)

def GrainSizeDistribution(sizeSet, hBin = 10):
    sdist = []
    from Scientific.Statistics import Histogram
    for sizes in sizeSet:
        for s in sizes:
            sdist.append(s)
    gdist = Histogram.Histogram(sdist, hBin)
    return gdist      

def GrainSizes(interscPointSet):
    sizeSet = []
    for pts in interscPointSet:
        segments = []
        for i in range(len(pts)-1):
            segments.append((pts[i+1]-pts[i]).length())        
        sizeSet.append(segments)
    return sizeSet    
                
def IntersecPointsSort(interscPoints, scanline):
    """Sort the intersection points according to the direction 
       of the scanline"""
    head = interscPoints[0]
    tail = interscPoints[0]
    for vec in interscPoints:
        if (Numeric.add.reduce(vec.array-head.array) >= 0):
            head = vec
        if (Numeric.add.reduce(vec.array-tail.array) <= 0):
            tail = vec
    if ((head-tail)*(scanline.head-scanline.tail) > 0): rorg = head
    else: rorg = tail
    for i in range(len(interscPoints)):
        interscPoints[i] = interscPoints[i] - rorg
    interscPoints.sort( lambda x, y: cmp(x.length(), y.length()) )      
    for i in range(len(interscPoints)):
        interscPoints[i] = interscPoints[i] + rorg    
    return interscPoints
       
def test(ngrains, rays):
    poly = PolycrystalSource.test(ngrains)
    return PolyhedronRayShoot(poly, rays, 50)
    
if __name__ == '__main__':
    from Scientific.Geometry.VectorModule import Vector
#    rays = [Line(Vector(0,0,0), Vector(1,1,1)), \
#            Line(Vector(0,0,0), Vector(1,0,1))]
    rays = [Line(Vector(0,0,0), Vector(1,1,1))]
    sizeSet = test(20, rays)
    i = 0
    for sizes in sizeSet:
        print "Grain sizes are: ray ", str(i)
        print sizes  
        i = i + 1
