#
# class to perform virtual EBSP scanning
#
from Scientific.Geometry.VectorModule import Vector

class VirtualEBSPScan:
    def __init__(self, poly):
        self.poly = poly
        
    def setRegularGridPoints(self, ex = Vector(0.,0,0), ey = Vector(0.,0,0), \
            origon = Vector(0.0, 0, 0), xspacing = 0.0, \
            nx = 0, yspacing = 0.0, ny = 0):
        self.pointSet = []
        if (ex == ey):
            raise ValueError, "Two base vectors cannot be the same"
        for j in range(ny+1):
            for i in range(nx+1):
                self.pointSet.append(origon + \
                    i * xspacing * ex + j * yspacing * ey)

    def ComputeOrientations(self):
        tesselation = poly.geometry
        self.orientations = []
        for pt in self.pointSet:
            ori = None
            for rID in tesselation._regions.keys():
                r = tesselation.Region(rID)
                if (r.isInside(pt)): ori = poly.GetOrientation(rID)
            self.orientations.append(ori)         
    
    def GetOrientation(self, index):
        return self.pointSet[index], self.orientations[index]      
    
    def GetOrientations(self):
        return self.orientations      
    

if __name__ == '__main__':
    from DigitalMaterial.PX.Structures.Polycrystals import PolycrystalSource
    
    ngrains = 10
    poly = PolycrystalSource.test(ngrains)
    vEBSP = VirtualEBSPScan(poly)
    vEBSP.setRegularGridPoints(ex = Vector(1.,0,0), ey = Vector(0.,1.,0), \
        origon = Vector(-0.501, -0.501, 0), xspacing = 0.1, \
        nx = 11, yspacing = 0.1, ny = 11)
    vEBSP.ComputeOrientations()
        
