import types, string

def numeval(str, typestring):
    """return eval(str) if typestring represents numerical type, i.e.,
    'Int', 'Float' or 'Complex'; otherwise, simply return str"""
    if typestring == 'Int' or \
       typestring == 'Float' or \
       typestring == 'Complex':
        return eval(str)
    else:
        return str

def TypenameToType(typename):
    """convert type base-string of Python type to associated Python type
    object; e.g., 'Int' -> types.IntType, 'Float' -> types.FloatType,
    'Instance' -> types.InstanceType"""
    if type(typename) != types.StringType:
        raise TypeError, "input type is neither a type object nor a string"
    typename = typename + 'Type'
    if typename not in dir(types): 
        raise ValueError, "input type is not defined in builtin types"
    return eval('types.'+typename)

def TypeToTypename(pytype):
    """convert Python type object to associated type base-string;
    e.g., types.IntType -> 'Int', types.FloatType -> 'Float',
    types.InstanceType -> 'Instance'"""
    if type(pytype) != types.TypeType:
        raise TypeError, "input type is neither a type object nor a string"
    typename = filter(lambda s, pytype=pytype: \
                      pytype==eval('types.'+s), dir(types))[0]
    typename = typename[:string.find(typename, 'Type', 1)]
    return typename


class Signature:

    def __init__(self, pyType, *argTypes, **kwTypes):
        self._type = pyType
        self._argTypes = argTypes
        self._kwTypes  = kwTypes

    def GetType(self): return self._type
    def GetArgTypes(self): return self._argTypes
    def GetKeywordTypes(self): return self._kwTypes

    def __repr__(self):
        if self._type != 'Function' and self._type != 'Class':
            return self._type
        else:
            return '(%s, %s, %s)' % (self._type, self._argTypes, self._kwTypes)

    def GetPythonType(self):
        return _lookupTypeFrom
    

class Variable:

    def __init__(self, name, value, signature, mode='rw'):
        self._name  = name
        self._value = value
        if type(signature) is types.StringType:
            self._signature = Signature(signature)
        else:
            self._signature = signature
        self._mode = mode

    def GetName(self):      return self._name
    def GetValue(self):     return self._value
    def GetSignature(self): return self._signature
    def GetType(self):      return self._signature.GetType()
    def GetMode(self):      return self._mode

    def SetValue(self, value):
        if 'w' in self._mode:
            self._value = value
        else:
            raise ValueError, "write permissions not allowed"
    def SetSignature(self, type): self._signature = type

    def __repr__(self):
        return '(%s, %s, %s)' % (`self._name`, self._value, self._signature)



class VariableTable:

    def __init__(self):
        self.records = {}

    def __getitem__(self, key):
        return self.records[key]
    
    def __setitem__(self, key, value):
        # transform (value, type) tuple to Variable instance if necessary
        if type(value) == types.TupleType: 
            value = apply(Variable, value)
        self.records[key] = value
    
    def __getattr__(self, attr):
        """['clear', 'copy', 'get', 'has_key', 'items', 'keys', 'update',
        'values']"""
        return getattr(self.records, attr)

    def __repr__(self):
        return `self.records`


class OrderedVariableTable:

    def __init__(self):
        self._records = {}
        self._keys = []

    def __getitem__(self, key):
        if type(key) == types.IntType:
            return self._records[self._keys[key]]
        else:
            return self._records[key]
    
    def __setitem__(self, key, value):
        # transform (value, type) tuple to Variable instance if necessary
        if type(value) == types.TupleType: 
            value = apply(Variable, value)
        self._records[key] = value
        self._keys.append(key)
    
    def items(self):
        r = []
        for k in self._keys:
            r.append((k,self._records[k]))
        return r

    def values(self):
        r = []
        for k in self._keys:
            r.append(self._records[k])
        return r

    def keys(self):
        return self._keys

    def __getattr__(self, attr):
        """['clear', 'copy', 'get', 'has_key', 'items', 'keys', 'update',
        'values']"""
        return getattr(self._records, attr)

    def __repr__(self):
        return `self._records`
    

class VariableTableReader:

    def __init__(self):
        self._filename = None
        self._table = None

    def SetFileName(self, filename):
        self._filename = filename

    def GetFileName(self):
        return self._filename

    def _read(self):
        self._table = VariableTable()
        import sys
        if self._filename is None:
            self._input = sys.stdin
        else:
            self._input = open(self._filename, 'r')

        lines = map(string.split, self._input.readlines())
        for line in lines:
            if len(line) < 3: break
            name = line[0]
            value_str = line[1]
            signature_args = tuple(line[2:])
            value = numeval(value_str, signature_args[0])
            self._table[name] = \
                 Variable(name, value, apply(Signature, signature_args))

    def GetTable(self):
        if self._table is None: self._read()
        return self._table


import Tkinter
#if 'tk' not in dir(): tk = Tkinter.Tk()

class LabeledEntry(Tkinter.Frame):

    def __init__(self, master, name='', type='', variable=None, mode=''):
        Tkinter.Frame.__init__(self, master)
        self.label = Tkinter.Label(self, text=name)
        self.label.pack(side=Tkinter.LEFT, fill=Tkinter.X)
        self.tkstr = Tkinter.StringVar()
        self.tkstr.set(variable)
        self.entry = Tkinter.Entry(self, textvariable=self.tkstr)
        self.entry.pack(side=Tkinter.RIGHT, fill=Tkinter.X)        
        if 'w' not in mode:
            self.label['fg'] = 'gray60'
            self.entry['fg'] = 'gray60'
        self.pack()
        

class TkVariableTable(Tkinter.Frame):

    def __init__(self, root, variableTable=None):
        #self._toplevel = Tkinter.Toplevel(root)
        self._toplevel = root
        Tkinter.Frame.__init__(self, self._toplevel)
        self._variableTable = variableTable
        self._configurationCopy = variableTable.copy()
        self.frames = {}
        self._buildWidgets()

    def __getitem__(self, key):
        return self._variableTable[key].GetValue()

    def _buildWidgets(self):
        self._buildEntryTable()

    def SetVariableTable(self, variableTable):
        self._variableTable = variableTable
        self._configurationCopy = variableTable.copy()
        
    def _buildEntryTable(self):
        if self._variableTable is None:
            return 
        
        for name, value in self._variableTable.items():
            self.frames[name] = \
                  LabeledEntry(self, name=name,
                               type=value.GetSignature().GetType(),
                               variable=value.GetValue(), mode=value.GetMode())
            self.frames[name].pack(anchor=Tkinter.NW, pady=2)
        self.pack(anchor=Tkinter.NW, pady=5)

    def _resetEntryTable(self):
        if self._variableTable is None:
            return 
        
        for name, value in self._variableTable.items():
            self.frames[name].tkstr.set(value.GetValue())
        self.pack(anchor=Tkinter.NW, pady=5)

    def _setFromEntryTable(self):
        for name, value in self._variableTable.items():
            strvalue = self.frames[name].entry.get()
            sig  = value.GetSignature()
            newvalue = numeval(strvalue, sig.GetType())
            self._configurationCopy[name].SetValue(newvalue)
        self._variableTable = self._configurationCopy.copy()
        

class TkRadioTable(Tkinter.Frame):

    def __init__(self, root, variableTable=None, labeltext=''):
        #self._toplevel = Tkinter.Toplevel(root)
        self._toplevel = root
        Tkinter.Frame.__init__(self, self._toplevel)
        self._variableTable = variableTable
        self._configurationCopy = variableTable.copy()
        self._labeltext = labeltext
        self.frames = {}
        self._buildWidgets()

    def _buildWidgets(self):
        self._buildLabel()
        self._buildRadioTable()

    def SetVariableTable(self, variableTable):
        self._variableTable = variableTable
        self._configurationCopy = variableTable.copy()
        
    def GetCurrentValue(self):
        return self.tkvariable.get()

    def _buildLabel(self):
        self.label = Tkinter.Label(self, text=self._labeltext)
        #self.label.pack(side=Tkinter.TOP, fill=Tkinter.X, anchor=Tkinter.NW)
        self.label.pack(anchor=Tkinter.NW)

    def _buildRadioTable(self):
        if self._variableTable is None:
            return 
        
        self.tkvariable = Tkinter.IntVar()
        defaultvalue = None
        for var in self._variableTable.values():
            Tkinter.Radiobutton(self, text=var.GetName(), value=var.GetValue(),
                                variable=self.tkvariable).pack(anchor=Tkinter.W)
            if defaultvalue is None: defaultvalue = var.GetValue()

        self.tkvariable.set(defaultvalue)
        self.pack(anchor=Tkinter.NW, pady=5)

        

