import VTK
import string
from math import fabs


class PlotAtoms:

    def __init__(self, filename, pointFilter=None):
        self.filename = filename
        self.pointFilter = pointFilter
        self.readPoints(self.filename)
        self.images = []

    def saveImage(self):
        import tempfile
        image_filename = tempfile.mktemp('.ppm')
        self.renWin.SetFileName(image_filename)
        self.renWin.SaveImageAsPPM()
        print "Saving image to %s" % image_filename
        self.images.append(image_filename)
        return

    def readPoints(self, filename):
        f = open(filename, 'r')
        self.inputPoints = VTK.vtkPoints()
        n = 0
        line = string.split(f.readline())
        while len(line) != 0:
            p = map(string.atof, line)
            if self.pointFilter is None:
                self.inputPoints.InsertPoint(n, p[0], p[1], p[2])
                n = n+1
            elif self.pointFilter(n,p):
                self.inputPoints.InsertPoint(n, p[0], p[1], p[2])
                n = n+1
            line = string.split(f.readline())
	
    def display(self,
                radius=1.5, phiResolution=10, thetaResolution=10,
                LevelOfDetail=0):

        inputData = VTK.vtkPolyData()
        inputData.SetPoints(self.inputPoints)

        balls = VTK.vtkSphereSource()
        balls.SetRadius(radius)
        balls.SetPhiResolution(phiResolution)
        balls.SetThetaResolution(thetaResolution)

        glyphPoints = VTK.vtkGlyph3D()
        glyphPoints.SetInput(inputData)
        glyphPoints.SetSource(balls.GetOutput())
        glyphMapper = VTK.vtkPolyDataMapper()
        glyphMapper.SetInput(glyphPoints.GetOutput())

        if LevelOfDetail:
            glyph = VTK.vtkLODActor()
        else:
            glyph = VTK.vtkActor()
        glyph.SetMapper(glyphMapper)
        glyph.GetProperty().SetDiffuseColor(0.9, 0.2, 0.2)
        glyph.GetProperty().SetSpecular(0.3)
        glyph.GetProperty().SetSpecularPower(30)

        self.ren = VTK.vtkRenderer()
        self.renWin = VTK.vtkRenderWindow()
        self.renWin.AddRenderer(self.ren)
        self.renWin.SetSize(500,500)
        self.ren.AddActor(glyph)

        self.iren = VTK.vtkRenderWindowInteractor()
        self.iren.SetRenderWindow(self.renWin)
        self.iren.SetUserMethod(self.saveImage)

        self.iren.Initialize()
        self.iren.Start()


def distanceCutoff(n, p):
    import math
    dist = math.sqrt(p[0]*p[0] + p[1]*p[1] + p[2]*p[2])
    val = dist < 40.
    return val

def identity(n, p):
    return 1

def test():
    plot = PlotAtoms('triplejct.dat', pointFilter=distanceCutoff)
    plot.display(LevelOfDetail=1)

if __name__ == '__main__':
    test()
    
