import VTK, vtkRenderWidget, Tkinter, VTKGraphics
from DigitalMaterial.PX.Structures.Polycrystals.Polycrystal import *
from DigitalMaterial.PX.Tools.Utils import VariableConfiguration
VC = VariableConfiguration
import string, Numeric
import Gnuplot
        

CellGeneratorTable = VC.OrderedVariableTable()
CellGeneratorTable[0] = \
                VC.Variable('Voronoi', 0, VC.Signature('Int'))
                
PointSetGeneratorTable = VC.OrderedVariableTable()
PointSetGeneratorTable[0] = \
                VC.Variable('Uniformly Random (rbox)', 0, VC.Signature('Int'))
PointSetGeneratorTable[1] = \
                VC.Variable('Nonuniformly Random', 1, VC.Signature('Int'))

OrientationGeneratorTable = VC.OrderedVariableTable()
OrientationGeneratorTable[0] = \
                VC.Variable('Uniform ODF', 0, VC.Signature('Int'))
OrientationGeneratorTable[1] = \
                VC.Variable('Textured ODF', 1, VC.Signature('Int'))

VTable1 = VC.OrderedVariableTable()
VTable1['Number Of Grains'] = \
                VC.Variable('NumberOfGrains', 0, VC.Signature('Int'))
VTable1['Input File for Nonuniform Point Generator'] = \
            VC.Variable('NonuniformInputFile', '',
                        VC.Signature('String'))
VTable1['ODF'] = \
            VC.Variable('ODF', '',
                        VC.Signature('String'))

VTable2 = VC.OrderedVariableTable()
VTable2['Number Of Bins For MODF'] = \
                VC.Variable('NumberOfMODFBins', 0, VC.Signature('Int'))
VTable2['Number of Line Scans for Grain Size Distribution'] = \
            VC.Variable('NumberOfLineScans', 100,
                        VC.Signature('Int'))
VTable2['Cutting Plane'] = \
            VC.Variable('CutPlane', 100,
                        VC.Signature('Instance'))


def Gnuplot2D(title='', *datasets):
    plotter = Gnuplot.Gnuplot()
    plotter.title(title)
    plotter('set data style linespoints')
    plotter.plot(datasets)
    return plotter


class PolycrystalGUI:

    def __init__(self, title='Polycrystal Inquisitor',
                 vtkwindowSize=(400,400)):
        self.title = title
        self.Initialize(vtkwindowSize)
        self.createWidgets()

    def Initialize(self, windowSize):
        self.master = Tkinter.Tk()
        self.master.title(self.title)
        self.root = Tkinter.Toplevel(self.master)
        self.tkwidget = VTK.vtkTkRenderWidget (self.root,
                                               width=windowSize[0],
                                               height=windowSize[1])
        self.tkwidget.pack (expand='true',fill='both')
        self.renwin = self.tkwidget.GetRenderWindow()
        self.ren = None
        self.actors  = []

    def AddActor(self, actor):
        self.actors.append(actor)

    def SaveImage(self, filename=None):
        if filename is None:
            import tempfile
            filename = tempfile.mktemp('.ppm')
            print "Saving image to %s" % filename
        self.renwin.SetFileName(filename)
        self.renwin.SaveImageAsPPM()
        return

    def Show(self):
        if self.ren is not None: self.renwin.RemoveRenderer(self.ren)
        self.ren = VTK.vtkRenderer()
        self.ren.TwoSidedLightingOn()
        self.renwin.AddRenderer(self.ren)
        for a in self.actors:
            self.ren.AddActor(a)
        self.renwin.Render()

    def createWidgets(self):
        Tkinter.Label(text="Polycrystal Generation", fg="blue").pack(anchor=Tkinter.NW,
                                                          pady=10)
        self._tk_Cell_rtable = VC.TkRadioTable(self.master,
                                               CellGeneratorTable,
                                               'Cell Generator')
        self._tk_Point_rtable = VC.TkRadioTable(self.master,
                                                PointSetGeneratorTable,
                                                'Point Set Generator')
        self._tk_Ori_rtable = VC.TkRadioTable(self.master,
                                                OrientationGeneratorTable,
                                                'Orientation Generator')
        self._tkvtable1 = VC.TkVariableTable(self.master, VTable1)
        self._buildGenerateButton()
        Tkinter.Label(text="Polycrystal Interrogation", fg="blue").pack(anchor=Tkinter.NW,                                                          pady=10)
        self._tkvtable2 = VC.TkVariableTable(self.master, VTable2)
        self._buildMisorientationButton()
        self._buildGrainSizeButton()
        self._buildCutButton()

    def _buildGenerateButton(self):
        self._generateButton = Tkinter.Button(self.master, text='Generate',
                                              command=self._Generate)
        self._generateButton.pack(anchor=Tkinter.NW, padx=5, pady=2)
                                            

    def _buildMisorientationButton(self):
        self._misorientationButton = \
             Tkinter.Button(self.master, text='Compute MODF',
                                              command=self._ComputeMODF)
        self._misorientationButton.pack(anchor=Tkinter.NW, padx=5, pady=2)
                                            
    def _buildGrainSizeButton(self):
        self._grainSizeButton = \
             Tkinter.Button(self.master, text='Compute Grain Sizes Along Line',
                                              command=self._ComputeGrainSizeDistribution)
        self._grainSizeButton.pack(anchor=Tkinter.NW, padx=5, pady=2)
                                            
    def _buildCutButton(self):
        self._grainSizeButton = \
             Tkinter.Button(self.master, text='Cut Along Specified Plane',
                                              command=self._Cut)
        self._grainSizeButton.pack(anchor=Tkinter.NW, padx=5, pady=2)
                                            

    def _Generate(self):
        self._CellGenerator = \
               CellGeneratorTable[self._tk_Cell_rtable.GetCurrentValue()]
        self._PointSetGenerator = \
               PointSetGeneratorTable[self._tk_Point_rtable.GetCurrentValue()]
        self._OrientationGenerator = \
               OrientationGeneratorTable[self._tk_Ori_rtable.GetCurrentValue()]
        self._tkvtable1._setFromEntryTable()

        self._generator = PolycrystalGenerator()

        if self._CellGenerator.GetName() == 'Voronoi':
            self._polycrystalSource = \
                  PolycrystalSource.VoronoiPolycrystalSource()

        if self._PointSetGenerator.GetName() == 'Uniformly Random (rbox)':
            self._pointSource = PointSetSource.RboxGenerator()
            numGrains = self._tkvtable1['Number Of Grains']
            if numGrains == 0:
                raise ValueError, "Must specify non-zero number of grains"
            self._pointSource.SetNumberOfPoints(numGrains)
        elif self._PointSetGenerator.GetName() == 'Nonuniformly Random':
            self._pointSource = PointSetSource.NonuniformPointSetGenerator()
            file = self._tkvtable1['Input File for Nonuniform Point Generator']
            #self._pointSource = PointSetSource.PointSetReader()
            self._pointSource.SetInputFile(file)
        
        if self._OrientationGenerator.GetName() == 'Uniform ODF':
            self._orientationSource = Orientations.RandomOrientationSource()
        elif self._OrientationGenerator.GetName() == 'Textured ODF':
            input = self._tkvtable1['ODF']
            self._orientationSource = Orientations.BungeOrientationReader(input)

        self._generator.SetPointSource(self._pointSource)
        self._generator.SetPolycrystalSource(self._polycrystalSource)
        self._generator.SetOrientationSource(self._orientationSource)

        self.polycrystal = self._generator.GetPolycrystal()

        from DigitalMaterial.PX.Structures.Polycrystals import VTKPolycrystal
        self.polycrystal.vtkpolycrystal = \
                   VTKPolycrystal.VTKPolycrystal(self.polycrystal)
        self._mapper = PolycrystalMapper(self.polycrystal)
        self._mapper.MapToActors(displayFunction='GetSurfaceGrains')
        for a in self._mapper.actors:
            self.AddActor(a)
        self.Show()
            

    def _Cut(self):
        self._tkvtable2._setFromEntryTable()
        plane = eval(self._tkvtable2['Cutting Plane'])
        from DigitalMaterial.PX.Tools.Polycrystals import Cut
        knife = Cut.PolycrystalCutter(self.polycrystal)
        normal = plane[0:3]
        offset = plane[3]
        knife.Cut(normal, offset)
        self._mapper = PolycrystalMapper(self.polycrystal)
        self._mapper.MapToActors(displayFunction='GetPlanarCut', args=(knife,))
        self.actors = []
        for a in self._mapper.actors:
            self.AddActor(a)
        self.Show()
            


    def _ComputeMODF(self):
        if not self.polycrystal.grainAttributes.has_key('orientations'):
            print "No orientations defined on sample."
        else:
            self._tkvtable2._setFromEntryTable()
            nbins = self._tkvtable2['Number Of Bins For MODF']
            miso_distribution = self._generator.GetMisorientationDistribution(nbins)
            self._misoplotter = Gnuplot2D('MODF', miso_distribution)
            self._tkvtable2._resetEntryTable()
            #import Gnuplot
            #self._misoplotter = Gnuplot.Gnuplot()
            #self._misoplotter('set linestyle 1')
            #self._misoplotter.plot((miso_distribution,))

    def _ComputeGrainSizeDistribution(self):
        self._tkvtable2._setFromEntryTable()
        nlines = self._tkvtable2['Number of Line Scans for Grain Size Distribution']
        plane = eval(self._tkvtable2['Cutting Plane'])
        sizes = []
        writer = PolycrystalIO.SanjayPolycrystalWriter('tmp')
        writer.write(self.polycrystal)

        for nl in range(nlines):
            import whrandom
            ppoint = float(nl)/float(nlines)
            point = [plane[3]] # x direction
            for d in range(2):
                point.append(whrandom.uniform(-0.5,0.5))
            vec = [0.]
            for d in range(2):
                vec.append(whrandom.uniform(-1.,1.))
            line = [point, vec]

            linestring = '[%f,%f,%f;%f,%f,%f]' % \
                         (line[0][0], line[0][1], line[0][2], \
                          line[1][0], line[1][1], line[1][2])
            os.system('rm -f lens')
            os.system("lgsize3d tmp_v.dat tmp_i.dat '" + linestring + "' 892734")
            try:
                f = open('lens', 'r')
                print f
                lines = f.readlines()
                print map(string.atof, lines)
                f.close()
                print lines
                sizes.extend(map(string.atof, lines))
            except:
                pass

        sizes = Numeric.array(sizes)
        print sizes
        from Scientific.Statistics import Histogram
        self.size_distribution = Histogram.Histogram(sizes, 10)
        self._grainsizeplotter = Gnuplot2D('Grain size distribution',
                                           self.size_distribution)
        
    def _ComputeRandomGrainSizeDistribution(self):
        self._tkvtable2._setFromEntryTable()
        nlines = self._tkvtable2['Number of Line Scans for Grain Size Distribution']
        sizes = []
        for nl in range(nlines):
            import whrandom
            point = [0.,0.,0.]
#            for d in range(3):
#                point.append(whrandom.uniform(-0.5,0.5))
            vec = []
            for d in range(3):
                vec.append(whrandom.uniform(-1.,1.))
            line = [point, vec]

            linestring = '[%f,%f,%f;%f,%f,%f]' % \
                         (line[0][0], line[0][1], line[0][2], \
                          line[1][0], line[1][1], line[1][2])
            writer = PolycrystalIO.SanjayPolycrystalWriter('tmp')
            writer.write(self.polycrystal)
            os.system("lgsize3d tmp_v.dat tmp_i.dat '" + linestring + "' 892734")
            f = open('lens', 'r')
            import string, Numeric
            lines = f.readlines()
            print map(string.atof, lines)
            f.close()
            os.system('rm -f lens')
            sizes.extend(map(string.atof, lines))
        sizes = Numeric.array(sizes)
        print sizes
        from Scientific.Statistics import Histogram
        size_distribution = Histogram.Histogram(sizes, 10)
        self._grainsizeplotter = Gnuplot2D('Grain size distribution',
                                           size_distribution)
        


class OrientationColoring:

    def __init__(self, rRange=[0,1.0], gRange=[0,1.0], bRange=[0,1.0],
                 resolution=8):
        self.rRange = rRange
        self.gRange = gRange
        self.bRange = bRange
        self.resolution = resolution
        self.maxIndex = resolution**3

    def GetLookupTable(self):
        lookupTable = VTK.vtkLookupTable()
        resolution = self.resolution
        lookupTable.SetNumberOfColors(resolution**3)
        lookupTable.Build()

        index = 0
        res = self.resolution
        rMin = self.rRange[0]
        rSlope = (self.rRange[1]-rMin)/(res-1)
        gMin = self.gRange[0]
        gSlope = (self.gRange[1]-gMin)/(res-1)
        bMin = self.bRange[0]
        bSlope = (self.bRange[1]-bMin)/(res-1)
        for red in xrange( res ):
            for green in xrange( res ):
                for blue in xrange( res ):
                    rVal = rMin+rSlope*red
                    gVal = gMin+gSlope*green
                    bVal = bMin+bSlope*blue
                    lookupTable.SetTableValue(index, rVal, gVal, bVal, 1.0)
                    index = index + 1

        #lookupTable.SetTableRange(0, index-1)
        return lookupTable

    def __call__(self, orientation):
        import math
        try:
            res = self.resolution
            red = int(res*orientation[0]/(2.*math.pi))
            green = int(res*(orientation[1]+math.pi)/(2*math.pi))
            blue = int(res*orientation[2]/(2*math.pi))
            val = (red*res + green)*res + blue
            return (float(val)/self.maxIndex)
        except ValueError:
            return 0.


class PolycrystalAttributeMap:

    def __init__(self, polycrystal, attributeType, attribute, colorFunction):
        self.polycrystal = polycrystal
        self.attributeType = attributeType
        self.attribute = attribute
        self.colorFunction = colorFunction
        if self.attributeType is not 'grain' and \
           self.attributeType is not 'grainBoundary':
            raise ValueError, "attributeType not correctly set"

    def __call__(self, index):
        if self.attributeType == 'grain':
            color = self.colorFunction(self.polycrystal.GetGrainAttribute(self.attribute, index))
        elif self.attributeType == 'grainBoundary':
            color = \
              self.colorFunction(self.polycrystal.GetGrainBoundaryAttribute(self.attribute,index))
                                                                       
        return color


class PolycrystalMapper:

    def __init__(self, polycrystal):

        self.polycrystal = polycrystal
        self.vtkpolycrystal = self.polycrystal.vtkpolycrystal
        self.GBPolyData = self.vtkpolycrystal.GBPolyData
        self.GBPolyData.Update()
        self.grainIndices = \
          self.GBPolyData.GetCellData().GetFieldData().GetArray('grainIndices')
        
        if self.polycrystal.grainAttributes.has_key('orientations'):
            self.attributeMap = PolycrystalAttributeMap(self.polycrystal,
                                                        'grain',
                                                        'orientations',
                                                       OrientationColoring())
        else:
            self.attributeMap = PolycrystalAttributeMap(self.polycrystal,
                                                        'grain',
                                                        'colors',
                                                       RandomColoring())

    def GetSurfaceGrains(self, attrMap=None):

        from DigitalMaterial.PX.Geometry.Tesselation import Tesselation
        interfaces = []
        tess = self.polycrystal.geometry
        for i in range(6):
            interfaces.extend(Tesselation.GetInterfacesOnComponentSurface(tess,
                                                                          i))
        surfaceGBPolyData = self._buildGBPolyDataFromGBIndexSet(interfaces)

        return self._colorGBPolyData(surfaceGBPolyData, attrMap)

    def GetGrains(self, indexSet=None, attrMap=None):
        if indexSet is None:
            return self.GetSurfaceGrains(attrMap)
        else:
            GBIndexSet = []
            for gID in indexSet:
                GBIndexSet.extend(\
                    self.polycrystal.GetGrain(gID).GrainBoundaries())
            return self.GetGrainBoundaries(GBIndexSet, attrMap)

    def GetGrainBoundaries(self, indexSet=None, attrMap=None):
        if indexSet is None:
            return self._colorGBPolyData(self.GBPolyData, attrMap)
        else:
            GBPolyDataSubset = self._buildGBPolyDataFromGBIndexSet(indexSet)
            return self._colorGBPolyData(GBPolyDataSubset, attrMap)
        
    def GetPlanarCut(self, knife, attrMap=None):

        planeNormal, planeOffset = knife.planeNormal, knife.planeOffset

        self.clipPlane = VTK.vtkPlane()
        self.clipPlane.SetOrigin(self.GBPolyData.GetCenter())
        apply(self.clipPlane.SetNormal, planeNormal)

        clipper = VTK.vtkClipPolyData()
        clipper.SetInput(self.GBPolyData)
        clipper.SetValue(planeOffset)
        clipper.SetClipFunction(self.clipPlane)
        #        clipper.GenerateClippedOutputOn()
        clipData = clipper.GetOutput()
        clipData.Update()
        clipGBData = self._colorGBPolyData(clipData, attrMap)
        clipGBData.Update()

        cutGBData = self._colorGBPolyData(knife.GetSurfaceGrains(), attrMap)
        cutGBData.Update()
        
        writer=VTK.vtkPolyDataWriter()
        writer.SetFileName('clipData.vtk')
        writer.SetInput(clipData)
        writer.Write()

        return clipGBData, cutGBData


    def _buildGBPolyDataFromGBIndexSet(self, GBIndexSet):

        vtkcells = []
        gIs = VTK.vtkIntArray()
        gIs.SetNumberOfComponents(2)
#        gIs.SetNumberOfTuples(0)

        for i in GBIndexSet:
            vtkcells.append(self.vtkpolycrystal._VTKGBMap[i])

        sampledPolyData = VTK.vtkPolyData()
        sampledPolyData.SetPoints(self.GBPolyData.GetPoints())
        sampledCells = VTK.vtkCellArray()

        for c in vtkcells:
            sampledCells.InsertNextCell(self.GBPolyData.GetCell(c))
            gIs.InsertNextValue(self.grainIndices.GetComponent(c,0))
            gIs.InsertNextValue(self.grainIndices.GetComponent(c,1))

        gIField = VTK.vtkFieldData()
        gIField.SetArray(0, gIs)
        gIField.SetArrayName(0, 'grainIndices')
        sampledPolyData.SetPolys(sampledCells)
        sampledPolyData.GetCellData().SetFieldData(gIField)
        return sampledPolyData


    def _colorGBPolyData(self, gbpolydata, attrMap):
        """attach scalar cell data suitable for coloring to input
        vtkPolyData instance; details of coloring are encapsulated in
        the AttributeMap instance passed to _colorGBPolyData"""

        if attrMap is None:
            attrMap = self.attributeMap

	GBFrontColors = VTK.vtkScalars()
	GBFrontColors.Allocate(gbpolydata.GetNumberOfPolys(), 10)

	GBBackColors = VTK.vtkScalars()
	GBBackColors.Allocate(gbpolydata.GetNumberOfPolys(), 10)

        grainIndices = \
            gbpolydata.GetCellData().GetFieldData().GetArray('grainIndices')

        for n in range(gbpolydata.GetNumberOfPolys()):
            i0 =  int(grainIndices.GetComponent(n,0))
            i1 =  int(grainIndices.GetComponent(n,1))
            try:
                color = attrMap(i0)
            except:
                color = 0
            GBFrontColors.InsertScalar(n, color)
            try:
                color = attrMap(i1)
            except:
                color = 0
            GBBackColors.InsertScalar(n, color)

        gbpolydata.GetCellData().SetScalars(GBFrontColors)
        return gbpolydata

    def MapToActors(self, displayFunction='GetSurfaceGrains', args=()):

        function = eval('self.' + displayFunction)
        datasets = apply(function, args)
        import types
        if type(datasets) is types.TupleType:
            datasets = list(datasets)
        elif type(datasets) is not types.ListType:
            datasets = [datasets]
        self.actors = []
        for d in datasets:
            self.actors.append(\
               VTKGraphics.mapDataToActor(d, mapperType = \
                  VTK.vtkPolyDataMapper,lookupTable = \
                    self.attributeMap.colorFunction.GetLookupTable()))


if __name__ == '__main__':

    gui = PolycrystalGUI()


