#include <stdio.h>
#include <math.h>
#include "SquareGridField.h"
#include "SimplePeriodicBC.h"
#include "GnuPlotter.h"
#include "AdvectionEquation.h"
#include "ExplicitEuler.h"
#include "FieldCollection.h"

#define NOOFFIELDS 1

int main(void);

class ThisFieldInitializer : public virtual FieldInitializer {
	public:
		Real FieldValue(const Point &x) const {
			int i;
			Real y;
			Real yy;
			for (i=0, y=0.0; i<DIMENSION; i++) {
				//y+=sin(2*yy*yy*yy*yy.0*x(i));
				//y*=x(i)*(1.0-x(i));
			}
			y=x(0)*(1-x(0)); y*=y*y*y;
			yy=x(1)*(1-x(1)); yy*=yy*yy*yy;
			y*=yy*4096.0*8.0*2.5;
			//	if (x(1)>0.4 && x(1)<0.9) {
			//		//y=((0.6-x(1))*(x(1)-0.9)*(0.6-x(1))*(x(1)-0.9));
			//		y=1.0;
			//	}
			//	else y=0.0;
			y=(x(0)>0.5)?0.0:1.0;
			return(y);
			}
};

int main(void) 
{
	int i, nn;
	Index boexle(32);
	Real hh=1.0e-8;
	Point velo;
	Point box(1.0);
	GnuSurfacePlot splotter1, splotter2;
	SquareGridField phi;
	FieldCollection Fields;
	SimplePeriodicBC spbc(box);
	ExplicitEuler euler;

	cout << "SetNumberOfFields(" << NOOFFIELDS << ")=" << phi.SetNumberOfFields(NOOFFIELDS) << endl;
	cout << "SetLogicalSize(" << boexle << ")=" << phi.SetLogicalSize(boexle) << endl;
	cout << "SetBoundaryCondition(spbc)=" << phi.SetBoundaryCondition(&spbc)  << endl;

	Fields.AddField(&phi);
	Fields[0].Attach(&splotter1);
	splotter1.SetTitle("Field(t)");

	ThisFieldInitializer sini;

	AdvectionEquation adveq;
	cout << "Enter velocity" << endl;
	cin >> velo;
	adveq.SetVelocity(velo);

	euler.SetWorkSpace(Fields);
	FieldCollection &WorkSpace=euler.GetWorkSpace();
	WorkSpace[0].Attach(&splotter2);
	splotter2.SetTitle("Euler Workspace");
	euler.SetStepSize(hh);
	euler.SetRightHandSide(&adveq);
	
	for (i=0; i<NOOFFIELDS; i++) cout << "Initialize(sini, " << i << ")=" << Fields[0].Initialize(sini, i) << endl;

	Fields[0].Notify(); WorkSpace[0].Notify();

	cout << "Enter initial step size" << endl;
	for (cin >> hh ; hh>=0.0; cin >> hh) {
		euler.SetStepSize(hh);
		for (nn=0; nn>=0; cin >> nn) {
			for (i=0; i<nn; i++) euler.Move(Fields);
			Fields[0].Notify(); WorkSpace[0].Notify();
			cout << "Enter number of steps (old=" << nn << ") -- negative for exit" << endl;
		}
		cout << "Enter stepsize (old=" << hh << ") -- negative for exit" << endl;
	}

	Fields[0].Detach(&splotter1);
	WorkSpace[0].Detach(&splotter2);

	return(1);
}
