#include <stdio.h>
#include <math.h>
#include "SquareGridField.h"
#include "SimplePeriodicBC.h"
#include "GnuPlotter.h"
#include "ConservedOP.h"
#include "ExplicitEuler.h"
#include "FieldCollection.h"

#define NOOFFIELDS 1

int main(void);

class ThisFieldInitializer : public virtual FieldInitializer {
	public:
		Real FieldValue(const Point &x) const {
			int i;
			Real y, yy;
			for (i=0, y=0.0; i<DIMENSION; i++) {
				//y+=sin(2*yy*yy*yy*yy.0*x(i));
				//y*=x(i)*(1.0-x(i));
			}
			y=x(0)*(1-x(0)); y*=y*y*y;
			yy=x(1)*(1-x(1)); yy*=yy*yy*yy;
			y*=yy*4096.0*8.0*2.5;
			return(y);
		}
};


int main(void) 
{
	int i;
	char ccc;
	Index boexle(32);
	Real hh=1.0e-8;
	Real wp;
	Point box(1.0);
	GnuSurfacePlot splotter1, splotter2, splotter3;
	SimplePeriodicBC spbc(box);
	ThisFieldInitializer sini;
	SquareGridField phi;
	FieldCollection Fields;

	cout << "SetNumberOfFields(" << NOOFFIELDS << ")=" << phi.SetNumberOfFields(NOOFFIELDS) << endl;
	cout << "SetLogicalSize(" << boexle << ")=" << phi.SetLogicalSize(boexle) << endl;
	cout << "SetBoundaryCondition(spbc)=" << phi.SetBoundaryCondition(&spbc)  << endl;
	for (i=0; i<NOOFFIELDS; i++) cout << "Initialize(sini, " << i << ")=" << phi.Initialize(sini, i) << endl;
	Fields.AddField(&phi);

	Fields[0].Attach(&splotter1); 
	splotter1.SetTitle("Field(t)");

	ConservedOP cop;
	cout << "Enter Diffusivity*Width " << endl;
	cin >> wp;
	cop.SetDiffusivity(wp);
	cout << "Enter Diffusivity*Potential " << endl;
	cin >> wp;
	cout << "cop.SetPotential(" << wp << ")=" << cop.SetPotential(wp) << endl;

	cout << "cop.SetWorkSpace(" << &Fields[0] << ")=" << cop.SetWorkSpace(Fields) << endl;
	FieldCollection &COPSpace=cop.GetWorkSpace();
	//COPSpace[0].Attach(&splotter3);
	splotter3.SetTitle("COP Work Space");
	
	ExplicitEuler euler;
	euler.SetStepSize(hh);
	euler.SetRightHandSide(&cop);
	euler.SetWorkSpace(Fields);
	FieldCollection &EulerSpace=euler.GetWorkSpace();
	//EulerSpace[0].Attach(&splotter2);
	splotter2.SetTitle("Euler Work Space");

	for ( ; hh>=0.0; cin >> hh) {
		euler.SetStepSize(hh);
		for (ccc=' '; ccc!='x'; ccc=getchar()) {
			for (i=0; i<100; i++) euler.Move(Fields);
			Fields[0].Notify(); EulerSpace[0].Notify(); COPSpace[0].Notify(); 
			cout << "Hit <ENTER> for next 100 time steps or <x> for exit" << endl;
			fflush(stdin); getchar();
		}
		cout << "Enter stepsize (old=" << hh << ") -- negative for exit" << endl;
	}

	Fields[0].Detach(&splotter1);
	EulerSpace[0].Detach(&splotter2);
	COPSpace[0].Detach(&splotter3);

	return(1);
}
