#include <stdio.h>
#include <math.h>
#include "SquareGridField.h"
#include "SimplePeriodicBC.h"
#include "FieldInitializer.h"
#include "GnuPlotter.h"
#include "DiffusionEquation.h"
#include "ExplicitEuler.h"
#include "FieldCollection.h"

#define NOOFFIELDS 1

int main(void);

class ThisFieldInitializer : public virtual FieldInitializer {
	public:
		Point heli;
		Real FieldValue(const Point & x) const {
			int i;
			Real y;
			//cout << "myfunc( ";
			for (i=0, y=0.0; i<DIMENSION; i++) {
				//y+=sin(2.0*x(i));
				//y*=x(i)*(1.0-x(i));
				//cout << x(i) << " ";
			}
			//cout << ")=" << y << endl;
			y=x(0)*x(0)*(1-x(0))*(1-x(0));
			y+=x(1)*x(1)*(1-x(1))*(1-x(1));
			y+=x(0)*heli(0)+x(1)*heli(1);
			return(y);
		}
};


int main(void) 
{
	int i;
	char ccc;
	Index boexle(40);
	boexle[0]=55;
	//for (i=0; i<DIMENSION; i++) boexle[i]*=(i+3);

	ThisFieldInitializer sini;
	Point box(1.0);
	Point heli;
	heli[0]=1.0; heli[1]=1.0;
	sini.heli=heli;
	SimplePeriodicBC spbc(box);
	spbc.SetHelicity(heli);

	DiffusionEquation deq;
	deq.SetDiffusivity(1.0);

	ExplicitEuler euler;
	euler.SetStepSize(0.0001);
	euler.SetRightHandSide(&deq);

	GnuContourPlot plotter1, plotter2;
	plotter1.SetContourLevels(20);
	plotter2.SetContourLevels(20);
	GnuSurfacePlot splotter1, splotter2, splotterdiff;
	SquareGridField phi;
	SquareGridField dxphi;
	//PartialDerivative diff;
	//AsymmetricDerivative diff;
	AverageTwo diff;
	//diff.SetBackward();
	MakeEqual Equal;
	SubtractField Sub;
	FieldCollection Fields;
	
	//cout << "spbc.AllocCopy()=" << spbc.AllocCopy() << " != " << &spbc << endl;

	cout << "SetNumberOfFields(" << NOOFFIELDS << ")=" << phi.SetNumberOfFields(NOOFFIELDS) << endl;
	cout << "SetLogicalSize(" << boexle << ")=" <<  phi.SetLogicalSize(boexle) << endl;
	cout << "SetBoundaryCondition(spbc)=" << phi.SetBoundaryCondition(&spbc) << " ";
	cout << phi.GetBoundaryCondition() << endl;
	cout << "SetBoundaryCondition(spbc)=" << dxphi.SetBoundaryCondition(&spbc) << " ";
	cout << dxphi.GetBoundaryCondition() << endl;
	for (i=0; i<NOOFFIELDS; i++) cout << "Initialize(sini, " << i << ")=" << phi.Initialize(sini, i) << endl;
	Fields.AddField(&phi);
	
	cout << "Fields[0].Attach(&plotter2)" << endl;
	Fields[0].Attach(&plotter2);
	plotter2.SetTitle("Field");
	cout << "Fields[0].Attach(&splotter2)" << endl;
	Fields[0].Attach(&splotter2);
	cout << "SetTitle('Field')";
	splotter2.SetTitle("Field");
	Fields[0].Notify();
	dxphi.Attach(&splotterdiff);
	splotterdiff.SetTitle("Derivative");

	euler.SetWorkSpace(Fields);
	FieldCollection &WorkSpace=euler.GetWorkSpace();

	cout << "WorkSpace[0].Attach(&plotter1)" << endl;
	WorkSpace[0].Attach(&plotter1);
	plotter1.SetTitle("WorkSpace[0]");
	cout << "WorkSpace[0].Attach(&splotter1)" << endl;
	WorkSpace[0].Attach(&splotter1);
	splotter1.SetTitle("WorkSpace[0]");
	WorkSpace[0].Notify(); 
	Equal[dxphi][Fields[0]];
	cout << "phi.GetBoundaryCondition()=" << phi.GetBoundaryCondition() << " Helicity=" << phi.GetBoundaryCondition()->GetHelicity() << endl;
	cout << "dxphi.GetBoundaryCondition()=" << dxphi.GetBoundaryCondition() << " Helicity=" << dxphi.GetBoundaryCondition()->GetHelicity() << endl;
	diff(1)[dxphi];
	Sub[dxphi][Fields[0]];
	cout << "phi.GetBoundaryCondition()=" << phi.GetBoundaryCondition() << " Helicity=" << phi.GetBoundaryCondition()->GetHelicity() << endl;
	cout << "dxphi.GetBoundaryCondition()=" << dxphi.GetBoundaryCondition() << " Helicity=" << dxphi.GetBoundaryCondition()->GetHelicity() << endl;
	dxphi.Notify();
	
	getchar();


	for (ccc=' '; ccc!='x'; ccc=getchar()) {
		for (i=0; i<10; i++) euler.Move(Fields);
		WorkSpace[0].Notify();
		Fields[0].Notify();
		Equal[dxphi][Fields[0]];
		cout << "phi.GetBoundaryCondition()=" << phi.GetBoundaryCondition() << " Helicity=" << phi.GetBoundaryCondition()->GetHelicity() << endl;
		cout << "dxphi.GetBoundaryCondition()=" << dxphi.GetBoundaryCondition() << " Helicity=" << dxphi.GetBoundaryCondition()->GetHelicity() << endl;
		diff(1)[dxphi];
		Sub[dxphi][Fields[0]];
		cout << "phi.GetBoundaryCondition()=" << phi.GetBoundaryCondition() << " Helicity=" << phi.GetBoundaryCondition()->GetHelicity() << endl;
		cout << "dxphi.GetBoundaryCondition()=" << dxphi.GetBoundaryCondition() << " Helicity=" << dxphi.GetBoundaryCondition()->GetHelicity() << endl;
		cout << "WorkSpace[0].GetBoundaryCondition()=" << WorkSpace[0].GetBoundaryCondition() << " Helicity=" << WorkSpace[0].GetBoundaryCondition()->GetHelicity() << endl;
		dxphi.Notify();
	}

	cout << "Fields[0].Detach(&plotter2)" << endl;
	Fields[0].Detach(&plotter2);
	cout << "Fields[0].Detach(&splotter2)" << endl;
	Fields[0].Detach(&splotter2);
	cout << "WorkSpace[0].Detach(&plotter1)" << endl;
	WorkSpace[0].Detach(&plotter1);
	cout << "WorkSpace[0].Detach(&splotter1)" << endl;
	WorkSpace[0].Detach(&splotter1);
	cout << "everything detached" << endl;

	return(1);
}
