#include <stdio.h>
#include <math.h>
#include "Point.h"
#include "Index.h"
#include "SquareGridField.h"
#include "SimpleDirichletBC.h"
#include "SimplePeriodicBC.h"
#include "FieldInitializer.h"
#include "ConstantFieldInitializer.h"
#include "OperatorOperand.h"
#include "UnaryOperators.h"
//#include "GnuPlotter.h"

#define NOOFFIELDS 2

class MyInitializer : public virtual FieldInitializer {
	protected:
		Point helicity;
	public:
		MyInitializer(void) : helicity(0.0) { ; }
		MyInitializer(const Point & h) : helicity(h) { ; }
		Real FieldValue(const Point & x) const {
			int i;
			Real y;
			y=sin(x(0));
			//y=sin(x(1));
			//for (i=0, y=0.0; i<DIMENSION; i++) y+=sin(x(i));
			for (i=0; i<DIMENSION; i++) y+=x(i)*helicity(i);
			return(y);
		}
};

int main(int argc, char **argv) 
{
	int i;
	//char title[128], ccc=' ';
	Index boexle(40);
	//for (i=0; i<DIMENSION-1; i++) boexle[i]=3*boexle[DIMENSION-1]+1;

	ConstantFieldInitializer cini;
	cini.SetValueOfField(-12.5);

	Point box(2.0*PI), heli(0.0);
	heli[0]=-1.0;
	heli[1]= 2.0;
	//Point box(1.0);
	SimplePeriodicBC spbc(box);
	spbc.SetHelicity(heli);
	MyInitializer mini(heli);

	//GnuContourPlot plotter;
	//plotter.SetContourLevels(20);
	//GnuSurfacePlot splotter;

	SquareGridField psi;
	
	//Point v(0.0);
	//v[0]=1.0/sqrt(2.0);
	//v[1]=1.0;
	//DirDerivative  DDeri(v);
	Laplacian Laplace;

	cout << "GetLogicalSize()=" << psi.GetLogicalSize() << "\n";
	cout << "SetLogicalSize(" << boexle << ")=" << (psi.SetLogicalSize(boexle)) << "\n";
	cout << "GetLogicalSize()=" << psi.GetLogicalSize() << "\n";
	cout << "GetNumberOfFields()=" << psi.GetNumberOfFields() << "\n";
	cout << "SetNumberOfFields(" << NOOFFIELDS << ")=" << psi.SetNumberOfFields(NOOFFIELDS) << "\n";
	cout << "GetNumberOfFields()=" << psi.GetNumberOfFields() << endl;

	//cout << "GetBoundaryCondition()=" << psi.GetBoundaryCondition() << "\n";
	cout << "SetBoundaryCondition(spbc)=" << psi.SetBoundaryCondition(&spbc) << "\n";
	cout << "GetBoundaryCondition()=" << psi.GetBoundaryCondition() << endl;

	for (i=0; i<NOOFFIELDS; i++) cout << "Initialize(cini, " << i << ")=" << psi.Initialize(cini, i) << "\n";
	cout << "GetLogicalSize()=" << psi.GetLogicalSize() << "\n";
	cout << "SetLogicalSize()=" << psi.SetLogicalSize(boexle) << "\n";
	cout << "GetLogicalSize()=" << psi.GetLogicalSize() << "\n";
	for (i=0; i<NOOFFIELDS; i++) cout << "Initialize(cini, " << i << ")=" << psi.Initialize(mini, i) << "\n";
	if (NOOFFIELDS>2) cout << "Initialize(cini, " << 2 << ")=" << psi.Initialize(mini, 2) << "\n";
	cout << endl;


	//psi.Attach(&plotter);
	//psi.Attach(&splotter);

	//for (i=0; i<NOOFFIELDS; i++) cout << "Initialize(cini, " << i << ")=" << psi.Initialize(cini, i) << endl;

	//psi.SurfacePlot(gplot); getchar(); psi.ContourPlot(gplot); getchar();
	//for (ccc=' ', i=0; i<10 && ccc!='x'; i++) {
	//	sprintf(title, "Laplace^%d", i);
	//	plotter.SetTitle(title);
	//	splotter.SetTitle(title);
	//	psi.Notify();
	//	fflush(stdin); ccc=getchar();
	//	psi.Laplace();
	//}

	//for (i=0; i<NOOFFIELDS; i++) cout << "Initialize(cini, " << i << ")=" << psi.Initialize(cini, i) << endl;
	//for (ccc=' ', i=0; i<10 && ccc!='x'; i++) {
	//	sprintf(title, "d^%d/dx_0^%d", i, i);
	//	plotter.SetTitle(title);
	//	splotter.SetTitle(title);
	//	psi.Notify();
	//	fflush(stdin); ccc=getchar();
	//	psi.Derivative(0);
	//}

	//for (i=0; i<NOOFFIELDS; i++) cout << "Initialize(cini, " << i << ")=" << psi.Initialize(cini, i) << endl;
	//for (ccc=' ', i=0; i<10 && ccc!='x'; i++) {
	//	sprintf(title, "d^%d/dx_1^%d", i, i);
	//	plotter.SetTitle(title);
	//	splotter.SetTitle(title);
	//	psi.Notify();
	//	fflush(stdin); ccc=getchar();
	//	psi.Derivative(1);
	//}

	//for (i=0; i<NOOFFIELDS; i++) cout << "Initialize(cini, " << i << ")=" << psi.Initialize(cini, i) << endl;
	//cout << "Enter direction" << endl;
	//Point  dir;
	//cin >> dir;
	//cout << dir << endl;
	//for (ccc=' ', i=0; i<10 && ccc!='x'; i++) {
	//	sprintf(title, "[(%g,%g)*grad]^%d", dir(0), dir(1), i);
	//	plotter.SetTitle(title);
	//	splotter.SetTitle(title);
	//	psi.Notify();
	//	fflush(stdin); ccc=getchar();
	//	psi.DirectionalDerivative(dir);
	//}
	//psi.SurfacePlot(gplot); getchar(); psi.ContourPlot(gplot); getchar();

	//psi.Detach(&plotter);
	//psi.Detach(&splotter);

	//DDeri[psi];
	Laplace[psi];
	psi.DumpToFile("out.put");
	cout << "Data dumped to file" << endl;

	return(1); 
}
