#ifndef BINARYOPERATORS_H
#define BINARYOPERATORS_H

#include "OperatorOperand.h"
#include <iostream.h>
#include <stdlib.h>

class AddField : virtual public BinOperator {
	public:
		virtual Operator & OperateOnFieldAnd(const Field *f);
		virtual Operator & OperateOnFieldAnd(const SquareGridField *f) 
			{ return(OperateOnFieldAnd((const Field *)f)); }

		virtual Operator & OperateOnSquareGridFieldAnd(const Field *f)
			{ return(OperateOnFieldAnd((const Field *)f)); }
		virtual Operator & OperateOnSquareGridFieldAnd(const SquareGridField *f);
};

class SubtractField : virtual public BinOperator {
	public:
		virtual Operator & OperateOnFieldAnd(const Field *f);
		virtual Operator & OperateOnFieldAnd(const SquareGridField *f) 
			{ return(OperateOnFieldAnd((const Field *)f)); }

		virtual Operator & OperateOnSquareGridFieldAnd(const Field *f)
			{ return(OperateOnFieldAnd((const Field *)f)); }
		virtual Operator & OperateOnSquareGridFieldAnd(const SquareGridField *f);
};

class MultiplyField : virtual public BinOperator {
	public:
		virtual Operator & OperateOnFieldAnd(const Field *f);
		virtual Operator & OperateOnFieldAnd(const SquareGridField *f) 
			{ return(OperateOnFieldAnd((const Field *)f)); }

		virtual Operator & OperateOnSquareGridFieldAnd(const Field *f)
			{ return(OperateOnFieldAnd((const Field *)f)); }
		virtual Operator & OperateOnSquareGridFieldAnd(const SquareGridField *f);
};


class MakeEqual : virtual public BinOperator {
	public:
		virtual Operator & OperateOnFieldAnd(const Field *f) {
			cout << "Please create a field of equal type first" << endl;
			exit(-1);
			return(*this);
		}
		virtual Operator & OperateOnFieldAnd(const SquareGridField *f) 
			{ return(OperateOnFieldAnd((const Field *)f)); }

		virtual Operator & OperateOnSquareGridFieldAnd(const Field *f)
			{ return(OperateOnFieldAnd((const Field *)f)); }
		virtual Operator & OperateOnSquareGridFieldAnd(const SquareGridField *f);
};

class GradientOperator : virtual public BinOperator {
	protected:
		int direction;
	public:
		GradientOperator(void);
		GradientOperator(int dir); 

		Operator & operator()(int dir) { 
			direction=dir; 
			#ifdef _DEBUG 
				if (abs(direction)>1) { 	
					cout << "GradientOperator::operator()(int " << dir << ") out of range" << endl;
					exit(-1);
				}
			#endif
			return(*this); 
		}
		virtual Operator & OperateOnFieldAnd(const Field *f) {
			cout << "Please create a field of equal type first" << endl;
			exit(-1);
			return(*this);
		}
		virtual Operator & OperateOnFieldAnd(const SquareGridField *f) 
			{ return(OperateOnFieldAnd((const Field *)f)); }

		virtual Operator & OperateOnSquareGridFieldAnd(const Field *f)
			{ return(OperateOnFieldAnd((const Field *)f)); }
		virtual Operator & OperateOnSquareGridFieldAnd(const SquareGridField *f);
		
		virtual bool SetDirection(int dir) {
			#ifdef _DEBUG
				if (dir==0 || dir==1 || dir==-1) { direction=dir; return(true); }
				else return(false);
			#else 
				direction=dir; return(true);
			#endif
		}
		int GetDirection(void) { return(direction); }
};

class L2NormOperator : virtual public BinOperator {
	public:
		virtual Operator & OperateOnFieldAnd(const Field *f) {
			cout << "Please create a field of equal type first" << endl;
			exit(-1);
			return(*this);
		}
		virtual Operator & OperateOnFieldAnd(const SquareGridField *f) 
			{ return(OperateOnFieldAnd((const Field *)f)); }

		virtual Operator & OperateOnSquareGridFieldAnd(const Field *f)
			{ return(OperateOnFieldAnd((const Field *)f)); }
		virtual Operator & OperateOnSquareGridFieldAnd(const SquareGridField *f);
};

#endif // BINARYOPERATORS_H
