#ifndef MAPLEPLOTTER_H
#define MAPLEPLOTTER_H

#include <stdio.h>
#include "SubjectObserver.h"
#include "DigitalMaterial.h"

class MaplePlotter : public Observer
{
	protected:
		char title[256];
		char *buffername;
		FILE* mapleplot_pipe;
		FILE* buffer;
	public:
		virtual bool SetTitle(const char * text) { 
			#ifdef _DEBUG 
				if (strlen(text)>=255) return(false);
			#endif
			strcpy(title, text);
			return(true);
		}
#ifndef SWIG
		MaplePlotter(void);
		MaplePlotter(FILE * gplotpipe);
		virtual ~MaplePlotter(void);

		virtual bool InitializeMapleplot(void)=0;
		virtual bool FlushMaplePlotter(void) { 
			#ifdef _DEBUG
				if (mapleplot_pipe==NULL) return(false);
			#endif 
			fflush(mapleplot_pipe);
			return(true);
		}
		virtual bool SendPlotCommand(void)=0;
		virtual bool SendPlotCommand(const char* commandstring);
		virtual void Update(Subject *theChangedSubject) = 0;
#endif //SWIG
};

class TwoDMapleplotter : public MaplePlotter
{
	public:
#ifndef SWIG
		TwoDMapleplotter();
		TwoDMapleplotter(FILE *gplotpipe);
		virtual ~TwoDMapleplotter();
		virtual bool InitializeMapleplot()=0;
		virtual bool SendData(Subject *theChangedSubject);
		virtual bool SendPlotCommand();
		virtual void Update(Subject *theChangedSubject);
#endif //SWIG
};

class MapleContourPlot : public TwoDMapleplotter
{
	protected:
		int contour_levels;
	public:
		MapleContourPlot();
		virtual ~MapleContourPlot();
		virtual bool SetContourLevels(int levels);
		virtual int GetContourLevels() const { return(contour_levels); }
#ifndef SWIG
		MapleContourPlot(FILE * gplotpipe);
		virtual bool InitializeMapleplot();
#endif //SWIG
};

class MapleSurfacePlot : public TwoDMapleplotter
{
	protected:
		double phi_x;
		double phi_z;
	public:
		MapleSurfacePlot();
		virtual ~MapleSurfacePlot();
#ifndef SWIG
		MapleSurfacePlot(FILE * gplotpipe);
		virtual bool InitializeMapleplot();
		virtual bool SetAngles(Real phix, Real phiz);
		virtual Real GetXAngle(void) { return(phi_x); }
		virtual Real GetZAngle(void) { return(phi_z); }
#endif //SWIG
};
#endif
