/*
	Adaption of the Visitor pattern to unary and binary operators on fields
*/


#ifndef OPERATOROPERAND_H
#define OPERATOROPERAND_H

#include <iostream.h>
#include <stdlib.h>

class Field;
class SquareGridField;
class BinOperator;
class Operator;

class Operand {
	public:
		virtual Operator & Apply(Operator &op)=0;
		virtual Operator & Apply(Operator &op) const=0;
};

class Operator {
	public:
		virtual Operator & OperateOn(Field *f)=0;
		virtual Operator & OperateOn(SquareGridField *f)=0;

		virtual Operator & OperateOn(const Field *f) { return(WantNonConst("Field")); }
		virtual Operator & OperateOn(const SquareGridField *f) { return(WantNonConst("SquareGridField")); }

		//virtual Operator & operator[](Operand &x) { return(x.Apply(*this)); }
		//virtual Operator & operator[](const Operand &x) { return(x.Apply(*this)); }
		virtual Operator & operator[](Operand &x);
		virtual Operator & operator[](const Operand &x);

		virtual Operator & WantNonConst(const char * what) { 
			cout << "Need a non-const " << what << endl; 
			exit(-1); 
			return(*this); 
		}
};

class First : virtual public Operator {
	protected:
		BinOperator *binop;
	public:
		First(BinOperator *bop); // : binop(bop)
		virtual Operator & OperateOn(Field *f) { return(OperateOn((const Field *)f)); }
		virtual Operator & OperateOn(SquareGridField *f) { return(OperateOn((const SquareGridField *)f)); }
		virtual Operator & OperateOn(const Field *f)=0;
		virtual Operator & OperateOn(const SquareGridField *f)=0;
};

class FirstField : virtual public First {
	public:
		FirstField(BinOperator *bop); // :First(bop)
		virtual Operator & OperateOn(const Field *f);
		virtual Operator & OperateOn(const SquareGridField *f);
};

class FirstSquareGridField : virtual public First {
	public:
		FirstSquareGridField(BinOperator *bop); //  :First(bop)
		virtual Operator & OperateOn(const Field *f);
		virtual Operator & OperateOn(const SquareGridField *f);
};

class BinOperator : virtual public Operator {
	protected:
		First *first;
		FirstField firstfield;
		FirstSquareGridField firstsquaregridfield;
		union {
			Operand *operand;
			Field *field;
			SquareGridField *squaregridfield;
		};
	public:
		BinOperator(void);
		
		Operator & OperateOn(Field *f);
		Operator & OperateOn(SquareGridField *f);

		Operator & OperateOn(const Field *f);
		Operator & OperateOn(const SquareGridField *f);

		virtual Operator & OperateOnFieldAnd(const Field *f)=0;
		virtual Operator & OperateOnFieldAnd(const SquareGridField *f)=0;

		virtual Operator & OperateOnSquareGridFieldAnd(const Field *f)=0;
		virtual Operator & OperateOnSquareGridFieldAnd(const SquareGridField *f)=0;
};



#endif //OPERATOROPERAND_H
