#ifndef POINT_H
#define POINT_H

#include <stdlib.h>
#include <fstream.h>

#ifdef _DEBUG
#include <assert.h>
#endif //_DEBUG

#include "DigitalMaterial.h"

class Point // : public virtual Introspective, public Cloneable // not yet!
{
private:	
	Real coordinate[DIMENSION];
public:
	Point();
//XXX: Swig 1.3 can handle this ?
//#if (DIMENSION==2)
	bool Initialize(const Real x, const Real y);
//#else // (DIMENSION==3)
//	bool Initialize(const Real x, const Real y, const Real z);
	// Only 2D and 3D version
//#endif

	char * GetPtrToArray() { return((char*)coordinate); }
	//Only used so that coordinate can be accessed from Python

#ifndef SWIG
	Point(const Real x);
	//~Point() { }
	Real & operator[](const int i) {
		#ifdef _DEBUG
			assert(i>=0 && i<DIMENSION);
		#endif //_DEBUG
			return(coordinate[i]);
	}
	Real operator()(const int i) const {
		#ifdef _DEBUG
			assert(i>=0 && i<DIMENSION);
		#endif //_DEBUG
			return(coordinate[i]);
	}

	Point(const Real x[DIMENSION]);
	Point(const Point & point);
#if DIMENSION==2
	Point(const Real x, const Real y);
#elif DIMENSION==3
	Point(const Real x, const Real y, const Real z);
#endif
	Real * operator()(Real *pp) const;

	bool operator==(const Point &right) const;
	bool operator!=(const Point &right) const { return(!(*this==right)); }
	bool IsPositive() const;
	bool IsNonNegative() const;
	bool IsInBox(const Point &box) const;
	Point & operator*=(const Real x);
	Point & operator/=(const Real x);
	Point & operator+=(const Point & addend);
	Point & operator-=(const Point & addend);
	Point & operator=(const Point & x);
	bool Zero() { return Initialize(0.0); }
	bool Initialize(const Real x);
	Real Volume() const;
	Real Norm(void) const;
	void Print(const char * text) const;
	friend ostream & operator<<(ostream &os, const Point & point);
	friend istream & operator>>(istream &is, Point & point);
#endif //SWIG
};
#endif // POINT_H
