#ifndef SEMIIMPLICITCONSERVEDOP_H
#define SEMIIMPLICITCONSERVEDOP_H

#ifdef _DEBUG
#include <assert.h>
#endif

#include "DigitalMaterial.h"
#include "Field.h"
#include "SplitRightHandSide.h"
#include "OperatorOperand.h"
#include "BinaryOperators.h"
#include "UnaryOperators.h"
#include "SplitConservedOP.h"
#include "ConservedOP.h" 


//#include "Introspective.h"
//#include "Clonable.h"


class COPImplicitPropagator : virtual public ImplicitPropagator {
	protected:
		Real diffusivity, part, strength;
		Real ptimess;
	public:
		COPImplicitPropagator(void);
		COPImplicitPropagator(const COPImplicitPropagator& prop);

		virtual Real ReDiagonal(const Point &k);
		virtual Real ImDiagonal(const Point &k) { return(0.0); }
		
		virtual Real GetDiffusivity(void) const { return(diffusivity); }
		virtual bool SetDiffusivity(Real d) { diffusivity=d; return(true); }
		virtual Real GetPart(void) const { return(part); }
		virtual bool SetPart(Real d) { part=d; ptimess=part*strength; return(true); }
		virtual Real GetStrength(void) const { return(strength); }
		virtual bool SetStrength(Real d) { strength=d; ptimess=part*strength; return(true); }
};

//class PartOfPhaseFieldTerm : public virtual ApplyFunction {
//	protected:
//		Real strength, part;
//	public:
//		PartOfPhaseFieldTerm(void);
//
//		Real function(Real x) { return(strength*(x*x*x-part*x)); }
//		Operator & operator()(Real s, Real p) { strength=s; part=1.0+p; return(*this); }
//		bool SetStrength(Real s) { strength=s; return(true); }
//		Real GetStrength(void) const { return(strength); }
//		bool SetPart(Real p) { part=1.0+p; return(true); }
//		Real GetPart(void) const { return(part-1.0); }
//};

class SemiImplicitConservedOP : virtual public SplitRightHandSide, virtual public ConservedOPParameters //, public Introspective, public Cloneable
{
	protected:
		FieldCollection WorkSpace;
		Real part;
		PhaseFieldPotential PhaseFieldTerm;
		MakeEqual Equal;
		Laplacian Laplace;
		AddField Add;
		MultiplyReal Times;
		COPImplicitPropagator LinProp;
		PartOfPhaseFieldTerm NonlinearPart;
	public:
		SemiImplicitConservedOP(void);
		SemiImplicitConservedOP(const SemiImplicitConservedOP &cop);
		virtual ~SemiImplicitConservedOP(void);

		virtual bool RHS(const FieldCollection &input, FieldCollection &output, int n);
		virtual bool Advance(FieldCollection & input, Real time, int n);
		
		virtual bool SetDiffusivity(Real dd) { 
			diffusivity=dd; 
			return(LinProp.SetDiffusivity(dd)); 
		}
		virtual Real GetPart(void) const { return(part); }
		virtual bool SetPart(Real p) {
			part=p;
			return(NonlinearPart.SetPart(p) && LinProp.SetPart(p));
		}
		virtual bool SetPotential(Real dd) { 
			potential=dd; 
			return(NonlinearPart.SetStrength(dd) && LinProp.SetStrength(dd)); 
		}
		virtual bool SetWorkSpace(const FieldCollection & Fields);
		virtual int GetNumberOfRHS(void) const { return(2); }
		virtual FieldCollection & GetWorkSpace() { return(WorkSpace); }
		virtual const FieldCollection & GetWorkSpace() const { return(WorkSpace); }
};


#endif
