// SubjectObserver.h: interface for the Subject and Observer classes
//
//////////////////////////////////////////////////////////////////////

// Adapted from the Subject-Observer section of 
// "Design Patterns, Elements of Reusable Object-Oriented Software"
// by E. Gamma, R. Helm, R. Johnson, and J. Vlissides, 
// Addison Wesley, Massachusetts, 1995
// ISBN 0-201-63361-2


#ifndef OBSERVER_H
#define OBSERVER_H

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif

#include <list>
#include <iostream.h>
#include "DigitalMaterial.h"

class Field ;
class SquareGridField ;
class Subject;
class FieldObserver;

class Subject  
{
public:
#ifndef SWIG
	Subject(void);
	virtual ~Subject(void);
#endif //SWIG
	virtual void  Attach(FieldObserver* o) { _observers->push_back(o); }
	virtual void Detach(FieldObserver* o) { _observers->remove(o); }
	virtual void LookAtMe(FieldObserver &obs) const =0;
	virtual void Notify(void);
private:
#ifndef SWIG // XXX SWIG doesn't support standard template libraries
	std::list<FieldObserver*> *_observers;
#endif // SWIG
};

class FieldObserver  
{
public:
#ifndef SWIG
	FieldObserver(void);
	virtual ~FieldObserver(void);
#endif //SWIG
	virtual void LookAt(const Subject *subject) { subject->LookAtMe(*this); }
	virtual void Update(const Field *theChangedSubject) { 
		cout << "Update(const Field *) not implemented!" << endl; 
		exit(-1);
	}
	virtual void Update(const SquareGridField *theChangedSubject) { Update((const Field*)theChangedSubject); }
};

#endif // !defined(OBSERVER_H)
