#include "AnalyticControlEuler.h"

AnalyticControlEuler::AnalyticControlEuler(void) : OperatorSplitFieldMover(), step_size(0.0) 
{
	// introspective things and so on
}

AnalyticControlEuler::AnalyticControlEuler(Real step, SplitRightHandSide *RHS) : OperatorSplitFieldMover(RHS), step_size(step)
{
	// introspective
}

AnalyticControlEuler::AnalyticControlEuler(const AnalyticControlEuler &exeuler) : OperatorSplitFieldMover(exeuler), step_size(exeuler.GetStepSize())
{ SetWorkSpace(exeuler.GetWorkSpace()); }

AnalyticControlEuler::~AnalyticControlEuler(void) { FreeWorkSpace(); }

void AnalyticControlEuler::FreeWorkSpace(void)
{
	int i;
	for (i=0;i<WorkSpace.GetNumberOfFields(); i++) if (&WorkSpace[i]!=ZERO) delete &WorkSpace[i];
	WorkSpace.RemoveAllFields();
}

void AnalyticControlEuler::FreeErrorEstimate(void)
{
	int i;
	for (i=0;i<ErrorEstimate.GetNumberOfFields(); i++) if (&ErrorEstimate[i]!=ZERO) delete &ErrorEstimate[i];
	ErrorEstimate.RemoveAllFields();
}

bool AnalyticControlEuler::SetWorkSpace(const FieldCollection &Fields)
{
	int i, number;
	bool flag=false;
	Field *NewField;

	number=Fields.GetNumberOfFields();

	if (number!=WorkSpace.GetNumberOfFields()) 											flag=true;
	else 			for (i=0; i<number; i++) if (&WorkSpace[i]==ZERO) 					flag=true;
	if (!flag) 	for (i=0; i<number; i++) if (typeid(WorkSpace[i])!=typeid(Fields[i])) 	flag=true;

	if (flag) {
		FreeWorkSpace();
		for (i=0; i<number; i++) {
			flag=WorkSpace.AddField(NewField=Fields[i].AllocCopy());
			if (!flag || NewField==ZERO) return(false);
		}
	}
	flag=false;
	if (number!=ErrorEstimate.GetNumberOfFields()) 											flag=true;
	else 			for (i=0; i<number; i++) if (&ErrorEstimate[i]==ZERO)					flag=true;
	if (!flag) 	for (i=0; i<number; i++) if (typeid(ErrorEstimate[i])!=typeid(Fields[i])) 	flag=true;

	if (flag) {
		FreeErrorEstimate();
		for (i=0; i<number; i++) {
			flag=ErrorEstimate.AddField(NewField=Fields[i].AllocCopy());
			if (!flag || NewField==ZERO) return(false);
		}
	}
	return(true);
}
bool AnalyticControlEuler::Move(FieldCollection & Fields)
{
	int i, j, n;

	for (i=0; i<ErrorEstimate.GetNumberOfFields(); i++) Equal[ErrorEstimate[i]][Fields[i]];

	for (n=0; n<split_rhs->GetNumberOfRHS(); n++) {
		if (!split_rhs->Advance(ErrorEstimate, step_size, n)) {
			if (!split_rhs->RHS(ErrorEstimate, WorkSpace, n)) return(false);
			for (i=0; i<ErrorEstimate.GetNumberOfFields(); i++) {
				Times(step_size)[WorkSpace[i]];
				Add[ErrorEstimate[i]][WorkSpace[i]];
			}
		}
	}
	for (j=0; j<2; j++) {
		for (n=0; n<split_rhs->GetNumberOfRHS(); n++) {
			if (!split_rhs->Advance(Fields, 0.5*step_size, n)) {
				if (!split_rhs->RHS(Fields, WorkSpace, n)) return(false);
				for (i=0; i<Fields.GetNumberOfFields(); i++) {
					Times(0.5*step_size)[WorkSpace[i]];
					Add[Fields[i]][WorkSpace[i]];
				}
			}
		}
	}
	for (i=0; i<ErrorEstimate.GetNumberOfFields(); i++) {
		Subtract[ErrorEstimate[i]][Fields[i]];
		Subtract[Fields[i]][ErrorEstimate[i]];
	}
	return(true);
}
