#include "DigitalMaterial.h"
#include "BinaryOperators.h"
#include "Point.h"
#include "Field.h"
#include "SquareGridField.h"
#include <math.h>
#include <new>

using namespace std;

Operator & AddField::OperateOnFieldAnd(const Field *addend)
{
	int i;
	Point x; 
	Real *y1 = new(nothrow) Real[field->GetNumberOfFields()];
	Real *y2 = new(nothrow) Real[field->GetNumberOfFields()];

	#ifdef _DEBUG
		assert(field->GetNumberOfFields()==addend->GetNumberOfFields());
		assert(y1!=ZERO && y2!=ZERO);
	#endif

	for (field->GotoFirstPoint(); !field->IsLastPoint(); field->GotoNextPoint()) {
		x=field->GetPointCoordinates();
		field->GetFieldValue(y1);
		if (addend->GetFieldValue(x, y2)) {
			for (i=0; i<field->GetNumberOfFields(); i++) y1[i]+=y2[i];
			field->SetFieldValue(y1);
		}
	}
	delete []y1;
	delete []y2;
	return(*this);
}

Operator & AddField::OperateOnSquareGridFieldAnd(const SquareGridField *addend)
{
	int i;
	size_t n;
	#ifdef _DEBUG
		assert(squaregridfield->GetBoundaryCondition()!=ZERO);
		assert(squaregridfield->GetNumberOfFields()==addend->GetNumberOfFields());
		assert(squaregridfield->GetBoundaryCondition()->GetPhysicalSize()==addend->GetBoundaryCondition()->GetPhysicalSize());
	#endif

	if (squaregridfield->GetLogicalSize()!=addend->GetLogicalSize()) return(OperateOnFieldAnd((const Field *)addend));
	else {
		for (i=0; i<squaregridfield->GetNumberOfFields(); i++)
			for (n=0; n<squaregridfield->GetNumberOfDataPoints(); n++) 
				squaregridfield->GetField(n, i)+=addend->GetFieldValue(n, i);
		return(*this);
	}
}


Operator & SubtractField::OperateOnFieldAnd(const Field *addend)
{
	int i;
	Point x; 
	Real *y1 = new(nothrow) Real[field->GetNumberOfFields()];
	Real *y2 = new(nothrow) Real[field->GetNumberOfFields()];

	#ifdef _DEBUG
		assert(field->GetNumberOfFields()==addend->GetNumberOfFields());
		assert(y1!=ZERO && y2!=ZERO);
	#endif

	for (field->GotoFirstPoint(); !field->IsLastPoint(); field->GotoNextPoint()) {
		x=field->GetPointCoordinates();
		field->GetFieldValue(y1);
		if (addend->GetFieldValue(x, y2)) {
			for (i=0; i<field->GetNumberOfFields(); i++) y1[i]-=y2[i];
			field->SetFieldValue(y1);
		}
	}
	delete []y1;
	delete []y2;
	return(*this);
}

Operator & SubtractField::OperateOnSquareGridFieldAnd(const SquareGridField *addend)
{
	int i;
	size_t n;
	#ifdef _DEBUG
		assert(squaregridfield->GetBoundaryCondition()!=ZERO);
		assert(squaregridfield->GetNumberOfFields()==addend->GetNumberOfFields());
		assert((squaregridfield->GetBoundaryCondition())->GetPhysicalSize()==(addend->GetBoundaryCondition())->GetPhysicalSize());
	#endif

	if (squaregridfield->GetLogicalSize()!=addend->GetLogicalSize()) {
		return( OperateOnFieldAnd((const Field *)addend) );
	}
	else {
		for (i=0; i<squaregridfield->GetNumberOfFields(); i++)
			for (n=0; n<squaregridfield->GetNumberOfDataPoints(); n++) 
				squaregridfield->GetField(n, i)-=addend->GetFieldValue(n, i);
		return(*this);
	}
}

Operator & MultiplyField::OperateOnFieldAnd(const Field *addend)
{
	int i;
	Point x; 
	Real *y1 = new(nothrow) Real[field->GetNumberOfFields()];
	Real *y2 = new(nothrow) Real[field->GetNumberOfFields()];

	#ifdef _DEBUG
		assert(field->GetNumberOfFields()==addend->GetNumberOfFields());
		assert(y1!=ZERO && y2!=ZERO);
	#endif

	for (field->GotoFirstPoint(); !field->IsLastPoint(); field->GotoNextPoint()) {
		x=field->GetPointCoordinates();
		field->GetFieldValue(y1);
		if (addend->GetFieldValue(x, y2)) {
			for (i=0; i<field->GetNumberOfFields(); i++) y1[i]*=y2[i];
			field->SetFieldValue(y1);
		}
	}
	delete []y1;
	delete []y2;
	return(*this);
}

Operator & MultiplyField::OperateOnSquareGridFieldAnd(const SquareGridField *addend)
{
	int i;
	size_t n;
	#ifdef _DEBUG
		assert(squaregridfield->GetBoundaryCondition()!=ZERO);
			assert(squaregridfield->GetNumberOfFields()==addend->GetNumberOfFields());
		assert(squaregridfield->GetBoundaryCondition()->GetPhysicalSize()==addend->GetBoundaryCondition()->GetPhysicalSize());
	#endif

	if (squaregridfield->GetLogicalSize()!=addend->GetLogicalSize()) return(OperateOnFieldAnd((const Field *)addend));
	else {
		for (i=0; i<squaregridfield->GetNumberOfFields(); i++)
			for (n=0; n<squaregridfield->GetNumberOfDataPoints(); n++) 
				squaregridfield->GetField(n, i)*=addend->GetFieldValue(n, i);
		return(*this);
	}
}


Operator & MakeEqual::OperateOnSquareGridFieldAnd(const SquareGridField *right)
{
	int i;
	size_t n;
	#ifdef _DEBUG
		assert(
			squaregridfield->SetNumberOfFields(right->GetNumberOfFields()) &&
			squaregridfield->SetLogicalSize(right->GetLogicalSize()) &&
			squaregridfield->SetBoundaryCondition(right->GetBoundaryCondition()));
	#else
		squaregridfield->SetNumberOfFields(right->GetNumberOfFields());
		squaregridfield->SetLogicalSize(right->GetLogicalSize());
		squaregridfield->SetBoundaryCondition(right->GetBoundaryCondition());
	#endif

	for (i=0; i<squaregridfield->GetNumberOfFields(); i++) {
		//cout << "MakeEqual::OperateOnSquareGridFieldAnd(const SquareGridField *) " << squaregridfield->GetPtrToFFTWReal()[i] << " " << squaregridfield->GetPtrToFFTComplex()[i] << endl;
		for (n=0; n<squaregridfield->GetNumberOfDataPoints(); n++)
			squaregridfield->GetField(n, i)=right->GetFieldValue(n, i);
	}
	return(*this);
}

GradientOperator::GradientOperator(void) : BinOperator() { ; }

GradientOperator::GradientOperator(int dir) : BinOperator(), direction(dir) 
{
	if (direction!=1 && direction!=0 && direction!=-1) {
		cout << "GradientOperator::GradientOperator(int " << dir << ") We have only forward, backward and central differences!" << endl;
		cout << "Setting to symmetric differentiation!" << endl;
	}
}

Operator & GradientOperator::OperateOnSquareGridFieldAnd(const SquareGridField *right)
{
	size_t n;
	int i, d, vv;
	Real rePhi, imPhi, reD, imD, vol, inv_dX, N2m;
	Index fftwcomplexbox(right->GetFFTWBox()), index, box=right->GetLogicalSize();
	const Point pp((right->GetBoundaryCondition())->GetPhysicalSize());
	const Point mm((right->GetBoundaryCondition())->GetHelicity());
	const Point null(0.0);
	fftwcomplexbox[DIMENSION-1]/=2;

	#ifdef _DEBUG
		assert(squaregridfield->GetReToCompPlan()!=NULL);
		assert(squaregridfield->GetCompToRePlan()!=NULL);
		assert(
			squaregridfield->SetNumberOfFields(DIMENSION) &&
			squaregridfield->SetLogicalSize(right->GetLogicalSize()) &&
			squaregridfield->SetBoundaryCondition(right->GetBoundaryCondition()));
	#else
		squaregridfield->SetNumberOfFields(DIMENSION);
		squaregridfield->SetLogicalSize(right->GetLogicalSize());
		squaregridfield->SetBoundaryCondition(right->GetBoundaryCondition());
	#endif
	fftw_real **real_data=squaregridfield->GetPtrToFFTWReal();
	fftw_complex **complex_data=squaregridfield->GetPtrToFFTComplex();
	#ifdef _DEBUG
		assert(real_data!=ZERO);
		assert(complex_data!=ZERO);
		for (i=0; i<squaregridfield->GetNumberOfFields(); i++) {
			assert(real_data[i]!=ZERO);
			assert(complex_data[i]!=ZERO);
		}
	#endif

	for (n=0; n<squaregridfield->GetNumberOfDataPoints(); n++) squaregridfield->GetField(n, DIMENSION-1)=right->GetFieldValue(n, 0);
	vol=(Real)box.Volume();

	rfftwnd_one_real_to_complex(squaregridfield->GetReToCompPlan(), real_data[DIMENSION-1], NULL);
	if (mm!=null) {
		for (d=0, vv=1; d<DIMENSION; d++) vv*=box(d);
		for (d=0; d<DIMENSION; d++) {
			N2m=0.5*mm(d)*pp(d)*vv/box(d);
			index.Zero(); 
			for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
				complex_data[DIMENSION-1][index(fftwcomplexbox)].re+=N2m;
				complex_data[DIMENSION-1][index(fftwcomplexbox)].im-=N2m/tan(PI*index(d)/box(d));
			}
		}
	}

	for (i=0; i<DIMENSION; i++) { 
		inv_dX=box(i)/pp(i);
		for (index.Zero(); index.IsInBox(fftwcomplexbox); index.NextInBox(fftwcomplexbox)) {
			rePhi=complex_data[DIMENSION-1][index(fftwcomplexbox)].re;
			imPhi=complex_data[DIMENSION-1][index(fftwcomplexbox)].im;
			reD=direction*(cos(2.0*PI*index(i)/box(i))-1);
			imD=           sin(2.0*PI*index(i)/box(i));
			complex_data[i][index(fftwcomplexbox)].re=inv_dX*(rePhi*reD-imPhi*imD)/vol;
			complex_data[i][index(fftwcomplexbox)].im=inv_dX*(rePhi*imD+imPhi*reD)/vol;
		}
	}

	index.Zero();
	for (i=0; i<DIMENSION; i++) {
		complex_data[i][index(fftwcomplexbox)].re+=mm(i);
		rfftwnd_one_complex_to_real(squaregridfield->GetCompToRePlan(), complex_data[i], NULL);
	}

	(squaregridfield->GetBoundaryCondition())->SetHelicity(null);
	return(*this);
}


Operator & L2NormOperator::OperateOnSquareGridFieldAnd(const SquareGridField *right)
{
	int i;
	size_t n;
	Real norm, y;

	#ifdef _DEBUG
		assert(
			squaregridfield->SetNumberOfFields(1) &&
			squaregridfield->SetLogicalSize(right->GetLogicalSize()) &&
			squaregridfield->SetBoundaryCondition(right->GetBoundaryCondition()));
	#else
		squaregridfield->SetNumberOfFields(1);
		squaregridfield->SetLogicalSize(right->GetLogicalSize());
		squaregridfield->SetBoundaryCondition(right->GetBoundaryCondition());
	#endif

	for (n=0; n<squaregridfield->GetNumberOfDataPoints(); n++) {
		norm=0.0;
		for (i=0; i<right->GetNumberOfFields(); i++) {
			y=right->GetFieldValue(n, i);
			norm+=y*y;
		}
		squaregridfield->GetField(n, 0)=sqrt(norm);
	}
	return(*this);
}

