#include "Field.h"
#include "UnaryOperators.h"
#include "BinaryOperators.h"

Field::Field() : number_of_fields(0), boundary_condition(ZERO)
{ 
	// Introspective things and so on 
}

Field::Field(const Field &field) : 	number_of_fields(field.GetNumberOfFields())
{
	boundary_condition=field.GetBoundaryCondition()->AllocCopy();
	// No introspective things because "field" must have registered already
}

Field::Field(int number, const BoundaryCondition * bc) : number_of_fields(number)
{
	boundary_condition=bc->AllocCopy();
	#ifdef _DEBUG
		assert(boundary_condition!=ZERO);
	#endif
}

Field::~Field(void) 
{	
	number_of_fields=0;
	if (boundary_condition!=ZERO) delete boundary_condition;
}

bool Field::GetFieldValue(const Point &x, Real *y) const
{
	int i;
	for (i=0; i<number_of_fields; i++) y[i]=GetFieldValue(x, i);
	return(true);
}


// ============ Here come the operators ==============
// calling the Operators directly should be more efficient

Field & Field::operator= (const Field &right)
{
	static MakeEqual is;
	//cout << "Field::operator=(const Field &)" << endl;
	is[*this][right];
	return(*this);
}

Field & Field::operator+=(const Field &right)
{
	static AddField add;
	//cout << "Field::operator+=(const Field &)" << endl;
	add[*this][right];
	return(*this);
}


Field & Field::operator-=(const Field &right)
{
	static SubtractField sub;
	//cout << "Field::operator-=(const Field &)" << endl;
	sub[*this][right];
	return(*this);
}

Field & Field::operator*=(const Field &right)
{
	static MultiplyField mult;
	//cout << "Field::operator*=(const Field &)" << endl;
	mult[*this][right];
	return(*this);
}

Field & Field::operator+=(Real addend)
{
	static AddReal add(addend);
	//cout << "Field::operator+=(Real)" << endl;
	add[*this];
	return(*this);
}

Field & Field::operator*=(Real factor)
{
	static MultiplyReal mult;
	//cout << "Field::operator*=(Real)" << endl;
	mult(factor)[*this];
	return(*this);
}

Field & Field::Laplace(void)
{
	static Laplacian delta;
	//cout << "Field::Laplace" << endl;
	delta[*this];
	return(*this);
}

Field & Field::Laplace2(void)
{
	static LaplacianSquared delta2;
	//cout << "Field::Laplace2" << endl;
	delta2[*this];
	return(*this);
}

Field & Field::DirectionalDerivative(const Point & dir)
{
	static DirDerivative ddir(dir);
	//cout << "Field::DirectionalDerivative(const Point &)" << endl;
	ddir[*this];
	return(*this);
}

Field & Field::Derivative(int dir)
{
	static PartialDerivative der(dir);
	//cout << "Field::Derivative(int)" << endl;
	der[*this];
	return(*this);
}

//Field & Field::Apply(Function *f)
//{
//	ApplyFunction appl(f);
//	appl[*this];
//	return(*this);
//}

//Field & Field::Multiply(Function *f)
//{
//	MultiplyFunction multf(f);
//	multf[*this];
//	return(*this);
//}

Field & Field::PhaseFieldTerm(Real strength)
{
	static PhaseFieldPotential phft(strength);
	//cout << "Field::PhaseFieldTerm(Real)" << endl;
	phft[*this];
	return(*this);
}
