#include <typeinfo>
#ifdef _WIN32
#include <windows.h>
#endif //_WIN32
#include "GnuPlotter.h"
#include "Field.h"
#include "SquareGridField.h"
#include "Index.h"
#include "Point.h"

GnuPlotter::GnuPlotter(void) : Observer(), buffername(NULL), gnuplot_pipe(NULL), buffer(NULL)
{
	strcpy(title, "");
#ifdef _WIN32
		gnuplot_pipe = _popen("I:\\gnuplot\\pgnuplot.exe", "w");	
#else
	#ifdef _DEBUG
		char name[128];
		sprintf(name, "tee gnuplot.protocol%d |  gnuplot -geometry 480x360", (int)this);
		gnuplot_pipe = popen(name, "w");
	#else
		gnuplot_pipe = popen("gnuplot -geometry 480x360", "w");
	#endif // _DEBUG
#endif // _WIN32
	if (gnuplot_pipe==NULL) {
		cout << "GnuPlotter::GnuPlotter(): Gnuplot couldn't be opened!" << endl;
		exit(-1);
	}
	buffername=tempnam(NULL, "gnuplotter");
	if (buffername==NULL) {
		cout << "GnuPlotter::GnuPlotter(): Buffername couldn't be created!" << endl;
		exit(-1);
	}
	buffer=fopen(buffername, "w");
	if (buffer==NULL) {
		cout << "GnuPlotter::GnuPlotter(): Buffer couldn't be created!" << endl;
		exit(-1);
	}
}

GnuPlotter::GnuPlotter(FILE * gplotpipe) : Observer(), buffername(NULL), gnuplot_pipe(gplotpipe)
{
	#ifdef _DEBUG
		assert(gnuplot_pipe!=NULL);
	#endif //_DEBUG
	buffername=tempnam(NULL, "gnuplotter");
	if (buffername==NULL) {
		cout << "GnuPlotter::GnuPlotter(): Buffername couldn't be created!" << endl;
		exit(-1);
	}
	buffer=fopen(buffername, "w");
	if (buffer==NULL) {
		cout << "GnuPlotter::GnuPlotter(): Buffer couldn't be created!" << endl;
		exit(-1);
	}
}

GnuPlotter::~GnuPlotter(void) { 
	if (gnuplot_pipe!=NULL) {
		//cout << "GnuPlotter::~GnuPlotter()" << endl;
		#ifdef _WIN32
			_pclose(gnuplot_pipe);
		#else
			pclose(gnuplot_pipe); 
		#endif //_WIN32
	}
	if (buffer!=NULL) fclose(buffer);
	if (buffername!=NULL) {
		unlink(buffername);
		free(buffername);
	}
	// else nothing to close
}

bool GnuPlotter::SendPlotCommand(const char* commandstring)
{
	if (gnuplot_pipe!=NULL) {
		fprintf(gnuplot_pipe, "%s", commandstring);
		FlushGnuPlotter();
		return(true);
	}
	else return(false);
}

TwoDGnuplotter::TwoDGnuplotter() : GnuPlotter()
{ 
}

TwoDGnuplotter::TwoDGnuplotter(FILE * gplotpipe) : GnuPlotter(gplotpipe)
{ 
}

TwoDGnuplotter::~TwoDGnuplotter()
{ ; }

bool TwoDGnuplotter::SendData(Subject *theChangedSubject)
{
	//if (DIMENSION!=2) return(false);
	// Throw exeption?
	SquareGridField *field;
	Index ix(0);
	Point x;
	field=dynamic_cast<SquareGridField *>(theChangedSubject);
	#ifdef _DEBUG
		assert(field!=ZERO);
		assert(gnuplot_pipe!=NULL);
		assert(freopen(buffername, "w", buffer)!=NULL);
	#else
		freopen(buffername, "w", buffer);
	#endif
	Index box=field->GetLogicalSize();
	switch (DIMENSION) {
		case 1: {
			for (ix[0]=0; ix[0]<box(0); ix[0]++) {
				x=field->GetPhysicalCoordinates(ix);
				fprintf(buffer, "%g %g \n", x[0], field->GetFieldValue(x, 0));
				fprintf(buffer, "\n");
				fflush(buffer);
			}
		} break;
		case 2: {
			for (ix[0]=0; ix[0]<box(0); ix[0]++) {
				for (ix[1]=0; ix[1]<box(1); ix[1]++) {
					x=field->GetPhysicalCoordinates(ix);
					fprintf(buffer, "%g %g %g \n", x[0], x[1], field->GetFieldValue(x, 0));
				}
				fprintf(buffer, "\n");
				fflush(buffer);
			}
		} break;
		default: {
			cout << "Only 1D and 2D plots supported!" << endl;
			exit(-1);
		}
	}
			
	//fprintf(gnuplot_pipe, "e\n");
	fflush(buffer);
	return(FlushGnuPlotter());
}

bool TwoDGnuplotter::SendPlotCommand(void)
{
	#ifdef _DEBUG
		if (gnuplot_pipe==NULL) return(false);
	#endif
	switch (DIMENSION) {
	case 1: { fprintf(gnuplot_pipe, "\n\nplot \"%s\"  w l\n\n", buffername); } break;
	case 2: { fprintf(gnuplot_pipe, "\n\nsplot \"%s\"  w l\n\n", buffername); } break;
	default: {
			cout << "Only 1D and 2D plots supported!" << endl;
			exit(-1);
		}
	}
	return(FlushGnuPlotter());
}

void TwoDGnuplotter::Update(Subject *theChangedSubject)
{
	InitializeGnuplot();
	SendData(theChangedSubject);
	SendPlotCommand();
	//SendPlotCommand();
	//SendData(theChangedSubject);
}

GnuContourPlot::GnuContourPlot(void) : TwoDGnuplotter(), contour_levels(20)
{ InitializeGnuplot(); }

GnuContourPlot::GnuContourPlot(FILE * gplotpipe) : TwoDGnuplotter(gplotpipe), contour_levels(20)
{ InitializeGnuplot(); }

GnuContourPlot::~GnuContourPlot(void)
{ contour_levels=-1; }

bool GnuContourPlot::InitializeGnuplot(void)
{	
	#ifdef _DEBUG
		if (gnuplot_pipe==NULL) return(false);
		if (contour_levels<1)  return(false);
	#endif

#ifndef _WIN32
	fprintf(gnuplot_pipe, "set terminal x11\n");
#endif
	fprintf(gnuplot_pipe, "set hidden3d\n");
	fprintf(gnuplot_pipe, "set title \"%s\"\n", title);
	fprintf(gnuplot_pipe, "set nosurface\n");
	fprintf(gnuplot_pipe, "set grid\n");
	fprintf(gnuplot_pipe, "set cntrparam levels %d\n", contour_levels); 
	fprintf(gnuplot_pipe, "set view 0,0\n");
	fprintf(gnuplot_pipe, "set contour\n");
	return(FlushGnuPlotter());
}


bool GnuContourPlot::SetContourLevels(int levels)
{
	if (levels<1) return(false);
	contour_levels=levels;
	return(true);
}

GnuSurfacePlot::GnuSurfacePlot(void) : TwoDGnuplotter(), phi_x(60.0), phi_z(30.0)
{ InitializeGnuplot(); }

GnuSurfacePlot::GnuSurfacePlot(FILE * gplotpipe) : TwoDGnuplotter(gplotpipe), phi_x(GetXAngle()), phi_z(GetZAngle())
{ InitializeGnuplot(); }

GnuSurfacePlot::~GnuSurfacePlot(void)
{ phi_x=0.0; phi_z=0.0; }


bool GnuSurfacePlot::SetAngles(Real phix, Real phiz) 
{ 
	phi_x=phix; 
	phi_z=phiz; 
	fprintf(gnuplot_pipe, "set view %g,%g\n", phi_x, phi_z);
	return(true);
}

bool GnuSurfacePlot::InitializeGnuplot(void)
{	
	#ifdef _DEBUG
		if (gnuplot_pipe==NULL) return(false);
	#endif

#ifndef _WIN32
	fprintf(gnuplot_pipe, "set terminal x11\n");
#endif
	fprintf(gnuplot_pipe, "set title \"%s\"\n", title);
	fprintf(gnuplot_pipe, "set hidden3d\n");
	fprintf(gnuplot_pipe, "set surface\n");
	fprintf(gnuplot_pipe, "set view %g,%g\n", phi_x, phi_z);
	fprintf(gnuplot_pipe, "set contour base \n");
	return(FlushGnuPlotter());
}

