#include "SplitConservedOP.h"
#include <typeinfo>
#include <stdio.h>

COPLinearPropagator::COPLinearPropagator(void) : DiagonalPropagator() { ; }
COPLinearPropagator::COPLinearPropagator(Real d, Real ps, Real t) : DiagonalPropagator(t), diffusivity(d), parttimesstrength(ps) { ; }
COPLinearPropagator::COPLinearPropagator(const COPLinearPropagator& prop) : DiagonalPropagator(prop), diffusivity(prop.GetDiffusivity()), parttimesstrength(prop.GetPartTimesStrength()) { ; }

Real COPLinearPropagator::ReDiagonal(const Point &k)
{
	int i;
	Real d;

	for (i=0, d=0; i<DIMENSION; i++) d+=2.0*(cos(k(i))-1)/delta(i)/delta(i); // diagonal elements of laplace

	d*=-d; // minus laplace squared

	return(d);
}
	

SplitConservedOP::SplitConservedOP(void) : SplitRightHandSide(), WorkSpace(), diffusivity(0.0), potential(0.0)
{
	// introspective
}

SplitConservedOP::SplitConservedOP(const SplitConservedOP &cop) : SplitRightHandSide(cop), WorkSpace(), diffusivity(cop.GetDiffusivity()), potential(cop.GetPotential())
{
	SetWorkSpace(cop.GetWorkSpace());
}

SplitConservedOP::~SplitConservedOP(void)
{	
	diffusivity=0.0;
	potential=0.0;
	if (&WorkSpace[0]!=ZERO) delete &WorkSpace[0];
	WorkSpace.RemoveAllFields();
}

bool SplitConservedOP::SetWorkSpace(const FieldCollection & field)
{
	int i;

	for (i=0; i<WorkSpace.GetNumberOfFields(); i++) if (&WorkSpace[i]!=ZERO) delete &WorkSpace[i];
	WorkSpace.RemoveAllFields();

	//cout << "ConservedOP::SetWorkSpace(" << field << ")" << endl;
	if (!WorkSpace.AddField(field[0].AllocCopy())) return(false);
	//cout << "ConservedOP::SetWorkSpace(" << field << ") .... workspace=" << workspace << endl;
	if (&WorkSpace[0]==ZERO) return(false);

	return(true);
}

bool SplitConservedOP::RHS(const FieldCollection &input, FieldCollection &output, int n)
{
	#ifdef _DEBUG
		//if (typeid(*workspace)!=typeid(*phi[0])) {
		//	cout << "ConservedOP::RHS workspace of different type as phi" << endl;
		//	cout << "workspace: id.name=" << (typeid(*workspace)).name() << endl;
		//	cout << "      phi: id.name=" << (typeid(*phi[0])).name() << endl;
		//	exit(-1);
		//}
		assert(&input[0]!=ZERO);
		assert(&output[0]!=ZERO);
		assert(&WorkSpace[0]!=ZERO);
		assert(0<=n && n<GetNumberOfRHS());
	#endif //_DEBUG

	Field &Phi=output[0], &Wsp=WorkSpace[0];

	switch(n) { 
	case 0: { // Laplace^2
			Equal[Phi][input[0]];
			Laplace[Phi];
			Times(-diffusivity)[Phi];
			Laplace[Phi];
			return(true);
		} break;
	case 1: { // PhaseField Term
			Equal[Phi][input[0]];
			PhaseFieldTerm(potential)[Phi];
			Laplace[Phi];
			return(true);
		} break;
	default: {
			cout << "SplitConservedOP::RHS(..., ..., n=" << n << ") n out of range - 0<=n<2" << endl;
			return(false);
		}
	}
	
	//Equal[Wsp][input[0]];
	//Equal[Phi][input[0]];

	//PhaseFieldTerm(potential)[Wsp];
	//Laplace[Phi];
	//Times(-diffusivity)[Phi];
	//Add[Phi][Wsp];
	//Laplace[Phi];
	return(true);
}


bool SplitConservedOP::Advance(const FieldCollection &input, FieldCollection &output, Real time, int n)
{
	#ifdef _DEBUG
		assert(&input[0]!=ZERO);
		assert(&output[0]!=ZERO);
		assert(0<=n && n<GetNumberOfRHS());
	#endif //_DEBUG

	Field &Phi=output[0], &Wsp=WorkSpace[0];

	switch(n) { 
	case 0: { // -D*Laplace^2 analytic ok
			Equal[output[0]][input[0]];
			L2Prop(time)[output[0]];
			return(true);
		} break;
	case 1: { // PhaseField Term not analytic
			return(false);
		} break;
	default: {
			cout << "AnalyticSplitConservedOP::RHS(..., ..., n=" << n << ") n out of range - 0<=n<2" << endl;
			return(false);
		}
	}
	return(true);
}
