#include <stdlib.h>
#include "OperatorOperand.h"
#include "DigitalMaterial.h"

Operator & Operator::operator[](Operand &x) { return(x.Apply(*this)); }
Operator & Operator::operator[](const Operand &x) { return(x.Apply(*this)); }

First::First(BinOperator *bop) : binop(bop) { ; }

FirstField::FirstField(BinOperator *bop) : First(bop) { ; }
Operator & FirstField::OperateOn(const Field *f) { return(binop->OperateOnFieldAnd(f)); }
Operator & FirstField::OperateOn(const SquareGridField *f) { return(binop->OperateOnFieldAnd(f)); }

FirstSquareGridField::FirstSquareGridField(BinOperator *bop) : First(bop) { ; }
Operator & FirstSquareGridField::OperateOn(const Field *f) { return(binop->OperateOnSquareGridFieldAnd(f)); }
Operator & FirstSquareGridField::OperateOn(const SquareGridField *f) { return(binop->OperateOnSquareGridFieldAnd(f)); }


BinOperator::BinOperator(void) : first(ZERO), firstfield(this), firstsquaregridfield(this), operand(ZERO) { ; }


//	-------------------------------------------------------------
//
//	Switches Operator back to initial state after operation is accomplished.
// 	It might be reasonable to have a BinOperator::Reset() function to save some dispatches
//	for example in
//
//	AddField Add;
//	SquareGridField Phi, Psi, Chi;
//
//	Add[Phi][Psi]; 	// Phi+=Psi;
//	Add[Phi][Chi]; 	// Phi+=Chi;
//
//
//	whereas without the "operand=ZERO;"
//
//
//	AddField Add;
//	SquareGridField Phi, Psi, Chi;
//	Add[Phi][Psi]; 	// Phi+=Psi;
//	Add[Chi];		// Phi+=Chi;
//
//	-------------------------------------------------------------


Operator & BinOperator::OperateOn(Field *f)
{
	if (operand) {
		first->OperateOn(f);
		operand=ZERO;  
	}
	else {
		first=&firstfield;
		field=f;
	}
	return(*this);
}
Operator & BinOperator::OperateOn(SquareGridField *f) 
{
	if (operand) {
		first->OperateOn(f);
		operand=ZERO;
	}
	else {
		first=&firstsquaregridfield;
		squaregridfield=f;
	}
	return(*this);
}


Operator & BinOperator::OperateOn(const Field *f)
{
	if (operand) {
		first->OperateOn(f);
		operand=ZERO;  
	}
	else WantNonConst("Field");
	return(*this);
}
Operator & BinOperator::OperateOn(const SquareGridField *f) 
{
	if (operand) {
		first->OperateOn(f);
		operand=ZERO;
	}
	else WantNonConst("SquareGridField");
	return(*this);
}




