#include <math.h>
#include <new>
#include "SimplePeriodicBC.h"

using namespace std;

SimplePeriodicBC::SimplePeriodicBC() : physical_size(0.0), helicity(0.0)
{
	//Introspective
}

SimplePeriodicBC::SimplePeriodicBC(const SimplePeriodicBC & spbc) : physical_size(spbc.GetPhysicalSize()), helicity(spbc.GetHelicity())
{
	//Introspective has been done by spbc!
}

BoundaryCondition * SimplePeriodicBC::AllocCopy(void) const
{
	SimplePeriodicBC * spbc = new(nothrow) SimplePeriodicBC(*this);
	#ifdef _DEBUG
		assert(spbc);
	#endif
	return(spbc);
}

SimplePeriodicBC::SimplePeriodicBC(const Point & box) : physical_size(box), helicity(0.0)
{
	//Introspective
	if (!physical_size.IsPositive()) physical_size.Zero();
	// Throw exeption???
}


bool SimplePeriodicBC::FoldBackIntoSupport(Point & point) const
{
	int i;
	for (i=0; i<DIMENSION; i++) {
		point[i]=(Real)fmod(point(i), physical_size(i));
		if (point(i)<0.0) point[i]+=physical_size(i);
	}
	return(true);
}

bool SimplePeriodicBC::SetPhysicalSize(const Point &x)
{
	if (x.IsPositive()) {
		physical_size=x;
		return(true);
	}
	else return(false);
}
