#include "SplitConservedOP.h"
#include <typeinfo>
#include <stdio.h>
#include <math.h>

COPLinearPropagator::COPLinearPropagator(void) : DiagonalPropagator() { ; }
COPLinearPropagator::COPLinearPropagator(const COPLinearPropagator& prop) : DiagonalPropagator(prop), 
	diffusivity(prop.GetDiffusivity()), 
	part(prop.GetPart()),
	strength(prop.GetStrength()), 
	ptimess(prop.GetPart()*prop.GetStrength())
{ ; }

Real COPLinearPropagator::ReDiagonal(const Point &k)
{
	int i;
	Real d;

	for (i=0, d=0; i<DIMENSION; i++) d+=2.0*(cos(k(i))-1)/delta(i)/delta(i); // diagonal elements of laplace

	d*=-d*diffusivity+ptimess; // minus laplace squared times diffusivity
	//d*=-d*diffusivity; // minus laplace squared times diffusivity

	return(d);
}
	
PartOfPhaseFieldTerm::PartOfPhaseFieldTerm(void) : ApplyFunction(), strength(0.0), part(0.0) { ; }

SplitConservedOP::SplitConservedOP(void) : ConservedOPParameters(0.0, 0.0), SplitRightHandSide(), WorkSpace()
{
	// introspective
}

SplitConservedOP::SplitConservedOP(const SplitConservedOP &cop) : SplitRightHandSide(cop), ConservedOPParameters(cop), WorkSpace(), part(cop.GetPart())
{
	SetWorkSpace(cop.GetWorkSpace());
}

SplitConservedOP::~SplitConservedOP(void)
{	
	diffusivity=0.0;
	potential=0.0;
	if (&WorkSpace[0]!=ZERO) delete &WorkSpace[0];
	WorkSpace.RemoveAllFields();
}

bool SplitConservedOP::SetWorkSpace(const FieldCollection & field)
{
	int i;

	for (i=0; i<WorkSpace.GetNumberOfFields(); i++) if (&WorkSpace[i]!=ZERO) delete &WorkSpace[i];
	WorkSpace.RemoveAllFields();

	//cout << "ConservedOP::SetWorkSpace(" << field << ")" << endl;
	if (!WorkSpace.AddField(field[0].AllocCopy())) return(false);
	//cout << "ConservedOP::SetWorkSpace(" << field << ") .... workspace=" << workspace << endl;
	if (&WorkSpace[0]==ZERO) return(false);

	return(true);
}

bool SplitConservedOP::RHS(const FieldCollection &input, FieldCollection &output, int n)
{
	#ifdef _DEBUG
		assert(&input[0]!=ZERO);
		assert(&output[0]!=ZERO);
		assert(&WorkSpace[0]!=ZERO);
		assert(0<=n && n<GetNumberOfRHS());
	#endif //_DEBUG

	Field &Phi=output[0], &Wsp=WorkSpace[0];

	switch(n) { 
	case 0: { // Laplace^2
			Equal[Phi][input[0]];
			Equal[Wsp][input[0]];
			Laplace[Phi];
			Times(-diffusivity)[Phi];
			Times(part*potential)[Wsp];
			Add[Phi][Wsp];
			Laplace[Phi];
			Times(0.5)[Phi];
			return(true);
		} break;
	case 1: { // PhaseField Term
			Equal[Phi][input[0]];
			NonlinearPart(potential, part)[Phi];
			Laplace[Phi];
			return(true);
		} break;
	case 2: { // Laplace^2
			Equal[Phi][input[0]];
			Equal[Wsp][input[0]];
			Laplace[Phi];
			Times(-diffusivity)[Phi];
			Times(part*potential)[Wsp];
			Add[Phi][Wsp];
			Laplace[Phi];
			Times(0.5)[Phi];
			return(true);
		} break;
//	case 0: { // PhaseField Term
//			Equal[Phi][input[0]];
//			NonlinearPart(potential, part)[Phi];
//			Laplace[Phi];
//			return(true);
//		} break;
//	case 1: { // Laplace^2
//			Equal[Phi][input[0]];
//			Equal[Wsp][input[0]];
//			Laplace[Phi];
//			Times(-diffusivity)[Phi];
//			Times(part*potential)[Wsp];
//			Add[Phi][Wsp];
//			Laplace[Phi];
//			return(true);
//		} break;
	default: {
			cout << "SplitConservedOP::RHS(..., ..., n=" << n << ") n out of range - 0<=n<2" << endl;
			return(false);
		}
	}
	return(true);
}

bool SplitConservedOP::Advance(FieldCollection &input, Real time, int n)
{
	#ifdef _DEBUG
		assert(&input[0]!=ZERO);
		assert(0<=n && n<GetNumberOfRHS());
	#endif //_DEBUG

	switch(n) { 
		case 0: { LinProp(0.5*time)[input[0]]; return(true); } break;
		case 1: { return(false); } break;
		case 2: { LinProp(0.5*time)[input[0]]; return(true); } break;
		//case 0: { return(false); } break;
		//case 1: { LinProp(time)[input[0]]; return(true); } break;
		default: {
			cout << "AnalyticSplitConservedOP::RHS(..., ..., n=" << n << ") n out of range - 0<=n<2" << endl;
			return(false);
		}
	}
	return(true);
}
