#include "SplitEuler.h"

SplitEuler::SplitEuler(void) : OperatorSplitFieldMover(), step_size(0.0) 
{
	// introspective things and so on
}

SplitEuler::SplitEuler(Real step, SplitRightHandSide *RHS) : OperatorSplitFieldMover(RHS), step_size(step)
{
	// introspective
}

SplitEuler::SplitEuler(const SplitEuler &exeuler) : OperatorSplitFieldMover(exeuler), step_size(exeuler.GetStepSize())
{
	SetWorkSpace(exeuler.GetWorkSpace());
}

SplitEuler::~SplitEuler(void) 
{
	//int i;
	//step_size=0.0;
	//for (i=0; i<number_of_workspaces; i++) if (work_space[i]!=ZERO) delete work_space[i];
	//delete []work_space;
	FreeWorkSpace();
}

void SplitEuler::FreeWorkSpace(void)
{
	int i;
	for (i=0;i<WorkSpace.GetNumberOfFields(); i++) if (&WorkSpace[i]!=ZERO) delete &WorkSpace[i];
	WorkSpace.RemoveAllFields();
}

bool SplitEuler::SetWorkSpace(const FieldCollection &Fields)
{
	int i, number;
	bool flag=false;
	Field *NewField;

	number=Fields.GetNumberOfFields();

	if (number!=WorkSpace.GetNumberOfFields()) 											flag=true;
	else 			for (i=0; i<number; i++) if (&WorkSpace[i]==ZERO) 					flag=true;
	if (!flag) 	for (i=0; i<number; i++) if (typeid(WorkSpace[i])!=typeid(Fields[i])) 	flag=true;

	if (flag) {
		FreeWorkSpace();
		for (i=0; i<number; i++) {
			flag=WorkSpace.AddField(NewField=Fields[i].AllocCopy());
			if (!flag || NewField==ZERO) return(false);
		}
	}
	return(true);
}
bool SplitEuler::Move(FieldCollection & Fields)
{
	int i;
	int n;

	#ifdef _DEBUG
		//if (work_space==ZERO)  return(false);
		//if (n!=number_of_workspaces) return(false);
		//for (i=0; i<number_of_workspaces; i++) 
		//	if (typeid(*work_space[i])!=typeid(*field[i])) return(false);
	#endif

	//for (i=0; i<number_of_workspaces; i++) Equal[*work_space[i]][*field[i]]; 

	for (n=0; n<split_rhs->GetNumberOfRHS(); n++) {
		if (!split_rhs->RHS(Fields, WorkSpace, n)) return(false);

		for (i=0; i<Fields.GetNumberOfFields(); i++) {
			Times(step_size)[WorkSpace[i]];
			Add[Fields[i]][WorkSpace[i]];
		}
	}
	return(true);
}

bool SplitEuler::SafeMove(FieldCollection & fields)
{ 
  int i;
  if (split_rhs==ZERO) return(false);
  for (i=0;i<fields.GetNumberOfFields();i++){
	 if (&fields[i]==ZERO) return(false);
  }
  if (!SetWorkSpace(fields)) return(false);
  
  return(Move(fields));
}
