%module IPAF

%{
#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif //_MSC_VER
#include "Numeric/arrayobject.h"
#include "AdvancingFront.h"
#include "Face.h"
#include "Tetrahedron.h"
#include "Vector.h"
typedef std::set<Face> Front ;
#ifndef DIMENSION
#define DIMENSION  3 
#endif
%}

%init
%{
    import_array() ;
%}

%include typemaps.i
%include DimensionArrayTypeMap.i

class AdvancingFront
{
public:
    void Advance();

%addmethods
{
    AdvancingFront(PyObject* front0, PyObject* pos0,  
                   int cutoff, int nAtomic = 0, int atomicCutoff = -1) 
    {
        if ((front0 == Py_None || PyArray_Check(front0)) && PyArray_Check(pos0) ) 
		{
            Front front ;
            if (front0 != Py_None) {
                PyArrayObject* frontArray = (PyArrayObject*)front0 ;
                const int frontSize = PyArray_Size(front0)/3 ; 
				int(*fdata)[3] = (int(*)[3])(frontArray->data);
                for (int f = 0; f< frontSize; ++f) 
                    front.insert(Face(fdata[f][0],fdata[f][1],fdata[f][2])) ;
            }
            PyArrayObject* posArray = (PyArrayObject*)pos0 ;
            const int posSize = PyArray_Size(pos0)/3 ; 
            std::vector<Vector<int> > positions ;
            int(*pdata)[3] =  (int(*)[3])(posArray->data);
            for (int p = 0; p< posSize; ++p) 
                positions.push_back(Vector<int>(pdata[p][0],pdata[p][1],pdata[p][2])) ;
            return new AdvancingFront(front, positions, cutoff, 
                 nAtomic, atomicCutoff) ;
        }
        else {
            PyErr_SetString(PyExc_TypeError, "not a numpy array") ;
            return (0) ;
        }
    }
    void SetBoxSize(const int box[3], PyObject *wrapPy = 0)
	{
		if(wrapPy == 0)
		{
			self->SetBoxSize(box);
			return;
		}
		bool wrap[3];
		if(PyList_Check(wrapPy))
		{
			for(int i = 0; i < 3; ++i)
			{
				PyObject *o = PyList_GetItem(wrapPy,i);
				if(PyInt_Check(o)) wrap[i] = (PyInt_AsLong(o) != 0);
				else
				{
					PyErr_SetString(PyExc_TypeError,"list must contain int");
				}
			}
		}
		else
		{
			PyErr_SetString(PyExc_TypeError,"expected a list of ints");
		}
		self->SetBoxSize(box, wrap);
	}

    PyObject* GetTetrahedra() {
        const std::list<Tetrahedron>& results = self->GetTetrahedra();
        int dim[2] ;
        dim[0] = results.size() ;
        dim[1] = 4 ;
        PyArrayObject* tets = (PyArrayObject *)PyArray_FromDims(2, dim, 'i');
        int(*data)[4] =  (int(*)[4])(tets->data);
        std::list<Tetrahedron>::const_iterator it ;
        for (it = results.begin(); it != results.end(); ++it) 
            memcpy(data++, it->nodes, 4*sizeof(int)) ;
        return (PyObject*) tets ;     
    }
}
    
}

