# -*- makefile -*-
# Processing
# CXX is C++ files.
CXXFILES = $(filter %.cpp, $(ALLFILES))
CXXOBJECTS = $(CXXFILES:%.cpp=$(ARCH)/%.o)

# Dependencies get their own variable so we don't try to include
# an empty list of dependencies.
DFILES = $(CXXFILES:%.cpp=deps/%.d) 

CFLAGS += $(OPTS)

INTERFACE = $(MODULE).i
WRAP = $(ARCH)/$(MODULE)_wrap.cpp
IOBJ = $(ARCH)/$(MODULE)_wrap.o
SWIGMODULE = $(ARCH)/$(MODULE)cmodule$(SO)

.PHONY: ALL clean $(SUBDIRS)

ECHO =@ # set to @ to hide everything.  Unset to see it all.

# You have to make the library before the subdirectories or the
# $(ARCH)  $(OBJDIR) subdirectory won't exist.
ALL: $(LIBRARY) $(SWIGMODULE)

# Ar only the updated objects. $? are files among $(OBJECTS) newer than $(LIBRARY).
$(LIBRARY): $(CXXOBJECTS)
	-mkdir -p $(ARCH)
	-$(AR) -r $@ $?

# Create python extension from SWIG

$(IOBJ) : $(ARCH) 
	$(ECHO)$(SWIG) -c++ -python $(SWIGOPT) $(INCLUDE) -DDIMENSION=$(DIM) -DDIMENSIONIS$(DIM) -module $(MODULE) -o $(WRAP) $(INTERFACE)
	$(ECHO)$(CXX) -c $(CCSHARED) $(OPTS) $(CFLAGS) $(WRAP) $(INCLUDE) $(PYTHON_INCLUDE) -o $@

$(SWIGMODULE): $(IOBJ) 
	$(ECHO)$(CXXSHARED) $(CFLAGS) $(IOBJ) $(LIBRARY) $(CPP_DLLIBS) -DDIMENSION=$(DIM) -DDIMENSIONIS$(DIM) -o $@


#
deps/%.d: %.cpp
	$(ECHO)echo Making dependencies for $<
	$(ECHO)-mkdir -p deps
	$(ECHO)-rm -f $@
	$(ECHO)$(CPP) $(CFLAGS) $(INCLUDE) $< \
	| sed 's/\($*\)\.o[ :]*/$(ARCH)\/\1.o deps\/$(notdir $@) : /g' | grep -v '/usr/include' > $@

ifneq ($(DFILES),)
include $(DFILES:%.cpp=deps/%.d)
endif

$(ARCH)/%.o: %.cpp
	$(ECHO)echo Compiling $<
	$(ECHO)-mkdir -p $(ARCH)
	$(ECHO)$(CXX) $(CFLAGS) -c $< $(INCLUDE) -o $@

clean::
	-rm deps/*
	-rm $(ARCH)/*.o $(ARCH)/*.a

clean::
	$(foreach dir, $(SUBDIRS), $(MAKE) -C $(dir) clean; )

distclean:: clean
	-rm -rf $(ARCH) deps



