#
# The stuff related to MD
#
import sys
#sys.path.append('H:\Users\\tcretegn\DigitalMaterial\src\MD\WinSWIGBuild')
sys.path.append('H:\Users\dchen\QC\SWIGBuild\WindowsBuild')

import QC
from Numeric import *
from LinearAlgebra import *
from math import *

def ClosestLatticeSite(lattice, position):
    tmpList = lattice.LatticeCoordFromCartCoord(position)
    (tmpList[0], tmpList[1]) = (floor(tmpList[0]+0.5), floor(tmpList[1]+0.5))
    return lattice.CartCoordFromLatticeCoord(tmpList)

def Interpolate(nodes0, nodesdef, pos):
    arr = zeros((3,3), Float)
    arr[:,0] = nodes0[:,0]
    arr[:,1] = nodes0[:,1]
    arr[:,2] = ones(3,Float)
    inv_arr = inverse(arr)
    posdef = zeros((len(pos),2), Float)
    for i in range(len(pos)):
        for j in range(3):
            posdef[i] = posdef[i] + nodesdef[j]*(inv_arr[0][j]*pos[i][0] + inv_arr[1][j]*pos[i][1] + inv_arr[2][j])
    return posdef


#Strategy: THE DECORATION IS IN THE UNDEFORMED CRYSTAL !
#
#* Let r_d the decoration radius (in general bigger than the cutoff distance in order to take the deformations
#   into account)
#* Instanciate a NeighborLocator for the master atoms: masterNL  (cutoffDistance unspecified at this time)
#* Instanciate the slaved atoms
#* For each element E in the list of elements \mathcal{E} :
#   * Instanciate a temporary ListOfAtoms : LOAE
#   * Find the center of the circumventing sphere: let \vec{x}_E and r_E the center and the radius of the sphere
#   * If r_E < r_d : decorate the whole element
#   * Else : 
#       * let \mathcal{S} the set of master atoms which are within a distance r_E + r_d from \vec{x}_E
#       * for each M in \mathcal{S} :
#           * for each face F_i of E : calculate distance between M and F_i and let dME the minimal distance over i.
#           * If dME <= r_d :
#               * Instanciate a temporary ListOfAtoms: LOAtmp
#               * Instanciate a SphericalClusterInitializer with a center at M and radius r_d.
#               * Create LOAtmp
#               * Remove all the atoms of LOAtmp that are out of the element
#               * Add LOAtmp to LOAE
#       * OverlapPruner on LOAE
#   * Add LOAE to the list of slave atoms

def Decorate(elemIndex, lattice, mastersPos, pyelems, rd, vtkMeshData,sphereCenterIndex=None):
    # for each node of element i, find the closest lattice site
    roundedPos = zeros((3,2), Float)
    for i in range(3):
        roundedPos[i] = ClosestLatticeSite(lattice, mastersPos[pyelems[elemIndex][i]]- \
                             mastersPos[pyelems[elemIndex][0]])

#    vtkMeshData.SetElementReferencePosition(elemIndex, roundedPos)
    clusterInit = QC.PolyClusterInitializer(roundedPos,lattice)
    LOAtmp = QC.ListOfSlavedAtoms()

    if sphereCenterIndex:
        sphereInit = QC.SphericalClusterInitializer(rd, lattice)
        compInit = QC.CompositeClusterInitializer(clusterInit, sphereInit, lattice)
        for sphereI in range(len(sphereCenterIndex)):
            sphereInit.SetCenter(ClosestLatticeSite(lattice, mastersPos[sphereCenterIndex[sphereI]] - \
                                                             mastersPos[pyelems[elemIndex][0]]))
            LOAtmp2 = QC.ListOfSlavedAtoms()
            compInit.Create(LOAtmp2)
            LOAtmp2.AllocateElements(vtkMeshData.GetElement(elemIndex))
            LOAtmp.AddList(LOAtmp2)
            del LOAtmp2
        del sphereInit
        del compInit
    else:        
        clusterInit.Create(LOAtmp)
        LOAtmp.AllocateElements(vtkMeshData.GetElement(elemIndex))
    del clusterInit
    # interpolation:
    if LOAtmp.GetNumber() > 0:
        pos = LOAtmp.GetCartesianPositions().CopyAsNumPyArray()
        LOAtmp.SetReferencePositionInsideElement(LOAtmp.GetCartesianPositions())
        pos2 = Interpolate(roundedPos, array([mastersPos[pyelems[elemIndex][0]],  \
                    mastersPos[pyelems[elemIndex][1]],mastersPos[pyelems[elemIndex][2]] ]), pos)

        for i in range(len(pos2)):
            for j in range(2):
                LOAtmp.SetCartesianPosition(i,j,pos2[i][j])
    del roundedPos

#    for i in xrange(LOAtmp.GetNumber()):
#        LOAtmp.SetElement(i, vtkMeshData.GetElement(elemIndex))
    return LOAtmp

def GetCenterAndRadiusOfTriangle(coords):
    d2 = sum(coords*coords,1)
    mat = zeros((2,2), Float)
    vect = zeros(2,Float)
    for i in range(2):
        mat[i] = coords[i+1]-coords[i]
        vect[i] = 0.5*(d2[i+1]-d2[i])
    center = matrixmultiply(inverse(mat), vect) 
    radius = sqrt(sum((coords[0]-center)**2))
    return [center, radius]

  
def CalculateNodesWeightFromSlaves(masters, slaves) :
    '''We cheat! We know slaves are everywhere'''
    
    weight = ones((masters.GetNumber()), Float)
    for i in xrange (slaves.GetNumber()):
        slavedShape = slaves.ShapeFunctions(i)
        for j in range(3):
            weight[slaves.GetElementNode(i, j)] = weight[slaves.GetElementNode(i, j)] + slavedShape[j]   

    return weight

def CalculateNodesWeight(mastersPos, masters, pyelems, lattice) :
    '''We use angle stuff to reduce the weight of atomistic resolution to unit.
    For other situations, we use continuum limit 1/3.
    We shall rewrite the routine so it uses shape function (recursively on the
    structural grids) to evaluate the weight'''
    
    weight = zeros((masters.GetNumber()), Float)
    roundedPos = zeros((3,2), Float)
    TOL = lattice.GetVolumeOfCell()*0.001
    for i in range(len(pyelems)):
        for j in range(3):
            roundedPos[j] = ClosestLatticeSite(lattice, mastersPos[pyelems[i][j]]- \
                                                        mastersPos[pyelems[i][0]])
        area = CalculateAreaOfTriangle(roundedPos)
        areaOnethird = 0.3333333333333333333333 * area
        if ( abs(2.0*area - lattice.GetVolumeOfCell()) < TOL ): #unit cell is a parallelepiped 
            for j in range(3):
                weight[pyelems[i][j]] = weight[pyelems[i][j]] + \
                            CalculateTriangleWeightFromAngle(roundedPos, j)
        else:
            for j in range(3):
                weight[pyelems[i][j]] = weight[pyelems[i][j]] + areaOnethird/lattice.GetVolumeOfCell()

    return weight

def CalculateAreaOfTriangle(corners) :
    """Use determinant form to calculate the area of triangle [O'Rourke]"""
    mat = ones((3,3), Float)
    for i in range(3):
        mat[i][0:2] = corners[i]
    area = 0.5*determinant(mat)
    if area < 0: print "Negative area for triangle. Something Error!!"
    return area

def CalculateTriangleWeightFromAngle(corners, index):
    vect0 = corners[(index+1)%3]-corners[index]
    vect1 = corners[(index+2)%3]-corners[index]
    angle = arccos(innerproduct(vect0,vect1)/sqrt(sum(vect0*vect0)*sum(vect1*vect1)))
    TWOPI = 2.0*3.141592653589793238462643
    return angle/TWOPI
    
def crossProduct2D(vectora, vectorb):
    return vectora[0]*vectorb[1] - vectora[1]*vectorb[0]

def ComputeGradientOfElement(elemIndex, lattice, pyelems, mastersPos, mastersNewPos):
    """compute deformation gradient for linear triangle"""
    # for each node of element i, find the closest lattice site
    # XXX if we really need this we should do vtkMeshData.GetElementReferencePosition
    roundedPos = zeros((3,2), Float)
    roundedNewPos = zeros((3,2), Float)
    elemNodesDisp = zeros((3,2), Float)
    for i in range(3):
        roundedPos[i] = ClosestLatticeSite(lattice, mastersPos[pyelems[elemIndex][i]]- \
                                                    mastersPos[pyelems[elemIndex][0]])
    for i in range(3):
        elemNodesDisp[i] = mastersNewPos[pyelems[elemIndex][i]] - roundedPos[i]

    a = zeros(3, Float)
    b = zeros(3, Float)
    c = zeros(3, Float)
    det = 2.*CalculateAreaOfTriangle(roundedPos)
    for i in range(3):
        j = (i+1)%3
        m = (i+2)%3
        a[i] = roundedPos[j][0]*roundedPos[m][1] - roundedPos[m][0]*roundedPos[j][1]
        b[i] = roundedPos[j][1]-roundedPos[m][1]
        c[i] = roundedPos[m][0]-roundedPos[j][0]
    
    a = a/det ;
    b = b/det ;
    c = c/det ;
    # deformation gradient organized as (4,1) vector with the Fxx Fxy Fyx and Fyy order
    defGrad = zeros(4, Float)   

    for i in range(3):
        defGrad[0] = defGrad[0] + (1.+b[i])*elemNodesDisp[i][0]
        defGrad[1] = defGrad[1] + c[i]*elemNodesDisp[i][0]
        defGrad[2] = defGrad[2] + b[i]*elemNodesDisp[i][1]
        defGrad[3] = defGrad[3] + (1.+c[i])*elemNodesDisp[i][1]

    dudx = dudy = dvdx = dvdy = 0
    for i in range(3):
        dudx = dudx + b[i]*elemNodesDisp[i][0]
        dudy = dudy + c[i]*elemNodesDisp[i][0]
        dvdx = dvdx + b[i]*elemNodesDisp[i][1]
        dvdy = dvdy + c[i]*elemNodesDisp[i][1]

    # epsilon_xx, epsilon_yy and epsilon_xy
    smallStrain = zeros(3, Float)
    
    smallStrain[0] = dudx
    smallStrain[1] = dvdy
    smallStrain[2] = 0.5*(dudy + dvdx) 

    lagrangianStrain = zeros(3, Float)
    lagrangianStrain[0] = dudx + 0.5 * (dudx**2+dvdx**2)
    lagrangianStrain[1] = dvdy + 0.5 * (dudy**2+dvdy**2)
    lagrangianStrain[2] = 0.5 * ( dudy + dvdx + dudx*dudy + dvdx*dvdy )
    
    return [defGrad, smallStrain, lagrangianStrain]

def computeSecondInvariantOfStrainsOfElements(strainOfElements):
    """compute second invariant"""
    tmp = transpose(strainOfElements)
    strainIndex = abs(tmp[2]**2+tmp[0]*tmp[1])
    for i in xrange(len(strainIndex)):
        strainIndex[i] = sqrt(strainIndex[i])

    return strainIndex        

def ComputeL2Norm(valueArray):
    """compute norm of an array"""
    return sqrt(matrixmultiply(transpose(valueArray), valueArray))


def ComputeDefGradOfNodes(masters, pyelems, elemsDefGrad):
    '''Compute deformation gradient at node. Here we just simply compute the nodal
    deformation gradient by averaging the deformation gradients over all elements
    in contact with the node'''
    
    numElementsOfNode = zeros((masters.GetNumber()), Int)
    nodesDefGrad = zeros((masters.GetNumber(), 4), Float)
    for i in xrange(len(pyelems)):
        for j in range(3):
            numElementsOfNode[pyelems[i][j]] = numElementsOfNode[pyelems[i][j]] + 1
            nodesDefGrad[pyelems[i][j]] = nodesDefGrad[pyelems[i][j]] + elemsDefGrad[i]

    for i in xrange(masters.GetNumber()):
        nodesDefGrad[i] = nodesDefGrad[i]/numElementsOfNode[i]

    return nodesDefGrad

def ComputeDefGradErrorNormOfElements(defGradOfElements, defGradOfNodes, pyelems):
    '''XXX we need to doublecheck with the formulation but for now let's take
    1/3*(F-Fe)T(F-Fe) '''
    
    onethird = 0.3333333333333333333333 
    errorNormOfElements = zeros(len(pyelems), Float)
    for i in xrange(len(defGradOfElements)):
        for j in xrange(3):
            errorNormOfElements[i] = errorNormOfElements[i] + onethird* \
                matrixmultiply(transpose(defGradOfNodes[pyelems[i][j]] - defGradOfElements[i]), \
                               (defGradOfNodes[pyelems[i][j]] - defGradOfElements[i]))

#    errorNorOfElements = sqrt(errorNorOfElements)
    return errorNormOfElements

def ComputeRefineRatioOfElements(errorNormOfElements, dispNormOfElements):
    prescribeError = 0.05 # 5% prescibed error
    refineRatioOfElements = zeros(len(dispNormOfElements), Float)
    errorMean = 0.
    for i in xrange(len(dispNormOfElements)):
        errorMean = errorMean + dispNormOfElements[i]**2 + errorNormOfElements[i]**2

    errorMean = prescribeError*sqrt(errorMean/len(dispNormOfElements))
    return errorNormOfElements/errorMean
    
def createNewNodesForRemesh(lattice, pyelems, mastersPos, strainIndex, prescNum):
    addedPositions = []
    sphereInit = QC.SphericalClusterInitializer(lattice.GetLatticeSpacing(), lattice)
    for elemIndex in xrange(len(pyelems)):
        roundedPos = zeros((3,2), Float)
        for i in range(3): roundedPos[i] = ClosestLatticeSite(lattice, mastersPos[pyelems[elemIndex][i]]- \
                             mastersPos[pyelems[elemIndex][0]]) 
        if (strainIndex[elemIndex] > prescNum) and CalculateAreaOfTriangle(roundedPos) > lattice.GetVolumeOfCell()/2.:
            center = zeros(2,Float)
            for j in range(3) : center = center + roundedPos[j]
            sphereInit.SetCenter(center / 3.)
            elemInit = QC.PolyClusterInitializer(roundedPos,lattice)    
            sphereInElem = QC.CompositeClusterInitializer(sphereInit, elemInit, lattice)
            tmpLOA = QC.PrimitiveListOfAtoms()
            sphereInElem.Create(tmpLOA)
            tmpPos = tmpLOA.GetCartesianPositions().CopyAsNumPyArray()
            minIndex, minimum = -1, 1.e10
            for i in range(tmpLOA.GetNumber()):
                if( sum((tmpPos[i]-center)**2) < minimum and sum((tmpPos[i]-roundedPos[0])**2) > 0.001*lattice.GetLatticeSpacing()**2
                    and sum((tmpPos[i]-roundedPos[1])**2) > 0.001*lattice.GetLatticeSpacing()**2 and
                    sum((tmpPos[i]-roundedPos[2])**2) > 0.001*lattice.GetLatticeSpacing()**2 ):
                    minIndex = i
                    minimum = sum((tmpPos[i]-center)**2)
            if minIndex > -1:
                tmp = Interpolate(roundedPos, array([mastersPos[pyelems[elemIndex][0]],  \
                    mastersPos[pyelems[elemIndex][1]],mastersPos[pyelems[elemIndex][2]] ]), array([tmpPos[minIndex]]))
                addedPositions.append(tmp[0])

    addedMasters = QC.ListOfMasterAtoms()
    addedMasters.SetNumber(len(addedPositions))
    for j in range(addedMasters.GetNumber()):
        addedMasters.SetCartesianPosition(j,0,addedPositions[j][0])
        addedMasters.SetCartesianPosition(j,1,addedPositions[j][1])
    return addedMasters


    