from Numeric import *
from VTK import *

class Mesh3DDisplay:
    """A Mesh3DDisplay show a a set of meshes in 3D."""

    def __init__(self, Delaunay = None, AtomData = None, CellColorMap = None, \
                 PointColorMap = None, ColorMapTitle = None, ColorMapRange = None):
        self.delny3d = Delaunay
        if (AtomData):
            self.dataToVTKPoints(AtomData)
        self.InitDisplay()
        self.images = []
        if (CellColorMap or PointColorMap):
            print "need to implement ColorMap for 3D"

    def ShowEdges(self,TubeSize = 0.1):
        extract = vtkExtractEdges()
        extract.SetInput(self.delny3d.GetOutput())
        tubes = vtkTubeFilter()
        tubes.SetInput(extract.GetOutput())
        tubes.SetRadius(TubeSize)
        tubes.SetNumberOfSides(6)
        mapEdges = vtkPolyDataMapper()
        mapEdges.SetInput(tubes.GetOutput())
        edgeActor = vtkActor()
        edgeActor.SetMapper(mapEdges)
        edgeActor.GetProperty().SetColor(0.2,0.63,0.79)
        edgeActor.GetProperty().SetSpecularColor(1,1,1)
        edgeActor.GetProperty().SetSpecular(0.3)
        edgeActor.GetProperty().SetSpecularPower(20)
        edgeActor.GetProperty().SetAmbient(0.2)
        edgeActor.GetProperty().SetDiffuse(0.8)
        self.ren.AddActor(edgeActor)

    def ShowNodes(self,BallSize = 0.05):
        ball = vtkSphereSource()
        ball.SetRadius(BallSize)
        ball.SetThetaResolution(12)
        ball.SetPhiResolution(12)
        balls = vtkGlyph3D()
        balls.SetInput(self.delny3d.GetOutput())
        balls.SetSource(ball.GetOutput())
        mapBalls = vtkPolyDataMapper()
        mapBalls.SetInput(balls.GetOutput())
        ballActor = vtkActor()
        ballActor.SetMapper(mapBalls)
        ballActor.GetProperty().SetColor(1.0,0.41,0.7)
        ballActor.GetProperty().SetSpecularColor(1,1,1)
        ballActor.GetProperty().SetSpecular(0.3)
        ballActor.GetProperty().SetSpecularPower(20)
        ballActor.GetProperty().SetAmbient(0.2)
        ballActor.GetProperty().SetDiffuse(0.8)
        self.ren.AddActor(ballActor)

    def dataToVTKPoints(self,data):
        self.inputPoints = vtkPoints()
        for i in range(len(data)):
            self.inputPoints.InsertPoint(i, data[i][0], data[i][1], data[i][2])

    def ShowAtoms(self,
                radius=0.05, phiResolution=5, thetaResolution=5):

        inputData = vtkPolyData()
        inputData.SetPoints(self.inputPoints)

        balls = vtkSphereSource()
        balls.SetRadius(radius)
        balls.SetPhiResolution(phiResolution)
        balls.SetThetaResolution(thetaResolution)

        glyphPoints = vtkGlyph3D()
        glyphPoints.SetInput(inputData)
        glyphPoints.SetSource(balls.GetOutput())
        glyphMapper = vtkPolyDataMapper()
        glyphMapper.SetInput(glyphPoints.GetOutput())

        glyph = vtkActor()
        glyph.SetMapper(glyphMapper)
        glyph.GetProperty().SetDiffuseColor(0.9, 0.2, 0.2)
        glyph.GetProperty().SetSpecular(0.3)
        glyph.GetProperty().SetSpecularPower(30)

        self.ren.AddActor(glyph)
        
        
    def Show(self):
        self.iren.Initialize()
        self.iren.Start()
        
    def InitDisplay(self):
        self.ren = vtkRenderer()
        self.renWin = vtkRenderWindow()
        self.renWin.AddRenderer(self.ren)
        self.renWin.SetSize(400,400)

        self.iren = vtkRenderWindowInteractor()
        self.iren.SetRenderWindow(self.renWin)

    def saveImage(self, filename):
#        image_filename = 'temp.ppm'
        image_filename = filename
        self.renWin.SetFileName(image_filename)
        self.renWin.SaveImageAsPPM()
        print "Saving image to %s" % image_filename
        self.images.append(image_filename)
        return

    def saveTIFFImage(self, filename = None):
        w2if = vtkWindowToImageFilter()
        w2if.SetInput(self.renWin)
        writer = vtkTIFFWriter()
        writer.SetInput(w2if.GetOutput())
        if (filename):
            writer.SetFileName(filename)
        else:
            writer.SetFileName("temp.tiff")
        writer.Write()
        return

    