#!/usr/local/bin/python

from VTK import *
from Numeric import *
from Mesh2DDisplay import Mesh2DDisplay

def vtkMesh2D(Positions=None,Polygons=None):
    """vtkMesh2D takes points in positions and edges define the outer boundary and
    return vtkDelaunay2D"""
    points = vtkPoints()
    for i in range(len(Positions)):
        points.InsertPoint(i,Positions[i][0],Positions[i][1],0.0)

    polydata = vtkPolyData()
    polydata.SetPoints(points)

    if (Polygons):
        polyconstraint = vtkPolyData()
        polypolygon = vtkPolygon()
        polyconstraint.Allocate(1,1)

        polypolygon.GetPointIds().SetNumberOfIds(len(Polygons))
        for i in range(len(Polygons)):
            polypolygon.GetPointIds().SetId(i,Polygons[i][0])

        polyconstraint.InsertNextCell(polypolygon.GetCellType(), polypolygon.GetPointIds())

    # triangulate them
    #
    delny = vtkDelaunay2D()
    delny.SetInput(polydata)
    if (Polygons):
        delny.SetSource(polyconstraint)
    delny.BoundingTriangulationOff()
    delny.SetTolerance(0.001)
    delny.SetAlpha(0.0)
    delny.Update()
    return delny

def vtkQueryElements(Delaunay =  None):
    """return connectivity of elements using NumPy array"""
    pdata = vtkPolyData()
    pdata = Delaunay.GetOutput()
    pyelems = zeros((pdata.GetNumberOfCells(),3))  # hack on 3 nodes
    ptIds =vtkIdList()
    for i in range(0,pdata.GetNumberOfCells()):
        pdata.GetCellPoints(i,ptIds)
        for j in range(0,ptIds.GetNumberOfIds()):
            assert (ptIds.GetNumberOfIds() == 3), "Number of nodes != 3"
#            print  ptIds.GetId(j)
            pyelems[i][j] = ptIds.GetId(j)
    return pyelems
        
    
if __name__ == '__main__':
    pynodes= array([[0,2],[1,2],[2,2],
                    [0,1],[1,1],[2,1],
                    [0,0],[1,0],[2,0]]) #2d
    pyedges = array([[0,3],[3,6],[6,4],[4,7],[7,8],[8,5],[5,2],[2,1],[1,0]])
    delny = vtkMesh2D(Positions=pynodes, Polygons=pyedges)
# query the connectivity
    pyelems = vtkQueryElements(Delaunay=delny)
# display the meshes
#    showmeshes = Mesh2DDisplay(Delaunay=delny)
#    showmeshes.ShowNodes(BallSize = 0.1)
#    showmeshes.ShowEdges(TubeSize = 0.05)
#    showmeshes.Show()
# save image
#    showmeshes.saveTIFFImage("junk.tiff")
# display color maps
    scalars = zeros(len(pyelems),Float)
    for i in xrange(len(pyelems)):
        scalars[i] = float(i)/len(pyelems)*0.5
    showcolormap = Mesh2DDisplay(Delaunay=delny, CellColorMap=scalars, ColorMapTitle = "My Color")        
    showcolormap.ShowEdges(TubeSize = 0.05)
    showcolormap.Show()      
