// ListOfSlavedAtoms.h: interface for the ListOfSlavedAtoms class.
//
//////////////////////////////////////////////////////////////////////

#ifndef LISTOFSLAVEDATOMS_H
#define LISTOFSLAVEDATOMS_H

#include "DigitalMaterial.h"
#include "ListOfAtoms.h"
#include "DMArray.h"
#include "Element.h"

class QCListOfAtoms;

//! This is the list of atoms for the slaves in the quasicontinuum module. 
/** The slave atoms belong to an Element, which they ask for the value of the shape 
* functions at their reference position. 
*/


class ListOfSlavedAtoms : public ListOfAtoms  
{
public:
    ListOfSlavedAtoms(QCListOfAtoms* masters);
    virtual ~ListOfSlavedAtoms();
    
    const Element* GetElementsPointer() const { return elements; }
    const DMArray<int>* GetElementIndices() const { return &elementIndices; }
    
    void GetShapeFunctions(int i, double shapeValue[DIMENSION+1])
        {elements[i].ComputeShapeFunctionsAtQueryLatticeIndices(latticeIndicesInsideElement[i], shapeValue);}
    
//    const Element* GetElement(int i) {return elems[i];}
    void SetElementsPointer(const Element* elems) {elements = elems;}
    void SetElement(int i, int elem, int latticeCoords[DIMENSION]);
//     void SetLatticeIndicesInsideElement(const int indices[DIMENSION], int index)
//     {
//         for(int coord = 0; coord < DIMENSION; coord++)
//             latticeIndicesInsideElement[index][coord] = indices[coord];
//     }
//     void SetLatticeIndicesInsideElement(const DMArray<int, DIMENSION>* latticeIndicesInsideElement) {
//         this->latticeIndicesInsideElement.Copy(latticeIndicesInsideElement) ;}
    DMArray<int, DIMENSION>* GetLatticeIndicesInsideElement(){
        return &latticeIndicesInsideElement;}
    
    //! We haven't decided how to Copy, Save or Load a ListOfSlavedAtoms because it isn't clear what to do with the Element vector
    virtual ListOfAtoms *Copy()  {throw DMException("ListOfSlavedAtoms", "Copy",
         "Copying a ListOfSlavedAtoms is undefined");}
    void Save(DMWriter *writer);
    void Load(DMReader *reader);
    virtual void Clear();
    virtual void RemoveSubList(ListOfAtoms *atomsToRemove);
    //! We check that the constraint we set is a QCSlaveConstraint. 
    virtual void SetConstraint(Constraint *constraint) ;
    void DistributeData(const double* elementData, double* slaveData);
    
    virtual void Merge(const ListOfAtoms *atoms) ;

    virtual void Attach(ListOfAtoms *atoms) ;

protected:
    virtual void Allocate(const Shape &newShape);   
    
private:
    //! We have decided that the slaves should know their masters.
    QCListOfAtoms *masters;
    const Element* elements;
    DMArray<int> elementIndices;
    DMArray<int, DIMENSION> latticeIndicesInsideElement;
};

#endif // !defined(LISTOFSLAVEDATOMS_H)
