// QCSlaveConstraint.cpp: implementation of the QCSlaveConstraint class.
//
//////////////////////////////////////////////////////////////////////
#include "DigitalMaterial.h"
#include "QCSlaveConstraint.h"
#include "BoundaryConditions.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

QCSlaveConstraint::QCSlaveConstraint(ListOfSlavedAtoms *slaves, const QCListOfAtoms *masters)
{
    this->slaves = slaves;
    this->masters = masters ;
}

QCSlaveConstraint::~QCSlaveConstraint()
{
}


void QCSlaveConstraint::AdjustPositions(DMArray<double,DIMENSION> *positions)
{
    const DMArray<double,DIMENSION>* mastersPos = masters->GetCartesianPositions();
    DMArrayIterator<double,DIMENSION> slavePosIt(positions);
    DMArrayIterator<int,DIMENSION> slaveRefIt(slaves->GetLatticeIndicesInsideElement());
//    DMArrayIterator<const Element *> slaveElemIt(slaves->GetElements());
    const Element* elements = slaves->GetElementsPointer();
    DMArrayIterator<int, 1> slaveElemIt(slaves->GetElementIndices());
    positions->Zero();
    //	std::cout << "slavesIter.Size(): " << slavesIter.Size() << " " << slavesRefIter.Size() << std::endl;
    slavePosIt.Init();
    slaveRefIt.Init();
    slaveElemIt.Init();
    while(!slavePosIt.IsDone())
    {
        const Element *elem = &elements[slaveElemIt.Current()];
        const int numNodes = DIMENSION + 1;
        const double* masterPos0 = (*mastersPos)[elem->GetNode(0)] ;
        double shapeFunctions[DIMENSION+1];
        elem->ComputeShapeFunctionsAtQueryLatticeIndices(slaveRefIt.Current(), shapeFunctions);
        double R[DIMENSION][DIMENSION];
        for(int node=1; node<numNodes; ++node)
        {
            const double* masterPos = (*mastersPos)[elem->GetNode(node)] ;
            for(int coord=0; coord<DIMENSION; ++coord)
                R[node-1][coord] = masterPos[coord] - masterPos0[coord];
        }
        masters->GetBoundaryConditions()->DifferenceBoundaryConditions(R, DIMENSION);
        double* current = slavePosIt.Current();
        for(int coord=0; coord<DIMENSION; ++coord)
            current[coord] = masterPos0[coord];
        for(int i=1; i<numNodes; ++i)
            for(int coord=0; coord<DIMENSION; ++coord)
                current[coord] += shapeFunctions[i]*R[i - 1][coord];
        //		std::cout << index << " " << slavesIter.Current()[0] << " " << slavesIter.Current()[1] <<std::endl; 
      	slavePosIt.Next();
        slaveRefIt.Next();
        slaveElemIt.Next();
    }
    masters->GetBoundaryConditions()->EnforceBoundaryConditions(positions);

}

