// Refiner.h: abstract interface for the Refiner class.
//
//////////////////////////////////////////////////////////////////////

#ifndef REFINER_H
#define REFINER_H

#include <set>
#include <list>
#include <vector>
#include "DigitalMaterial.h"

class Element;
class MeshData;
class PolyClusterInitializer;
class Refiner
{
 public:
    Refiner(MeshData &meshData);
    virtual ~Refiner();
    void Refine();
 private:
#ifndef SWIG
    MeshData &meshData;
    class Cluster
	{
	public:
	    struct Master
	    {
		int id;
		int index[DIMENSION];
		inline bool operator==(const Master &master)const;
		bool operator<(const Master &master) const {return (id < master.id);}
	    };
	    Cluster(Element *elem);
	    void Merge(const Cluster &cluster);
	    std::vector<std::set<int> > GetExternalFaces() const;
	    std::set<Element *> elems;
	    std::set<Master> masters;
	};
    
    std::list<Cluster> clusters;
    void CreateClustersOfElementsToRefine();
    void RefineCluster(Cluster &cluster);
    bool FindClosestLatticeSiteInCluster(double point[DIMENSION],
					 PolyClusterInitializer &poly);
    
#endif //SWIG
};

#endif // REFINER_H
